@@ -31,6 +31,7 @@
 class AV_View;
 class XAP_BeOSApp;
 class XAP_BeOSFrame;
+class BPopUpMenu;
 
 /*****************************************************************/
 
@@ -41,11 +42,33 @@ public:
 				const char * szMenuLayoutName,
 				const char * szMenuLabelSetName);
 	~EV_BeOSMenu(void);
-	UT_Bool synthesize(void);
+	UT_Bool synthesize();
+	UT_Bool synthesizeMenuBar();
 
 protected:
 	XAP_BeOSApp 		*m_pBeOSApp;
 	XAP_BeOSFrame 		*m_pBeOSFrame;
+	
+	class BMenuBar* pMenuBar;
+};
+
+/*****************************************************************/
+
+class EV_BeOSMenuPopup : public EV_BeOSMenu
+{
+public:
+	EV_BeOSMenuPopup(XAP_BeOSApp * pWin32App, XAP_BeOSFrame * pBeOSFrame,
+					  const char * szMenuLayoutName,
+					  const char * szMenuLabelSetName);
+	~EV_BeOSMenuPopup(void);
+
+	UT_Bool				synthesizeMenuPopup(XAP_BeOSFrame * pFrame);
+	
+	BPopUpMenu* GetHandle();
+	
+protected:
+	
+	BPopUpMenu* m_pPopup;
 };
 
 #endif /* EV_BEOSMENU_H */
