@@ -35,9 +35,9 @@
 #include "xap_UnixToolbar_Icons.h"
 #include "ev_UnixToolbar_ViewListener.h"
 #include "xav_View.h"
-#include "xmlparse.h"
 #include "xap_Prefs.h"
-
+#include "xap_EncodingManager.h"
+#include"ap_Toolbar_Id.h"
 
 /*****************************************************************/
 #define COMBO_BUF_LEN 256
@@ -95,17 +95,19 @@ public:									// we create...
 			}
 			else // widget has no ->parent, so use the buffer's results
 			{
-				UT_uint32 size = strlen(wd->m_comboEntryBuffer);
 
 				// TODO : do a real conversion to UT_UCSChar or figure out the casting
 
 				// don't do changes for empty combo texts
 				if (UT_stricmp(wd->m_comboEntryBuffer, ""))
 				{
-					UT_UCSChar * text = (UT_UCSChar *) wd->m_comboEntryBuffer;
-					UT_ASSERT(text);
-
-					wd->m_pUnixToolbar->toolbarEvent(wd, text, size);
+					UT_UCSChar * text = (UT_UCSChar *) 
+					    (wd->m_id == AP_TOOLBAR_ID_FMT_SIZE ? 
+					    XAP_EncodingManager::fontsizes_list.getFirst(wd->m_comboEntryBuffer) :
+					    wd->m_comboEntryBuffer);
+					
+					UT_ASSERT(text);					
+					wd->m_pUnixToolbar->toolbarEvent(wd, text, strlen((char*)text));
 				}
 			}
 		}
@@ -142,6 +144,7 @@ EV_UnixToolbar::EV_UnixToolbar(XAP_UnixApp * pUnixApp, XAP_UnixFrame * pUnixFram
 	m_pUnixApp = pUnixApp;
 	m_pUnixFrame = pUnixFrame;
 	m_pViewListener = 0;
+	m_wToolbar = 0;
 	m_lid = 0;							// view listener id
 }
 
@@ -600,9 +603,12 @@ UT_Bool EV_UnixToolbar::refreshToolbar(AV_View * pView, AV_ChangeMask mask)
 					// Block the signal, set the contents
 					bool wasBlocked = wd->m_blockSignal;
 					wd->m_blockSignal = true;
-					if (szState)
-						gtk_entry_set_text(GTK_ENTRY(item->entry), szState);
-					else
+					if (szState) {
+						gtk_entry_set_text(GTK_ENTRY(item->entry), 
+						    wd->m_id==AP_TOOLBAR_ID_FMT_SIZE ?
+						    XAP_EncodingManager::fontsizes_list.getSecond(szState) 
+						    : szState);
+					} else
 						gtk_entry_set_text(GTK_ENTRY(item->entry), "");
 					wd->m_blockSignal = wasBlocked;
 					
@@ -647,3 +653,15 @@ XAP_UnixFrame * EV_UnixToolbar::getFrame(void)
 {
 	return m_pUnixFrame;
 }
+
+void EV_UnixToolbar::show(void)
+{
+	if (m_wToolbar)
+		gtk_widget_show (m_wToolbar->parent);
+}
+
+void EV_UnixToolbar::hide(void)
+{
+	if (m_wToolbar)
+		gtk_widget_hide (m_wToolbar->parent);
+}
