@@ -1,3 +1,5 @@
+/* -*- mode: C++; tab-width: 4; c-basic-offset: 4; -*- */
+
 /* AbiSource Program Utilities
  * Copyright (C) 1998-2000 AbiSource, Inc.
  * 
@@ -17,13 +19,7 @@
  * 02111-1307, USA.
  */
  
-//extern "C" {
 #include <gnome.h>
-#ifdef HAVE_BONOBO
-  // #include <libgnorba/gnorba.h>
-#include <bonobo.h>
-#endif
-//}
 
 #include <string.h>
 #include <stdio.h>
@@ -44,6 +40,11 @@
 #include "ev_Menu_Labels.h"
 #include "ev_EditEventMapper.h"
 
+// hack to get the print-preview menu icon
+#include "tb_menu_print_preview.xpm"
+
+static UT_Bool s_init = UT_FALSE;
+
 /*****************************************************************/
 
 struct __Aux {
@@ -143,6 +144,34 @@ EV_UnixGnomeMenu::EV_UnixGnomeMenu(XAP_UnixApp * pUnixApp,
 	: EV_UnixMenu(pUnixApp, pUnixFrame, szMenuLayoutName, szMenuLabelSetName)
 {
 	m_pUIInfo = NULL;
+
+	if(!s_init)
+		{
+			/* register non-standard pixmaps with the gnome-stock engine */
+			s_init = UT_TRUE;
+
+			static struct AbiWordStockPixmap{
+				int width, height;
+				char const * const name;
+				gchar **xpm_data;
+			} const entry_names [] = {
+				{ 16, 16, "Menu_AbiWord_PrintPreview", tb_menu_print_preview_xpm },
+				{ 0, 0, NULL, NULL}
+			};
+
+			static GnomeStockPixmapEntry entry[sizeof(entry_names)/sizeof(struct AbiWordStockPixmap)-1];
+
+			int i = 0;
+
+			for (i = 0; entry_names[i].name != NULL ; i++) {
+				entry[i].data.type = GNOME_STOCK_PIXMAP_TYPE_DATA;
+				entry[i].data.width = entry_names[i].width;
+				entry[i].data.height = entry_names[i].height;
+				entry[i].data.xpm_data = entry_names[i].xpm_data;
+				gnome_stock_pixmap_register (entry_names[i].name,
+											 GNOME_STOCK_PIXMAP_REGULAR, entry + i);
+			}
+		}
 }
 
 EV_UnixGnomeMenu::~EV_UnixGnomeMenu(void)
@@ -152,7 +181,15 @@ EV_UnixGnomeMenu::~EV_UnixGnomeMenu(void)
 
 /**********************************************************************/
 
-void EV_UnixGnomeMenu::s_getStockPixmapFromName (const char *name, char *pixmap_name, int n)
+/**
+ * This function tries to find the "gnome_stock" name of the pixmap
+ * associate to the name of the menu entry.  Usually the name of the
+ * gnome stock pixmap is only Menu_"name", but there are some exceptions
+ * that are listed in the static exceptions var.
+ */
+void EV_UnixGnomeMenu::s_getStockPixmapFromName (const char *name,
+												 char *pixmap_name,
+												 int n)
 {
 	char *true_name;
 	char *tmp;
@@ -169,6 +206,8 @@ void EV_UnixGnomeMenu::s_getStockPixmapFromName (const char *name, char *pixmap_
 		{"_Spelling...", GNOME_STOCK_MENU_SPELLCHECK},
 		{"_Options...", GNOME_STOCK_MENU_PREF},
 		{"Date and _Time...", GNOME_STOCK_MENU_TIMER},
+		{"Page Set_up...", GNOME_STOCK_MENU_PRINT},
+		{"Print P_review", "Menu_AbiWord_PrintPreview"},
 		{"_About", GNOME_STOCK_MENU_ABOUT},
 		{NULL, NULL}
 	};
@@ -195,6 +234,12 @@ void EV_UnixGnomeMenu::s_getStockPixmapFromName (const char *name, char *pixmap_
 	g_free (true_name);
 }
 
+/**
+ * Callback called when the user "selects" a menu item (ie. the mouse pointer
+ * is over this menu item).  It prints the statusbar message in the status bar.
+ * The statusbar message of an item is saved in his associated hash table, with
+ * the key "abi_statusbar_text".
+ */
 void EV_UnixGnomeMenu::s_onMenuItemSelect(GtkWidget * widget, gpointer data)
 {
 	__Aux *aux = (__Aux *) data;
@@ -211,6 +256,10 @@ void EV_UnixGnomeMenu::s_onMenuItemSelect(GtkWidget * widget, gpointer data)
 	pFrame->setStatusMessage(szMsg);
 };
 
+/**
+ * Callback called when the user "deselects" a menu item (ie. the mouse pointer
+ * leaves this menu item).  It erases the menubar.
+ */
 void EV_UnixGnomeMenu::s_onMenuItemDeselect(GtkWidget * widget, gpointer data)
 {
 	__Aux *aux = (__Aux *) data;
@@ -222,6 +271,9 @@ void EV_UnixGnomeMenu::s_onMenuItemDeselect(GtkWidget * widget, gpointer data)
 	pFrame->setStatusMessage(NULL);
 };
 
+/**
+ * Callback called each time the menu is displayed.
+ */
 void EV_UnixGnomeMenu::s_onInitMenu(GtkMenuItem * menuItem, gpointer data)
 {
 	__Aux *aux = (__Aux *) data;
@@ -234,6 +286,9 @@ void EV_UnixGnomeMenu::s_onInitMenu(GtkMenuItem * menuItem, gpointer data)
 	aux->me->refreshMenu(pFrame->getCurrentView ());
 };
 
+/**
+ * Callback called each time that one menu disappears.
+ */
 void EV_UnixGnomeMenu::s_onDestroyMenu(GtkMenuItem * menuItem, gpointer data)
 {
 	__Aux *aux = (__Aux *) data;
@@ -259,23 +314,30 @@ void EV_UnixGnomeMenu::s_onDestroyPopupMenu(GtkMenuItem * menuItem, gpointer cal
 };
 #endif
 
-void EV_UnixGnomeMenu::_convertString2Accel(const char *str, guint &accel_key, GdkModifierType &ac_mods)
+/**
+ * This subroutine calcules the gdk accel_key, ac_mods associated to
+ * a given string (for instance, str = "Ctrl+A" -> accel_key = 'A'
+ * ac_mods = GDK_CONTROL_MASK)
+ */
+void EV_UnixGnomeMenu::_convertString2Accel(const char *str,
+											guint &accel_key,
+											GdkModifierType &ac_mods)
 {
 	if (str == NULL || *str == '\0')
 		return;
 
 	if (strncmp (str, "Ctrl+", 5) == 0) {
-		ac_mods |= GDK_CONTROL_MASK;
+		ac_mods = (GdkModifierType) (ac_mods | GDK_CONTROL_MASK);
 		str += 5;
 	}
 
 	if (strncmp (str, "Alt+", 4) == 0) {
-		ac_mods |= GDK_MOD1_MASK;
+		ac_mods = (GdkModifierType) (ac_mods | GDK_MOD1_MASK);
 		str += 4;
 	}
 
 	if (strncmp (str, "Shift+", 6) == 0) {
-		ac_mods |= GDK_SHIFT_MASK;
+		ac_mods = (GdkModifierType) (ac_mods | GDK_SHIFT_MASK);
 		str += 6;
 	}
 
@@ -292,6 +354,13 @@ void EV_UnixGnomeMenu::_convertString2Accel(const char *str, guint &accel_key, G
 	}
 }
 
+/**
+ * This function does the "hard" work.  It converts the "AbiSource menu" into
+ * a GnomeUIInfo structure (the "gnome format" to code the menus).  The function
+ * starts the conversion in the @pos "item" (an item can be a menu item,
+ * a submenu, etc...) of the menu.  Due to the recursive nature of this function,
+ * the pos variable is passed by reference... <-- TODO: Explain that...
+ */
 GnomeUIInfo * EV_UnixGnomeMenu::_convertMenu2UIInfo (int &pos)
 {
 	const EV_Menu_ActionSet * pMenuActionSet = m_pUnixApp->getMenuActionSet();
@@ -336,7 +405,7 @@ GnomeUIInfo * EV_UnixGnomeMenu::_convertMenu2UIInfo (int &pos)
 				if ( !pAction->isCheckable() ) {
 					retval[i].type = GNOME_APP_UI_ITEM;
 					retval[i].pixmap_type = GNOME_APP_PIXMAP_STOCK;
-					retval[i].pixmap_info = g_new (char, 64);
+					retval[i].pixmap_info = g_new0 (char, 64);
 					s_getStockPixmapFromName (buf, (char *) retval[i].pixmap_info, 64);
 				}
 				else {
@@ -346,7 +415,7 @@ GnomeUIInfo * EV_UnixGnomeMenu::_convertMenu2UIInfo (int &pos)
 				retval[i].label = g_strdup (buf);
 				retval[i].hint = g_strdup (tooltip);
 				retval[i].moreinfo = (void*)menuEvent;
-				retval[i].user_data = g_malloc (sizeof(__Aux));
+				retval[i].user_data = g_new0 (__Aux, 1);
 				((__Aux *) retval[i].user_data)->me = this;
 				((__Aux *) retval[i].user_data)->id = id;
 			}
@@ -366,16 +435,16 @@ GnomeUIInfo * EV_UnixGnomeMenu::_convertMenu2UIInfo (int &pos)
 			if (szLabelName && *szLabelName)
 			{
 				char buf[1024];
-				const char *tooltip;
+				//const char *tooltip;
 				
 				// convert label into underscored version
 				_ev_convert(buf, szLabelName);
-				tooltip = pLabel->getMenuStatusMessage();
+				//tooltip = pLabel->getMenuStatusMessage();
 
 				retval[i].type = GNOME_APP_UI_SUBTREE;
 				retval[i].label = g_strdup (buf);
 				
-				retval[i].user_data = g_malloc (sizeof(__Aux));
+				retval[i].user_data = g_new0 (__Aux, 1);
 				((__Aux *) retval[i].user_data)->me = this;
 				((__Aux *) retval[i].user_data)->id = id;
 				retval[i].moreinfo = _convertMenu2UIInfo (++pos);
@@ -409,11 +478,14 @@ GnomeUIInfo * EV_UnixGnomeMenu::_convertMenu2UIInfo (int &pos)
 		pos++;
 	}
 
-	g_renew(GnomeUIInfo, retval, i + 1);
+	retval = g_renew(GnomeUIInfo, retval, i + 1);
 	  
 	return (retval);
 }
 
+/**
+ * Free the mem associate with the uiinfo structure (in a recursive way).
+ */
 void EV_UnixGnomeMenu::_destroyUIInfo(GnomeUIInfo * uiinfo)
 {
 	g_return_if_fail (uiinfo != NULL);
@@ -464,7 +536,7 @@ UT_Bool EV_UnixGnomeMenu::_refreshMenu(AV_View * pView, GtkWidget * wMenuRoot)
 		{
 		case EV_MLF_Normal:
 		{
-			// TODO: Dinamic labels and widgets
+			// TODO: Dynamic labels and widgets
 			// see if we need to enable/disable and/or check/uncheck it.
 			UT_Bool bEnable = UT_TRUE;
 			UT_Bool bCheck = UT_FALSE;
@@ -481,7 +553,7 @@ UT_Bool EV_UnixGnomeMenu::_refreshMenu(AV_View * pView, GtkWidget * wMenuRoot)
 			// Get the dynamic label
 			const char ** data = _ev_GetLabelName(m_pUnixApp, m_pUnixFrame, pAction, pLabel);
 			const char * szLabelName = data[0];
-			
+
 			if (szLabelName && *szLabelName)
 			{
 				char buf[1024];
@@ -498,8 +570,7 @@ UT_Bool EV_UnixGnomeMenu::_refreshMenu(AV_View * pView, GtkWidget * wMenuRoot)
 			if (GTK_IS_CHECK_MENU_ITEM(item))
 				GTK_CHECK_MENU_ITEM(item)->active = bCheck;
 			
-			gtk_widget_set_sensitive(GTK_WIDGET(item), bEnable);
-			
+			gtk_widget_set_sensitive(GTK_WIDGET(item), bEnable);			
 			break;
 		}
 		case EV_MLF_BeginSubMenu:
@@ -564,6 +635,10 @@ void EV_UnixGnomeMenu::_attachWidgetsAndSignals(GtkWidget * wMenuRoot, GnomeUIIn
 		}
 
 		if (uiinfo->type == GNOME_APP_UI_SUBTREE) {
+			// hack - refresh all of the menus whenever one gets selected
+			// because we can't refresh just one
+			gtk_signal_connect(GTK_OBJECT(((GnomeUIInfo *) uiinfo->widget)), "select",
+							   GTK_SIGNAL_FUNC(s_onInitMenu), uiinfo->user_data);		
 			gtk_signal_connect(GTK_OBJECT(((GnomeUIInfo *) uiinfo->moreinfo)->widget), "map",
 							   GTK_SIGNAL_FUNC(s_onInitMenu), uiinfo->user_data);
 			gtk_signal_connect(GTK_OBJECT(((GnomeUIInfo *) uiinfo->moreinfo)->widget), "unmap",
