@@ -20,7 +20,7 @@
 #ifndef EV_UNIXGNOMETOOLBAR_H
 #define EV_UNIXGNOMETOOLBAR_H
 
-#include <gnome.h>
+#include <gtk/gtkwidget.h>
 #include "ut_types.h"
 #include "ut_vector.h"
 #include "xap_Types.h"
@@ -44,6 +44,20 @@ public:
 	virtual ~EV_UnixGnomeToolbar(void);
 
 	virtual UT_Bool synthesize(void);
+	virtual void show(void);
+	virtual void hide(void);
+	virtual UT_Bool refreshToolbar(AV_View * pView, AV_ChangeMask mask);
+
+	// consider this function protected - it's needed in a callback
+	virtual void _orient_changed(GtkToolbar *toolbar,
+				GtkOrientation dir);
+private:
+	GtkWidget *_makeToolbar(void);
+	UT_Bool _addToolbar (GtkWidget *toolbar);
+
+	static int nbBands;
+	int nbToolbarsInBand;
+	UT_Vector m_vecToolbars;
 };
 
 #endif /* EV_UNIXGNOMETOOLBAR_H */
