@@ -45,15 +45,15 @@
 EV_Win32Toolbar::EV_Win32Toolbar(XAP_Win32App * pWin32App, XAP_Win32Frame * pWin32Frame,
 								 const char * szToolbarLayoutName,
 								 const char * szToolbarLabelSetName)
-	: EV_Toolbar(pWin32App->getEditMethodContainer(),
-				 szToolbarLayoutName,
-				 szToolbarLabelSetName)
+:	EV_Toolbar(pWin32App->getEditMethodContainer(),
+				szToolbarLayoutName,
+				szToolbarLabelSetName),
+	m_pWin32App(pWin32App),
+	m_pWin32Frame(pWin32Frame),
+	m_pViewListener(NULL),
+	m_lid(0),				// view listener id
+	m_hwnd(0)
 {
-	m_pWin32App = pWin32App;
-	m_pWin32Frame = pWin32Frame;
-	m_pViewListener = NULL;
-	m_lid = 0;							// view listener id
-	m_hwnd = NULL;
 }
 
 EV_Win32Toolbar::~EV_Win32Toolbar(void)
@@ -165,8 +165,17 @@ LRESULT CALLBACK EV_Win32Toolbar::_ComboWndProc( HWND hWnd, UINT uMessage, WPARA
 					// yep, we're done 
 					SendMessage(hWnd, CB_SHOWDROPDOWN, (WPARAM) FALSE, 0);
 
-					// now that we know dropdown is gone, this should be ok
-					PostMessage(hWnd, WM_KEYDOWN, VK_RETURN, 0);
+					UT_sint32 iSelected = SendMessage(hWnd, CB_GETCURSEL, 0, 0);
+					
+					if(iSelected != -1)
+						{
+						// now that we know dropdown is gone, this should be ok
+						PostMessage(hWnd, WM_KEYDOWN, VK_RETURN, 0);
+						}
+					else
+						{
+						PostMessage(hWnd, WM_KEYDOWN, VK_ESCAPE, 0);
+						}
 					break;
 				}
 			}
@@ -219,6 +228,7 @@ LRESULT CALLBACK EV_Win32Toolbar::_ComboEditWndProc( HWND hWnd, UINT uMessage, W
 			break;
 		}
 
+
 		case WM_KILLFOCUS:
 		{
 			// for now, we fire an event any time we lose focus
@@ -234,7 +244,14 @@ LRESULT CALLBACK EV_Win32Toolbar::_ComboEditWndProc( HWND hWnd, UINT uMessage, W
 
 			UT_UCSChar * pData = (UT_UCSChar *) buf;	// HACK: should be void *
 			UT_uint32 dataLength = GetWindowText(hWnd, buf, COMBO_BUF_LEN);
-			t->toolbarEvent(id, pData, dataLength);
+			if(dataLength)
+				{
+				t->toolbarEvent(id, pData, dataLength);
+				}
+			else
+				{
+				SendMessage(hWnd, WM_KEYDOWN, VK_ESCAPE, 0);
+				}
 			break;
 		}
 
@@ -544,12 +561,13 @@ UT_Bool EV_Win32Toolbar::synthesize(void)
 					if (bIcons)
 					{
 						HBITMAP hBitmap;
-						UT_Bool bFoundIcon = m_pWin32ToolbarIcons->getBitmapForIcon(m_hwnd,
-																					MY_MAXIMUM_BITMAP_X,
-																					MY_MAXIMUM_BITMAP_Y,
-																					&backgroundColor,
-																					pLabel->getIconName(),
-																					&hBitmap);
+						const UT_Bool bFoundIcon =
+							AP_Win32Toolbar_Icons::getBitmapForIcon(m_hwnd,
+																	MY_MAXIMUM_BITMAP_X,
+																	MY_MAXIMUM_BITMAP_Y,
+																	&backgroundColor,
+																	pLabel->getIconName(),
+																	&hBitmap);
 						UT_ASSERT(bFoundIcon);
 						TBADDBITMAP ab;
 						ab.hInst = 0;
