@@ -40,6 +40,14 @@
 #define DRAW_START DrawSetup();
 #define DRAW_END   DrawTeardown();
 
+/* Global font cache */
+struct _fcache {
+	struct _fcache *next;
+	struct _fcache *prev;
+	char 		   name[36];
+} *g_pFCache = NULL;	
+
+
 int GR_QNXGraphics::DrawSetup() {
 
 	//Don't do any of this if you are printing
@@ -102,14 +110,13 @@ int GR_QNXGraphics::DrawTeardown() {
 	m_OffsetPoint.y *= -1; 									
 	PgSetTranslation(&m_OffsetPoint, 0); 						
 
-	/* Debugging  
-	PgFlush();
-	*/
 	return 0;
 }
 
-GR_QNXGraphics::GR_QNXGraphics(PtWidget_t * win, PtWidget_t * draw)
+
+GR_QNXGraphics::GR_QNXGraphics(PtWidget_t * win, PtWidget_t * draw, XAP_App *app)
 {
+	m_pApp = app;
 	m_pWin = win;
 	m_pDraw = draw;
 	m_pFont = NULL;
@@ -118,6 +125,7 @@ GR_QNXGraphics::GR_QNXGraphics(PtWidget_t * win, PtWidget_t * draw)
 	m_iLineWidth = 1;
 	m_currentColor = Pg_BLACK;
 	m_pPrintContext = NULL;
+	m_iAscentCache = m_iDescentCache = -1;
 
 	m_cs = GR_Graphics::GR_COLORSPACE_COLOR;
 	m_cursor = GR_CURSOR_INVALID;
@@ -151,75 +159,62 @@ void GR_QNXGraphics::drawChars(const UT_UCSChar* pChars, int iCharOffset,
 				 int iLength, UT_sint32 xoff, UT_sint32 yoff)
 {
 	PhPoint_t pos;
+	int i,j;
 	
 	pos.x = xoff; 
 	pos.y = yoff + getFontAscent();
 
 	DRAW_START
 
-//	printf("Drawing with font [%s] \n", m_pFont->getFont());
 	PgSetFont(m_pFont->getFont());
 	PgSetTextColor(m_currentColor);
 	PgFlush();		/* Just to clear out any drawing to be done ... */
 
-#if 0
-	char      *buffer;
-
-	if (!(buffer = (char *)malloc(iLength + 1))) {
-		printf("Failed buffer allocation \n");
-		return;
+#if 0	//Our widecharacter support doesn't work for printing
+	UT_UCSChar *pNChars, utb[150];  // arbitrary biggish size for utb
+	if (iLength < (sizeof(utb) / sizeof(utb[0]))) {
+		pNChars = utb;
+	} else {
+		pNChars = new UT_UCSChar[iLength];
+	}
+	for (i = 0; i < iLength; i++) {
+		pNChars[i] = remapGlyph(pChars[i + iCharOffset], UT_FALSE);
 	}
 
-	int i;
-	for (i = 0; i<iLength; i++) {
-		buffer[i] = (char)pChars[i];	//Bad I know convert from UTC later
-	}
-	buffer[i] = '\0';
-	
-	
-	//For some reason, I _NEED_ to set the BACK_FILL flag, and
-	//if I don't set the fill color things just work fine because
-	//the last fill that happened was to wipe out the text area
-	//with the appropriate color. 
-
-	//This method is no more than a stop gap.  It fails for use
-	//on the rulers and it fails for use when scrolling text.
-	//A new method must be found!
- 
-	//This is the prefered way
-	//PgDrawTextmx(buffer, iLength, &pos, 0);
-
-	//This way would be okay if it worked
-	//PgSetFillColor(Pg_TRANSPARENT);	
-	//PgDrawTextmx(buffer, iLength, &pos, Pg_BACK_FILL);
-
-	//This way is a total kludge (assumes last draw was for background)
-	//PgSetFillColor(Pg_PURPLE);
-	//PgSetStrokeColor(Pg_PURPLE);
-	//PgDrawTextmx(buffer, iLength, &pos, Pg_BACK_FILL);
-
-	//This way is even more of a kludge
-	//PgSetFillColor(Pg_CYAN);
-	//PgSetStrokeColor(Pg_CYAN);
-	//PgChromaOn();
-	//PgSetChroma(Pg_CYAN, Pg_CHROMA_DEST_MATCH | Pg_CHROMA_NODRAW); 
-	//PgDrawTextmx(buffer, iLength, &pos, Pg_BACK_FILL);
-	//PgChromaOff();
-
-
-	//This way seems to work ok, good to use
-	PgDrawTextmx(buffer, iLength, &pos, 0);
-	printf("Drawing %d,%d [%s] \n", pos.x, pos.y, buffer);
-
-	//We always need to flush the buffer since we free it
-	PgFlush();
-	free(buffer);	
+	//Was: (char *)(&pChars[iCharOffset]), iLength*sizeof(UT_UCSChar), 
+	PgDrawTextmx((char *)pNChars, iLength*sizeof(UT_UCSChar), &pos, Pg_TEXT_WIDECHAR);
 #else
-	PgDrawTextmx((char *)(&pChars[iCharOffset]), iLength*sizeof(UT_UCSChar), 
-		   		  &pos, Pg_TEXT_WIDECHAR);
+	/* We have no idea in advance how big this result is going 
+	   to convert into, so we will guestimate with the MB_CUR_MAX,
+       which in theory is the max number of characters a wc will expand to.
+	*/
+	char *pNChars, utb[MB_CUR_MAX * 150];  // arbitrary biggish size for utb
+	int  blen, ipos;
 
-	PgFlush();
+	if ((iLength * MB_CUR_MAX) < (sizeof(utb) / sizeof(utb[0]))) {
+		blen = sizeof(utb) / sizeof(utb[0]);
+		pNChars = utb;
+	} else {
+		blen = iLength * MB_CUR_MAX;
+		pNChars = new char[blen];
+	}
+
+	for (i = ipos = 0; i < iLength; i++) {
+		int tlen;
+
+		UT_ASSERT((ipos + MB_CUR_MAX) < blen);
+		tlen = wctomb(&pNChars[ipos], remapGlyph(pChars[i + iCharOffset], UT_FALSE));
+		UT_ASSERT(tlen > 0);
+		ipos += tlen;
+	}
+
+	PgDrawTextmx(pNChars, ipos, &pos, 0);
 #endif
+	PgFlush();
+
+	if (pNChars != utb) {
+		delete [] pNChars;
+	}
 
 	DRAW_END
 }
@@ -233,17 +228,13 @@ void GR_QNXGraphics::drawChar(UT_UCSChar Char, UT_sint32 xoff, UT_sint32 yoff)
 
 void GR_QNXGraphics::setFont(GR_Font * pFont)
 {
-	/*
-  	Fonts listed in /usr/photon/font and take
-	the form of face size [style] passed in
-	to PgSetFont(char *font);
-	*/
 	QNXFont *qnxFont = (QNXFont *)pFont;
 	const char *backupfont = { "helv10" };
 	const char *font;
 
 	if (!qnxFont || !(font = qnxFont->getFont())) {
-		fprintf(stderr, "No font found, using helv10 as default \n");
+		UT_DEBUGMSG(("No font found, using helv10 as default"));
+		UT_ASSERT(0);
 		font = backupfont;
 	}
 
@@ -252,105 +243,91 @@ void GR_QNXGraphics::setFont(GR_Font * pFont)
 		delete(m_pFont);
 	}
 
-	/*
- 	 We need to copy the font here, not just save the pointer
-	 m_pFont = qnxFont;
-	*/
 	m_pFont = new QNXFont(font);
+
+	/* At the same time, load the font metrics into
+       a local cache to speed up access as we use them! */
+	struct _fcache *tmp;
+	for (tmp = g_pFCache; tmp; tmp = tmp->next) {
+		if (strcmp(tmp->name, font) == 0) {
+			break;
+		}
+	}
+	if (tmp == NULL &&
+		(tmp = (struct _fcache *)malloc(sizeof(*tmp))) != NULL) {
+		strcpy(tmp->name, font);
+		tmp->next = g_pFCache;
+		tmp->prev = NULL;
+		g_pFCache = tmp;	
+
+		PfLoadMetrics(font);
+	} else if (tmp && tmp->prev) {
+		if ((tmp->prev->next = tmp->next)) {
+			tmp->next->prev = tmp->prev;
+		}
+		if ((tmp->next = g_pFCache)) {
+			g_pFCache->prev = tmp;
+		}
+		tmp->prev = NULL;
+	}
+
+	m_iAscentCache = m_iDescentCache = -1;
 }
 
 UT_uint32 GR_QNXGraphics::getFontHeight()
 {
-	PhRect_t rect;
-	const char *font;
-
-	if (!m_pFont || !(font = m_pFont->getFont())) {
-		return(0);
-	}
-
-	PfExtentText(&rect, NULL, font, "A", 0);
-	//printf("GR: Height:%d + %d \n", MY_ABS(rect.lr.y), MY_ABS(rect.ul.y));
-	return(MY_ABS(rect.ul.y) + MY_ABS(rect.lr.y) + 1);
+	return getFontAscent() + getFontDescent();
 }
 
-UT_uint32 GR_QNXGraphics::measureString(const UT_UCSChar* s, int iOffset,
-					  int num,  unsigned short* pWidths)
+UT_uint32 GR_QNXGraphics::measureUnRemappedChar(const UT_UCSChar c)
 {
+	PhRect_t rect;
 	const char *font;
-	int  i, charWidth = 0;
-	char c[2], *buffer;
+	char 	 buffer[MB_CUR_MAX + 1];
+	int 	 len, indices, penpos;
 
 	if (!m_pFont || !(font = m_pFont->getFont())) {
-		return(0);
+		return 0;
 	}
-	//printf("GR: Measure string font %s \n", (font) ? font : "NULL");
 
-	//CHANGE THIS TO MAKE IT UNICODE
-	if (num == 1) {
-		buffer = c;
-	}
-	else {
-		if (!(buffer = (char *)malloc(num + 1))) {
-			fprintf(stderr, "Can't allocate memory \n");
-			return(0);
-		}
-	}
-	memset(buffer, 0, num + 1);
+	len = wctomb(buffer, c);
+	UT_ASSERT(len > 0);
+	buffer[len] = '\0';
+	indices = 1;			
+	penpos = 0;			
 
+
+	PfExtentTextCharPositions(&rect, 		/* Rect extent */
+				  NULL,			/* Position offset */
+				  buffer,	   	/* Buffer to hit */
+				  font, 		/* Font buffer uses */
+				  &indices,		/* Where to get pen pos from */
+				  &penpos, 		/* Where to store pen pos */
+				  1,			/* Number of indices */
+				  0,			/* Flags TODO: PF_WIDE_CHARS and save convert? */
+				  len,			/* Length of buffer (0 = use strlen) */
+				  0, 			/* Number of characters to skip */
+				  NULL);		/* Clipping rectangle? */
 #if 0
-	for (i = 0; i < num; i++) {
-		PhRect_t rect;
-
-		buffer[i] = (char)(s[i+iOffset]);
-		PfExtentText(&rect, NULL, font, &buffer[i], 0);
-		pWidths[i] = (rect.lr.x - min(rect.ul.x, 0) + 1) - rect.ul.x;
-		charWidth += pWidths[i];
-		//printf("Width of %s is %d \n", &buffer[i], pWidths[i]);
-	}
-#else
-	PhRect_t rect;
-	int indices, penpos;
-
-	for (i = 0; i < num; i++) {
-
-		buffer[i] = (char)(s[i+iOffset]);
-		memset(&rect, 0, sizeof(rect));
-		indices = 1;
-		penpos = 0;
-
-		PfExtentTextCharPositions(&rect, 			/* Rect extent */
-							 		NULL,			/* Position offset */
-									&buffer[i],		/* Buffer to hit */
-									font, 			/* Font buffer uses */
-									&indices,		/* Where to get pen pos from */
-									&penpos, 		/* Where to store pen pos */
-									1,				/* Number of indices */
-									0,				/* Flags */
-									0,				/* Length of buffer (use strlen) */
-									0, 				/* Number of characters to skip */
-									NULL);			/* Clipping rectangle? */
-		if (pWidths)
-			pWidths[i] = penpos;
-		charWidth += penpos;
-		//printf("Width of %s is %d (%d)\n", &buffer[i], pWidths[i], charWidth);
-	}
-
+	printf("%s : %s gives width %d \n", font, buffer, penpos);
 #endif
 
-	if (num != 1) {
-		free(buffer);
-	}
-
-	//printf("GR: Width:%d \n", charWidth);
-	return charWidth;
+	return penpos;
 }
 
 UT_uint32 GR_QNXGraphics::_getResolution(void) const
 {
+	//Unix comment
 	// this is hard-coded at 100 for X now, since 75 (which
 	// most X servers return when queried for a resolution)
 	// makes for tiny fonts on modern resolutions.
 
+	/*Photon comment
+	   This resolution should be variable between the printer
+	   and the screen.  For now we return a fixed value of 
+	   72 and use that for the source resolution when printing. 
+	*/
+
 	return 72 /* 100 */;
 }
 
@@ -384,7 +361,7 @@ GR_Font * GR_QNXGraphics::findFont(const char* pszFontFamily,
 	UT_ASSERT(pszFontWeight);
 	UT_ASSERT(pszFontSize);
 	
-	UT_DEBUGMSG(("GR: findFont [%s] [%s] [%s] [%s] \n", 
+	UT_DEBUGMSG(("GR: findFont [%s] [%s] [%s] [%s] ", 
 			pszFontFamily, pszFontStyle, pszFontWeight, pszFontSize));
 
 	char fname[MAX_FONT_TAG];
@@ -415,53 +392,49 @@ GR_Font * GR_QNXGraphics::findFont(const char* pszFontFamily,
 			break;
 		}
 	}
-	UT_DEBUGMSG(("Setting to font name [%s] \n", fname));
+	UT_DEBUGMSG(("Setting to font name [%s] ", fname));
 
 	return(new QNXFont(fname));
 }
 
 UT_uint32 GR_QNXGraphics::getFontAscent()
 {
-	FontQueryInfo info;
-	memset(&info, 0, sizeof(info));
+	if(m_iAscentCache == -1) {
+		FontQueryInfo info;
 
-	if (!m_pFont || !m_pFont->getFont()) {
-		UT_ASSERT(0);
-		return(0);
+		if (!m_pFont || !m_pFont->getFont()) {
+			UT_ASSERT(0);
+			return(0);
+		}
+
+		if (PfQueryFont(m_pFont->getFont(), &info) == -1) {
+			UT_ASSERT(0);
+			return(0);
+		}
+		m_iAscentCache = MY_ABS(info.ascender);
 	}
 
-	if (PfQueryFont(m_pFont->getFont(), &info) == -1) {
-		UT_ASSERT(0);
-		return(0);
-	}
-/*
-	printf("Name: [%s] \n", info.font);
-	printf("Desc: [%s] \n", info.desc);
-	printf("Size: [%d] \n", info.size);
-	printf("Style: [0x%x] \n", info.style);
-	printf("Asc:  [%d] \n", info.ascender);
-	printf("Dsc:  [%d] \n", info.descender);
-	printf("Width: [%d] \n", info.width);
-	printf("Range: [0x%x - 0x%x] \n", info.lochar, info.hichar);
-*/		
-	return(MY_ABS(info.ascender));
+	return m_iAscentCache;
 }
 
 UT_uint32 GR_QNXGraphics::getFontDescent()
 {
-	FontQueryInfo info;
+	if (m_iDescentCache == -1) {
+		FontQueryInfo info;
 
-	if (!m_pFont || !m_pFont->getFont()) {
-		UT_ASSERT(0);
-		return(0);
+		if (!m_pFont || !m_pFont->getFont()) {
+			UT_ASSERT(0);
+			return(0);
+		}
+	
+		if (PfQueryFont(m_pFont->getFont(), &info) == -1) {
+			UT_ASSERT(0);
+			return(0);
+		}
+		m_iDescentCache = MY_ABS(info.descender);
 	}
 
-	if (PfQueryFont(m_pFont->getFont(), &info) == -1) {
-		UT_ASSERT(0);
-		return(0);
-	}
-		
-	return(MY_ABS(info.descender));
+	return m_iDescentCache;
 }
 
 void GR_QNXGraphics::drawLine(UT_sint32 x1, UT_sint32 y1,
@@ -542,8 +515,6 @@ void GR_QNXGraphics::setClipRect(const UT_Rect* pRect)
 		r.lr.x = r.ul.x + pRect->width;
 		r.lr.y = r.ul.y + pRect->height;
 
-		//printf("GR: Set clipping %d,%d %d/%d \n", 
-		//	pRect->left, pRect->top, pRect->width, pRect->height);
 		UT_ASSERT(!m_pClipList);		//Only one item for now	
 	
 		if (m_pClipList || (m_pClipList = PhGetTile())) {
@@ -553,7 +524,6 @@ void GR_QNXGraphics::setClipRect(const UT_Rect* pRect)
 	}
 	else
 	{
-		//printf("GR: Clear Clipping \n");
 		if (m_pClipList) {
 			PhFreeTiles(m_pClipList);
 			m_pClipList = NULL;
@@ -572,13 +542,6 @@ void GR_QNXGraphics::fillRect(UT_RGBColor& c, UT_sint32 x, UT_sint32 y,
 	PgColor_t newc;
 
 	//Why do we have to do this, why can't the xap code figure it right?
-	//printf("GR Fill Rect %d,%d %d/%d 0x%x\n", x, y, w, h, newc);
-/*
-	if ((w < 0) || (h < 0) || x < 0 || y < 0) {
-		//printf("BAILING on the fill \n");
-		return;
-	}
-*/
 	w = (w < 0) ? -1*w : w;
 	h = (h < 0) ? -1*h : h;
 
@@ -619,8 +582,10 @@ void GR_QNXGraphics::scroll(UT_sint32 dx, UT_sint32 dy)
 	offset.x = -1*dx;
 	offset.y = -1*dy;
 
+/*
 	UT_DEBUGMSG(("GR Scroll1 Before %d,%d %d,%d  by %d,%d",
 			rect.ul.x, rect.ul.y, rect.lr.x, rect.lr.y, offset.x, offset.y));
+*/
 	
 	//This does the blit on the region, so the rect must be in
 	//the windows co-ordinates.  But it doesn't automatically
@@ -635,8 +600,10 @@ void GR_QNXGraphics::scroll(UT_sint32 dx, UT_sint32 dy)
 	//way easier though.
 	adjust_rect(&rect, &offset);
 
+/*
 	UT_DEBUGMSG(("GR Scroll1 %d,%d %d,%d  by %d,%d",
 			rect.ul.x, rect.ul.y, rect.lr.x, rect.lr.y, offset.x, offset.y));
+*/
 	PhBlit(PtWidgetRid(PtFindDisjoint(m_pDraw)), &rect, &offset);
 	//to get an expose call PtDamageExtent(region_widget, damage_rect)
 }
@@ -645,25 +612,38 @@ void GR_QNXGraphics::scroll(UT_sint32 x_dest, UT_sint32 y_dest,
 						  UT_sint32 x_src, UT_sint32 y_src,
 						  UT_sint32 width, UT_sint32 height)
 {
-//TF NOTE: This is still screwed up ... lots of screen dirt!
-#if 0
 	PhRect_t 	rect, widgetrect;
 	PhPoint_t 	offset;
 
 	PtBasicWidgetCanvas(m_pDraw, &widgetrect);
 
-	rect.ul.x = 
-	rect.lr.x = (width >= 0) ? x_src : x_src + width;
-	rect.ul.y = 
-	rect.lr.y = (height >= 0) ? y_src : y_src + height;
-	rect.lr.x += (width >= 0) ? width : -1*width;
-	rect.lr.y += (height >= 0) ? height : -1*height;
+/*
+	UT_DEBUGMSG(("GR Scroll2 dest:%d,%d src:%d,%d %dx%d ", 
+					x_dest, y_dest, x_src, y_src, width, height));
+*/
+	if (width < 0) {
+		UT_ASSERT(0);
+		x_src -= width;
+		width = -1*width;
+	}
+	if (height < 0) {
+		UT_ASSERT(0);
+		y_src -= height;
+		height = -1*height;
+	}
 
-	offset.x = -1*(x_src - x_dest);
-	offset.y = -1*(y_src - y_dest);
+	rect.ul.x = rect.lr.x = x_src;
+	rect.ul.y = rect.lr.y = y_src;
+	rect.lr.x += width;
+	rect.lr.y += height;
 
+	offset.x = x_dest - x_src;
+	offset.y = y_dest - y_src;
+
+/*
 	UT_DEBUGMSG(("GR Scroll2 Before %d,%d %d,%d  by %d,%d",
 			rect.ul.x, rect.ul.y, rect.lr.x, rect.lr.y, offset.x, offset.y));
+*/
 
 	if (PtRectIntersect(&widgetrect, &rect) == 0) {
 		UT_DEBUGMSG(("No intersection! "));
@@ -677,15 +657,12 @@ void GR_QNXGraphics::scroll(UT_sint32 x_dest, UT_sint32 y_dest,
 	PtWidgetOffset(m_pDraw, &shift);
 	PtTranslateRect(&rect, &shift);					
 
-//	adjust_rect(&rect, &offset);
-
+/*
 	UT_DEBUGMSG(("GR Scroll2 %d,%d %d,%d  by %d,%d",
 			rect.ul.x, rect.ul.y, rect.lr.x, rect.lr.y, offset.x, offset.y));
+*/
 
 	PhBlit(PtWidgetRid(PtFindDisjoint(m_pDraw)), &rect, &offset);
-#else
-	UT_ASSERT(0);
-#endif
 }
 
 void GR_QNXGraphics::clearArea(UT_sint32 x, UT_sint32 y,
@@ -695,19 +672,28 @@ void GR_QNXGraphics::clearArea(UT_sint32 x, UT_sint32 y,
 	fillRect(clrWhite, x, y, width, height);
 }
 
-GR_Image* GR_QNXGraphics::createNewImage(const char* pszName, const UT_ByteBuf* pBBPNG, UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight)
+GR_Image* GR_QNXGraphics::createNewImage(const char* pszName, const UT_ByteBuf* pBBPNG, UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight, GR_Image::GRType iType)
 {
-	GR_QNXImage* pImg = new GR_QNXImage(NULL, pszName);
-
-	pImg->convertFromPNG(pBBPNG, iDisplayWidth, iDisplayHeight);
-
+	/* GR_QNXImage* pImg = NULL; */
+	GR_Image* pImg = NULL;
+   	if (iType == GR_Image::GRT_Raster)
+     		pImg = new GR_QNXImage(pszName);
+   	else
+     		pImg = new GR_VectorImage(pszName);
+   
+	pImg->convertFromBuffer(pBBPNG, iDisplayWidth, iDisplayHeight);
 	return pImg;
 }
 
 void GR_QNXGraphics::drawImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDest)
 {
 	UT_ASSERT(pImg);
-	
+
+   	if (pImg->getType() != GR_Image::GRT_Raster) {
+      		pImg->render(this, xDest, yDest);
+      		return;
+   	}
+   
 	GR_QNXImage * pQNXImage = (GR_QNXImage *)(pImg);
 	Fatmap * image = pQNXImage->getData();
 	PhPoint_t pos;
@@ -816,6 +802,8 @@ void GR_QNXGraphics::setCursor(GR_Graphics::Cursor c)
 	GdkCursor * cursor = gdk_cursor_new(cursor_number);
 	gdk_window_set_cursor(m_pWin, cursor);
 	gdk_cursor_destroy(cursor);
+#else
+	UT_DEBUGMSG(("TODO: Set the cursor type "));
 #endif
 }
 
@@ -903,14 +891,12 @@ void GR_QNXGraphics::setPrintContext(PpPrintContext_t *context) {
 }
 
 UT_Bool GR_QNXGraphics::startPrint(void) {
-
-	/* Somehow I need to get the print context here */
+	UT_DEBUGMSG(("GR: Start print"));
 	UT_ASSERT(m_pPrintContext);
+
 	if (m_pPrintContext) {
-#if 1
-		PpPrintOpen(m_pPrintContext);	
-		PpPrintStart(m_pPrintContext);	
-#endif
+		PpStartJob(m_pPrintContext);	
+		PpContinueJob(m_pPrintContext);	
 	}
 
  	m_bPrintNextPage = UT_FALSE;    
@@ -920,11 +906,15 @@ UT_Bool GR_QNXGraphics::startPrint(void) {
 UT_Bool GR_QNXGraphics::startPage(const char * szPageLabel, UT_uint32 pageNumber,
 									UT_Bool bPortrait, UT_uint32 iWidth, UT_uint32 iHeight) {
 
-	printf("Portrait %d W/H %d/%d \n", bPortrait, iWidth, iHeight);
-
+	UT_DEBUGMSG(("GR: Start Page %d W/H %d/%d (portrait:%d label:%s nextpage:%d)", 
+		pageNumber, iWidth, iHeight, bPortrait, (szPageLabel) ? szPageLabel : "", m_bPrintNextPage));
 	UT_ASSERT(m_pPrintContext);
-	if (m_pPrintContext && !m_bPrintNextPage) {		/* First page do setup */
-		printf("Print first page! \n");
+
+	if (!m_pPrintContext) {
+		return UT_FALSE;
+	}
+
+	if (!m_bPrintNextPage) {		/* First page do setup */
 #if 0
 		PhDim_t 	size;
 
@@ -938,9 +928,7 @@ UT_Bool GR_QNXGraphics::startPage(const char * szPageLabel, UT_uint32 pageNumber
 		PpPrintOpen(m_pPrintContext);	
 		PpPrintStart(m_pPrintContext);	
 #endif
-	}
- 	else if (m_pPrintContext && m_bPrintNextPage) {
-		printf("Print a new page! \n");
+	} else {
 		PpPrintNewPage(m_pPrintContext);
 	}
 
@@ -949,11 +937,11 @@ UT_Bool GR_QNXGraphics::startPage(const char * szPageLabel, UT_uint32 pageNumber
 }
 
 UT_Bool GR_QNXGraphics::endPrint(void) {
-
+	UT_DEBUGMSG(("GR: End print"));
 	UT_ASSERT(m_pPrintContext);
+
 	if (m_pPrintContext) {
-		PpPrintStop(m_pPrintContext);
-		PpPrintClose(m_pPrintContext);
+		PpEndJob(m_pPrintContext);
 	}
 
 	return UT_TRUE;
