@@ -50,7 +50,7 @@ class QNXFont : public GR_Font {
 class GR_QNXGraphics : public GR_Graphics
 {
  public:
-	GR_QNXGraphics(PtWidget_t * win, PtWidget_t * draw);
+	GR_QNXGraphics(PtWidget_t * win, PtWidget_t * draw, XAP_App * app);
 	~GR_QNXGraphics();
 
 	virtual void drawChar(UT_UCSChar Char, UT_sint32 xoff, UT_sint32 yoff);
@@ -58,8 +58,8 @@ class GR_QNXGraphics : public GR_Graphics
 						   int iLength, UT_sint32 xoff, UT_sint32 yoff);
 	virtual void setFont(GR_Font* pFont);
 	virtual UT_uint32 getFontHeight();
-	virtual UT_uint32 measureString(const UT_UCSChar*s, int iOffset, int num,
-									unsigned short* pWidths);
+	virtual UT_uint32 measureUnRemappedChar(const UT_UCSChar c);
+	//virtual UT_uint32 measureString(const UT_UCSChar*s, int iOffset, int num, unsigned short* pWidths);
 	virtual void setColor(UT_RGBColor& clr);
 
 	virtual GR_Font* getGUIFont();
@@ -87,7 +87,7 @@ class GR_QNXGraphics : public GR_Graphics
 	virtual void clearArea(UT_sint32, UT_sint32, UT_sint32, UT_sint32);
   
 	virtual void drawImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDest);
-	virtual GR_Image* createNewImage(const char* pszName, const UT_ByteBuf* pBBPNG, UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight);
+   	virtual GR_Image* createNewImage(const char* pszName, const UT_ByteBuf* pBBPNG, UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight, GR_Image::GRType iType = GR_Image::GRT_Raster);
   
 	virtual UT_Bool queryProperties(GR_Graphics::Properties gp) const;
 	virtual UT_Bool startPrint(void);
@@ -127,11 +127,10 @@ class GR_QNXGraphics : public GR_Graphics
 	QNXFont * 			m_pFontGUI;
   
 	void*  				m_pColormap;
-	int					m_aCharWidths[256];
-	int          		m_iWindowHeight, m_iWindowWidth;
 	UT_sint32			m_iLineWidth;
-	GR_Graphics::Cursor		m_cursor;
+	UT_sint32			m_iAscentCache, m_iDescentCache;
 
+	GR_Graphics::Cursor		m_cursor;
 	GR_Graphics::ColorSpace	m_cs;
 	
 	PgColor_t			m_currentColor;
