@@ -34,9 +34,12 @@
 #include "ut_misc.h"
 #include "ut_string.h"
 #include "ut_dialogHelper.h"
+#include "ut_wctomb.h"
+#include "xap_EncodingManager.h"
 
-GR_UnixGraphics::GR_UnixGraphics(GdkWindow * win, XAP_UnixFontManager * fontManager)
+GR_UnixGraphics::GR_UnixGraphics(GdkWindow * win, XAP_UnixFontManager * fontManager, XAP_App * app)
 {
+	m_pApp = app;
 	m_pWin = win;
 	m_pFontManager = fontManager;
 	m_pFont = NULL;
@@ -92,14 +95,40 @@ UT_Bool GR_UnixGraphics::queryProperties(GR_Graphics::Properties gp) const
 	}
 }
 
+/* let's cache this, since construction of UT_Wctomb is rather slow */
+static UT_Wctomb* w = NULL;
+static char text[MB_LEN_MAX];
+static int text_length;
+static UT_Bool fallback_used;
+
+#define WCTOMB_DECLS \
+	if (!w) {	\
+	    w = new UT_Wctomb;	\
+	} else	\
+	    w->initialize();	
+	    
+#define CONVERT_TO_MBS(c)	\
+    	if (c<=0xff) {	\
+		/* this branch is to allow Lists to function */	\
+		text[0] = (unsigned char)c;			\
+		text_length = 1;				\
+		fallback_used = 0;				\
+	} else	{\
+		fallback_used = 0;	\
+		if (!w->wctomb(text,text_length,(wchar_t)c)) {	\
+		    w->wctomb_or_fallback(text,text_length,(wchar_t)c);	\
+		    fallback_used = 1;	\
+		}	\
+	}	
+
 // HACK: I need more speed
 void GR_UnixGraphics::drawChar(UT_UCSChar Char, UT_sint32 xoff, UT_sint32 yoff)
 {
-	GdkWChar Wide_char = Char;
-	
-	GdkFont *font = m_pFont->getGdkFont();
-	gdk_draw_text_wc (m_pWin, font, m_pGC,
-				   xoff, yoff + font->ascent, &Wide_char, 1);
+	UT_UCSChar Wide_char = remapGlyph(Char, UT_FALSE);
+	WCTOMB_DECLS;
+	CONVERT_TO_MBS(Wide_char);
+	GdkFont *font = m_pFont->getGdkFontForUCSChar(Wide_char);
+	gdk_draw_text(m_pWin,font,m_pGC,xoff,yoff+font->ascent,text,text_length);
 }
 
 void GR_UnixGraphics::drawChars(const UT_UCSChar* pChars, int iCharOffset,
@@ -107,36 +136,23 @@ void GR_UnixGraphics::drawChars(const UT_UCSChar* pChars, int iCharOffset,
 {
 	if (!m_pFontManager)
 		return;
-
 	UT_ASSERT(m_pFont);
-
-	/*
-	  TODO -  We need to seriously look for a way to avoid this.
-	  Doing a memory allocation on every draw is painful.
-	*/
-	GdkFont *font = m_pFont->getGdkFont();
-
-	// Blargh... GDK wants strings in 32 bits, we use 16 internally
-    GdkWChar *pNChars = new GdkWChar[iLength];
-
-	for (int i = 0; i < iLength; i++)
-    {
-		pNChars[i] = pChars[i + iCharOffset];
-    }
-  
-
-	// Use "wide-char" function
-	gdk_draw_text_wc (m_pWin, font, m_pGC,
-					  xoff, yoff + font->ascent, pNChars, iLength);
-	//XDrawString16 (drawable_private->xdisplay, drawable_private->xwindow,
-	//			   gc_private->xgc, xoff, yoff + xfont->ascent, pNChars,
-	//			   iLength);
-
-	delete pNChars;
-
-#if 1
+	WCTOMB_DECLS;
+	GdkFont *font;
+	GdkFont *EnglishFont;
+	GdkFont *ChineseFont;
+	m_pFont->explodeGdkFonts(EnglishFont,ChineseFont);
+	UT_sint32 x;
+	const UT_UCSChar *pC;
+  	for(pC=pChars+iCharOffset, x=xoff; pC<pChars+iCharOffset+iLength; ++pC)
+	  {
+		UT_UCSChar actual = remapGlyph(*pC,UT_FALSE);
+		font=XAP_EncodingManager::instance->is_cjk_letter(actual)? ChineseFont: EnglishFont;
+		CONVERT_TO_MBS(actual);
+		gdk_draw_text(m_pWin,font,m_pGC,x,yoff+font->ascent,text,text_length);
+		x+=gdk_text_width(font, text, text_length);
+	  }	
 	flush();
-#endif	
 }
 
 void GR_UnixGraphics::setFont(GR_Font * pFont)
@@ -171,10 +187,34 @@ UT_uint32 GR_UnixGraphics::getFontHeight()
 
 	UT_ASSERT(m_pFont);
 
-	GdkFont* font = m_pFont->getGdkFont();
-	return font->ascent + font->descent;
+	return getFontAscent()+getFontDescent();
 }
 
+UT_uint32 GR_UnixGraphics::measureUnRemappedChar(const UT_UCSChar c)
+{
+	// measureString() could be defined in terms of measureUnRemappedChar()
+	// but its not (for presumed performance reasons).  Also, a difference
+	// is that measureString() uses remapping to get past zero-width
+	// character cells.
+	if (!m_pFontManager)
+		return 0;
+
+	UT_ASSERT(m_pFont);
+	UT_ASSERT(m_pGC);
+	UT_UCSChar Wide_char = c;
+	WCTOMB_DECLS;
+	CONVERT_TO_MBS(Wide_char);
+	if (fallback_used)
+	    return 0;
+	GdkFont *font = m_pFont->getGdkFontForUCSChar(Wide_char);
+	return gdk_text_width(font, text, text_length);
+
+}
+
+#if 0
+/*
+    WARNING: this code doesn't support non-latin1 chars.
+*/
 UT_uint32 GR_UnixGraphics::measureString(const UT_UCSChar* s, int iOffset,
 										 int num,  unsigned short* pWidths)
 {
@@ -199,10 +239,8 @@ UT_uint32 GR_UnixGraphics::measureString(const UT_UCSChar* s, int iOffset,
 	
 	for (int i = 0; i < num; i++)
     {
-		cChar = s[i + iOffset];
-		
-		width = gdk_text_width_wc (pFont, &cChar, 1);
-		
+		cChar = remapGlyph(s[i + iOffset], UT_TRUE);
+		width = gdk_char_width_wc (pFont, cChar);
 		charWidth += width;
 		if (pWidths)
 			pWidths[i] = width;
@@ -210,7 +248,7 @@ UT_uint32 GR_UnixGraphics::measureString(const UT_UCSChar* s, int iOffset,
   
 	return charWidth;
 }
-
+#endif
 UT_uint32 GR_UnixGraphics::_getResolution(void) const
 {
 	// this is hard-coded at 100 for X now, since 75 (which
@@ -257,7 +295,7 @@ GR_Font * GR_UnixGraphics::getGUIFont(void)
 		UT_ASSERT(font);
 
 		// bury it in a new font handle
-		m_pFontGUI = new XAP_UnixFontHandle(font, 10);
+		m_pFontGUI = new XAP_UnixFontHandle(font, 12); // Hardcoded GUI font size
 		UT_ASSERT(m_pFontGUI);
 	}
 
@@ -310,7 +348,6 @@ GR_Font * GR_UnixGraphics::findFont(const char* pszFontFamily,
 
 	// Request the appropriate XAP_UnixFont
 	XAP_UnixFont * unixfont = m_pFontManager->getFont(pszFontFamily, s);	
-
 	if (!unixfont)
 	{
 		// Oops!  We don't have that font here.  substitute something
@@ -348,8 +385,8 @@ UT_uint32 GR_UnixGraphics::getFontAscent()
 	UT_ASSERT(m_pGC);
 
 	GdkFont* pFont = m_pFont->getGdkFont();
-
-	return pFont->ascent;
+	GdkFont* pMatchFont=m_pFont->getMatchGdkFont();
+	return MAX(pFont->ascent, pMatchFont->ascent);
 }
 
 UT_uint32 GR_UnixGraphics::getFontDescent()
@@ -361,8 +398,8 @@ UT_uint32 GR_UnixGraphics::getFontDescent()
 	UT_ASSERT(m_pGC);
 
 	GdkFont* pFont = m_pFont->getGdkFont();
-
-	return pFont->descent;
+	GdkFont* pMatchFont=m_pFont->getMatchGdkFont();
+	return MAX(pFont->descent, pMatchFont->descent);
 }
 
 void GR_UnixGraphics::drawLine(UT_sint32 x1, UT_sint32 y1,
@@ -532,7 +569,7 @@ void GR_UnixGraphics::scroll(UT_sint32 x_dest, UT_sint32 y_dest,
 void GR_UnixGraphics::clearArea(UT_sint32 x, UT_sint32 y,
 							 UT_sint32 width, UT_sint32 height)
 {
-//	UT_DEBUGMSG(("ClearArea: %d %d %d %d\n", x, y, width, height));
+  //	UT_DEBUGMSG(("ClearArea: %d %d %d %d\n", x, y, width, height));
 	if (width > 0)
 	{
 #define TURBOSLOW	0
@@ -576,36 +613,45 @@ UT_Bool GR_UnixGraphics::endPrint(void)
 	return UT_FALSE;
 }
 
-GR_Image* GR_UnixGraphics::createNewImage(const char* pszName, const UT_ByteBuf* pBBPNG, UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight)
+GR_Image* GR_UnixGraphics::createNewImage(const char* pszName, const UT_ByteBuf* pBB, UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight, GR_Image::GRType iType)
 {
-	GR_UnixImage* pImg = new GR_UnixImage(NULL, pszName);
-
-	pImg->convertFromPNG(pBBPNG, iDisplayWidth, iDisplayHeight);
-
-	return pImg;
+   	GR_Image* pImg = NULL;
+   	if (iType == GR_Image::GRT_Raster)
+   		pImg = new GR_UnixImage(pszName);
+   	else
+	   	pImg = new GR_VectorImage(pszName);
+	     
+	pImg->convertFromBuffer(pBB, iDisplayWidth, iDisplayHeight);
+   	return pImg;
 }
 
 void GR_UnixGraphics::drawImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDest)
 {
 	UT_ASSERT(pImg);
-	
-	GR_UnixImage * pUnixImage = static_cast<GR_UnixImage *>(pImg);
 
-	Fatmap * image = pUnixImage->getData();
+   	if (pImg->getType() != GR_Image::GRT_Raster) {
+	   	pImg->render(this, xDest, yDest);
+	   	return;
+	}
+	   
+   	GR_UnixImage * pUnixImage = static_cast<GR_UnixImage *>(pImg);
 
-	UT_sint32 iImageWidth = pUnixImage->getDisplayWidth();
-	UT_sint32 iImageHeight = pUnixImage->getDisplayHeight();
+   	Fatmap * image = pUnixImage->getData();
+
+   	UT_sint32 iImageWidth = pUnixImage->getDisplayWidth();
+   	UT_sint32 iImageHeight = pUnixImage->getDisplayHeight();
+
+   	gdk_draw_rgb_image(m_pWin,
+			   m_pGC,
+			   xDest,
+			   yDest,
+			   iImageWidth,
+			   iImageHeight,
+			   GDK_RGB_DITHER_NORMAL,
+			   image->data,
+			   image->width * 3); // This parameter is the total bytes across one row,
+                                              // which is pixels * 3 (we use 3 bytes per pixel).
 
-	gdk_draw_rgb_image(m_pWin,
-					   m_pGC,
-					   xDest,
-					   yDest,
-					   iImageWidth,
-					   iImageHeight,
-					   GDK_RGB_DITHER_NORMAL,
-					   image->data,
-					   image->width * 3); // This parameter is the total bytes across one row,
-	                                      // which is pixels * 3 (we use 3 bytes per pixel).
 }
 
 void GR_UnixGraphics::flush(void)
@@ -726,6 +772,42 @@ void GR_UnixGraphics::fillRect(GR_Color3D c, UT_Rect &r)
 	fillRect(c,r.left,r.top,r.width,r.height);
 }
 
+void GR_UnixGraphics::polygon(UT_RGBColor& c,UT_Point *pts,UT_uint32 nPoints)
+{
+	// save away the current color, and restore it after we draw the polygon
+	GdkGCValues gcValues;
+	GdkColor oColor;
+
+	memset(&oColor, 0, sizeof(GdkColor));
+  
+	gdk_gc_get_values(m_pGC, &gcValues);
+
+	oColor.pixel = gcValues.foreground.pixel;
+
+	// get the new color
+	GdkColor nColor;
+
+	nColor.red = c.m_red << 8;
+	nColor.blue = c.m_blu << 8;
+	nColor.green = c.m_grn << 8;
+
+	gdk_color_alloc(m_pColormap, &nColor);
+  
+	gdk_gc_set_foreground(m_pGC, &nColor);
+
+	GdkPoint* points = new GdkPoint[nPoints];
+    UT_ASSERT(points);
+
+    for (UT_uint32 i = 0;i < nPoints;i++){
+        points[i].x = pts[i].x;
+        points[i].y = pts[i].y;
+    }
+	gdk_draw_polygon(m_pWin, m_pGC, 1, points, nPoints);
+	delete[] points;
+
+	gdk_gc_set_foreground(m_pGC, &oColor);
+}
+
 //////////////////////////////////////////////////////////////////
 // This is a static method in the GR_Font base class implemented
 // in platform code.
