@@ -48,15 +48,17 @@ void GR_Win32Graphics::_constructorCommonCode(HDC hdc)
 	setCursor(GR_CURSOR_DEFAULT);
 }
 
-GR_Win32Graphics::GR_Win32Graphics(HDC hdc, HWND hwnd)
+GR_Win32Graphics::GR_Win32Graphics(HDC hdc, HWND hwnd, XAP_App * app)
 {
 	_constructorCommonCode(hdc);
+	m_pApp = app;
 	m_hwnd = hwnd;
 }
 
-GR_Win32Graphics::GR_Win32Graphics(HDC hdc, const DOCINFO * pDocInfo)
+GR_Win32Graphics::GR_Win32Graphics(HDC hdc, const DOCINFO * pDocInfo, XAP_App * app)
 {
 	_constructorCommonCode(hdc);
+	m_pApp = app;
 	m_bPrint = UT_TRUE;
 	m_pDocInfo = pDocInfo;
 }
@@ -171,7 +173,19 @@ void GR_Win32Graphics::drawChar(UT_UCSChar Char, UT_sint32 xoff, UT_sint32 yoff)
 	SetTextAlign(m_hdc, TA_LEFT | TA_TOP);
 	SetBkMode(m_hdc, TRANSPARENT);		// TODO: remember and reset?
 
-	ExtTextOutW(m_hdc, xoff, yoff, 0, NULL, &Char, 1, NULL);
+	UT_UCSChar currentChar = remapGlyph(Char, UT_FALSE);
+	if(UT_IsWinNT())
+	{
+		ExtTextOutW(m_hdc, xoff, yoff, 0, NULL, &currentChar, 1, NULL);
+	}
+	else
+	{
+		char str[sizeof(UT_UCSChar)];
+		int iConverted = WideCharToMultiByte(CP_ACP, NULL, 
+			&currentChar, 1,
+			str, sizeof(str), NULL, NULL);
+		ExtTextOutA(m_hdc, xoff, yoff, 0, NULL, str, iConverted, NULL);
+	}
 }
 
 void GR_Win32Graphics::drawChars(const UT_UCSChar* pChars,
@@ -184,8 +198,20 @@ void GR_Win32Graphics::drawChars(const UT_UCSChar* pChars,
 	SetTextAlign(m_hdc, TA_LEFT | TA_TOP);
 	SetBkMode(m_hdc, TRANSPARENT);		// TODO: remember and reset?
 
-	ExtTextOutW(m_hdc, xoff, yoff, 0, NULL, pChars + iCharOffset, iLength, NULL);
-	//TextOutW(m_hdc, xoff, yoff, pChars + iCharOffset, iLength);
+	// TODO: need remapGlyph() before the following call
+	if(UT_IsWinNT())
+	{
+		ExtTextOutW(m_hdc, xoff, yoff, 0, NULL, pChars + iCharOffset, iLength, NULL);
+	}
+	else
+	{
+		char* str = new char[iLength * sizeof(UT_UCSChar)];
+		int iConverted = WideCharToMultiByte(CP_ACP, NULL, 
+			pChars + iCharOffset, iLength, 
+			str, iLength * sizeof(UT_UCSChar), NULL, NULL);
+		ExtTextOutA(m_hdc, xoff, yoff, 0, NULL, str, iConverted, NULL);
+		delete [] str;
+	}
 }
 
 void GR_Win32Graphics::setFont(GR_Font* pFont)
@@ -217,12 +243,18 @@ UT_uint32 GR_Win32Graphics::getFontDescent()
 	return m_pFont->getDescent();
 }
 
+UT_uint32 GR_Win32Graphics::measureUnRemappedChar(const UT_UCSChar c)
+{
+	UT_ASSERT(m_pFont);
+	return m_pFont->measureUnRemappedChar(c);
+}
+#if 0
 UT_uint32 GR_Win32Graphics::measureString(const UT_UCSChar* s, int iOffset, int num, unsigned short* pWidths)
 {
 	UT_ASSERT(m_pFont);
 	return m_pFont->measureString(s,iOffset,num,pWidths);
 }
-
+#endif
 UT_uint32 GR_Win32Graphics::_getResolution(void) const
 {
 	int result = GetDeviceCaps(m_hdc, LOGPIXELSY); // NOTE: assumes square pixels
@@ -448,12 +480,17 @@ void GR_Win32Graphics::setClipRect(const UT_Rect* pRect)
 	UT_ASSERT(res != ERROR);
 }
 
-GR_Image* GR_Win32Graphics::createNewImage(const char* pszName, const UT_ByteBuf* pBBPNG,
-										   UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight)
+GR_Image* GR_Win32Graphics::createNewImage(const char* pszName, const UT_ByteBuf* pBB,
+					   UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight,
+					   GR_Image::GRType iType)
 {
-	GR_Win32Image* pImg = new GR_Win32Image(NULL, pszName);
-
-	pImg->convertFromPNG(pBBPNG, iDisplayWidth, iDisplayHeight);
+	GR_Image* pImg = NULL;
+   	if (iType == GR_Image::GRT_Raster)
+     		pImg = new GR_Win32Image(pszName);
+   	else
+     		pImg = new GR_VectorImage(pszName);
+   
+	pImg->convertFromBuffer(pBB, iDisplayWidth, iDisplayHeight);
 
 	return pImg;
 }
@@ -462,6 +499,11 @@ void GR_Win32Graphics::drawImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDes
 {
 	UT_ASSERT(pImg);
 	
+   	if (pImg->getType() != GR_Image::GRT_Raster) {
+	   	pImg->render(this, xDest, yDest);
+	   	return;
+	}
+   
 	GR_Win32Image* pWin32Img = static_cast<GR_Win32Image*>(pImg);
 
 	BITMAPINFO* pDIB = pWin32Img->getDIB();
@@ -670,6 +712,26 @@ void GR_Font::s_getGenericFontProperties(const char * szFontName,
 	return;
 }
 
+UT_uint32 GR_Win32Font::measureUnRemappedChar(const UT_UCSChar c)
+{
+	int iCharWidth = 0;
+	int iWidth;
+	// try to get cached value for the width of this char.
+	// if that fails, force fill the cache for this char
+	// and then re-hit the cache.
+	// if that fails, the char is probably not defined in
+	// the font, so we substitute the default char width.
+	iWidth = m_cw.getWidth(c);
+	if (!iWidth)
+	{
+		m_cw.setCharWidthsOfRange(m_oldHDC, c, c);
+		iWidth = m_cw.getWidth(c);
+		// [[Why the default width?  I think zero is better in that case?]]
+		if (!iWidth) iWidth = 0 /* m_defaultCharWidth */;
+	}
+	return iWidth;
+}
+#if 0
 UT_uint32 GR_Win32Font::measureString(const UT_UCSChar* s, int iOffset, int num, unsigned short* pWidths)
 {
 	UT_ASSERT(s);
@@ -685,14 +747,13 @@ UT_uint32 GR_Win32Font::measureString(const UT_UCSChar* s, int iOffset, int num,
 		// and then re-hit the cache.
 		// if that fails, the char is probably not defined in
 		// the font, so we substitute the default char width.
-		
-		iWidth = m_cw.getWidth(s[i+iOffset]);
+		UT_UCSChar currentChar;
+		currentChar = remapGlyph(s[i + iOffset]);
+		iWidth = m_cw.getWidth(currentChar);
 		if (!iWidth)
 		{
-			m_cw.setCharWidthsOfRange(m_oldHDC,s[i+iOffset],s[i+iOffset]);
-			iWidth = m_cw.getWidth(s[i+iOffset]);
-			if (!iWidth)
-				iWidth = m_defaultCharWidth;
+			m_cw.setCharWidthsOfRange(m_oldHDC,currentChar,currentChar);
+			iWidth = m_cw.getWidth(currentChar);
 		}
 		
 		iCharWidth += iWidth;
@@ -702,7 +763,7 @@ UT_uint32 GR_Win32Font::measureString(const UT_UCSChar* s, int iOffset, int num,
 
 	return iCharWidth;
 }
-
+#endif
 GR_Win32Font::GR_Win32Font(HFONT hFont)
 {
 	m_hFont = hFont;
@@ -741,3 +802,30 @@ void GR_Win32Font::selectFontIntoDC(HDC hdc)
 	m_cw.setCharWidthsOfRange(hdc,d,d);
 	m_defaultCharWidth = m_cw.getWidth(d);
 }
+
+void GR_Win32Graphics::polygon(UT_RGBColor& c,UT_Point *pts,UT_uint32 nPoints)
+{
+    HPEN hPen = CreatePen(PS_SOLID,1,RGB(c.m_red,c.m_grn,c.m_blu));
+    HPEN hOldPen = (HPEN)SelectObject(m_hdc,hPen);
+
+    HBRUSH hBrush = CreateSolidBrush(RGB(c.m_red,c.m_grn,c.m_blu));
+    HBRUSH hOldBrush = (HBRUSH)SelectObject(m_hdc,hBrush);
+
+    POINT * points = new POINT[nPoints];
+    UT_ASSERT(points);
+
+    for (UT_uint32 i = 0;i < nPoints;i++){
+        points[i].x = pts[i].x;
+        points[i].y = pts[i].y;
+    }
+
+    Polygon(m_hdc,points,nPoints);
+
+    (void) SelectObject(m_hdc,hOldPen);
+    DeleteObject(hPen);
+
+    (void) SelectObject(m_hdc,hOldBrush);
+    DeleteObject(hBrush);
+
+    delete[] points;
+}
