@@ -35,6 +35,8 @@
 #include"xap_App.h"
 
 
+
+
 // default GTK message box button width, in GTK screen units (pixels)
 #define DEFAULT_BUTTON_WIDTH	85
 
@@ -48,6 +50,14 @@ static gboolean focus_in_event(GtkWidget *widget,GdkEvent */*event*/,gpointer /*
       return FALSE;
 }
 
+static gboolean destroy_event(GtkWidget *widget,GdkEvent */*event*/,gpointer /*user_data*/)
+{
+      XAP_Frame *pFrame=(XAP_Frame *)gtk_object_get_data(GTK_OBJECT(widget), "frame");
+      if(pFrame == NULL) return FALSE;
+      
+      return FALSE;
+}
+
 static gboolean focus_out_event(GtkWidget *widget,GdkEvent */*event*/,gpointer /*user_data*/)
 {
       XAP_Frame *pFrame=(XAP_Frame *)gtk_object_get_data(GTK_OBJECT(widget), "frame");
@@ -112,6 +122,33 @@ static gboolean focus_in_event_Modeless(GtkWidget *widget,GdkEvent */*event*/,gp
       return FALSE;
 }
 
+
+static gboolean focus_in_event_ModelessOther(GtkWidget *widget,GdkEvent */*event*/,gboolean (*other_function)(void) )
+{
+      XAP_App *pApp=(XAP_App *)gtk_object_get_data(GTK_OBJECT(widget), "pApp");
+      XAP_Frame *pFrame= pApp->getLastFocussedFrame();
+      if(pFrame ==(XAP_Frame *)  NULL) 
+      {
+             UT_uint32 nframes =  pApp->getFrameCount();
+             if(nframes > 0 && nframes < 10)
+	     {     
+	            pFrame = pApp->getFrame(0);
+	     }
+             else
+	     {
+	            return FALSE;
+	      }
+      }
+      if(pFrame == (XAP_Frame *) NULL) return FALSE;
+      AV_View * pView = pFrame->getCurrentView();
+      if(pView!= NULL)
+      {
+            pView->focusChange(AV_FOCUS_MODELESS);
+            (*other_function)();
+      }
+      return FALSE;
+}
+
 void connectFocus(GtkWidget *widget,const XAP_Frame *frame)
 {
       gtk_object_set_data(GTK_OBJECT(widget), "frame",
@@ -121,9 +158,23 @@ void connectFocus(GtkWidget *widget,const XAP_Frame *frame)
       gtk_signal_connect(GTK_OBJECT(widget), "focus_out_event",
 					 GTK_SIGNAL_FUNC(focus_out_event), NULL);
       gtk_signal_connect(GTK_OBJECT(widget), "destroy",
-					 GTK_SIGNAL_FUNC(focus_out_event), NULL);
+					 GTK_SIGNAL_FUNC(destroy_event), NULL);
 }
 
+void connectFocusModelessOther(GtkWidget *widget,const XAP_App * pApp, 
+			       gboolean(*other_function)(void))
+{
+      gtk_object_set_data(GTK_OBJECT(widget), "pApp",
+					  (void *)pApp);
+      gtk_signal_connect(GTK_OBJECT(widget), "focus_in_event",
+					 GTK_SIGNAL_FUNC(focus_in_event_ModelessOther), (gpointer) other_function);
+      gtk_signal_connect(GTK_OBJECT(widget), "focus_out_event",
+					 GTK_SIGNAL_FUNC(focus_out_event_Modeless), NULL);
+      gtk_signal_connect(GTK_OBJECT(widget), "destroy",
+					 GTK_SIGNAL_FUNC(focus_out_event_Modeless), NULL);
+}
+
+
 void connectFocusModeless(GtkWidget *widget,const XAP_App * pApp)
 {
       gtk_object_set_data(GTK_OBJECT(widget), "pApp",
@@ -133,7 +184,7 @@ void connectFocusModeless(GtkWidget *widget,const XAP_App * pApp)
       gtk_signal_connect(GTK_OBJECT(widget), "focus_out_event",
 					 GTK_SIGNAL_FUNC(focus_out_event_Modeless), NULL);
       gtk_signal_connect(GTK_OBJECT(widget), "destroy",
-					 GTK_SIGNAL_FUNC(focus_out_event_Modeless), NULL);
+					 GTK_SIGNAL_FUNC(destroy_event), NULL);
 }
 
 
@@ -267,40 +318,8 @@ void centerDialog(GtkWidget * parent, GtkWidget * child)
 	UT_ASSERT(parent);
 	UT_ASSERT(child);
 
-	UT_ASSERT(parent->window);
-	gint parentx = 0;
-	gint parenty = 0;
-	gint parentwidth = 0;
-	gint parentheight = 0;
-	gdk_window_get_origin(parent->window, &parentx, &parenty);
-	gdk_window_get_size(parent->window, &parentwidth, &parentheight);
-	UT_ASSERT(parentwidth > 0 && parentheight > 0);
-
-	// this message box's geometry (it won't have a ->window yet, so don't assert it)
-	gint width = 0;
-	gint height = 0;
-	gtk_widget_size_request(child, &child->requisition);
-	width = child->requisition.width;
-	height = child->requisition.height;
-	UT_ASSERT(width > 0 && height > 0);
-
-	// set new place
-	gint newx = parentx + ((parentwidth - width) / 2);
-	gint newy = parenty + ((parentheight - height) / 2);
-
-	// measure the root window
-	gint rootwidth = gdk_screen_width();
-	gint rootheight = gdk_screen_height();
-	// if the dialog won't fit on the screen, panic and center on the root window
-	if ((newx + width) > rootwidth || (newy + height) > rootheight)
-	{
-		UT_DEBUGMSG(("Dialog [%p] at coordiantes [%d, %d] is in non-viewable area; "
-					 "centering on desktop instead.\n", child, newx, newy));
-		gtk_window_set_position(GTK_WINDOW(child), GTK_WIN_POS_CENTER);
-	}
-	else
-		gtk_widget_set_uposition(child, newx, newy);
-	
+	gtk_window_set_transient_for(GTK_WINDOW(child),
+				     GTK_WINDOW(parent));
 }
 
 gint searchCList(GtkCList * clist, char * compareText)
@@ -594,3 +613,4 @@ void createLabelAccelerators( GtkWidget *widget )
 
 	gtk_window_add_accel_group (GTK_WINDOW (widget), data.accel_group);
 }
+
