@@ -17,10 +17,15 @@
  * 02111-1307, USA.
  */
  
+#include <stdio.h>
+
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
 #include <ctype.h>
+#ifdef HAVE_LIBXML2
+#include <glib.h>
+#endif
 
 #include "ut_types.h"
 #include "ut_misc.h"
@@ -29,6 +34,74 @@
 #include "ut_debugmsg.h"
 #include "ut_growbuf.h"
 
+/*
+    If WITHOUT_MB is defined, UT_Mbtowc and UT_Wctomb won't be used.
+    I don't there there could be reason for defining WITHOUT_MB, since
+    UT_Mbtowc and UT_Wctomb use iconv internally, which works fine everywhere.
+*/
+
+#ifndef WITHOUT_MB
+#include "ut_mbtowc.h"
+#include "ut_wctomb.h"
+#endif
+
+#include "xap_EncodingManager.h"
+
+/*
+ * This is cut & pasted from glib 1.3 (c) RedHat
+ * We need this for to convert UTF16 to UTF8
+ */
+int
+unichar_to_utf8 (int c, unsigned char *outbuf)
+{
+  size_t len = 0;
+  int first;
+  int i;
+
+  if (c < 0x80)
+    {
+      first = 0;
+      len = 1;
+    }
+  else if (c < 0x800)
+    {
+      first = 0xc0;
+      len = 2;
+    }
+  else if (c < 0x10000)
+    {
+      first = 0xe0;
+      len = 3;
+    }
+   else if (c < 0x200000)
+    {
+      first = 0xf0;
+      len = 4;
+    }
+  else if (c < 0x4000000)
+    {
+      first = 0xf8;
+      len = 5;
+    }
+  else
+    {
+      first = 0xfc;
+      len = 6;
+    }
+
+  if (outbuf)
+    {
+      for (i = len - 1; i > 0; --i)
+	{
+	  outbuf[i] = (c & 0x3f) | 0x80;
+	  c >>= 6;
+	}
+      outbuf[0] = c | first;
+    }
+
+  return len;
+}
+
 //////////////////////////////////////////////////////////////////
 // char * UT_catPathname(const char * szPath, const char * szFile);
 // is defined in platform-specific code.
@@ -49,7 +122,7 @@ UT_sint32 UT_stricmp(const char * s1, const char * s2)
 {
 	UT_ASSERT(s1);
 	UT_ASSERT(s2);
-	
+
 	while(tolower(*s1) == tolower(*s2) && *s1 != '\0' && *s2 != '\0')
 	{
 		s1++;
@@ -167,6 +240,12 @@ UT_sint32 UT_XML_stricmp(const XML_Char * sz1, const XML_Char * sz2)
 	return UT_stricmp(sz1,sz2);
 }
 
+UT_sint32 UT_XML_strnicmp(const XML_Char * sz1, const XML_Char * sz2, const UT_uint32 n)
+{
+	UT_ASSERT(sizeof(char) == sizeof(XML_Char));
+	return UT_strnicmp(sz1,sz2, n);
+}
+
 UT_sint32 UT_XML_strcmp(const XML_Char * sz1, const XML_Char * sz2)
 {
 	UT_ASSERT(sizeof(char) == sizeof(XML_Char));
@@ -445,11 +524,24 @@ UT_UCSChar UT_UCS_tolower(UT_UCSChar c)
 {
 	if (c < 128)
 		return tolower(c);
+#if 0		
 	if (c >= 256)
 		return c;  /* Unicode but not Latin-1 - don't know what to do */
 	if (c >= 0xC0 && c <= 0xDE && c != 0xD7)  /* uppercase Latin-1 chars */
 		return c + 0x20;
 	return c;
+#else
+	if (c < 128)
+		return tolower(c);
+	if (XAP_EncodingManager::instance->single_case())
+		return c;
+	/*let's trust libc!*/
+	UT_UCSChar local = XAP_EncodingManager::instance->try_UToNative(c);
+	if (!local || local>0xff)
+		return c;
+	local = XAP_EncodingManager::instance->try_nativeToU(tolower(local));
+	return local ? local : c;
+#endif
 }
 
 
@@ -547,7 +639,7 @@ UT_uint32 UT_UCS_strlen(const UT_UCSChar * string)
 {
 	UT_uint32 i;
 
-	for(i = 0; *string != NULL; string++, i++)
+	for(i = 0; *string != 0; string++, i++)
 		;
 
 	return i;
@@ -561,9 +653,9 @@ UT_UCSChar * UT_UCS_strcpy(UT_UCSChar * dest, const UT_UCSChar * src)
 	UT_UCSChar * d = dest;
 	UT_UCSChar * s = (UT_UCSChar *) src;
 
-	while (*s != NULL)
+	while (*s != 0)
 		*d++ = *s++;
-	*d = NULL;
+	*d = 0;
 
 	return dest;
 }
@@ -578,9 +670,21 @@ UT_UCSChar * UT_UCS_strcpy_char(UT_UCSChar * dest, const char * src)
 	UT_UCSChar * d 		= dest;
 	unsigned char * s	= (unsigned char *) src;
 
-	while (*s != NULL)
-		*d++ = *s++;
-	*d = NULL;
+#ifndef WITHOUT_MB
+	UT_Mbtowc m;
+	wchar_t wc;
+#endif
+
+	while (*s != 0)
+	  {
+#ifdef WITHOUT_MB
+	    *d++ = *s++;   
+#else
+		if(m.mbtowc(wc,*s))*d++=wc;
+		s++;
+#endif
+	  }
+	*d = 0;
 
 	return dest;
 }
@@ -593,9 +697,21 @@ char * UT_UCS_strcpy_to_char(char * dest, const UT_UCSChar * src)
 	char * 			d = dest;
 	UT_UCSChar * 	s = (UT_UCSChar *) src;
 
-	while (*s != NULL)
-	    *d++ = *s++;
-	*d = NULL;
+#ifndef WITHOUT_MB
+	UT_Wctomb w;
+#endif
+
+	while (*s != 0)
+	  {
+#ifdef WITHOUT_MB
+	    *d++ = *s++;   
+#else
+		int length;
+		if(w.wctomb(d,length,*s++))
+		  d+=length;
+#endif
+	  }
+	*d = 0;
 	
 	return dest;
 }
@@ -613,22 +729,49 @@ UT_Bool UT_UCS_cloneString(UT_UCSChar ** dest, const UT_UCSChar * src)
 
 UT_Bool UT_UCS_cloneString_char(UT_UCSChar ** dest, const char * src)
 {
-	UT_uint32 length = strlen(src) + 1;
-	*dest = (UT_UCSChar *)calloc(length,sizeof(UT_UCSChar));
-	if (!*dest)
-		return UT_FALSE;
-	UT_UCS_strcpy_char(*dest, src);
 
-	return UT_TRUE;
+#ifdef WITHOUT_MB
+
+		UT_uint32 length = strlen(src) + 1;   
+		*dest = (UT_UCSChar *)calloc(length,sizeof(UT_UCSChar)); 
+		if (!*dest)
+				return UT_FALSE;
+		UT_UCS_strcpy_char(*dest, src);
+
+		return UT_TRUE;          
+
+#else
+
+		UT_uint32 length = MB_LEN_MAX*strlen(src) + 1;
+		*dest = (UT_UCSChar *)calloc(length,sizeof(UT_UCSChar));
+		if (!*dest)
+				return UT_FALSE;
+		UT_UCSChar * d= *dest;
+		unsigned char * s	= (unsigned char *) src;
+		
+		UT_Mbtowc m;
+		wchar_t wc;
+		
+		while (*s != 0)
+		{
+				if(m.mbtowc(wc,*s))*d++=wc;
+				s++;
+		}
+		*d = 0;  
+		
+		return UT_TRUE;
+
+#endif
+
 }
 
 // convert each character in a string to ASCII uppercase
 char * UT_upperString(char * string)
 {
 	if (!string)
-		return NULL;
+		return 0;
 
-	for (char * ch = string; *ch != NULL; ch++)
+	for (char * ch = string; *ch != 0; ch++)
 		*ch = toupper(*ch);
 
 	return string;
@@ -638,9 +781,9 @@ char * UT_upperString(char * string)
 char * UT_lowerString(char * string)
 {
 	if (!string)
-		return NULL;
+		return 0;
 
-	for (char * ch = string; *ch != NULL; ch++)
+	for (char * ch = string; *ch != 0; ch++)
 		*ch = tolower(*ch);
 
 	return string;
@@ -760,4 +903,143 @@ XML_Char* UT_encodeUTF8char(UT_UCSChar cIn)
 }
 #endif // --jeff
 
-	
+#ifndef HAVE_LIBXML2
+static void endElement(void *userData, const XML_Char *name)
+{
+}
+
+static void startElement(void *userData, const XML_Char *name, const XML_Char **atts)
+{
+	XML_Char **pout = (XML_Char **)userData;
+	//What do we do with this, is this cast safe!?
+	*pout = (XML_Char *)atts[1];
+}
+#endif /* HAVE_LIBXML2 */
+
+XML_Char *UT_decodeXMLstring(XML_Char *in)
+{
+	// There has *got* to be an easier way to do this with expat, but I
+	// didn't spot it from looking at the expat source code.  Anyhow, this
+	// is just used during init to chomp the preference default value
+	// strings, so the amount of work done probably doesn't matter too
+	// much.
+	XML_Char *out = 0;
+#ifdef HAVE_LIBXML2
+	static xmlDocPtr dok;
+	//UT_DEBUGMSG(("BLAH in:%s\n",in));
+	char* in2 = 
+	  g_strdup_printf("<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n"
+			  "<fake blah=\"%s\"/>\n",in);
+	dok = xmlParseDoc(in2);
+	if (dok == NULL)
+	  UT_DEBUGMSG(("XML parsing error\n"));
+	else
+	  {
+	    xmlNodePtr node = xmlDocGetRootElement(dok);
+	    out = UT_strdup(node->properties->children->content);
+	    //UT_DEBUGMSG(("BLAH out:%s\n",out));
+	  }
+	g_free(in2);
+	xmlFreeDoc(dok);
+	return out;
+#else
+	const char s1[] = "<fake blah=\"";
+	const char s2[] = "\"/>";
+	XML_Parser parser = 0;
+	parser = XML_ParserCreate(0);
+	XML_SetUserData(parser, &out);
+	XML_SetElementHandler(parser, startElement, endElement);
+	if (!XML_Parse(parser, s1, sizeof(s1)-1, 0)
+	||  !XML_Parse(parser, in, strlen(in), 0)
+	||  !XML_Parse(parser, s2, sizeof(s2)-1, 0))
+	{
+		UT_DEBUGMSG(("XML parsing error %s; %s:%d\n",
+			     XML_ErrorString(XML_GetErrorCode(parser)),
+			     __FILE__, __LINE__));
+	}
+	// TODO: who owns the storage for this?
+	out = UT_strdup(out);
+	if (parser) XML_ParserFree(parser);
+	return out;
+#endif
+}
+
+UT_Bool UT_isSmartQuotableCharacter(UT_UCSChar c)
+{
+	// TODO:  this is anglo-centric; really need a locale argument or
+	// TODO:  something to get smart quote rules for the rest of the world
+	UT_Bool result;
+	switch (c)
+	{
+	case '"':
+	case '`':
+	case '\'':
+		result = UT_TRUE;
+		break;
+	default:
+		result = UT_FALSE;
+		break;
+	}
+	return (result);
+}
+
+UT_Bool UT_isSmartQuotedCharacter(UT_UCSChar c)
+{
+	// TODO:  this is anglo-centric; really need a locale argument or
+	// TODO:  something to get smart quote rules for the rest of the world
+	UT_Bool result;
+	switch (c)
+	{
+	case UCS_LQUOTE:
+	case UCS_RQUOTE:
+	case UCS_LDBLQUOTE:
+	case UCS_RDBLQUOTE:
+		result = UT_TRUE;
+		break;
+	default:
+		result = UT_FALSE;
+		break;
+	}
+	return (result);
+}
+
+UT_Bool UT_UCS_isupper(UT_UCSChar c)
+{
+	if (XAP_EncodingManager::instance->single_case())
+	    return 1;/* FIXME: anyone has better idea? */
+	UT_UCSChar local = XAP_EncodingManager::instance->try_UToNative(c);
+	return local && local <0xff ? isupper(local)!=0 : 0;
+};
+
+UT_Bool UT_UCS_islower(UT_UCSChar c)
+{
+	if (XAP_EncodingManager::instance->single_case())
+	    return 1;/* FIXME: anyone has better idea? */
+	UT_UCSChar local = XAP_EncodingManager::instance->try_UToNative(c);
+	return local && local <0xff ? islower(local)!=0 : 0;
+};
+
+UT_Bool UT_UCS_isalpha(UT_UCSChar c)
+{
+	UT_UCSChar local = XAP_EncodingManager::instance->try_UToNative(c);
+	return local && local < 0xff ? 
+		isalpha(local)!=0 : 
+		local > 0xff /* we consider it alpha if it's > 0xff */;
+};
+
+/*
+ this one prints floating point value but using dot as fractional serparator
+ independent of the current locale's settings.
+*/
+const char* std_size_string(float f)
+{
+  static char string[10];
+  int i=(int)f;
+  if(f-i<0.1)
+	sprintf(string, "%d", i);
+  else {
+          int fr = int(10*(f-i));
+	sprintf(string,"%d.%d", i,fr);
+  };
+  return string;
+};
