@@ -31,10 +31,11 @@ UT_BEGIN_EXTERN_C
 #define UT_LAYOUT_UNITS						1440
 
 
-typedef enum _ut_dimension { DIM_IN, DIM_CM, DIM_PI, DIM_PT, DIM_none } UT_Dimension;
+typedef enum _ut_dimension { DIM_IN, DIM_CM, DIM_MM, DIM_PI, DIM_PT, DIM_PX, DIM_PERCENT, DIM_none } UT_Dimension;
 
 double UT_convertToInches(const char* s);
 double UT_convertToPoints(const char* s);
+double UT_convertToDimension(const char* s, UT_Dimension dim);
 UT_sint32 UT_convertToLayoutUnits(const char* s);
 double UT_convertDimensionless(const char * sz);
 double UT_convertInchesToDimension(double inches, UT_Dimension dim);
@@ -42,15 +43,19 @@ double UT_convertInchesToDimension(double inches, UT_Dimension dim);
 UT_sint32 UT_paperUnits(const char * sz);
 UT_sint32 UT_docUnitsFromPaperUnits(GR_Graphics * pG, UT_sint32 iPaperUnits);
 UT_sint32 UT_layoutUnitsFromPaperUnits(UT_sint32 iPaperUnits);
+UT_sint32 UT_paperUnitsFromLayoutUnits(UT_sint32 iLayoutUnits);
 
 UT_Dimension UT_determineDimension(const char * sz, UT_Dimension fallback = DIM_IN);
 const char * UT_dimensionName(UT_Dimension dim);
-const char * UT_convertToDimensionString(UT_Dimension, double value, const char * szPrecision = NULL);
+const char * UT_convertInchesToDimensionString(UT_Dimension, double valueInInches, const char * szPrecision = NULL);
+const char * UT_formatDimensionString(UT_Dimension, double value, const char * szPrecision = NULL);
+const char * UT_reformatDimensionString(UT_Dimension dim, const char *sz, const char * szPrecision = NULL);
 const char * UT_convertToDimensionlessString(double value, const char * szPrecision = NULL);
 const char * UT_formatDimensionedValue(double value, const char * szUnits, const char * szPrecision = NULL);
 
 UT_Bool UT_hasDimensionComponent(const char * sz);
 
+
 UT_END_EXTERN_C
 
 #endif /* UT_UNITS_H */
