@@ -34,6 +34,7 @@
 #include "xav_View.h"
 #include "xad_Document.h"
 
+
 #include "ap_FrameData.h"
 #include "gr_BeOSGraphics.h"
 
@@ -60,13 +61,17 @@ void TFScrollBar::ValueChanged(float newValue) {
 		if (!pView) {
 			return;
 		}
-        if (Orientation() == B_VERTICAL &&
+        
+	float minRange , maxRange;
+	GetRange(&minRange , &maxRange);
+
+	if (Orientation() == B_VERTICAL &&
 			(UT_sint32)newValue != pView->getYScrollOffset()) {
-				pView->sendVerticalScrollEvent((UT_sint32) newValue);
+				pView->sendVerticalScrollEvent((UT_sint32) newValue, (UT_sint32)maxRange);
         }
         else if (Orientation() == B_HORIZONTAL &&
 				 (UT_sint32)newValue != pView->getXScrollOffset()) {
-				pView->sendHorizontalScrollEvent((UT_sint32) newValue);
+				pView->sendHorizontalScrollEvent((UT_sint32) newValue, (UT_sint32)maxRange);
         }
 }
 
@@ -84,6 +89,7 @@ XAP_BeOSFrame::XAP_BeOSFrame(XAP_BeOSApp * app)
 	m_pView = NULL;
 	m_pBeWin = NULL;
 	m_pBeDocView = NULL;
+	m_pBeOSPopup = NULL;
 }
 
 // TODO when cloning a new frame from an existing one
@@ -200,7 +206,6 @@ void XAP_BeOSFrame::_createTopLevelWindow(void)
 			      	B_NORMAL_WINDOW_FEEL, 0);
 	UT_ASSERT(m_pBeWin);
 	m_pBeWin->_createWindow(m_szMenuLayoutName, m_szMenuLabelSetName);
-
 	// we let our caller decide when to show m_wTopLevelWindow.
 	return;
 }
@@ -265,43 +270,6 @@ UT_Bool XAP_BeOSFrame::updateTitle()
 	return UT_TRUE;
 }
 
-UT_Bool XAP_BeOSFrame::runModalContextMenu(AV_View * /* pView */, const char * szMenuName, UT_sint32 x, UT_sint32 y) {
-#if 0
-	UT_Bool bResult = UT_TRUE;
-
-	UT_ASSERT(!m_pBeOSPopup);
-
-	m_pBeOSPopup = new EV_BeOSMenuPopup(m_pBeOSApp,this,szMenuName,m_szMenuLabelSetName);
-	if (m_pBeOSPopup && m_pBeOSPopup->synthesizeMenuPopup())
-	{
-		// the popup will steal the mouse and so we won't get the
-		// button_release_event and we won't know to release our
-		// grab.  so let's do it here.  (when raised from a keyboard
-		// context menu, we may not have a grab, but that should be ok.
-
-		GtkWidget * w = gtk_grab_get_current();
-		if (w)
-		{
-			UT_DEBUGMSG(("Ungrabbing mouse [before popup].\n"));
-			gtk_grab_remove(w);
-		}
-
-		translateDocumentToScreen(x,y);
-
-		UT_DEBUGMSG(("ContextMenu: %s at [%d,%d]\n",szMenuName,x,y));
-		UT_Point pt;
-		pt.x = x;
-		pt.y = y;
-		gtk_menu_popup(GTK_MENU(m_pBeOSPopup->getMenuHandle()), NULL, NULL,
-					   s_gtkMenuPositionFunc, &pt, 3, 0);
-	}
-
-	DELETEP(m_pBeOSPopup);
-	return bResult;
-#endif
-	return(UT_FALSE);
-}
-
 UT_Vector * XAP_BeOSFrame::VecToolbarLayoutNames() {
 	return(&m_vecToolbarLayoutNames); 
 }
@@ -319,6 +287,8 @@ void XAP_BeOSFrame::setScrollBars(TFScrollBar *h, TFScrollBar *v) {
 	m_vScroll = v;
 }	
 
+
+
 /*********************************************************
  Local Window/View Class Stuff 
 *********************************************************/
@@ -332,6 +302,7 @@ be_Window::be_Window(XAP_BeOSApp *theApp, XAP_BeOSFrame *theFrame,
 	m_pBeOSFrame = theFrame;
 	m_pBeOSApp = theApp;
 }
+
 #if 0
 void be_Window::MessageReceived(BMessage *msg) {
 	switch(msg->what) {
@@ -342,6 +313,7 @@ void be_Window::MessageReceived(BMessage *msg) {
 		BWindow::MessageReceived(msg);
 	}
 }
+
 #endif
 
 bool be_Window::QuitRequested(void) {
@@ -363,22 +335,56 @@ bool be_Window::QuitRequested(void) {
 	}
 
 	// let the window be destroyed
-	return(true);
+	return(false);
+}
+
+void be_Window::MessageReceived(BMessage *pMsg)
+{
+	float smallStep, bigStep;
+	float deltaX, deltaY;
+	
+	switch(pMsg->what)
+	{
+		case 'inme': // Inme is a "Create Menu" message sent when the view is established.
+			m_pBeOSMenu->synthesize();
+			break;
+#if (B_BEOS_VERSION > B_BEOS_VERSION_4_5)		
+		case B_MOUSE_WHEEL_CHANGED:
+		
+			if( pMsg->FindFloat("be:wheel_delta_y" , &deltaY) == B_OK)
+			{		
+			m_vScroll->GetSteps(&smallStep, &bigStep);
+			m_vScroll->SetValue(m_vScroll->Value() + smallStep * deltaY);
+			}
+			
+			if( pMsg->FindFloat("be:wheel_delta_x" , &deltaX) == B_OK)
+			{
+			m_hScroll->GetSteps(&smallStep, &bigStep);
+			m_hScroll->SetValue(m_hScroll->Value() + smallStep * deltaX);
+			}
+
+			break;
+#else
+#warning "You should really upgrade your BeOS version to get wheel support"
+#endif
+	}
+	
+	BWindow::MessageReceived(pMsg);
 }
 
 bool be_Window::_createWindow(const char *szMenuLayoutName,
 			   const char *szMenuLabelSetName) {
 	BRect r;
-	
+
 	m_winRectAvailable = Bounds();
 	//printf("Initial Available Rect: "); m_winRectAvailable.PrintToStream();
-
 	//Add the menu
+	
 	m_pBeOSMenu = new EV_BeOSMenu(m_pBeOSApp, m_pBeOSFrame,
 				      szMenuLayoutName, szMenuLabelSetName);
-	UT_ASSERT(m_pBeOSMenu);
-	UT_Bool bResult = m_pBeOSMenu->synthesize();
-	UT_ASSERT(bResult);
+	
+	m_pBeOSMenu->synthesizeMenuBar();
+		
 	//printf("After Adding Menu Available Rect: "); m_winRectAvailable.PrintToStream();
 	
 	//Add the toolbars
@@ -395,23 +401,19 @@ bool be_Window::_createWindow(const char *szMenuLayoutName,
 
         _createDocumentWindow();
 
-#if 0
-
         // Let the app-specific frame code create the status bar
         // if it wants to.  we will put it below the document
         // window (a peer with toolbars and the overall sunkenbox)
         // so that it will appear outside of the scrollbars.
-	m_wStatusBar = _createStatusBarWindow();
-        if (m_wStatusBar) {
-                gtk_widget_show(m_wStatusBar);
-                gtk_box_pack_end(GTK_BOX(m_wVBox), m_wStatusBar, FALSE, FALSE, 0
-);
-        }                    
-#endif
+       m_pBeOSStatusBarView =  _createStatusBarWindow();
+	
+    if (!m_pBeOSStatusBarView)
+	 return (false);
 
 	return(true);	
 }
 
+
 be_DocView::be_DocView(BRect frame, const char *name, uint32 resizeMask, uint32 flags)
 	:BView(frame, name, resizeMask, flags | B_FRAME_EVENTS) {
 	m_pBPicture = NULL;
@@ -422,8 +424,8 @@ void be_DocView::FrameResized(float new_width, float new_height) {
 	GR_BeOSGraphics *pG;
 	pBWin = (be_Window *)Window();
 	if (!pBWin || !pBWin->m_pBeOSFrame)
-		return;
- 	pBWin->DisableUpdates(); 
+		return; 
+// 	pBWin->DisableUpdates(); 
 	pG = (GR_BeOSGraphics *)pBWin->m_pBeOSFrame->Graphics();
 	if (!pG)
 	{
@@ -438,10 +440,10 @@ void be_DocView::FrameResized(float new_width, float new_height) {
 	AV_View *pView = pBWin->m_pBeOSFrame->getCurrentView();
 	if (pView) {
 		pView->setWindowSize(rect.Width(), rect.Height());
-		pView->draw();
+		pView->draw(); //TODO do we need this???
  		/* Methinks it can handle itself*/
 	}
-	pBWin->EnableUpdates();
+//	pBWin->EnableUpdates();
 	pBWin->Sync(); //Maybe Sync? We'll see
 }
 
@@ -452,6 +454,25 @@ void be_DocView::Draw(BRect updateRect) {
 	if (!pBWin || !pBWin->m_pBeOSFrame)
 		return;
 	
+	// Okay, there is a bug here, that when the BeOS wants a single line of horizontal text drawn, it (more)
+	// passes updateRect.top and updateRect.bottom as an equal value, so how do we get around this (more)
+	// other than pre-decreasing the top value.
+	// The same thing also occurs for a vertical line..
+	
+	updateRect.top -= 1.0f;
+	if( updateRect.top < 0.0f)
+	{
+		updateRect.bottom += 1.0f;
+		updateRect.top = 0.0f;
+	}
+	
+	updateRect.left -= 1.0f;
+	if( updateRect.left < 0.0f)
+	{
+		updateRect.left = 0.0f;
+		updateRect.right += 1.0f;
+	}
+
 #if defined(USE_BACKING_BITMAP)
 	GR_BeOSGraphics 	*pG;
 	BBitmap 		*pBitmap;	
@@ -509,6 +530,52 @@ Things to do to speed this up, make it less flashy:
 #endif
 }
 
+void be_DocView::WindowActivated(bool activated)
+{
+	be_Window *pBWin;
+	pBWin = (be_Window *)Window();
+	AV_View *pView = pBWin->m_pBeOSFrame->getCurrentView();
+	if(pView)
+		pView->focusChange(activated ? AV_FOCUS_HERE : AV_FOCUS_NONE);
+}
+
+/*****************************************************************/
+UT_Bool XAP_BeOSFrame::runModalContextMenu(AV_View * /* pView */, const char * szMenuName,
+										   UT_sint32 x, UT_sint32 y)
+{
+	UT_Bool bResult = UT_TRUE;
+	UT_ASSERT(!m_pBeOSPopup);
+
+	m_pBeOSPopup = new EV_BeOSMenuPopup(m_pBeOSApp, this, 
+									  szMenuName, 
+									  m_szMenuLabelSetName);
+									  
+	if (m_pBeOSPopup && m_pBeOSPopup->synthesizeMenuPopup(this))
+	{
+		UT_DEBUGMSG(("ContextMenu: %s at [%d,%d]\n",szMenuName,x,y));
+		
+		translateDocumentToScreen(x,y);
+	 
+		BPoint screenPoint(x,y);
+		BPopUpMenu* pMenu = m_pBeOSPopup->GetHandle();
+		BMenuItem* selectedItem = pMenu->Go(screenPoint , false , false, false);
+	
+		// Send the menu item invocation message to the window
+		
+		if(selectedItem) // The user clicked an item, will be NULL if they click away from the popup.
+		{
+			if(m_pBeWin) // Should always be valid here, but just in case.
+				m_pBeWin->PostMessage(selectedItem->Message());	
+		}
+		
+	}
+	
+	DELETEP(m_pBeOSPopup);
+
+	return bResult;
+
+}
+
 EV_Toolbar * XAP_BeOSFrame::_newToolbar(XAP_App *app, XAP_Frame *frame,
 					const char *szLayout,
 					const char *szLanguage)
