@@ -154,7 +154,7 @@ void XAP_QNXDialog_About::runModal(XAP_Frame * pFrame)
 	
 
 	// attach a new graphics context
-	m_gc = new GR_QNXGraphics(mainWindow, m_drawingareaGraphic);
+	m_gc = new GR_QNXGraphics(mainWindow, m_drawingareaGraphic, pFrame->getApp());
 
 	UT_QNXCenterWindow(/*parentWindow | */ NULL, mainWindow);
 	UT_QNXBlockWidget(parentWindow, 1);
@@ -310,8 +310,8 @@ void XAP_QNXDialog_About::_preparePicture(void)
 		
 	UT_PNG_getDimensions(pBB, iImageWidth, iImageHeight);
 	
-	m_pGrImageSidebar = new GR_QNXImage(NULL,NULL);
-	m_pGrImageSidebar->convertFromPNG(pBB, iImageWidth, iImageHeight);
+	m_pGrImageSidebar = new GR_QNXImage(NULL);
+	m_pGrImageSidebar->convertFromBuffer(pBB, iImageWidth, iImageHeight);
 
 	DELETEP(pBB);
 }
