@@ -143,6 +143,15 @@ static int s_SymbolMap_clicked(PtWidget_t *widget, void *data, PtCallbackInfo_t
 	return Pt_CONTINUE;
 }
 
+static int s_CurrentSymbol_clicked(PtWidget_t *widget, void *data, PtCallbackInfo_t *info)
+{
+	XAP_QNXDialog_Insert_Symbol *dlg = (XAP_QNXDialog_Insert_Symbol *)data;
+	UT_ASSERT(widget && dlg);
+	dlg->CurrentSymbol_clicked(info);
+	return Pt_CONTINUE;
+}
+
+
 static int s_new_font(PtWidget_t *widget, void *data, PtCallbackInfo_t *info)
 {
 	XAP_QNXDialog_Insert_Symbol *dlg = (XAP_QNXDialog_Insert_Symbol *)data;
@@ -179,15 +188,29 @@ void XAP_QNXDialog_Insert_Symbol::runModal(XAP_Frame * pFrame)
 void XAP_QNXDialog_Insert_Symbol::activate(void)
 {
 	UT_ASSERT(m_windowMain);
+	ConstructWindowName();
+	PtSetResource(m_windowMain, Pt_ARG_WINDOW_TITLE, m_WindowName, 0);
 	PtWindowFocus(m_windowMain);
 }
 
 void XAP_QNXDialog_Insert_Symbol::destroy(void)
 {
+	if (!m_windowMain) {
+		return;
+	}
+
 	modeless_cleanup();
 
-	PtDestroyWidget(m_windowMain);
+	PtWidget_t *win = m_windowMain;
 	m_windowMain = NULL;
+	PtDestroyWidget(win);
+}
+
+void XAP_QNXDialog_Insert_Symbol::notifyActiveFrame(XAP_Frame *pFrame) {
+	activate();
+}
+
+void XAP_QNXDialog_Insert_Symbol::notifyCloseFrame(XAP_Frame *pFrame) {
 }
 
 void XAP_QNXDialog_Insert_Symbol::runModeless(XAP_Frame * pFrame)
@@ -224,7 +247,7 @@ void XAP_QNXDialog_Insert_Symbol::runModeless(XAP_Frame * pFrame)
 
 	// make a new QNX GC
 	DELETEP (m_qnxGraphics);
-	m_qnxGraphics = new GR_QNXGraphics(mainWindow, m_SymbolMap);
+	m_qnxGraphics = new GR_QNXGraphics(mainWindow, m_SymbolMap, m_pApp);
 
 	// let the widget materialize TODO: get a real size!
 	UT_QNXGetWidgetArea(m_SymbolMap, NULL, NULL, &w, &h);
@@ -232,7 +255,7 @@ void XAP_QNXDialog_Insert_Symbol::runModeless(XAP_Frame * pFrame)
 
 	// make a new QNX GC
 	DELETEP (m_qnxarea);
-	m_qnxarea = new GR_QNXGraphics(mainWindow, m_areaCurrentSym );
+	m_qnxarea = new GR_QNXGraphics(mainWindow, m_areaCurrentSym, m_pApp);
 		
 	// let the widget materialize
 	UT_QNXGetWidgetArea(m_areaCurrentSym, NULL, NULL, &w, &h);
@@ -268,27 +291,12 @@ void XAP_QNXDialog_Insert_Symbol::runModeless(XAP_Frame * pFrame)
 	UT_QNXCenterWindow(parentWindow, mainWindow);
 	PtRealizeWidget(mainWindow);
 	PgFlush();
-
-#if 0
-	UT_QNXBlockWidget(parentWindow, 1);
-	
-	int count = PtModalStart();
-	done = 0;
-	while(!done) {
-		PtProcessEvent();
-	}
-	PtModalEnd(MODAL_END_ARG(count));
-
-	UT_QNXBlockWidget(parentWindow, 0);
-	PtDestroyWidget(mainWindow);
-#endif
 }
 
 void XAP_QNXDialog_Insert_Symbol::event_OK(void)
 {
 	m_answer = XAP_Dialog_Insert_Symbol::a_OK;
 	m_Inserted_Symbol = m_CurrentSymbol;
-	printf("Calling insert on OK \n");
 	_onInsertButton();
 }
 
@@ -367,8 +375,9 @@ void XAP_QNXDialog_Insert_Symbol::SymbolMap_clicked( PtCallbackInfo_t * e)
 	PhRect_t         *rect;
 	UT_uint32 		 x, y;
 
-/* Global co-ordinates ... no good
 	ptrevent = (PhPointerEvent_t *)PhGetData(e->event);
+
+/* Global co-ordinates ... no good
 	x = (UT_uint32) event->pos.x;
 	y = (UT_uint32) event->pos.y;
 */
@@ -381,6 +390,17 @@ void XAP_QNXDialog_Insert_Symbol::SymbolMap_clicked( PtCallbackInfo_t * e)
 	m_PreviousSymbol = m_CurrentSymbol;
 	m_CurrentSymbol = iDrawSymbol->calcSymbol(x, y);
 	iDrawSymbol->drawarea(m_CurrentSymbol, m_PreviousSymbol);
+
+	/* Double clicking should also insert the symbol */
+	if (ptrevent->click_count > 1) {
+		event_OK();
+	}
+}
+
+void XAP_QNXDialog_Insert_Symbol::CurrentSymbol_clicked( PtCallbackInfo_t * e)
+{
+	//Single clicks will insert the symbol
+	event_OK();
 }
 
 
@@ -518,6 +538,8 @@ PtWidget_t * XAP_QNXDialog_Insert_Symbol::_constructWindow(void)
 		PtSetArg(&args[n++], Pt_ARG_USER_DATA, &data, sizeof(this)); 
 		PtSetArg(&args[n++], Pt_ARG_RAW_DRAW_F, &s_Symbolarea_exposed, 1); 
 		areaCurrentSym = PtCreateWidget(PtRaw, symgroup, n, args);
+		PtAddEventHandler(areaCurrentSym, Ph_EV_BUT_PRESS /* | Ph_EV_BUT_RELEASE */, 
+							s_CurrentSymbol_clicked, this);
    	}
 
 	n = 0;
