@@ -145,7 +145,6 @@ static int s_radio_Percent_clicked(PtWidget_t *w, void *data, PtCallbackInfo_t *
 }
 static int s_spin_Percent_changed(PtWidget_t *w, void *data, PtCallbackInfo_t *info)
 {
-	printf("Spin changed! \n");
 	XAP_QNXDialog_Zoom *dlg = (XAP_QNXDialog_Zoom *)data;
 	UT_ASSERT(dlg);
 	dlg->event_SpinPercentChanged();
@@ -194,7 +193,7 @@ void XAP_QNXDialog_Zoom::runModal(XAP_Frame * pFrame)
 	// Populate the window's data items
 	_populateWindowData();
 	
-	m_qnxGraphics = new GR_QNXGraphics(mainWindow, m_previewArea);
+	m_qnxGraphics = new GR_QNXGraphics(mainWindow, m_previewArea, pFrame->getApp());
 	unsigned short w, h;
 	UT_QNXGetWidgetArea(m_previewArea, NULL, NULL, &w, &h);
 	_createPreviewFromGC(m_qnxGraphics, w, h);
@@ -241,7 +240,7 @@ void XAP_QNXDialog_Zoom::event_WindowDelete(void)
 void XAP_QNXDialog_Zoom::event_Radio200Clicked(void)
 {
 	//m_zoomType = XAP_Dialog_Zoom::zoomType::z_200;
-	m_zoomType = z_200;
+	m_zoomType = XAP_Frame::z_200;
 	_enablePercentSpin(UT_FALSE);
 	_updatePreviewZoomPercent(200);
 	PtDamageWidget(m_previewArea);
@@ -250,7 +249,7 @@ void XAP_QNXDialog_Zoom::event_Radio200Clicked(void)
 void XAP_QNXDialog_Zoom::event_Radio100Clicked(void)
 {
 	//m_zoomType = XAP_Dialog_Zoom::zoomType::z_100;
-	m_zoomType = z_100;
+	m_zoomType = XAP_Frame::z_100;
 	_enablePercentSpin(UT_FALSE);
 	_updatePreviewZoomPercent(100);
 	PtDamageWidget(m_previewArea);
@@ -258,7 +257,7 @@ void XAP_QNXDialog_Zoom::event_Radio100Clicked(void)
 
 void XAP_QNXDialog_Zoom::event_Radio75Clicked(void)
 {
-	m_zoomType = z_75;
+	m_zoomType = XAP_Frame::z_75;
 	_enablePercentSpin(UT_FALSE);
 	_updatePreviewZoomPercent(75);
 	PtDamageWidget(m_previewArea);
@@ -266,7 +265,7 @@ void XAP_QNXDialog_Zoom::event_Radio75Clicked(void)
 
 void XAP_QNXDialog_Zoom::event_RadioPageWidthClicked(void)
 {
-	m_zoomType = z_PAGEWIDTH;
+	m_zoomType = XAP_Frame::z_PAGEWIDTH;
 	_enablePercentSpin(UT_FALSE);
 	// TODO : figure out the dimensions
 	PtDamageWidget(m_previewArea);
@@ -274,7 +273,7 @@ void XAP_QNXDialog_Zoom::event_RadioPageWidthClicked(void)
 
 void XAP_QNXDialog_Zoom::event_RadioWholePageClicked(void)
 {
-	m_zoomType = z_WHOLEPAGE;
+	m_zoomType = XAP_Frame::z_WHOLEPAGE;
 	_enablePercentSpin(UT_FALSE);
 	// TODO : figure out the dimensions
 	PtDamageWidget(m_previewArea);
@@ -282,7 +281,7 @@ void XAP_QNXDialog_Zoom::event_RadioWholePageClicked(void)
 
 void XAP_QNXDialog_Zoom::event_RadioPercentClicked(void)
 {
-	m_zoomType = z_PERCENT;
+	m_zoomType = XAP_Frame::z_PERCENT;
 	_enablePercentSpin(UT_TRUE);
 	// call event_SpinPercentChanged() to do the fetch and update work
 	event_SpinPercentChanged();
@@ -302,6 +301,7 @@ void XAP_QNXDialog_Zoom::event_SpinPercentChanged(void)
 {
 	m_zoomPercent = get_numeric_value(m_spinPercent);
 	_updatePreviewZoomPercent(m_zoomPercent);
+	PtSetResource(m_radioPercent, Pt_ARG_FLAGS, Pt_SET, Pt_SET);
 }
 
 void XAP_QNXDialog_Zoom::event_PreviewAreaExposed(void)
@@ -354,33 +354,43 @@ PtWidget_t * XAP_QNXDialog_Zoom::_constructWindow(void)
 	windowZoom = PtCreateWidget(PtWindow, NULL, n, args);
 	PtAddCallback(windowZoom, Pt_CB_WINDOW_CLOSING, s_delete_clicked, this);
 
-	n = 0;
 #define MARGIN_SIZE 10 
-	PtSetArg(&args[n++], Pt_ARG_GROUP_ORIENTATION, Pt_GROUP_HORIZONTAL, Pt_GROUP_HORIZONTAL); 
+
+	n = 0;
+	PtSetArg(&args[n++], Pt_ARG_GROUP_ORIENTATION, Pt_GROUP_VERTICAL, 0); 
 	PtSetArg(&args[n++], Pt_ARG_MARGIN_HEIGHT, MARGIN_SIZE, 0); 
 	PtSetArg(&args[n++], Pt_ARG_MARGIN_WIDTH, MARGIN_SIZE, 0); 
+	PtSetArg(&args[n++], Pt_ARG_GROUP_SPACING_Y, MARGIN_SIZE, 0); 
+	PtWidget_t *mainvgroup = PtCreateWidget(PtGroup, windowZoom, n, args);
+
+
+	n = 0;
+	PtSetArg(&args[n++], Pt_ARG_GROUP_ORIENTATION, Pt_GROUP_HORIZONTAL, Pt_GROUP_HORIZONTAL); 
 	PtSetArg(&args[n++], Pt_ARG_GROUP_SPACING_X, MARGIN_SIZE, 0); 
-	hboxFrames = PtCreateWidget(PtGroup, windowZoom, n, args);
+	PtSetArg(&args[n++], Pt_ARG_GROUP_FLAGS, Pt_GROUP_EQUAL_SIZE_VERTICAL, Pt_GROUP_EQUAL_SIZE_VERTICAL);
+	hboxFrames = PtCreateWidget(PtGroup, mainvgroup, n, args);
 
 	n = 0;
 	PtSetArg(&args[n++], Pt_ARG_GROUP_ORIENTATION, Pt_GROUP_VERTICAL, Pt_GROUP_VERTICAL); 
 	PtSetArg(&args[n++], Pt_ARG_GROUP_FLAGS, 
 				Pt_GROUP_EXCLUSIVE | Pt_GROUP_EQUAL_SIZE_HORIZONTAL, 
 				Pt_GROUP_EXCLUSIVE | Pt_GROUP_EQUAL_SIZE_HORIZONTAL);
-	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, "Zoom To", 0); 
 	vboxZoomTo = PtCreateWidget(PtGroup, hboxFrames, n, args);
+	pretty_group(vboxZoomTo, pSS->getValue(XAP_STRING_ID_DLG_Zoom_RadioFrameCaption));
 
 	n = 0;
 	PtSetArg(&args[n++], Pt_ARG_GROUP_ORIENTATION, Pt_GROUP_VERTICAL, Pt_GROUP_VERTICAL); 
-	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, "Preview", 0); 
 	PtSetArg(&args[n++], Pt_ARG_GROUP_SPACING_Y, 10, 0); 
 	vboxZoomPreview = PtCreateWidget(PtGroup, hboxFrames, n, args);
+	pretty_group(vboxZoomPreview, pSS->getValue(XAP_STRING_ID_DLG_Zoom_PreviewFrame));
 
+#if 0
 	n = 0;
 	UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(XAP_STRING_ID_DLG_Zoom_RadioFrameCaption));
 	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, unixstr, 0);
 	PtCreateWidget(PtLabel, vboxZoomTo, n, args);
 	FREEP(unixstr);
+#endif
 
 	n = 0;
 	PtSetArg(&args[n++], Pt_ARG_FLAGS, 0, Pt_GETS_FOCUS);
@@ -441,14 +451,18 @@ PtWidget_t * XAP_QNXDialog_Zoom::_constructWindow(void)
 	PtSetArg(&args[n++], Pt_ARG_NUMERIC_MAX, 500, 0);
 	PtSetArg(&args[n++], Pt_ARG_NUMERIC_MIN,   1, 0);
 	spinbuttonPercent = PtCreateWidget(PtNumericInteger, vboxZoomTo, n, args);
-	PtAddCallback(radiobuttonPercent, Pt_CB_NUMERIC_CHANGED, s_spin_Percent_changed, this);
+	PtAddCallback(spinbuttonPercent, Pt_CB_NUMERIC_CHANGED, s_spin_Percent_changed, this);
+	PtAddCallback(spinbuttonPercent, Pt_CB_ACTIVATE, s_spin_Percent_changed, this);
 
+#if 0
 	n = 0;
 	UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(XAP_STRING_ID_DLG_Zoom_PreviewFrame));
 	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, unixstr, 0);
 	PtCreateWidget(PtLabel, vboxZoomPreview, n, args);
 	FREEP(unixstr);
+#endif
 
+	//This should be set by some string ...
 	n = 0;
 	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, "10 pt Times New Roman", 0);
 	frameSampleText = PtCreateWidget(PtLabel, vboxZoomPreview, n, args);
@@ -456,12 +470,14 @@ PtWidget_t * XAP_QNXDialog_Zoom::_constructWindow(void)
 	n = 0;
 	PtSetArg(&args[n++], Pt_ARG_WIDTH, 150, 0);
 	PtSetArg(&args[n++], Pt_ARG_HEIGHT, 150, 0);
+/*
 	PtSetArg(&args[n++], Pt_ARG_GROUP_ORIENTATION, Pt_GROUP_HORIZONTAL, Pt_GROUP_HORIZONTAL); 
 #define _VS_ANCHOR_ (Pt_LEFT_ANCHORED_RIGHT | Pt_RIGHT_ANCHORED_RIGHT | \
 		     Pt_TOP_ANCHORED_TOP | Pt_BOTTOM_ANCHORED_BOTTOM)
 	PtSetArg(&args[n++], Pt_ARG_ANCHOR_FLAGS, _VS_ANCHOR_, _VS_ANCHOR_); 
 #define _VS_STRETCH_ (Pt_GROUP_STRETCH_HORIZONTAL | Pt_GROUP_STRETCH_VERTICAL)
 	PtSetArg(&args[n++], Pt_ARG_GROUP_FLAGS, _VS_STRETCH_, _VS_STRETCH_); 
+*/
 	PtWidget_t *rawgroup = PtCreateWidget(PtGroup, vboxZoomPreview, n, args);
 
 	n = 0;
@@ -474,7 +490,11 @@ PtWidget_t * XAP_QNXDialog_Zoom::_constructWindow(void)
 
 	n = 0;
 	PtSetArg(&args[n++], Pt_ARG_GROUP_ORIENTATION, Pt_GROUP_HORIZONTAL, Pt_GROUP_HORIZONTAL); 
-	PtWidget_t *vboxButtons = PtCreateWidget(PtGroup, vboxZoomPreview, n, args);
+	PtWidget_t *vboxButtons = PtCreateWidget(PtGroup, mainvgroup, n, args);
+
+	n = 0;
+	PtSetArg(&args[n++], Pt_ARG_WIDTH, 1.5 * ABI_DEFAULT_BUTTON_WIDTH, 0);
+	PtCreateWidget(PtLabel, vboxButtons, n, args);
 
 	n = 0;
 	PtSetArg(&args[n++], Pt_ARG_WIDTH, ABI_DEFAULT_BUTTON_WIDTH, 0);
@@ -548,25 +568,25 @@ void XAP_QNXDialog_Zoom::_populateWindowData(void)
 	_enablePercentSpin(UT_FALSE);	// default
 	switch(getZoomType())
 	{
-	case XAP_QNXDialog_Zoom::z_200:
+	case XAP_Frame::z_200:
 		set_toggle_button(m_radio200, UT_TRUE);
 		_updatePreviewZoomPercent(200);
 		break;
-	case XAP_QNXDialog_Zoom::z_100:
+	case XAP_Frame::z_100:
 		set_toggle_button(m_radio100, UT_TRUE);
 		_updatePreviewZoomPercent(100);		
 		break;
-	case XAP_QNXDialog_Zoom::z_75:
+	case XAP_Frame::z_75:
 		set_toggle_button(m_radio75, UT_TRUE);
 		_updatePreviewZoomPercent(75);
 		break;
-	case XAP_QNXDialog_Zoom::z_PAGEWIDTH:
+	case XAP_Frame::z_PAGEWIDTH:
 		set_toggle_button(m_radioPageWidth, UT_TRUE);
 		break;
-	case XAP_QNXDialog_Zoom::z_WHOLEPAGE:
+	case XAP_Frame::z_WHOLEPAGE:
 		set_toggle_button(m_radioWholePage, UT_TRUE);
 		break;
-	case XAP_QNXDialog_Zoom::z_PERCENT:
+	case XAP_Frame::z_PERCENT:
 		set_toggle_button(m_radioPercent, UT_TRUE);
 		_enablePercentSpin(UT_TRUE);	// override
 		_updatePreviewZoomPercent(getZoomPercent());
@@ -586,7 +606,7 @@ void XAP_QNXDialog_Zoom::_storeWindowData(void)
 	{
 		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(item->data)))
 		{
-			m_zoomType = (XAP_Dialog_Zoom::zoomType)
+			m_zoomType = (XAP_Frame::tZoomType)
 				GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(item->data), WIDGET_ID_TAG_KEY));
 			break;
 		}
