@@ -36,6 +36,9 @@
 #include "xad_Document.h"
 #include "ut_qnxHelper.h"
 
+/* This is required for dynamic zoom implimentation */
+#include "fv_View.h"
+
 //THIS IS BAD!!!
 #include "ap_FrameData.h"
 
@@ -55,7 +58,7 @@ int XAP_QNXFrame::_fe::focus_in_event(PtWidget_t *w, void *data, PtCallbackInfo_
 																		 : AV_FOCUS_NEARBY);
 	*/
 	//I don't understand the AV_FOCUS_HERE vs NEARBY
-	pFrame->getCurrentView()->focusChange(AV_FOCUS_NEARBY);
+	pFrame->getCurrentView()->focusChange(AV_FOCUS_HERE/*AV_FOCUS_NEARBY*/);
 	return Pt_CONTINUE;
 }
 
@@ -121,6 +124,7 @@ int XAP_QNXFrame::_fe::resize(PtWidget_t * w, void *data, PtCallbackInfo_t *info
 
 	XAP_QNXFrame * pQNXFrame = (XAP_QNXFrame *)data;
 	AV_View * pView = pQNXFrame->getCurrentView();
+	FV_View * pfView = static_cast<FV_View*>(pView);
 
 	if (pView) {
 		UT_DEBUGMSG(("Resizing to %d,%d %d,%d \n",
@@ -130,6 +134,23 @@ int XAP_QNXFrame::_fe::resize(PtWidget_t * w, void *data, PtCallbackInfo_t *info
 				     cbinfo->new_size.lr.y - cbinfo->new_size.ul.y);
 	}
 
+	// Dynamic Zoom Implimentation
+	UT_uint32 newZoom = 100;
+	switch(pQNXFrame->getZoomType())
+	{
+	case XAP_Frame::z_PAGEWIDTH:
+		newZoom = pfView->calculateZoomPercentForPageWidth();
+		pQNXFrame->setZoomPercentage(newZoom);
+		break;
+	case XAP_Frame::z_WHOLEPAGE:
+		newZoom = pfView->calculateZoomPercentForWholePage();
+		pQNXFrame->setZoomPercentage(newZoom);
+		break;
+	default:
+		break;
+	}
+
+
 	return Pt_CONTINUE;
 }
 		
@@ -423,7 +444,7 @@ void XAP_QNXFrame::_createTopLevelWindow(void)
 	}
 	//PtAddEventHandler(m_wTopLevelWindow, Ph_EV_KEY, _fe::key_press_event, this);
 	PtAddCallback(m_wTopLevelWindow, Pt_CB_GOT_FOCUS, _fe::focus_in_event, this);
-	PtAddCallback(m_wTopLevelWindow, Pt_CB_GOT_FOCUS, _fe::focus_out_event, this);
+	PtAddCallback(m_wTopLevelWindow, Pt_CB_LOST_FOCUS, _fe::focus_out_event, this);
 
 	/* TODO: Menu and the Toolbars all go into the same Toolbar "group" */
 #if 0
