@@ -160,7 +160,7 @@ void XAP_UnixGnomeFrame::_createTopLevelWindow(void)
 						   m_pUnixApp->getApplicationName());
 
 	gtk_drag_dest_set (m_wTopLevelWindow,
-					   GTK_DEST_DEFAULT_MOTION | GTK_DEST_DEFAULT_DROP,
+					   (GtkDestDefaults)(GTK_DEST_DEFAULT_MOTION|GTK_DEST_DEFAULT_DROP),
 					   drag_types, n_drag_types, GDK_ACTION_COPY);
 
 	gtk_signal_connect(GTK_OBJECT(m_wTopLevelWindow), "drag_data_received",
@@ -196,8 +196,6 @@ void XAP_UnixGnomeFrame::_createTopLevelWindow(void)
 
 	UT_ASSERT(m_pUnixMenu);
 
-
-
 	// create a toolbar instance for each toolbar listed in our base class.
 	// TODO for some reason, the toolbar functions require the TLW to be
 	// TODO realized (they reference m_wTopLevelWindow->window) before we call them.
@@ -243,9 +241,13 @@ void XAP_UnixGnomeFrame::_createTopLevelWindow(void)
 	// Set the size if requested
 	
 	if (f & XAP_UnixApp::GEOMETRY_FLAG_SIZE)
-		gtk_widget_set_usize(m_wTopLevelWindow,
-							 width,
-							 height);
+	{
+                gint abi_width = UT_MIN( gdk_screen_width() - 30, 950);
+                gint abi_height = UT_MIN( gdk_screen_height() - 100, 1200);
+	        gtk_widget_set_usize(m_wTopLevelWindow,
+							 abi_width,
+							 abi_height);
+	}
 
 	// Because we're clever, we only honor this flag when we
 	// are the first (well, only) top level frame available.
@@ -276,8 +278,7 @@ UT_Bool XAP_UnixGnomeFrame::openURL(const char * szURL)
 
 /*****************************************************************/
 
-UT_Bool XAP_UnixGnomeFrame::runModalContextMenu(AV_View * /* pView */, const char * szMenuName,
- 												UT_sint32 x, UT_sint32 y)
+UT_Bool XAP_UnixGnomeFrame::runModalContextMenu(AV_View *  pView, const char * szMenuName,	UT_sint32 x, UT_sint32 y)
 {
  	UT_Bool bResult = UT_TRUE;
  	UT_ASSERT(!m_pUnixPopup);
@@ -298,6 +299,7 @@ UT_Bool XAP_UnixGnomeFrame::runModalContextMenu(AV_View * /* pView */, const cha
  		// button_release_event and we won't know to release our
  		// grab.  so let's do it here.  (when raised from a keyboard
  		// context menu, we may not have a grab, but that should be ok.
+                pUnixPopup->refreshMenu(pView);
 		gnome_popup_menu_do_popup_modal (GTK_WIDGET (pUnixPopup->getMenuHandle ()),
 										 NULL, NULL, NULL, NULL);
  	}
