@@ -34,6 +34,8 @@
 #include "xap_UnixApp.h"
 #include "xap_UnixFrame.h"
 #include "xap_Strings.h"
+#include "xap_Prefs.h"
+#include "ut_debugmsg.h"
 
 /*****************************************************************/
 XAP_Dialog * XAP_UnixDialog_FileOpenSaveAs::static_constructor(XAP_DialogFactory * pFactory,
@@ -101,7 +103,8 @@ UT_Bool XAP_UnixDialog_FileOpenSaveAs::_run_gtk_main(XAP_Frame * pFrame,
 	  bCheckWritePermission.
 	*/
 
-	char * szFinalPathname = NULL;		// this is the file name returned from the dialog
+	char * szDialogFilename = NULL;		// this is the file name returned from the dialog
+	char * szFinalPathname = NULL;		// this is the file name after suffix addition, if any
 	char * szFinalPathnameCopy = NULL;	// one to mangle when looking for dirs, etc.
 
 	char * pLastSlash;
@@ -129,8 +132,8 @@ UT_Bool XAP_UnixDialog_FileOpenSaveAs::_run_gtk_main(XAP_Frame * pFrame,
 			// a file, so we have to catch it, change the dialog, and not return
 			// any filename yet.
 
-			UT_cloneString(szFinalPathname, gtk_file_selection_get_filename(pFS));
-			UT_ASSERT(szFinalPathname);
+			UT_cloneString(szDialogFilename, gtk_file_selection_get_filename(pFS));
+			UT_ASSERT(szDialogFilename);
 
 			err = stat(szFinalPathname, &buf);
 			UT_ASSERT(err == 0 || err == -1);
@@ -139,7 +142,7 @@ UT_Bool XAP_UnixDialog_FileOpenSaveAs::_run_gtk_main(XAP_Frame * pFrame,
 			// set the filter properly and continue in the selection
 			if (err == 0 && S_ISDIR(buf.st_mode))
 			{
-				GString * s = g_string_new(szFinalPathname);
+				GString * s = g_string_new(szDialogFilename);
 				if (s->str[s->len - 1] != '/')
 				{
 					g_string_append_c(s, '/');
@@ -148,11 +151,11 @@ UT_Bool XAP_UnixDialog_FileOpenSaveAs::_run_gtk_main(XAP_Frame * pFrame,
 				g_string_free(s, TRUE);
 
 				// free the string and continue along
-				FREEP(szFinalPathname);
+				FREEP(szDialogFilename);
 				continue;
 			}
 
-			UT_cloneString(m_szFinalPathnameCandidate, szFinalPathname);
+			UT_cloneString(m_szFinalPathnameCandidate, szDialogFilename);
 			
 			// if we got here, the text wasn't a directory, so it's a file,
 			// and life is good
@@ -170,16 +173,14 @@ UT_Bool XAP_UnixDialog_FileOpenSaveAs::_run_gtk_main(XAP_Frame * pFrame,
 
 		// Give us a filename we can mangle
 
-		UT_cloneString(szFinalPathname, gtk_file_selection_get_filename(pFS));
-		UT_ASSERT(szFinalPathname);
+		UT_cloneString(szDialogFilename, gtk_file_selection_get_filename(pFS));
+		UT_ASSERT(szDialogFilename);
 
-		// OLD // We append the suffix of the default type, so the user doesn't
-		// OLD // have to.  This is adapted from the Windows front-end code
-		// OLD // (xap_Win32Dlg_FileOpenSaveAs.cpp), since it should act the same.
+		// We append the suffix of the default type, so the user doesn't
+	        // have to.  This is adapted from the Windows front-end code
+		// (xap_Win32Dlg_FileOpenSaveAs.cpp), since it should act the same.
+		// If, however, the user doesn't want suffixes, they don't have to have them.  
 
-		// Sorry, this is UNIX, not some crappy MS "OS". If the user wants the file
-		// to be named foo, name it foo. It is unacceptably inconsistant and
-		// disobediant to do otherwise.
 		{
 			//UT_uint32 end = UT_pointerArrayLength((void **) m_szSuffixes);
 
@@ -204,8 +205,51 @@ UT_Bool XAP_UnixDialog_FileOpenSaveAs::_run_gtk_main(XAP_Frame * pFrame,
 					break;
 				}
 			}
-		}
+			UT_Bool wantSuffix = UT_TRUE;
+			
+			XAP_Prefs *pPrefs= pFrame->getApp()->getPrefs();
 
+			pPrefs->getPrefsValueBool((const XML_Char *)XAP_PREF_KEY_UseSuffix, &wantSuffix);
+
+			UT_DEBUGMSG(("UseSuffix: %d\n", wantSuffix));
+
+			// if the file doesn't have a suffix already, and the file type
+			// is normal (special types are negative, like auto detect),
+			// and the user wants extensions, slap a suffix on it.   
+			if ((!UT_pathSuffix(szDialogFilename)) && 
+			    (nFileType > 0) && wantSuffix)                                
+				{                                                       
+					// add suffix based on selected file type       
+					const char * szSuffix = UT_pathSuffix(m_szSuffixes[nIndex]);
+					UT_ASSERT(szSuffix);                            
+					UT_uint32 length = strlen(szDialogFilename) + strlen(szSuffix) + 1;
+					
+					szFinalPathname = (char *)calloc(length,sizeof(char));
+					
+					if (szFinalPathname)                            						
+						{                                               
+							char * p = szFinalPathname;             
+							strcpy(p,szDialogFilename);             
+							strcat(p,szSuffix);                     
+						}                                               
+					
+				}                                                       
+			else                                                    
+				{                                                       
+					// the file type is special (auto detect)       
+					// set to plain name, and let the auto detector in the
+					// exporter figure it out                       
+					UT_cloneString(szFinalPathname,szDialogFilename);
+				}                                                       
+			// free szDialogFilename since it's been put into szFinalPathname (with
+			// or without changes) and it's invalid (missing an extension which
+			// might have been appended)                            
+			
+			FREEP(szDialogFilename);   
+
+			
+		}
+		
 		UT_cloneString(szFinalPathnameCopy, szFinalPathname);
 		
 		err = stat(szFinalPathnameCopy, &buf);
@@ -351,6 +395,7 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 	const XML_Char * szFileTypeLabel = NULL;
 	switch (m_id)
 	{
+	case XAP_DIALOG_ID_INSERT_PICTURE:
 	case XAP_DIALOG_ID_FILE_OPEN:
 	{
 		szTitle = pSS->getValue(XAP_STRING_ID_DLG_FOSA_OpenTitle);
@@ -555,7 +600,6 @@ void XAP_UnixDialog_FileOpenSaveAs::runModal(XAP_Frame * pFrame)
 	UT_ASSERT(parent);
 	// center it
     centerDialog(parent, GTK_WIDGET(pFS));
-	gtk_window_set_transient_for(GTK_WINDOW(pFS), GTK_WINDOW(parent));
 	
 	gtk_widget_show(GTK_WIDGET(pFS));
 	gtk_grab_add(GTK_WIDGET(pFS));
