@@ -230,7 +230,6 @@ void XAP_UnixDialog_Print::_raisePrintDialog(XAP_Frame * pFrame)
 	
 	gtk_window_set_title (GTK_WINDOW (window), pSS->getValue(XAP_STRING_ID_DLG_UP_PrintTitle));
 	gtk_container_set_border_width (GTK_CONTAINER (window), 0);
-	gtk_widget_set_usize (window, 370, 275);
 
 	// Add a main vbox
 	vbox1 = gtk_vbox_new (FALSE, 0);
@@ -492,7 +491,6 @@ void XAP_UnixDialog_Print::_raisePrintDialog(XAP_Frame * pFrame)
 	UT_ASSERT(parent);
 	// center it
     centerDialog(parent, window);
-	gtk_window_set_transient_for(GTK_WINDOW(window), GTK_WINDOW(parent));
 	
 	gtk_widget_show (window);
 	gtk_grab_add(GTK_WIDGET(window));
@@ -573,14 +571,14 @@ void XAP_UnixDialog_Print::_getGraphics(void)
 		m_pPSGraphics = new PS_Graphics(m_szPrintToFilePathname, m_szDocumentTitle,
 										m_pUnixFrame->getApp()->getApplicationName(),
 										fontmgr,
-										UT_TRUE);
+										UT_TRUE, app);
 	}
 	else
 	{		
 		m_pPSGraphics = new PS_Graphics(m_szPrintCommand, m_szDocumentTitle,
 										m_pUnixFrame->getApp()->getApplicationName(),
 										fontmgr,
-										UT_FALSE);
+										UT_FALSE, app);
 	}
 
 	UT_ASSERT(m_pPSGraphics);
