@@ -168,7 +168,6 @@ void XAP_UnixDialog_Zoom::runModal(XAP_Frame * pFrame)
 	// Center our new dialog in its parent and make it a transient
 	// so it won't get lost underneath
     centerDialog(parentWindow, mainWindow);
-	gtk_window_set_transient_for(GTK_WINDOW(mainWindow), GTK_WINDOW(parentWindow));
 
 	// Show the top level dialog,
 	gtk_widget_show(mainWindow);
@@ -185,7 +184,7 @@ void XAP_UnixDialog_Zoom::runModal(XAP_Frame * pFrame)
 		UT_ASSERT(m_previewArea && m_previewArea->window);
 
 		// make a new Unix GC
-		m_unixGraphics = new GR_UnixGraphics(m_previewArea->window, unixapp->getFontManager());
+		m_unixGraphics = new GR_UnixGraphics(m_previewArea->window, unixapp->getFontManager(), m_pApp);
 		
 		// let the widget materialize
 		_createPreviewFromGC(m_unixGraphics,
@@ -343,7 +342,7 @@ GtkWidget * XAP_UnixDialog_Zoom::_constructWindow(void)
 	FREEP(tmp);
 	vboxZoomTo_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton200));
 	gtk_object_set_data (GTK_OBJECT (windowZoom), "radiobutton200", radiobutton200);
-	gtk_object_set_data (GTK_OBJECT (radiobutton200), WIDGET_ID_TAG_KEY, GINT_TO_POINTER(z_200));
+	gtk_object_set_data (GTK_OBJECT (radiobutton200), WIDGET_ID_TAG_KEY, GINT_TO_POINTER(XAP_Frame::z_200));
 	gtk_widget_show (radiobutton200);
 	gtk_box_pack_start (GTK_BOX (vboxZoomTo), radiobutton200, FALSE, TRUE, 0);
 
@@ -352,7 +351,7 @@ GtkWidget * XAP_UnixDialog_Zoom::_constructWindow(void)
 	FREEP(tmp);
 	vboxZoomTo_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton100));
 	gtk_object_set_data (GTK_OBJECT (windowZoom), "radiobutton100", radiobutton100);
-	gtk_object_set_data (GTK_OBJECT (radiobutton100), WIDGET_ID_TAG_KEY, GINT_TO_POINTER(z_100));
+	gtk_object_set_data (GTK_OBJECT (radiobutton100), WIDGET_ID_TAG_KEY, GINT_TO_POINTER(XAP_Frame::z_100));
 	gtk_widget_show (radiobutton100);
 	gtk_box_pack_start (GTK_BOX (vboxZoomTo), radiobutton100, FALSE, TRUE, 0);
 
@@ -361,7 +360,7 @@ GtkWidget * XAP_UnixDialog_Zoom::_constructWindow(void)
 	FREEP(tmp);
 	vboxZoomTo_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton75));
 	gtk_object_set_data (GTK_OBJECT (windowZoom), "radiobutton75", radiobutton75);
-	gtk_object_set_data (GTK_OBJECT (radiobutton75), WIDGET_ID_TAG_KEY, GINT_TO_POINTER(z_75));
+	gtk_object_set_data (GTK_OBJECT (radiobutton75), WIDGET_ID_TAG_KEY, GINT_TO_POINTER(XAP_Frame::z_75));
 	gtk_widget_show (radiobutton75);
 	gtk_box_pack_start (GTK_BOX (vboxZoomTo), radiobutton75, TRUE, TRUE, 0);
 
@@ -370,7 +369,7 @@ GtkWidget * XAP_UnixDialog_Zoom::_constructWindow(void)
 	FREEP(tmp);
 	vboxZoomTo_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobuttonPageWidth));
 	gtk_object_set_data (GTK_OBJECT (windowZoom), "radiobuttonPageWidth", radiobuttonPageWidth);
-	gtk_object_set_data (GTK_OBJECT (radiobuttonPageWidth), WIDGET_ID_TAG_KEY, GINT_TO_POINTER(z_PAGEWIDTH));
+	gtk_object_set_data (GTK_OBJECT (radiobuttonPageWidth), WIDGET_ID_TAG_KEY, GINT_TO_POINTER(XAP_Frame::z_PAGEWIDTH));
 	gtk_widget_show (radiobuttonPageWidth);
 	gtk_box_pack_start (GTK_BOX (vboxZoomTo), radiobuttonPageWidth, TRUE, TRUE, 0);
 
@@ -379,7 +378,7 @@ GtkWidget * XAP_UnixDialog_Zoom::_constructWindow(void)
 	FREEP(tmp);
 	vboxZoomTo_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobuttonWholePage));
 	gtk_object_set_data (GTK_OBJECT (windowZoom), "radiobuttonWholePage", radiobuttonWholePage);
-	gtk_object_set_data (GTK_OBJECT (radiobuttonWholePage), WIDGET_ID_TAG_KEY, GINT_TO_POINTER(z_WHOLEPAGE));
+	gtk_object_set_data (GTK_OBJECT (radiobuttonWholePage), WIDGET_ID_TAG_KEY, GINT_TO_POINTER(XAP_Frame::z_WHOLEPAGE));
 	gtk_widget_show (radiobuttonWholePage);
 	gtk_box_pack_start (GTK_BOX (vboxZoomTo), radiobuttonWholePage, TRUE, TRUE, 0);
 
@@ -388,7 +387,7 @@ GtkWidget * XAP_UnixDialog_Zoom::_constructWindow(void)
 	FREEP(tmp);
 	vboxZoomTo_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobuttonPercent));
 	gtk_object_set_data (GTK_OBJECT (windowZoom), "radiobuttonPercent", radiobuttonPercent);
-	gtk_object_set_data (GTK_OBJECT (radiobuttonPercent), WIDGET_ID_TAG_KEY, GINT_TO_POINTER(z_PERCENT));
+	gtk_object_set_data (GTK_OBJECT (radiobuttonPercent), WIDGET_ID_TAG_KEY, GINT_TO_POINTER(XAP_Frame::z_PERCENT));
 	gtk_widget_show (radiobuttonPercent);
 	gtk_box_pack_start (GTK_BOX (vboxZoomTo), radiobuttonPercent, TRUE, TRUE, 0);
 
@@ -411,7 +410,10 @@ GtkWidget * XAP_UnixDialog_Zoom::_constructWindow(void)
 	gtk_object_set_data (GTK_OBJECT (windowZoom), "frameSampleText", frameSampleText);
 	gtk_widget_show (frameSampleText);
 	gtk_container_add (GTK_CONTAINER (framePreview), frameSampleText);
+#if 0	
+	/*setting size of 'frameSampleText' makes no sense - VH*/
 	gtk_widget_set_usize (frameSampleText, 221, 97);
+#endif	
 	gtk_container_border_width (GTK_CONTAINER (frameSampleText), 10);
 
 	// *** This is how we do a preview widget ***
@@ -538,25 +540,25 @@ void XAP_UnixDialog_Zoom::_populateWindowData(void)
 	_enablePercentSpin(UT_FALSE);	// default
 	switch(getZoomType())
 	{
-	case XAP_UnixDialog_Zoom::z_200:
+	case XAP_Frame::z_200:
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_radio200), TRUE);
 		_updatePreviewZoomPercent(200);
 		break;
-	case XAP_UnixDialog_Zoom::z_100:
+	case XAP_Frame::z_100:
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_radio100), TRUE);
 		_updatePreviewZoomPercent(100);		
 		break;
-	case XAP_UnixDialog_Zoom::z_75:
+	case XAP_Frame::z_75:
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_radio75), TRUE);
 		_updatePreviewZoomPercent(75);
 		break;
-	case XAP_UnixDialog_Zoom::z_PAGEWIDTH:
+	case XAP_Frame::z_PAGEWIDTH:
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_radioPageWidth), TRUE);
 		break;
-	case XAP_UnixDialog_Zoom::z_WHOLEPAGE:
+	case XAP_Frame::z_WHOLEPAGE:
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_radioWholePage), TRUE);
 		break;
-	case XAP_UnixDialog_Zoom::z_PERCENT:
+	case XAP_Frame::z_PERCENT:
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_radioPercent), TRUE);
 		_enablePercentSpin(UT_TRUE);	// override
 		_updatePreviewZoomPercent(getZoomPercent());
@@ -575,7 +577,7 @@ void XAP_UnixDialog_Zoom::_storeWindowData(void)
 	{
 		if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(item->data)))
 		{
-			m_zoomType = (XAP_Dialog_Zoom::zoomType)
+			m_zoomType = (XAP_Frame::tZoomType)
 				GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(item->data), WIDGET_ID_TAG_KEY));
 			break;
 		}
