@@ -31,6 +31,7 @@
 #include "xap_UnixApp.h"
 #include "xap_UnixFontManager.h"
 #include "xap_UnixFontXLFD.h"
+#include "xap_EncodingManager.h"
 
 
 // TODO get this from some higher-level place
@@ -67,14 +68,19 @@ UT_Bool XAP_UnixFontManager::scavengeFonts(void)
 	UT_uint32 totaldirs = 0;
 	UT_uint32 totalfonts = 0;
 	
+	const char** subdirs = localeinfo_combinations("","","");
+	/* If j is even, we open fonts.dir in encoding-specific directory */
 	for (i = 0; i < count; i++)
 	{
-		UT_ASSERT(m_searchPaths.getNthItem(i));
-
-		char * filename = (char *) calloc(strlen((char *) m_searchPaths.getNthItem(i)) +
-										  strlen((char *) FONTS_DIR_FILE) + 1, sizeof(char));
-
-		sprintf(filename, "%s%s", (char *) m_searchPaths.getNthItem(i), FONTS_DIR_FILE);
+	    UT_ASSERT(m_searchPaths.getNthItem(i));	
+	    char* basedirname = (char *) m_searchPaths.getNthItem(i);
+	    int basedirname_len = strlen(basedirname);
+	    char* filename;
+	    
+	    for(const char** subdir = subdirs;*subdir;++subdir)
+	    {	    	
+		filename = (char *) calloc(basedirname_len + 1 + strlen(*subdir) + strlen((char *) FONTS_DIR_FILE) + 1, sizeof(char));
+		sprintf(filename, "%s/%s%s", basedirname, *subdir, FONTS_DIR_FILE);
 
 		FILE * file;
 
@@ -112,6 +118,10 @@ UT_Bool XAP_UnixFontManager::scavengeFonts(void)
 				UT_DEBUGMSG(("File says %d fonts should follow...\n", fontcount));
 			}
 #endif
+
+			char* lastslash = strrchr(filename,'/');
+			if (lastslash)
+				*lastslash = '\0';
 			
 			// every line after is a font name / XLFD pair
 			UT_sint32 line;
@@ -130,8 +140,10 @@ UT_Bool XAP_UnixFontManager::scavengeFonts(void)
 					fclose(file);
 					return UT_TRUE;
 				}
-				_allocateThisFont((const char *) buffer,
-								  (const char *) m_searchPaths.getNthItem(i));
+				if (subdir != subdirs && XAP_EncodingManager::instance->cjk_locale())
+				    _allocateCJKFont((const char *) buffer,line);
+				else
+				    _allocateThisFont((const char *) buffer, filename,line);
 			}
 
 			totalfonts += line;
@@ -144,6 +156,7 @@ UT_Bool XAP_UnixFontManager::scavengeFonts(void)
 		FREEP(filename);
 		if (file)
 			fclose(file);
+	    }
 	}
 
 	if (totaldirs == 0)
@@ -277,7 +290,7 @@ XAP_UnixFont * XAP_UnixFontManager::getFont(const char * fontname,
 }
 
 void XAP_UnixFontManager::_allocateThisFont(const char * line,
-											const char * workingdir)
+											const char * workingdir,int iLine)
 {
 	/*
 	  Each line comes in as:
@@ -365,27 +378,20 @@ void XAP_UnixFontManager::_allocateThisFont(const char * line,
 	UT_ASSERT(slant);
 
 	XAP_UnixFont::style s = XAP_UnixFont::STYLE_NORMAL;
-	// sort from most common down
-	if (!UT_stricmp(weight, "regular") &&
-		!UT_stricmp(slant, "r"))
-	{
-		s = XAP_UnixFont::STYLE_NORMAL;
-	}
-	else if (!UT_stricmp(weight, "bold") &&
-			!UT_stricmp(slant, "r"))
-	{
-		s = XAP_UnixFont::STYLE_BOLD;
-	}
-	else if (!UT_stricmp(weight, "regular") &&
-			!UT_stricmp(slant, "i"))
-	{
-		s = XAP_UnixFont::STYLE_ITALIC;
-	}
-	else if (!UT_stricmp(weight, "bold") &&
-			!UT_stricmp(slant, "i"))
-	{
-		s = XAP_UnixFont::STYLE_BOLD_ITALIC;
-	}
+	if(!UT_stricmp(slant, "r"))
+	  {
+		if(!UT_stricmp(weight, "bold"))
+		  s = XAP_UnixFont::STYLE_BOLD;
+		else
+		  s = XAP_UnixFont::STYLE_NORMAL;
+	  }
+	else if(!UT_stricmp(slant, "i"))
+	  {
+		if(!UT_stricmp(weight, "bold"))
+		  s = XAP_UnixFont::STYLE_BOLD_ITALIC;
+		else
+		  s= XAP_UnixFont::STYLE_ITALIC;
+	  }
     else
     {
         UT_DEBUGMSG(("XAP_UnixFontManager::_allocateThisFont() - can't guess "
@@ -417,11 +423,15 @@ void XAP_UnixFontManager::_allocateThisFont(const char * line,
 
 	// build a font and load it up
 	XAP_UnixFont * font = new XAP_UnixFont;
+	font->set_CJK_font(0);
 	if (font->openFileAs((const char *) fontfile,
 						 (const char *) metricfile,
 						 (const char *) xlfd,
 						 s))
 	{
+		UT_ASSERT(iLine>=0);
+		if(iLine<4)
+			XAP_UnixFont::s_defaultNonCJKFont[iLine]=font;
 		_addFont(font);
 	}
 	else
@@ -434,9 +444,100 @@ void XAP_UnixFontManager::_allocateThisFont(const char * line,
 	FREEP(linedup);
 }
 	  
-void XAP_UnixFontManager::_addFont(XAP_UnixFont * font)
+void XAP_UnixFontManager::_addFont(XAP_UnixFont * newfont)
 {
 	// we index fonts by a combined "name" and "style"
+	const char* fontkey = newfont->getFontKey();
+	UT_HashEntry* curfont_entry = m_fontHash.findEntry(fontkey);
+	if (!curfont_entry || !curfont_entry->pData)
+	{
+		m_fontHash.addEntry(fontkey, NULL, (void *) newfont);
+	} 
+	else 
+	{
+		/* 
+                  since "standard fonts" folder is read first and then
+                  current charset-specific subdirectory, it's obvious that
+                  the recent version is current charset-specific font, 
+                  so we replace original font (that is standard) 
+                  unconditionally.
+		*/
+                XAP_UnixFont* curfont = static_cast<XAP_UnixFont*>(curfont_entry->pData);               
+                delete curfont;
+                curfont_entry->pData = (void*)newfont;
 
-	m_fontHash.addEntry(font->getFontKey(), NULL, (void *) font);
+	       
+	}
 }
+
+void XAP_UnixFontManager::_allocateCJKFont(const char * line,
+											   int iLine)
+{
+	gchar **sa= g_strsplit(line,",",5);
+
+	g_strstrip(sa[0]);
+		
+	gchar * xlfd =sa[1];
+	if (!xlfd)
+	{
+		UT_DEBUGMSG(("XAP_UnixFontManager::_allocateThisFont() - missing XLFD "
+					 "file name at second position in line.\n"));
+		g_strfreev(sa);
+		return;		
+	}
+	// clean up that XLFD by removing surrounding spaces
+	xlfd = g_strstrip(xlfd);
+
+	// let this class munch on all the fields
+	XAP_UnixFontXLFD descriptor(xlfd);
+
+	// get the weight (should be "regular" or "bold") and the slant (should be
+	// "r" for Roman or "i" for Italic) to discern an internal "style"
+	const char * weight = descriptor.getWeight();
+	UT_ASSERT(weight);
+	const char * slant = descriptor.getSlant();
+	UT_ASSERT(slant);
+
+	XAP_UnixFont::style s = XAP_UnixFont::STYLE_NORMAL;
+	// sort from most common down
+
+	if(!UT_stricmp(slant, "r"))
+	  {
+		if(!UT_stricmp(weight, "bold"))
+		  s = XAP_UnixFont::STYLE_BOLD;
+		else
+		  s = XAP_UnixFont::STYLE_NORMAL;
+	  }
+	else if(!UT_stricmp(slant, "i"))
+	  {
+		if(!UT_stricmp(weight, "bold"))
+		  s = XAP_UnixFont::STYLE_BOLD_ITALIC;
+		else
+		  s= XAP_UnixFont::STYLE_ITALIC;
+	  }
+	
+	// build a font and load it up
+	XAP_UnixFont * font = new XAP_UnixFont;
+	font->set_CJK_Ascent(atoi(sa[2]));
+	font->set_CJK_Descent(atoi(sa[3]));
+	font->set_CJK_Width(atoi(sa[4]));
+	font->set_CJK_font(1);
+	if (font->openFileAs((const char *) sa[0],
+						 (const char *) sa[0],
+						 (const char *) xlfd,
+						 s))
+	{
+	  
+	  UT_ASSERT(iLine>=0);
+	  if(iLine<4)
+		XAP_UnixFont::s_defaultCJKFont[iLine]=font;
+	  _addFont(font);
+	}
+	else
+	{
+		DELETEP(font);
+	}
+
+	g_strfreev(sa);
+}
+
