@@ -20,6 +20,8 @@
 #include <gtk/gtk.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
+#include <sys/stat.h>
 
 #include "ut_types.h"
 #include "ut_debugmsg.h"
@@ -36,6 +38,9 @@
 #include "xav_View.h"
 #include "xad_Document.h"
 
+/* This is required for dynamic zoom implimentation */
+#include "fv_View.h"
+
 
 /*****************************************************************/
 
@@ -172,7 +177,8 @@ gboolean XAP_UnixFrame::_fe::focus_in_event(GtkWidget *w,GdkEvent */*event*/,gpo
 	UT_ASSERT(pFrame);
 	gtk_object_set_data(GTK_OBJECT(w), "toplevelWindowFocus",
 						GINT_TO_POINTER(TRUE));
-	pFrame->getCurrentView()->focusChange(gtk_grab_get_current() == NULL || gtk_grab_get_current() == w ? AV_FOCUS_HERE : AV_FOCUS_NEARBY);
+	if (pFrame->getCurrentView())
+		pFrame->getCurrentView()->focusChange(gtk_grab_get_current() == NULL || gtk_grab_get_current() == w ? AV_FOCUS_HERE : AV_FOCUS_NEARBY);
 	return FALSE;
 }
 
@@ -182,7 +188,8 @@ gboolean XAP_UnixFrame::_fe::focus_out_event(GtkWidget *w,GdkEvent */*event*/,gp
 	UT_ASSERT(pFrame);
 	gtk_object_set_data(GTK_OBJECT(w), "toplevelWindowFocus",
 						GINT_TO_POINTER(FALSE));
-	pFrame->getCurrentView()->focusChange(AV_FOCUS_NONE);
+	if (pFrame->getCurrentView())
+		pFrame->getCurrentView()->focusChange(AV_FOCUS_NONE);
 	return FALSE;
 }
 
@@ -206,6 +213,7 @@ gint XAP_UnixFrame::_fe::button_release_event(GtkWidget * w, GdkEventButton * e)
 	XAP_UnixFrame * pUnixFrame = (XAP_UnixFrame *)gtk_object_get_user_data(GTK_OBJECT(w));
 	pUnixFrame->setTimeOfLastEvent(e->time);
 	AV_View * pView = pUnixFrame->getCurrentView();
+
 	EV_UnixMouse * pUnixMouse = static_cast<EV_UnixMouse *>(pUnixFrame->getMouse());
 
 	//UT_DEBUGMSG(("Ungrabbing mouse.\n"));
@@ -223,10 +231,27 @@ gint XAP_UnixFrame::_fe::configure_event(GtkWidget* w, GdkEventConfigure *e)
 		
 	XAP_UnixFrame * pUnixFrame = (XAP_UnixFrame *)gtk_object_get_user_data(GTK_OBJECT(w));
 	AV_View * pView = pUnixFrame->getCurrentView();
+	FV_View * pfView = static_cast<FV_View*>(pView);
 
 	if (pView)
 		pView->setWindowSize(e->width, e->height);
 
+	// Dynamic Zoom Implimentation
+	UT_uint32 newZoom = 100;
+	switch(pUnixFrame->getZoomType())
+	{
+	case XAP_Frame::z_PAGEWIDTH:
+		newZoom = pfView->calculateZoomPercentForPageWidth();
+		pUnixFrame->setZoomPercentage(newZoom);
+		break;
+	case XAP_Frame::z_WHOLEPAGE:
+		newZoom = pfView->calculateZoomPercentForWholePage();
+		pUnixFrame->setZoomPercentage(newZoom);
+		break;
+	default:
+		;
+	}
+
 	return 1;
 }
 	
@@ -579,11 +604,16 @@ void XAP_UnixFrame::_createTopLevelWindow(void)
 	m_pUnixApp->getGeometry(&x, &y, &width, &height, &f);
 
 	// Set the size if requested
-	
+
 	if (f & XAP_UnixApp::GEOMETRY_FLAG_SIZE)
-		gtk_widget_set_usize(m_wTopLevelWindow,
-							 width,
-							 height);
+	{
+                gint abi_width = UT_MIN( gdk_screen_width() - 30, 950);
+                gint abi_height = UT_MIN( gdk_screen_height() - 100, 1200);
+	        gtk_widget_set_usize(m_wTopLevelWindow,
+							 abi_width,
+							 abi_height);
+	}
+
 
 	// Because we're clever, we only honor this flag when we
 	// are the first (well, only) top level frame available.
@@ -626,17 +656,31 @@ UT_Bool XAP_UnixFrame::show()
 
 UT_Bool XAP_UnixFrame::openURL(const char * szURL)
 {
-	// TODO : FIX THIS.  Find a better way to search for
-	// TODO : other browsers on your machine.
+  	char *execstring;
+	struct stat * statbuf = (struct stat *) malloc(sizeof(struct stat));
 
-	// Try to connect to a running Netscape, if not, start new one
-
-	char execstring[4096];
-
-	g_snprintf(execstring, 4096, "netscape -remote openURL\\(%s\\) "
-			   "|| netscape %s &", szURL, szURL);
+	//
+	// The gnome-help-browser sucks right now. Instead open with netscape
+	// or kde.
+	// When it gets better we should restore this code.
+	/*
+	//if (!stat("/opt/gnome/bin/gnome-help-browser", statbuf) || !stat("/usr/local/bin/gnome-help-browser", statbuf) || !stat("/usr/bin/gnome-help-browser", statbuf))
+	{
+		execstring = g_strdup_printf("gnome-help-browser %s &", szURL);
+	}
+	*/
+	if (!stat("/opt/kde/bin/kdehelp", statbuf) || !stat("/usr/local/kde/bin/kdehelp", statbuf) || !stat("/usr/local/bin/kdehelp", statbuf) || !stat("/usr/bin/kdehelp", statbuf))
+	{
+		execstring = g_strdup_printf("kdehelp %s &", szURL);
+	}
+	else
+	{
+		// Try to connect to a running Netscape, if not, start new one
+		execstring = g_strdup_printf("netscape -remote openURL\\(%s\\) || netscape %s &", szURL, szURL);
+	}
 	system(execstring);
-	
+	g_free(execstring);
+	FREEP(statbuf);
 	return UT_FALSE;
 }
 
@@ -728,3 +772,9 @@ EV_Toolbar * XAP_UnixFrame::_newToolbar(XAP_App *app, XAP_Frame *frame,
 							   static_cast<XAP_UnixFrame *>(frame), 
 							   szLayout, szLanguage));
 }
+
+void XAP_UnixFrame::queue_resize()
+{
+	UT_DEBUGMSG(("XAP_UnixFrame::queue_resize\n"));	
+	gtk_widget_queue_resize(m_wTopLevelWindow);
+}
