@@ -19,12 +19,14 @@
 
 #include <stdio.h>
 #include <string.h>
+#include <locale.h>
 
 #include "ut_types.h"
 #include "ut_assert.h"
 #include "ut_debugmsg.h"
 #include "ut_string.h"
 #include "ut_misc.h"
+#include "ut_wctomb.h"
 #include "ut_dialogHelper.h"
 
 #include "xap_UnixPSGenerate.h"
@@ -35,6 +37,7 @@
 #include "xap_UnixFont.h"
 #include "xap_UnixFontManager.h"
 #include "xap_UnixFontXLFD.h"
+#include "xap_EncodingManager.h"
 
 // the resolution that we report to the application (pixels per inch).
 #define PS_RESOLUTION		7200
@@ -50,9 +53,11 @@ PS_Graphics::PS_Graphics(const char * szFilename,
 						 const char * szTitle,
 						 const char * szSoftwareNameAndVersion,
 						 XAP_UnixFontManager * fontManager,						 
-						 UT_Bool	  bIsFile)
+						 UT_Bool	  bIsFile,
+						 XAP_App *pApp)
 {
 	UT_ASSERT(szFilename && *szFilename);
+	m_pApp = pApp;
 	m_szFilename = szFilename;
 	m_szTitle = szTitle;
 	m_szSoftwareNameAndVersion = szSoftwareNameAndVersion;
@@ -109,64 +114,84 @@ void PS_Graphics::setFont(GR_Font* pFont)
 
 UT_uint32 PS_Graphics::getFontAscent()
 {
-	UT_ASSERT(m_pCurrentFont);
-
-	GlobalFontInfo * gfi = m_pCurrentFont->getMetricsData()->gfi;
-	UT_ASSERT(gfi);
-
+  UT_ASSERT(m_pCurrentFont);
+  PSFont *pEnglishFont;
+  PSFont *pChineseFont;
+  explodePSFonts(pEnglishFont,pChineseFont);
+  UT_ASSERT(pEnglishFont && pChineseFont);
+  int e,c;
+  GlobalFontInfo * gfi = pEnglishFont->getMetricsData()->gfi;
+  UT_ASSERT(gfi);
 #if 0
-	return _scale(gfi->ascender);
+  e = _scale(gfi->ascender);
 #else
-	return _scale(gfi->fontBBox.ury);
+  e = _scale(gfi->fontBBox.ury);
 #endif	
+  c= pChineseFont == pEnglishFont ? 0 :_scale(pChineseFont->getUnixFont()->get_CJK_Ascent());
+  return MAX(e,c);
 }
 
 UT_uint32 PS_Graphics::getFontDescent()
 {
-	UT_ASSERT(m_pCurrentFont);
-
-	GlobalFontInfo * gfi = m_pCurrentFont->getMetricsData()->gfi;
-	UT_ASSERT(gfi);
-
+  UT_ASSERT(m_pCurrentFont);
+  PSFont *pEnglishFont;
+  PSFont *pChineseFont;
+  explodePSFonts(pEnglishFont,pChineseFont);  
+  UT_ASSERT(pEnglishFont && pChineseFont);
+  int e,c;
+  GlobalFontInfo * gfi = pEnglishFont->getMetricsData()->gfi;
+  UT_ASSERT(gfi);
 #if 0	
-	UT_ASSERT(gfi->descender <= 0);
-	return _scale(-gfi->descender);
+  UT_ASSERT(gfi->descender <= 0);
+  e=_scale(-gfi->descender);
 #else
-	UT_ASSERT(gfi->fontBBox.lly <= 0);
-	return _scale(-(gfi->fontBBox.lly));
+  UT_ASSERT(gfi->fontBBox.lly <= 0);
+  e=_scale(-(gfi->fontBBox.lly));
 #endif	
+  c= pChineseFont == pEnglishFont ? 0 :_scale(pChineseFont->getUnixFont()->get_CJK_Descent());
+  return MAX(e,c);
 }
 
 UT_uint32 PS_Graphics::getFontHeight()
 {
-	UT_ASSERT(m_pCurrentFont);
-
-	GlobalFontInfo * gfi = m_pCurrentFont->getMetricsData()->gfi;
-	UT_ASSERT(gfi);
-
-#if 0
-	UT_ASSERT(gfi->descender <= 0);
-	return _scale(gfi->ascender - gfi->descender);
-#else
-	UT_ASSERT(gfi->fontBBox.lly <= 0);
-	UT_ASSERT((gfi->fontBBox.ury - gfi->fontBBox.lly) >= (gfi->ascender - gfi->descender));
-	return _scale(gfi->fontBBox.ury - gfi->fontBBox.lly);
-#endif
+  UT_ASSERT(m_pCurrentFont);
+  return getFontAscent()+getFontDescent();
 }
 	
+UT_uint32 PS_Graphics::measureUnRemappedChar(const UT_UCSChar c)
+{
+ 	UT_ASSERT(m_pCurrentFont);
+	PSFont *pEnglishFont;
+	PSFont *pChineseFont;
+	explodePSFonts(pEnglishFont,pChineseFont);  
+	const UT_uint16 * cwi = pEnglishFont->getUniWidths();
+	if (XAP_EncodingManager::instance->is_cjk_letter(c))
+	    return _scale(pChineseFont->getUnixFont()->get_CJK_Width());
+	else {	
+	    UT_Byte recoded = c <= 0xff ? c : XAP_EncodingManager::instance->try_UToNative(c);
+	    return recoded <= 0xff && recoded!=0  ? _scale(cwi[recoded]) : 0;
+	};
+}
+#if 0
+/*
+    WARNING: this code doesn't support non-latin1 chars.
+*/
 UT_uint32 PS_Graphics::measureString(const UT_UCSChar* s, int iOffset, 
 									int num, unsigned short* pWidths)
 {
 	UT_ASSERT(m_pCurrentFont);
 	UT_ASSERT(s);
 	const UT_UCSChar * p = s+iOffset;
-	const int * cwi = m_pCurrentFont->getMetricsData()->cwi;
+	const UT_uint16 * cwi = m_pCurrentFont->getUniWidths();
 
 	UT_uint32 iCharWidth = 0;
 	for (int k=0; k<num; k++)
 	{
-		UT_ASSERT(p[k] < 256);			// TODO deal with Unicode
-		register int x = _scale(cwi[p[k]]);
+		//UT_ASSERT(p[k] < 256);			// TODO deal with Unicode
+		register int x;
+		UT_UCSChar currentChar;
+		currentChar = remapGlyph(p[k], UT_FALSE);
+		x = (currentChar < 256 ? _scale(cwi[currentChar]) : 0;
 		
 		iCharWidth += x;
 		pWidths[k] = x;
@@ -174,7 +199,7 @@ UT_uint32 PS_Graphics::measureString(const UT_UCSChar* s, int iOffset,
 		
 	return iCharWidth;
 }
-	
+#endif
 UT_uint32 PS_Graphics::_getResolution(void) const
 {
 	return PS_RESOLUTION;
@@ -273,12 +298,13 @@ GR_Font* PS_Graphics::findFont(const char* pszFontFamily,
 		PSFont * psf = (PSFont *) m_vecFontList.getNthItem(k);
 		UT_ASSERT(psf);
 		// is this good enough for a match?
-		if (psf->getUnixFont() == pFont->getUnixFont() &&
+		if (!strcmp(psf->getUnixFont()->getFontKey(),pFont->getUnixFont()->getFontKey()) &&		
 			psf->getSize() == pFont->getSize())
 		{
 			// don't return the one in the vector, even though
 			// it matches, but return the copy, since they're
 			// disposable outside our realm.
+			pFont->setIndex(psf->getIndex());
 			return pFont;
 		}
 	}
@@ -298,6 +324,46 @@ GR_Font* PS_Graphics::findFont(const char* pszFontFamily,
 void PS_Graphics::drawChars(const UT_UCSChar* pChars, int iCharOffset, 
 							int iLength, UT_sint32 xoff, UT_sint32 yoff)
 {
+  PSFont *pEnglishFont;
+  PSFont *pChineseFont;
+  explodePSFonts(pEnglishFont,pChineseFont);  
+  UT_ASSERT(pEnglishFont && pChineseFont);
+
+  const UT_uint16 *cwi = pEnglishFont->getUniWidths();
+  const UT_UCSChar *pS;
+  const UT_UCSChar *pE=pChars+iCharOffset;
+  const UT_UCSChar *pEnd=pChars+iCharOffset+iLength;
+  UT_sint32 xS;
+  do
+	{
+	  for(pS=pE,xS=xoff; pE<pEnd && XAP_EncodingManager::instance->is_cjk_letter(*pE); ++pE)
+	          xoff+=_scale(pChineseFont->getUnixFont()->get_CJK_Width());
+	  if(pE>pS)
+		{
+		  _emit_SetFont(pChineseFont);
+		  drawCharsCJK(pS,0,pE-pS,xS,yoff);
+		}
+	  for(pS=pE,xS=xoff; pE<pEnd && !XAP_EncodingManager::instance->is_cjk_letter(*pE); ++pE)
+	          xoff+=_scale(cwi[XAP_EncodingManager::instance->UToNative(remapGlyph(*pE,*pS > 0xff))]);
+	  if(pE>pS)
+		{
+		  _emit_SetFont(pEnglishFont);
+		  drawCharsCJK(pS,0,pE-pS,xS,yoff);
+		}
+	}
+  while(pE<pEnd);
+}
+							 
+static UT_Wctomb* pWctomb = NULL;
+
+void PS_Graphics::drawCharsCJK(const UT_UCSChar* pChars, int iCharOffset, 
+							 int iLength, UT_sint32 xoff, UT_sint32 yoff)
+{
+	if (!pWctomb)
+	    pWctomb = new UT_Wctomb;
+	else
+	    pWctomb->initialize();
+	    
 	UT_ASSERT(m_pCurrentFont);
 
 	// The GR classes are expected to take yoff as the upper-left of
@@ -312,6 +378,9 @@ void PS_Graphics::drawChars(const UT_UCSChar* pChars, int iCharOffset,
 
 	const UT_UCSChar * pS = pChars+iCharOffset;
 	const UT_UCSChar * pEnd = pS+iLength;
+	UT_UCSChar currentChar;
+	char _bytes[30];
+	int _bytes_len;
 
 	*pD++ = '(';
 	while (pS<pEnd)
@@ -326,18 +395,34 @@ void PS_Graphics::drawChars(const UT_UCSChar* pChars, int iCharOffset,
 		}
 
 		// TODO deal with Unicode issues.
-
-		switch (*pS)
-		{
-		case 0x08:		*pD++ = '\\';	*pD++ = 'b';	break;
-		case UCS_TAB:	*pD++ = '\\';	*pD++ = 't';	break;
-		case UCS_LF:	*pD++ = '\\';	*pD++ = 'n';	break;
-		case UCS_FF:	*pD++ = '\\';	*pD++ = 'f';	break;
-		case UCS_CR:	*pD++ = '\\';	*pD++ = 'r';	break;
-		case '\\':		*pD++ = '\\';	*pD++ = '\\';	break;
-		case '(':		*pD++ = '\\';	*pD++ = '(';	break;
-		case ')':		*pD++ = '\\';	*pD++ = ')';	break;
-		default:		*pD++ = (unsigned char) *pS; 	break;
+		if (XAP_EncodingManager::instance->is_cjk_letter(*pS)) 
+		{		
+			pWctomb->wctomb_or_fallback(_bytes,_bytes_len,*pS);
+			if (pD+_bytes_len-buf > OUR_LINE_LIMIT)
+			{
+			    *pD++ = '\\';
+			    *pD++ = '\n';
+			    *pD++ = 0;
+			    m_ps->writeBytes(buf);
+			    pD = buf;
+			}
+			for(int i=0;i<_bytes_len;++i)
+			    *pD++ = _bytes[i];
+		} else 	{
+		    currentChar = remapGlyph(*pS, *pS >= 256 ? UT_TRUE : UT_FALSE);
+		    currentChar = currentChar <= 0xff ? currentChar : XAP_EncodingManager::instance->UToNative(currentChar);
+		    switch (currentChar)
+		    {
+			case 0x08:		*pD++ = '\\';	*pD++ = 'b';	break;
+			case UCS_TAB:	*pD++ = '\\';	*pD++ = 't';	break;
+			case UCS_LF:	*pD++ = '\\';	*pD++ = 'n';	break;
+			case UCS_FF:	*pD++ = '\\';	*pD++ = 'f';	break;
+			case UCS_CR:	*pD++ = '\\';	*pD++ = 'r';	break;
+			case '\\':		*pD++ = '\\';	*pD++ = '\\';	break;
+			case '(':		*pD++ = '\\';	*pD++ = '(';	break;
+			case ')':		*pD++ = '\\';	*pD++ = ')';	break;
+			default:		*pD++ = (unsigned char)currentChar; 	break;
+		    }
 		}
 		pS++;
 	}
@@ -357,7 +442,7 @@ void PS_Graphics::drawLine(UT_sint32 x1, UT_sint32 y1, UT_sint32 x2, UT_sint32 y
 	
 	char buf[OUR_LINE_LIMIT*2];
 //	UT_sint32 nA = getFontAscent();
-	sprintf(buf,"%d %d %d %d ML\n", x2, y2, x1, y1);
+	g_snprintf(buf,sizeof (buf),"%d %d %d %d ML\n", x2, y2, x1, y1);
 	m_ps->writeBytes(buf);
 }
 
@@ -392,8 +477,10 @@ void PS_Graphics::invertRect(const UT_Rect* /*pRect*/)
 
 void PS_Graphics::setClipRect(const UT_Rect* /*pRect*/)
 {
-	// setClipRect is only used for handling screen exposes, which do not happen on paper.
-	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+    // setClipRect is used for clipping images, even when printing to
+    // PostScript.  Just ignore these for now.
+
+    // TODO : Can PostScript clip regions?  Does it make sense for our images?
 }
 
 void PS_Graphics::clearArea(UT_sint32 /*x*/, UT_sint32 /*y*/,
@@ -518,7 +605,7 @@ UT_Bool PS_Graphics::_startPage(const char * szPageLabel, UT_uint32 pageNumber,
 	// emit stuff prior to each page
 	
 	char buf[1024];
-	sprintf(buf,"%d",pageNumber);
+	g_snprintf(buf, sizeof(buf),"%d",pageNumber);
 
 	const char * argv[2] = { buf, buf };
 	m_ps->formatComment("Page",argv,2);
@@ -527,7 +614,7 @@ UT_Bool PS_Graphics::_startPage(const char * szPageLabel, UT_uint32 pageNumber,
 
 	m_ps->formatComment("BeginPageSetup");
 
-	sprintf(buf,"%d %d %d %s\n",iWidth,iHeight,PS_RESOLUTION,((bPortrait) ? "BPP" : "BPL"));
+	g_snprintf(buf,sizeof (buf),"%d %d %d %s\n",iWidth,iHeight,PS_RESOLUTION,((bPortrait) ? "BPP" : "BPL"));
 	m_ps->writeBytes(buf);
 	
 	// TODO add page-setup stuff here
@@ -588,7 +675,8 @@ void PS_Graphics::_emit_DocumentNeededResources(void)
 	{
 		vec.addItem((void *) "font");
 		PSFont * psf = (PSFont *)m_vecFontList.getNthItem(k);
-		vec.addItem(psf->getMetricsData()->gfi->fontName);
+		if(!psf->getUnixFont()->is_CJK_font())
+		    vec.addItem(psf->getMetricsData()->gfi->fontName);
 	}
 
 	// TODO add any other resources here
@@ -600,6 +688,9 @@ void PS_Graphics::_emit_IncludeResource(void)
 {
 	UT_uint32 k;
 	UT_uint32 kLimit = m_vecFontList.getItemCount();
+	typedef char* pchar_t;
+	char **fontKey= new pchar_t[kLimit];
+	int fontKeyCount=0;
 
 	for (k=0; k<kLimit; k++)
 	{
@@ -613,6 +704,18 @@ void PS_Graphics::_emit_IncludeResource(void)
 		// into the document.  This looks really slow... perhaps buffer line
 		// by line or in larger chunks the font data.
 		XAP_UnixFont * unixfont = psf->getUnixFont();
+		if(unixfont->is_CJK_font())
+		  continue;
+		int match=0;
+		for(int i=0;i<fontKeyCount;++i)
+		  if(!strcmp(unixfont->getFontKey(),fontKey[i]))
+			{
+			  match=1;
+			  break;
+			}
+		if(match)
+		  continue;
+		fontKey[fontKeyCount++]=UT_strdup(unixfont->getFontKey());
 
 		// Make sure the font file will open, maybe it disappeared...
 		if(!unixfont->openPFA())
@@ -646,22 +749,24 @@ void PS_Graphics::_emit_IncludeResource(void)
 		XAP_UnixFontXLFD myXLFD(unixfont->getXLFD());
 
 		// write findfont
-		g_snprintf(buf, 128, "/%s findfont\n", psf->getMetricsData()->gfi->fontName);
+		g_snprintf(buf, sizeof (buf), "/%s findfont\n", psf->getMetricsData()->gfi->fontName);
 		m_ps->writeBytes(buf);
 
 		// Compare with iso8859, and emit LAT for that font
 		if (!UT_stricmp(myXLFD.getRegistry(), "iso8859") && !UT_stricmp(myXLFD.getEncoding(), "1"))
 		{
-			g_snprintf(buf, 128, "LAT\n");
+			g_snprintf(buf, sizeof (buf), "LAT\n");
 			m_ps->writeBytes(buf);
 		}
 
 		// exec the swapper macro
-		g_snprintf(buf, 128, "/%s EXC\n", psf->getMetricsData()->gfi->fontName);
+		g_snprintf(buf, sizeof (buf), "/%s EXC\n", psf->getMetricsData()->gfi->fontName);
 		m_ps->writeBytes(buf);
 
 	}
-
+	for(int i=0;i<fontKeyCount;++i)
+	  free(fontKey[i]);
+	delete []fontKey;
 	// TODO add any other IncludeResource's here
 }
 
@@ -689,7 +794,7 @@ void PS_Graphics::_emit_PrologMacros(void)
 	
 	for (unsigned int k=0; k<NrElements(t); k++)
 	{
-		sprintf(buf,"  %s\n",t[k]);
+		g_snprintf(buf,sizeof(buf),"  %s\n",t[k]);
 		m_ps->writeBytes(buf);
 	}
 }
@@ -703,23 +808,24 @@ void PS_Graphics::_emit_FontMacros(void)
 	for (k=0; k<kLimit; k++)
 	{
 		PSFont * psf = (PSFont *)m_vecFontList.getNthItem(k);
-		sprintf(buf,"  /F%d {%d /%s FSF setfont} bind def\n", k,
-				psf->getSize(), psf->getMetricsData()->gfi->fontName);
+		g_snprintf(buf,sizeof(buf),"  /F%d {%d /%s FSF setfont} bind def\n", k,
+				psf->getSize(), psf->getUnixFont()->is_CJK_font() ?
+					psf->getUnixFont()->getFontfile() : 
+					psf->getMetricsData()->gfi->fontName);
 		m_ps->writeBytes(buf);
 	}
 }
 
+
 void PS_Graphics::_emit_SetFont(void)
 {
-	char buf[1024];
-	g_snprintf(buf, 1024, "F%d\n", m_pCurrentFont->getIndex());
-	m_ps->writeBytes(buf);
+        _emit_SetFont(m_pCurrentFont);
 }
 
 void PS_Graphics::_emit_SetLineWidth(void)
 {
 	char buf[1024];
-	g_snprintf(buf, 1024, " %d setlinewidth\n", m_iLineWidth);
+	g_snprintf(buf, sizeof(buf), " %d setlinewidth\n", m_iLineWidth);
 	m_ps->writeBytes(buf);
 }
 
@@ -735,59 +841,71 @@ void PS_Graphics::_emit_SetColor(void)
 	char buf[128];
 	// used for any averaging
 	unsigned char newclr;
+
+	setlocale(LC_NUMERIC,"C");
 	switch(m_cs)
 	{
 	case GR_Graphics::GR_COLORSPACE_COLOR:
-		sprintf(buf,"0.%.0f 0.%.0f 0.%.0f setrgbcolor\n",					// Write the radix character as hard wired periods in PS files, LC_NUMERIC can make sprintf choose non period values for it.
-				((float) m_currentColor.m_red / (float) 255.0)*100000000.0,
-				((float) m_currentColor.m_grn / (float) 255.0)*100000000.0,
-				((float) m_currentColor.m_blu / (float) 255.0)*100000000.0);
+        g_snprintf(buf, sizeof (buf), "%.8f %.8f %.8f setrgbcolor\n",
+                ((float) m_currentColor.m_red / (float) 255.0),
+                ((float) m_currentColor.m_grn / (float) 255.0),
+                ((float) m_currentColor.m_blu / (float) 255.0));
 		break;
 	case GR_Graphics::GR_COLORSPACE_GRAYSCALE:
 		newclr = (unsigned char) (( (float) m_currentColor.m_red +
 									(float) m_currentColor.m_grn +
 									(float) m_currentColor.m_blu ) /
 								  (float) 3.0);
-		sprintf(buf,"0.%.0f setgray\n",								// Write the radix character as hard wired periods in PS files, LC_NUMERIC can make sprintf choose non period values for it.
-				(float) newclr / (float) 255.0) * 10000000.0;
+		g_snprintf(buf, sizeof(buf), "%.8f setgray\n", (float) newclr / (float) 255.0);
 		break;
 	case GR_Graphics::GR_COLORSPACE_BW:
 		// Black & White is a special case of the Gray color space where
 		// all colors are 0 (black) and all absence of color is 1 (white)
-		sprintf(buf,"0 setgray\n");
+		g_snprintf(buf,sizeof(buf),"0 setgray\n");
 		break;
 	default:
 		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 	}
+	setlocale(LC_NUMERIC,""); // restore original locale
 	
 	m_ps->writeBytes(buf);
 }
 
-GR_Image* PS_Graphics::createNewImage(const char* pszName, const UT_ByteBuf* pBBPNG, UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight)
+ GR_Image* PS_Graphics::createNewImage(const char* pszName, const UT_ByteBuf* pBB, UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight, GR_Image::GRType iType)
 {
-	PS_Image* pImg = new PS_Image(NULL, pszName);
-
-	pImg->convertFromPNG(pBBPNG, iDisplayWidth, iDisplayHeight);
+	GR_Image* pImg = NULL;
+   
+   	if (iType == GR_Image::GRT_Raster)
+     		pImg = new PS_Image(pszName);
+   	else if (iType == GR_Image::GRT_Vector)
+     		pImg = new GR_VectorImage(pszName);
+   
+	pImg->convertFromBuffer(pBB, iDisplayWidth, iDisplayHeight);
 
 	return pImg;
 }
 
 void PS_Graphics::drawImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDest)
 {
-	switch(m_cs)
-	{
-	case GR_Graphics::GR_COLORSPACE_COLOR:
+   	if (pImg->getType() != GR_Image::GRT_Raster) {
+	   pImg->render(this, xDest, yDest);
+	   return;
+	}
+   
+   	switch(m_cs)
+     	{
+       	case GR_Graphics::GR_COLORSPACE_COLOR:
 		drawRGBImage(pImg, xDest, yDest);
 		break;
-	case GR_Graphics::GR_COLORSPACE_GRAYSCALE:
+      	case GR_Graphics::GR_COLORSPACE_GRAYSCALE:
 		drawGrayImage(pImg, xDest, yDest);
 		break;
-	case GR_Graphics::GR_COLORSPACE_BW:
+      	case GR_Graphics::GR_COLORSPACE_BW:
 		drawBWImage(pImg, xDest, yDest);
 		break;
-	default:
+      	default:
 		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
-	}
+     }
 }
 	
 void PS_Graphics::drawRGBImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDest)
@@ -807,28 +925,28 @@ void PS_Graphics::drawRGBImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDest)
 	char buf[128];
 
 	// remember all the context information 
-	sprintf(buf, "gsave\n");
+	g_snprintf(buf, sizeof(buf), "gsave\n");
 	m_ps->writeBytes(buf);	
 
 	// this is the number of bytes in a "row" of image data, which
 	// is image->width times 3 bytes per pixel
-	sprintf(buf, "/rowdata %d string def\n", image->width * 3);
+	g_snprintf(buf, sizeof(buf),"/rowdata %d string def\n", image->width * 3);
 	m_ps->writeBytes(buf);
 	
 	// translate for quadrant 2, so Y values are negative; land us at
 	// lower left of image (baseline), which is twice the height
-	sprintf(buf, "%d %d translate\n", xDest, m_iRasterPosition - yDest  - iDestHeight);
+	g_snprintf(buf, sizeof (buf), "%d %d translate\n", xDest, m_iRasterPosition - yDest  - iDestHeight);
 	m_ps->writeBytes(buf);
 
-	sprintf(buf, "%d %d scale\n", iDestWidth, iDestHeight);
+	g_snprintf(buf, sizeof (buf),"%d %d scale\n", iDestWidth, iDestHeight);
 	m_ps->writeBytes(buf);
 
 	// use true image source data dimensions for matrix 
-	sprintf(buf, "%d %d 8 [%d 0 0 %d 0 %d]\n", image->width, image->height,
+	g_snprintf(buf, sizeof (buf),"%d %d 8 [%d 0 0 %d 0 %d]\n", image->width, image->height,
 			image->width, image->height * -1, image->height);
 	m_ps->writeBytes(buf);
 	
-	sprintf(buf, "{currentfile\n  rowdata readhexstring pop}\nfalse 3\ncolorimage\n");
+	g_snprintf(buf, sizeof (buf),"{currentfile\n  rowdata readhexstring pop}\nfalse 3\ncolorimage\n");
 	m_ps->writeBytes(buf);
 	
 	// "image" is full of 24 bit data; throw the data specifications
@@ -841,21 +959,21 @@ void PS_Graphics::drawRGBImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDest)
 	for (cursor = start, col = 0 ; cursor < end; cursor++, col++)
 	{
 		// fetch a byte and convert to hex
-		g_snprintf((char *) hexbuf, 3, "%.2X", *cursor);
+		g_snprintf((char *) hexbuf, sizeof (hexbuf), "%.2X", *cursor);
 		m_ps->writeBytes(hexbuf, 2);
 
 		if (col == 40) // 2 chars per round, 80 columns total
 		{
 			col = -1;
-			sprintf((char *) hexbuf, "\n");
+			g_snprintf((char *) hexbuf, sizeof(hexbuf), "\n");
 			m_ps->writeBytes(hexbuf, 1);
 		}
 	}
-	sprintf(buf, "\n");
+	g_snprintf(buf, sizeof(buf), "\n");
 	m_ps->writeBytes(buf);
 
 	// recall all that great info
-	sprintf(buf, "grestore\n");
+	g_snprintf(buf, sizeof(buf),"grestore\n");
 	m_ps->writeBytes(buf);
 	
 }
@@ -877,28 +995,28 @@ void PS_Graphics::drawGrayImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDest
 	char buf[128];
 
 	// remember all the context information 
-	sprintf(buf, "gsave\n");
+	g_snprintf(buf, sizeof(buf),"gsave\n");
 	m_ps->writeBytes(buf);	
 
 	// this is the number of bytes in a "row" of image data, which
 	// is image->width in a grayscale image
-	sprintf(buf, "/rowdata %d string def\n", image->width);
+	g_snprintf(buf, sizeof(buf),"/rowdata %d string def\n", image->width);
 	m_ps->writeBytes(buf);
 	
 	// translate for quadrant 2, so Y values are negative; land us at
 	// lower left of image (baseline), which is twice the height
-	sprintf(buf, "%d %d translate\n", xDest, m_iRasterPosition - yDest  - iDestHeight);
+	g_snprintf(buf, sizeof(buf),"%d %d translate\n", xDest, m_iRasterPosition - yDest  - iDestHeight);
 	m_ps->writeBytes(buf);
 
-	sprintf(buf, "%d %d scale\n", iDestWidth, iDestHeight);
+	g_snprintf(buf, sizeof(buf),"%d %d scale\n", iDestWidth, iDestHeight);
 	m_ps->writeBytes(buf);
 
 	// use true image source data dimensions for matrix 
-	sprintf(buf, "%d %d 8 [%d 0 0 %d 0 %d]\n", image->width, image->height,
+	g_snprintf(buf, sizeof(buf),"%d %d 8 [%d 0 0 %d 0 %d]\n", image->width, image->height,
 			image->width, image->height * -1, image->height);
 	m_ps->writeBytes(buf);
 
-	sprintf(buf, "{currentfile\n  rowdata readhexstring pop}\nimage\n");
+	g_snprintf(buf, sizeof(buf),"{currentfile\n  rowdata readhexstring pop}\nimage\n");
 	// color image does:
 	// sprintf(buf, "{currentfile\n  rowdata readhexstring pop}\nfalse 3\ncolorimage\n");
 	m_ps->writeBytes(buf);
@@ -927,11 +1045,11 @@ void PS_Graphics::drawGrayImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDest
 #if 0
 		// We can use the Y channel from the YIQ spec, which weights
 		// the R, G, and B channels to be perceptually more balanced.
-		g_snprintf((char *) hexbuf, 3, "%.2X", ( (UT_Byte) ( ( (float) (*cursor++) * (float) (0.299) +
+		g_snprintf((char *) hexbuf, sizeof(hexbuf), "%.2X", ( (UT_Byte) ( ( (float) (*cursor++) * (float) (0.299) +
 															   (float) (*cursor++) * (float) (0.587) +
 															   (float) (*cursor++) * (float) (0.114) ) ) ));
 #endif
-		g_snprintf((char *) hexbuf, 3, "%.2X", ( (UT_Byte) ( ( (float) (*cursor++) * (float) (1) +
+		g_snprintf((char *) hexbuf, sizeof(hexbuf), "%.2X", ( (UT_Byte) ( ( (float) (*cursor++) * (float) (1) +
 															   (float) (*cursor++) * (float) (1) +
 															   (float) (*cursor++) * (float) (1) ) /
 															 (float) (3.0) )) );
@@ -945,11 +1063,11 @@ void PS_Graphics::drawGrayImage(GR_Image* pImg, UT_sint32 xDest, UT_sint32 yDest
 		else
 			col++;
 	}
-	sprintf(buf, "\n");
+	g_snprintf(buf, sizeof(buf),"\n");
 	m_ps->writeBytes(buf);
 
 	// recall all that great info
-	sprintf(buf, "grestore\n");
+	g_snprintf(buf, sizeof(buf),"grestore\n");
 	m_ps->writeBytes(buf);
 	
 }
@@ -1003,3 +1121,67 @@ void PS_Graphics::fillRect(GR_Color3D /*c*/, UT_Rect & /*r*/)
 }
 
 
+PSFont *PS_Graphics::findMatchPSFontCJK(PSFont * pFont)
+{
+  if (!XAP_EncodingManager::instance->cjk_locale())
+	return pFont;
+  UT_uint32 k, count;
+  int s;
+  switch(pFont->getUnixFont()->getStyle())
+	{
+	case XAP_UnixFont::STYLE_NORMAL:
+	  s=0;
+	  break;
+	case XAP_UnixFont::STYLE_BOLD:
+	  s=1;
+	  break;
+	case XAP_UnixFont::STYLE_ITALIC:
+	  s=2;
+	  break;
+	case XAP_UnixFont::STYLE_BOLD_ITALIC:
+	  s=3;
+	  break;
+	default:
+	  s=0;
+	}
+  for (k = 0, count = m_vecFontList.getItemCount(); k < count; k++)
+	{
+	  PSFont * psf = (PSFont *) m_vecFontList.getNthItem(k);
+	  UT_ASSERT(psf);
+	  if (!strcmp(psf->getUnixFont()->getFontKey(),
+			(pFont->getUnixFont()->is_CJK_font() ? 
+			    XAP_UnixFont::s_defaultNonCJKFont[s] : 
+			    XAP_UnixFont::s_defaultCJKFont[s])->getFontKey()) &&
+		  psf->getSize() == pFont->getSize())
+		return psf;
+	}  
+  PSFont * p = new PSFont(pFont->getUnixFont()->is_CJK_font() ? 
+          XAP_UnixFont::s_defaultNonCJKFont[s] : 
+	  XAP_UnixFont::s_defaultCJKFont[s] , pFont->getSize() );
+  UT_ASSERT(p);
+  m_vecFontList.addItem((void *) p);
+  p->setIndex(m_vecFontList.getItemCount() - 1);
+  return p;
+};
+
+void PS_Graphics::_emit_SetFont(PSFont *pFont)
+{
+	char buf[1024];
+	g_snprintf(buf, 1024, "F%d\n", pFont->getIndex());
+	m_ps->writeBytes(buf);
+};
+
+
+void PS_Graphics::explodePSFonts(PSFont*& pEnglishFont,PSFont*& pChineseFont)
+{
+  if(m_pCurrentFont->getUnixFont()->is_CJK_font())
+	{
+	  pChineseFont=m_pCurrentFont;
+	  pEnglishFont=findMatchPSFontCJK(pChineseFont);
+	}
+  else
+	{
+	  pEnglishFont=m_pCurrentFont;
+	  pChineseFont=findMatchPSFontCJK(pEnglishFont);
+	}
+};
