@@ -36,6 +36,9 @@
 #include "xav_View.h"
 #include "xad_Document.h"
 
+#include "fv_View.h"
+
+#pragma warning(disable:4355)
 
 /*****************************************************************/
 
@@ -75,21 +78,21 @@ UT_Bool XAP_Win32Frame::RegisterClass(XAP_Win32App * app)
 /*****************************************************************/
 
 XAP_Win32Frame::XAP_Win32Frame(XAP_Win32App * app)
-:	XAP_Frame(app),
-	m_dialogFactory(this, app),
-	m_iBarHeight(0),
-	m_iStatusBarHeight(0)
+:       XAP_Frame(app),
+        m_pWin32App(app),
+        m_pWin32Menu(0),
+        m_pWin32Popup(0),
+        m_iBarHeight(0),
+        m_iStatusBarHeight(0),
+        m_hwndFrame(0),
+        m_hwndRebar(0),
+        m_hwndContainer(0),
+        m_hwndStatusBar(0),
+        m_dialogFactory(this, app),
+        m_mouseWheelMessage(0),
+        m_iSizeWidth(0),
+        m_iSizeHeight(0)
 {
-	m_pWin32App = app;
-	m_pWin32Menu = NULL;
-	m_pWin32Popup = NULL;
-	m_pView = NULL;
-	m_hwndFrame = NULL;
-	m_hwndRebar = NULL;
-	m_hwndContainer = NULL;
-	m_iSizeWidth = 0;
-	m_iSizeHeight = 0;
-	m_mouseWheelMessage = 0;
 }
 
 // TODO when cloning a new frame from an existing one
@@ -97,19 +100,21 @@ XAP_Win32Frame::XAP_Win32Frame(XAP_Win32App * app)
 // TODO dialog data ??
 
 XAP_Win32Frame::XAP_Win32Frame(XAP_Win32Frame * f)
-	: XAP_Frame(static_cast<XAP_Frame *>(f)),
-	m_dialogFactory(this, static_cast<XAP_App *>(f->m_pWin32App))
+:       XAP_Frame(f),
+        m_pWin32App(f->m_pWin32App),
+        m_pWin32Menu(0),
+        m_pWin32Popup(0),
+        m_iBarHeight(0),
+        m_iStatusBarHeight(0),
+        m_hwndFrame(0),
+        m_hwndRebar(0),
+        m_hwndContainer(0),
+        m_hwndStatusBar(0),
+        m_dialogFactory(this, f->m_pWin32App),
+        m_mouseWheelMessage(0),
+        m_iSizeWidth(0),
+        m_iSizeHeight(0)
 {
-	m_pWin32App = f->m_pWin32App;
-	m_pWin32Menu = NULL;
-	m_pWin32Popup = NULL;
-	m_pView = NULL;
-	m_hwndFrame = NULL;
-	m_hwndRebar = NULL;
-	m_hwndContainer = NULL;
-	m_iSizeWidth = 0;
-	m_iSizeHeight = 0;
-	m_mouseWheelMessage = 0;
 }
 
 XAP_Win32Frame::~XAP_Win32Frame(void)
@@ -350,6 +355,8 @@ LRESULT CALLBACK XAP_Win32Frame::_FrameWndProc(HWND hwnd, UINT iMsg, WPARAM wPar
 
 	pView = f->m_pView;
 
+	FV_View* pfView = static_cast<FV_View*>(pView);
+
 	if(iMsg == f->m_mouseWheelMessage)
 	{
 		wParam = MAKEWPARAM(0, (short)(int)wParam);
@@ -363,6 +370,7 @@ LRESULT CALLBACK XAP_Win32Frame::_FrameWndProc(HWND hwnd, UINT iMsg, WPARAM wPar
 		if (pView)
 		{
 			pView->focusChange(AV_FOCUS_HERE);
+			SetFocus(f->m_hwndContainer);
 		}
 		return 0;
 
@@ -373,6 +381,7 @@ LRESULT CALLBACK XAP_Win32Frame::_FrameWndProc(HWND hwnd, UINT iMsg, WPARAM wPar
 			pView->focusChange(AV_FOCUS_NONE);
 		}
 		return 0;
+
 	case WM_CREATE:
 		return 0;
 
@@ -421,22 +430,6 @@ LRESULT CALLBACK XAP_Win32Frame::_FrameWndProc(HWND hwnd, UINT iMsg, WPARAM wPar
 			return 0;
 		return DefWindowProc(hwnd,iMsg,wParam,lParam);
 
-	case WM_SYSKEYDOWN:
-	case WM_KEYDOWN:
-	{
-	    ev_Win32Keyboard *pWin32Keyboard = static_cast<ev_Win32Keyboard *>(f->m_pKeyboard);
-		if (pWin32Keyboard->onKeyDown(pView,hwnd,iMsg,wParam,lParam))
-			return 0;
-		return DefWindowProc(hwnd,iMsg,wParam,lParam);
-	}
-	case WM_SYSCHAR:
-	case WM_CHAR:
-	{
-	    ev_Win32Keyboard *pWin32Keyboard = static_cast<ev_Win32Keyboard *>(f->m_pKeyboard);
-		if (pWin32Keyboard->onChar(pView,hwnd,iMsg,wParam,lParam))
-			return 0;
-		return DefWindowProc(hwnd,iMsg,wParam,lParam);
-	}
 	case WM_NOTIFY:
 		switch (((LPNMHDR) lParam)->code) 
 		{
@@ -554,6 +547,25 @@ LRESULT CALLBACK XAP_Win32Frame::_FrameWndProc(HWND hwnd, UINT iMsg, WPARAM wPar
 			
 		f->m_iSizeWidth = nWidth;
 		f->m_iSizeHeight = nHeight;
+
+
+		UT_uint32 newZoom = 0;
+
+		switch(f->getZoomType())
+		{
+//		// special cases
+		case XAP_Frame::z_PAGEWIDTH:
+			newZoom = pfView->calculateZoomPercentForPageWidth();
+			f->setZoomPercentage(newZoom);
+
+			break;
+		case XAP_Frame::z_WHOLEPAGE:
+			newZoom = pfView->calculateZoomPercentForWholePage();
+			f->setZoomPercentage(newZoom);
+			break;
+		default:
+			;
+		}
 		
 		return 0;
 	}
