@@ -20,6 +20,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <signal.h>
+#include <time.h>
 
 #include "ut_types.h"
 #include "ut_assert.h"
@@ -38,6 +39,8 @@
 #include "xap_LoadBindings.h"
 #include "xap_Dictionary.h"
 #include "xap_Prefs.h"
+#include "xap_EncodingManager.h"
+
 
 /*****************************************************************/
 
@@ -56,6 +59,11 @@ XAP_App::XAP_App(XAP_Args * pArgs, const char * szAppName) : m_hashClones(5)
 	m_pToolbarActionSet = NULL;
 	m_pDict = NULL;
 	m_prefs = NULL;
+	m_pEncMgr = new XAP_EncodingManager();/* HACK: this is done in order 
+		not to update the code for each platform. If platform specific
+		code has its own implementation of EncodingManager, then it
+		should just simply delete an instance pointed by this member
+		and assign ptr to platform-specific implementation. - hvv */
        
 	m_pApp = this;
 	m_lastFocussedFrame = NULL;
@@ -79,8 +87,14 @@ XAP_App::~XAP_App(void)
 	DELETEP(m_pToolbarActionSet);
 	DELETEP(m_pDict);
 	DELETEP(m_prefs);
+	DELETEP(m_pEncMgr);
 }
 
+const XAP_EncodingManager* XAP_App::getEncodingManager(void) const 
+{ 
+    return m_pEncMgr;
+};
+
 UT_Bool XAP_App::initialize(void)
 {
 	// create application-wide resources that
@@ -112,7 +126,11 @@ UT_Bool XAP_App::initialize(void)
 
 	// TODO use m_pArgs->{argc,argv} to process any command-line
 	// TODO options that we need.
-
+	//
+	// Need to initialize the random number generator. 
+	//
+	UT_uint32 t = (UT_uint32) time( NULL);
+	srand(t);
 	return UT_TRUE;
 }
 
@@ -443,6 +461,16 @@ XAP_Prefs * XAP_App::getPrefs(void) const
 	return m_prefs;
 }
 
+#ifdef HAVE_LIBXML2
+UT_Bool XAP_App::getPrefsValue(const char * szKey, const XML_Char ** pszValue) const
+{
+	if (!m_prefs)
+		return UT_FALSE;
+
+	return m_prefs->getPrefsValue((XML_Char*) (szKey), pszValue);
+}
+#endif
+
 UT_Bool XAP_App::getPrefsValue(const XML_Char * szKey, const XML_Char ** pszValue) const
 {
 	if (!m_prefs)
@@ -451,6 +479,16 @@ UT_Bool XAP_App::getPrefsValue(const XML_Char * szKey, const XML_Char ** pszValu
 	return m_prefs->getPrefsValue(szKey,pszValue);
 }
 
+#ifdef HAVE_LIBXML2
+UT_Bool XAP_App::getPrefsValueBool(const char * szKey, UT_Bool * pbValue) const
+{
+	if (!m_prefs)
+		return UT_FALSE;
+
+	return m_prefs->getPrefsValueBool((XML_Char*) (szKey), pbValue);
+}
+#endif
+
 UT_Bool XAP_App::getPrefsValueBool(const XML_Char * szKey, UT_Bool * pbValue) const
 {
 	if (!m_prefs)
