@@ -24,7 +24,7 @@
 #include "ut_types.h"
 #include "ut_vector.h"
 #include "ut_hash.h"
-#include "xmlparse.h"
+#include "ut_xml.h"
 #include "xap_Dialog.h"
 
 #define NUM_MODELESSID 39
@@ -43,6 +43,7 @@ class XAP_StringSet;
 class XAP_Dictionary;
 class PD_DocumentRange;
 class AV_View;
+class XAP_EncodingManager;
 
 /*****************************************************************
 ******************************************************************
@@ -88,6 +89,7 @@ public:
 	EV_EditBindingMap *				getBindingMap(const char * szName);
 	const EV_Menu_ActionSet *		getMenuActionSet(void) const;
 	const EV_Toolbar_ActionSet *	getToolbarActionSet(void) const;
+	const XAP_EncodingManager*			getEncodingManager(void) const;
 
 	XAP_Args *						getArgs(void) const;
 
@@ -95,6 +97,10 @@ public:
 	UT_Bool							isWordInDict(const UT_UCSChar * pWord, UT_uint32 len) const;
 
 	XAP_Prefs *						getPrefs(void) const;
+#ifdef HAVE_LIBXML2
+	UT_Bool							getPrefsValue(const char * szKey, const XML_Char ** pszValue) const;
+	UT_Bool							getPrefsValueBool(const char * szKey, UT_Bool * pbValue) const;
+#endif
 	UT_Bool							getPrefsValue(const XML_Char * szKey, const XML_Char ** pszValue) const;
 	UT_Bool							getPrefsValueBool(const XML_Char * szKey, UT_Bool * pbValue) const;
 
@@ -154,6 +160,7 @@ protected:
 	       XAP_Dialog_Modeless * pDialog;
         } m_IdTable[NUM_MODELESSID+1]; 
         
+	XAP_EncodingManager*				m_pEncMgr;
 	static XAP_App *				m_pApp;
 };
 
