@@ -83,35 +83,37 @@ XAP_Dialog * XAP_DialogFactory::requestDialog(XAP_Dialog_Id id)
 	UT_uint32 index;
 	XAP_Dialog * pDialog = NULL;
 	
-	_findDialogInTable(id,&index);
-
-	switch (m_dlg_table[index].m_type)
+	if(_findDialogInTable(id,&index))
 	{
-	case XAP_DLGT_NON_PERSISTENT:	// construct a non-persistent dialog and just return it.
-		goto CreateItSimple;
 
-	case XAP_DLGT_FRAME_PERSISTENT:						// if requested frame-persistent dialog
-		if (m_dialogType == XAP_DLGT_FRAME_PERSISTENT)	// from a frame-persistent factory.
-			goto CreateItPersistent;
-		break;
-		
-	case XAP_DLGT_APP_PERSISTENT:						// if requested app-persistent dialog
-		if (m_dialogType == XAP_DLGT_APP_PERSISTENT)		//   if from a app-persistent factory
-			goto CreateItPersistent;
-		if (m_dialogType == XAP_DLGT_FRAME_PERSISTENT)	//   if from a frame-persistent factory,
-			goto HandToAppFactory;						//     let the app's factory do it....
-		break;
-		
-	case XAP_DLGT_MODELESS:						// if requested app-persistent dialog
-		if (m_dialogType == XAP_DLGT_APP_PERSISTENT)		//   if from a app-persistent factory
-			goto CreateItPersistent;
-		if (m_dialogType == XAP_DLGT_FRAME_PERSISTENT)	//   if from a frame-persistent factory,
-			goto HandToAppFactory;						//     let the app's factory do it....
-		break;
+		switch (m_dlg_table[index].m_type)
+		{
+		case XAP_DLGT_NON_PERSISTENT:	// construct a non-persistent dialog and just return it.
+			goto CreateItSimple;
 
+		case XAP_DLGT_FRAME_PERSISTENT:						// if requested frame-persistent dialog
+			if (m_dialogType == XAP_DLGT_FRAME_PERSISTENT)	// from a frame-persistent factory.
+				goto CreateItPersistent;
+			break;
+			
+		case XAP_DLGT_APP_PERSISTENT:						// if requested app-persistent dialog
+			if (m_dialogType == XAP_DLGT_APP_PERSISTENT)		//   if from a app-persistent factory
+				goto CreateItPersistent;
+			if (m_dialogType == XAP_DLGT_FRAME_PERSISTENT)	//   if from a frame-persistent factory,
+				goto HandToAppFactory;						//     let the app's factory do it....
+			break;
+			
+		case XAP_DLGT_MODELESS:						// if requested app-persistent dialog
+			if (m_dialogType == XAP_DLGT_APP_PERSISTENT)		//   if from a app-persistent factory
+				goto CreateItPersistent;
+			if (m_dialogType == XAP_DLGT_FRAME_PERSISTENT)	//   if from a frame-persistent factory,
+				goto HandToAppFactory;						//     let the app's factory do it....
+			break;
+
+		}
 	}
 
-	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+//	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 	return NULL;
 
 CreateItSimple:
