@@ -21,7 +21,7 @@
 #define XAP_DIALOG_FONTCHOOSER_H
 
 #include "ut_types.h"
-#include "xmlparse.h"
+#include "ut_xml.h"
 #include "xap_Dialog.h"
 class GR_Graphics;
 
