@@ -27,6 +27,7 @@
 #include "ut_debugmsg.h"
 #include "ut_vector.h"
 #include "ut_string.h"
+#include "ut_growbuf.h"
 #include "xap_App.h"
 #include "xap_Frame.h"
 #include "xap_Prefs.h"
@@ -51,58 +52,55 @@
 /*****************************************************************/
 
 XAP_Frame::XAP_Frame(XAP_App * app)
+:       m_app(app),
+        m_pDoc(0),
+        m_pView(0),
+        m_pViewListener(0),
+        m_lid((AV_ListenerId)-1),
+        m_pScrollObj(0),
+        m_szMenuLayoutName(0),
+        m_szMenuLabelSetName(0),
+        m_szToolbarLabelSetName(0),
+        m_szToolbarAppearance(0),
+        m_nView(0),
+        m_iUntitled(0),
+        m_pMouse(0),
+        m_pKeyboard(0),
+        m_pScrollbarViewListener(0),
+        m_lidScrollbarViewListener((AV_ListenerId)-1),
+        m_zoomType(z_100),
+        m_pData(0),
+        m_pInputModes(0)
 {
-	m_app = app;
-
-	m_pData = NULL;
-	m_pDoc = NULL;
-	m_pView = NULL;
-	m_pViewListener = NULL;
-	m_pScrollObj = NULL;
-	m_szMenuLayoutName = NULL;
-	m_szMenuLabelSetName = NULL;
-	m_szToolbarLabelSetName = NULL;
-	m_szToolbarAppearance = NULL;
-	m_iUntitled = 0;
-	m_nView = 0;
-	m_pScrollbarViewListener = NULL;
-	m_pInputModes = NULL;
-	m_pKeyboard = NULL;
-	m_pMouse = NULL;
-	m_app->rememberFrame(this);
-	memset(m_szTitle,0,sizeof(m_szTitle));
-	memset(m_szNonDecoratedTitle,0,sizeof(m_szNonDecoratedTitle));
-
-	m_lid = (AV_ListenerId) -1;
+        m_app->rememberFrame(this);
+        memset(m_szTitle,0,sizeof(m_szTitle));
+        memset(m_szNonDecoratedTitle,0,sizeof(m_szNonDecoratedTitle));
 }
 
 XAP_Frame::XAP_Frame(XAP_Frame * f)
+:       m_app(f->m_app),                        // only clone a few things
+        m_pDoc(REFP(f->m_pDoc)),
+        m_pView(0),
+        m_pViewListener(0),
+        m_lid((AV_ListenerId)-1),
+        m_pScrollObj(0),
+        m_szMenuLayoutName(0),
+        m_szMenuLabelSetName(0),
+        m_szToolbarLabelSetName(0),
+        m_szToolbarAppearance(0),
+        m_nView(0),
+        m_iUntitled(f->m_iUntitled),
+        m_pMouse(0),
+        m_pKeyboard(0),
+        m_pScrollbarViewListener(0),
+        m_lidScrollbarViewListener((AV_ListenerId)-1),
+        m_zoomType(z_100),
+        m_pData(0),
+        m_pInputModes(0)
 {
-	// only clone a few things
-	m_app = f->m_app;
-	m_pDoc = REFP(f->m_pDoc);
-	m_iUntitled = f->m_iUntitled;
-
-	// everything else gets recreated
-	m_pData = NULL;
-	m_pView = NULL;
-	m_pViewListener = NULL;
-	m_pScrollObj = NULL;
-	m_szMenuLayoutName = NULL;
-	m_szMenuLabelSetName = NULL;
-	m_szToolbarLabelSetName = NULL;
-	m_szToolbarAppearance = NULL;
-	m_nView = 0;
-	m_pScrollbarViewListener = NULL;
-	m_pInputModes = NULL;
-	m_pKeyboard = NULL;
-	m_pMouse = NULL;
-	
-	m_app->rememberFrame(this, f);
-	memset(m_szTitle,0,sizeof(m_szTitle));
-	memset(m_szNonDecoratedTitle,0,sizeof(m_szNonDecoratedTitle));
-
-	m_lid = (AV_ListenerId) -1;
+        m_app->rememberFrame(this, f);
+        memset(m_szTitle,0,sizeof(m_szTitle));
+        memset(m_szNonDecoratedTitle,0,sizeof(m_szNonDecoratedTitle));
 }
 
 XAP_Frame::~XAP_Frame(void)
@@ -162,7 +160,9 @@ UT_Bool XAP_Frame::initialize(const char * szKeyBindingsKey, const char * szKeyB
 	const char * szBindings = NULL;
 	EV_EditBindingMap * pBindingMap = NULL;
 
-	if ((pApp->getPrefsValue(szKeyBindingsKey,&szBindings)) && (szBindings) && (*szBindings))
+	if ((pApp->getPrefsValue(szKeyBindingsKey,
+				 (const XML_Char**)&szBindings)) && 
+	    (szBindings) && (*szBindings))
 		pBindingMap = m_app->getBindingMap(szBindings);
 	if (!pBindingMap)
 		pBindingMap = m_app->getBindingMap(szKeyBindingsDefaultValue);
@@ -183,7 +183,9 @@ UT_Bool XAP_Frame::initialize(const char * szKeyBindingsKey, const char * szKeyB
 	//////////////////////////////////////////////////////////////////
 
 	const char * szMenuLayoutName = NULL;
-	if ((pApp->getPrefsValue(szMenuLayoutKey,&szMenuLayoutName)) && (szMenuLayoutName) && (*szMenuLayoutName))
+	if ((pApp->getPrefsValue(szMenuLayoutKey,
+				 (const XML_Char**)&szMenuLayoutName)) &&
+	    (szMenuLayoutName) && (*szMenuLayoutName))
 		;
 	else
 		szMenuLayoutName = szMenuLayoutDefaultValue;
@@ -194,7 +196,9 @@ UT_Bool XAP_Frame::initialize(const char * szKeyBindingsKey, const char * szKeyB
 	//////////////////////////////////////////////////////////////////
 
 	const char * szMenuLabelSetName = NULL;
-	if ((pApp->getPrefsValue(szMenuLabelSetKey,&szMenuLabelSetName)) && (szMenuLabelSetName) && (*szMenuLabelSetName))
+	if ((pApp->getPrefsValue(szMenuLabelSetKey,
+				 (const XML_Char**)&szMenuLabelSetName)) &&
+	    (szMenuLabelSetName) && (*szMenuLabelSetName))
 		;
 	else
 		szMenuLabelSetName = szMenuLabelSetDefaultValue;
@@ -205,7 +209,9 @@ UT_Bool XAP_Frame::initialize(const char * szKeyBindingsKey, const char * szKeyB
 	//////////////////////////////////////////////////////////////////
 
 	const char * szToolbarLayouts = NULL;
-	if ((pApp->getPrefsValue(szToolbarLayoutsKey,&szToolbarLayouts)) && (szToolbarLayouts) && (*szToolbarLayouts))
+	if ((pApp->getPrefsValue(szToolbarLayoutsKey,
+				 (const XML_Char**)&szToolbarLayouts)) &&
+	    (szToolbarLayouts) && (*szToolbarLayouts))
 		;
 	else
 		szToolbarLayouts = szToolbarLayoutsDefaultValue;
@@ -234,7 +240,9 @@ UT_Bool XAP_Frame::initialize(const char * szKeyBindingsKey, const char * szKeyB
 	//////////////////////////////////////////////////////////////////
 
 	const char * szToolbarLabelSetName = NULL;
-	if ((pApp->getPrefsValue(szToolbarLabelSetKey,&szToolbarLabelSetName)) && (szToolbarLabelSetName) && (*szToolbarLabelSetName))
+	if ((pApp->getPrefsValue(szToolbarLabelSetKey,
+				 (const XML_Char**)&szToolbarLabelSetName)) &&
+	    (szToolbarLabelSetName) && (*szToolbarLabelSetName))
 		;
 	else
 		szToolbarLabelSetName = szToolbarLabelSetDefaultValue;
@@ -245,7 +253,8 @@ UT_Bool XAP_Frame::initialize(const char * szKeyBindingsKey, const char * szKeyB
 	//////////////////////////////////////////////////////////////////
 
 	const char * szToolbarAppearance = NULL;
-	pApp->getPrefsValue(XAP_PREF_KEY_ToolbarAppearance,&szToolbarAppearance);
+	pApp->getPrefsValue(XAP_PREF_KEY_ToolbarAppearance,
+			    (const XML_Char**)&szToolbarAppearance);
 	UT_ASSERT((szToolbarAppearance) && (*szToolbarAppearance));
 	UT_cloneString((char *&)m_szToolbarAppearance,szToolbarAppearance);
 	
@@ -353,7 +362,7 @@ const char * XAP_Frame::getViewKey(void) const
 	// Why "+3"?  For the "0x" and the null. 
 	static char buf[(sizeof(void *) * 2) + 3];
 
-	sprintf(buf, "%p", m_pDoc);
+	sprintf(buf, "%p", (void *)m_pDoc);
 
 	return buf;
 }
@@ -537,11 +546,11 @@ XAP_Dialog_MessageBox::tAnswer XAP_Frame::showMessageBox(XAP_String_Id id,
 
 	if(p_str1)
 	{
-		pDialog->setMessage(pSS->getValue(id), p_str1);
+		pDialog->setMessage((char*)pSS->getValue(id), p_str1);
 	}
 	else
 	{
-		pDialog->setMessage(pSS->getValue(id));
+		pDialog->setMessage((char*)pSS->getValue(id));
 	}
 
 
@@ -557,6 +566,32 @@ XAP_Dialog_MessageBox::tAnswer XAP_Frame::showMessageBox(XAP_String_Id id,
 	return ans;
 }
 
+UT_Error XAP_Frame::backup()
+{
+	char * ext = ".bak"; // TODO: Make this a preference
+	char * oldName = (char *) m_pDoc->getFilename();
+	char * backupName;
+	if (!oldName || !(*oldName))
+	{
+		const XAP_StringSet * pSS = m_app->getStringSet();
+		oldName = (char *) malloc(strlen(pSS->getValue(XAP_STRING_ID_UntitledDocument)) + strlen(ext) + 1);
+		sprintf(oldName, pSS->getValue(XAP_STRING_ID_UntitledDocument), m_iUntitled);
+		UT_ASSERT(oldName);
+	}
+	backupName = (char *) malloc(strlen(oldName) + strlen(ext) + 1);
+	UT_ASSERT(backupName);
+	backupName = strcat(oldName, ext);
+	UT_ASSERT(backupName);
+	
+	UT_Error error = m_pDoc->saveAs(backupName, m_pDoc->getLastType());
+	UT_DEBUGMSG(("File %s saved.\n", backupName));
+	
+	FREEP(backupName);
+	FREEP(oldName);
+	
+	return error;
+}
+
 //////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////
 
