@@ -24,6 +24,8 @@
 #include <string.h>
 
 #include "ut_debugmsg.h"
+#include "ut_growbuf.h"
+#include "ut_string.h"
 #include "xap_Prefs.h"
 
 /*****************************************************************/
@@ -32,6 +34,7 @@ XAP_PrefsScheme::XAP_PrefsScheme( XAP_Prefs *pPrefs, const XML_Char * szSchemeNa
 	: m_hash(13)
 {
 	m_pPrefs = pPrefs;
+	m_uTick = 0;
 
 	if (szSchemeName && *szSchemeName)
 		UT_XML_cloneString((XML_Char *&)m_szName,szSchemeName);
@@ -57,18 +60,19 @@ UT_Bool XAP_PrefsScheme::setSchemeName(const XML_Char * szNewSchemeName)
 
 UT_Bool XAP_PrefsScheme::setValue(const XML_Char * szKey, const XML_Char * szValue)
 {
-	UT_HashEntry * pEntry = m_hash.findEntry(szKey);
+	++m_uTick;
+	UT_HashEntry * pEntry = m_hash.findEntry((char*)szKey);
 	if (pEntry)
 	{
 		if (UT_stricmp(szValue,pEntry->pszRight) == 0)
 			return UT_TRUE;				// equal values, no changes required
 		
-		m_hash.setEntry(pEntry, szValue, NULL); // update with new value
+		m_hash.setEntry(pEntry, (char*)szValue, NULL); // update with new value
 	}
 	else
 	{
 		// otherwise, need to add a new entry
-		m_hash.addEntry(szKey,szValue,NULL);
+		m_hash.addEntry((char*)szKey,(char*)szValue,NULL);
 	}
 
 	m_pPrefs->_markPrefChange( szKey );
@@ -78,12 +82,12 @@ UT_Bool XAP_PrefsScheme::setValue(const XML_Char * szKey, const XML_Char * szVal
 
 UT_Bool XAP_PrefsScheme::setValueBool(const XML_Char * szKey, UT_Bool bValue)
 {
-	return setValue(szKey, ((bValue) ? "1" : "0"));
+	return setValue(szKey, (XML_Char*) ((bValue) ? "1" : "0"));
 }
 
 UT_Bool XAP_PrefsScheme::getValue(const XML_Char * szKey, const XML_Char ** pszValue) const
 {
-	UT_HashEntry * pEntry = m_hash.findEntry(szKey);
+	UT_HashEntry * pEntry = m_hash.findEntry((char*)szKey);
 	if (!pEntry)
 		return UT_FALSE;
 
@@ -390,6 +394,8 @@ UT_Bool XAP_Prefs::setCurrentScheme(const XML_Char * szSchemeName)
 }
 
 /*****************************************************************/
+static const XML_Char DEBUG_PREFIX[] = "DeBuG";  // case insensitive
+static const XML_Char NO_PREF_VALUE[] = "";
 
 UT_Bool XAP_Prefs::getPrefsValue(const XML_Char * szKey, const XML_Char ** pszValue) const
 {
@@ -401,6 +407,14 @@ UT_Bool XAP_Prefs::getPrefsValue(const XML_Char * szKey, const XML_Char ** pszVa
 		return UT_TRUE;
 	if (m_builtinScheme->getValue(szKey,pszValue))
 		return UT_TRUE;
+	// It is legal for there to be arbitrary preference tags that start with 
+	// "Debug", and Abi apps won't choke.  The idea is that developers can use
+	// these to selectively trigger development-time behaviors.
+	if (UT_XML_strnicmp(szKey, DEBUG_PREFIX, sizeof(DEBUG_PREFIX) - 1) == 0)
+	{
+		*pszValue = NO_PREF_VALUE;
+		return UT_TRUE;
+	}
 
 	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 	return UT_FALSE;
@@ -416,6 +430,14 @@ UT_Bool XAP_Prefs::getPrefsValueBool(const XML_Char * szKey, UT_Bool * pbValue)
 		return UT_TRUE;
 	if (m_builtinScheme->getValueBool(szKey,pbValue))
 		return UT_TRUE;
+	// It is legal for there to be arbitrary preference tags that start with 
+	// "Debug", and Abi apps won't choke.  The idea is that developers can use
+	// these to selectively trigger development-time behaviors.
+	if (UT_XML_strnicmp(szKey, DEBUG_PREFIX, sizeof(DEBUG_PREFIX) - 1) == 0)
+	{
+		*pbValue = UT_FALSE;
+		return UT_TRUE;
+	}
 
 	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 	return UT_FALSE;
@@ -427,6 +449,7 @@ UT_Bool XAP_Prefs::getPrefsValueBool(const XML_Char * szKey, UT_Bool * pbValue)
 ******************************************************************
 *****************************************************************/
 
+#ifndef HAVE_LIBXML2
 static void startElement(void *userData, const XML_Char *name, const XML_Char **atts)
 {
 	XAP_Prefs * pPrefs = (XAP_Prefs *)userData;
@@ -450,6 +473,7 @@ static void startElement_SystemDefaultFile(void *userData, const XML_Char *name,
 	XAP_Prefs * pPrefs = (XAP_Prefs *)userData;
 	pPrefs->_startElement_SystemDefaultFile(name,atts);
 }
+#endif /* HAVE_LIBXML2 */
 
 /*****************************************************************/
 
@@ -664,12 +688,15 @@ void XAP_Prefs::_charData(const XML_Char * /* s */, int /* len */)
 
 /*****************************************************************/
 
+
 UT_Bool XAP_Prefs::loadPrefsFile(void)
 {
+#ifndef HAVE_LIBXML2 
+	XML_Parser parser = NULL;
+#endif
 	UT_Bool bResult = UT_FALSE;			// assume failure
 	const char * szFilename;
 	FILE * fp = NULL;
-	XML_Parser parser = NULL;
 	int done = 0;
 	char buf[4096];
 
@@ -685,7 +712,25 @@ UT_Bool XAP_Prefs::loadPrefsFile(void)
 		UT_DEBUGMSG(("could not get pathname for preferences file.\n"));
 		goto Cleanup;
 	}
-
+#ifdef HAVE_LIBXML2
+	else
+	{
+	  xmlDocPtr dok = xmlParseFile(szFilename);
+	  if (dok == NULL)
+	    {
+	      UT_DEBUGMSG(("Could not open and parse file %s\n",
+			   szFilename));
+	    }
+	  else
+	    {
+	      xmlNodePtr node = xmlDocGetRootElement(dok);
+	      _scannode(dok,node,0,UT_FALSE);
+	      xmlFreeDoc(dok);
+	      bResult = UT_TRUE;
+	    }
+	}
+ Cleanup:
+#else
 	fp = fopen(szFilename, "r");
 	if (!fp)
 	{
@@ -752,13 +797,13 @@ UT_Bool XAP_Prefs::loadPrefsFile(void)
 	}
 
 	bResult = UT_TRUE;
-
 Cleanup:
 	FREEP(m_parserState.m_szSelectedSchemeName);
 	if (parser)
 		XML_ParserFree(parser);
 	if (fp)
 		fclose(fp);
+#endif /* HAVE_LIBXML2 */
 	return bResult;
 }
 
@@ -871,6 +916,8 @@ UT_Bool XAP_Prefs::savePrefsFile(void)
 							"\t**** defaults and adjusted by the installation system defaults.  This scheme\n"
 							"\t**** is only written here as a reference.  Any schemes following this one\n"
 							"\t**** only list values that deviate from the built-in values.\n"
+							"\t**** Items values must observe XML encoding for double quote (&quot;),\n"
+							"\t**** ampersand (&amp;), and angle brackets (&lt; and &gt;).\n"
 							"\t-->\n"),
 						szBuiltinSchemeName);
 			}
@@ -882,20 +929,64 @@ UT_Bool XAP_Prefs::savePrefsFile(void)
 			UT_uint32 j;
 			for (j=0; (p->getNthValue(j,&szKey,&szValue)); j++)
 			{
+				UT_Bool need_print;
+				need_print = UT_FALSE;
 				if (bIsBuiltin)
 				{
 					// for the builtin set, we print every value
-					fprintf(fp,"\t\t%s=\"%s\"\n",szKey,szValue);
+					need_print = UT_TRUE;
 				}
 				else
 				{
 					// for non-builtin sets, we only print the values which are different
 					// from the builtin set.
-					const XML_Char * szBuiltinValue;
+					const XML_Char * szBuiltinValue = NO_PREF_VALUE;
 					UT_Bool bHaveBuiltinValue = m_builtinScheme->getValue(szKey,&szBuiltinValue);
-					UT_ASSERT(bHaveBuiltinValue);
+					if (UT_XML_strnicmp(szKey, DEBUG_PREFIX, sizeof(DEBUG_PREFIX) - 1))
+					{
+						UT_DEBUGMSG(("Strange preference '%s=\"%s\"', ASSERT follows...\n", szKey, szValue));
+						UT_ASSERT(bHaveBuiltinValue);
+					}
+					else
+					{
+						need_print = UT_TRUE;  // always print "Debug..." values
+					}
 					if (UT_XML_strcmp(szValue,szBuiltinValue) != 0)
-						fprintf(fp,"\t\t%s=\"%s\"\n",szKey,szValue);
+						need_print = UT_TRUE;
+				}
+				if (need_print == UT_TRUE)
+				{
+					// szValue is UTF8.  Convert to Unicode and then
+					// do XML-encoding of XML-special characters and
+					// non-ASCII characters.  The printed value
+					// strings will get XML parsing and conversion to
+					// UTF8 the next time the application reads the
+					// prefs file.
+					UT_GrowBuf gb;
+					UT_decodeUTF8string(szValue, UT_XML_strlen(szValue), &gb);
+					UT_uint32 length = gb.getLength();
+					fprintf(fp,"\t\t%s=\"",szKey);
+					for (UT_uint32 udex=0; udex<length; ++udex)
+					{
+						UT_UCSChar ch = *(gb.getPointer(udex));
+						switch (ch)
+						{
+						case '&':   fputs("&amp;", fp);  break;
+						case '<':   fputs("&lt;", fp);  break;
+						case '>':   fputs("&gt;", fp);  break;
+						case '"':   fputs("&quot;", fp);  break;
+						default:
+							if (ch < ' ' || ch >= 128)
+							{
+								fprintf(fp, "&#x%x;", ch);
+							}
+							else
+							{
+								putc(ch, fp);
+							}
+						}
+					}
+					fputs("\"\n", fp);
 				}
 			}
 				
@@ -984,13 +1075,26 @@ UT_Bool XAP_Prefs::loadSystemDefaultPrefsFile(const char * szSystemDefaultPrefsP
 	UT_ASSERT(szSystemDefaultPrefsPathname && *szSystemDefaultPrefsPathname);
 	
 	UT_Bool bResult = UT_FALSE;			// assume failure
+	m_parserState.m_parserStatus = UT_TRUE;
+#ifdef HAVE_LIBXML2
+	xmlDocPtr dok = xmlParseFile(szSystemDefaultPrefsPathname);
+	if (dok == NULL)
+	  {
+	    UT_DEBUGMSG(("Could not open and parse file %s\n",
+			 szSystemDefaultPrefsPathname));
+	  }
+	else
+	  {
+	    xmlNodePtr node = xmlDocGetRootElement(dok);
+	    _scannode(dok,node,0,UT_TRUE);
+	    xmlFreeDoc(dok);
+	    bResult = UT_TRUE;
+	  }
+#else
 	FILE * fp = NULL;
-	XML_Parser parser = NULL;
 	int done = 0;
 	char buf[4096];
-
-	m_parserState.m_parserStatus = UT_TRUE;
-
+	XML_Parser parser = NULL;
 	fp = fopen(szSystemDefaultPrefsPathname, "r");
 	if (!fp)
 	{
@@ -1038,6 +1142,7 @@ Cleanup:
 		XML_ParserFree(parser);
 	if (fp)
 		fclose(fp);
+#endif
 	return bResult;
 }
 
@@ -1080,18 +1185,18 @@ void XAP_Prefs::_markPrefChange( const XML_Char *szKey )
 {
 	if ( m_bInChangeBlock )
 	{
-		UT_HashEntry *uth_e = m_ahashChanges.findEntry( szKey );
+		UT_HashEntry *uth_e = m_ahashChanges.findEntry((char*) szKey );
 		if ( uth_e ) 
 			m_ahashChanges.setEntry( uth_e, (char *)NULL, (void *)1 );	
 		else
-			m_ahashChanges.addEntry( szKey, (char *)NULL, (void *)1 );	
+			m_ahashChanges.addEntry((char*) szKey, (char *)NULL, (void *)1 );	
 
 		// notify later
 	}
 	else
 	{
 		UT_AlphaHashTable	changes(3);
-		changes.addEntry( szKey, (char *)NULL, (void *)1 );	
+		changes.addEntry((char*) szKey, (char *)NULL, (void *)1 );	
 
 		_sendPrefsSignal( (UT_AlphaHashTable *)&changes );
 	}
@@ -1124,3 +1229,35 @@ void XAP_Prefs::_sendPrefsSignal( UT_AlphaHashTable *hash  )
 		(p->m_pFunc)(m_pApp, this, hash, p->m_pData);
 	}
 }
+
+#ifdef HAVE_LIBXML2
+void XAP_Prefs::_scannode(xmlDocPtr dok, xmlNodePtr cur, int c, char sys /*boolean*/)
+{
+  while (cur != NULL)
+    {
+      if (strcmp("text", (char*) cur->name) == 0)
+	{
+	  xmlChar* s = cur->content; // xmlNodeListGetString(dok, cur, 1);
+	  _charData(s, strlen((char*) s));
+	}
+      else
+	{
+	  xmlChar *prop = NULL;
+	  const xmlChar* props[3] = { NULL, NULL, NULL };
+	  if (cur->properties)
+	    {
+	      props[0] = cur->properties->name;
+	      props[1] = cur->properties->children->content;
+	    }
+	  if (sys) _startElement_SystemDefaultFile(cur->name, props);
+	  else     _startElement(cur->name, props);
+	  if (prop) free(prop);
+	}
+      _scannode(dok, cur->children, c + 1, sys);
+      if (strcmp("text", (char*) cur->name) != 0)
+	_endElement(cur->name);
+      cur = cur->next;
+    }
+}
+#endif /* HAVE_LIBXML2 */
+
