@@ -50,6 +50,8 @@ char		g_szBrowseDir[1024];
 
 HFONT		g_hfontMain;
 
+static int	s_bInstallationSuccessfull = 0;
+
 #define		ID_PANE			1000
 #define		ID_BTN_NEXT		1001
 #define		ID_BTN_CANCEL	1002
@@ -158,11 +160,8 @@ void _updateDirAndSpace(char* pszPath)
 {
 	char szBuf[256];
 	char* p;
-	DWORD iSectorsPerCluster;
-	DWORD iBytesPerSector;
-	DWORD iNumberOfFreeClusters;
-	DWORD iTotalNumberOfClusters;
-	DWORD iBytes;
+	long iBytesLow, iBytesHigh;
+	int iResult;
 
 	if (g_pSet_BrowseDir->pszDirName &&g_pSet_BrowseDir->pszDirName[0])
 	{
@@ -184,17 +183,23 @@ void _updateDirAndSpace(char* pszPath)
 	p++;
 	*p = 0;
 	
-	GetDiskFreeSpace(szBuf,
-					 &iSectorsPerCluster,
-					 &iBytesPerSector,
-					 &iNumberOfFreeClusters,
-					 &iTotalNumberOfClusters);
-
-	iBytes = iNumberOfFreeClusters * iSectorsPerCluster * iBytesPerSector;
+	iResult = ASK_getDiskFreeSpace(szBuf, &iBytesHigh, &iBytesLow);
 
 	// and set it into the static text display
-	ASK_convertBytesToString(iBytes, szBuf);
+	ASK_convert64BitsToString(iBytesHigh, iBytesLow, szBuf);
 	SetWindowText(g_hwndStatic_DiskSpace, szBuf);
+
+	if(iResult == 0)
+	{
+		EnableWindow(g_hwndButtonNext, FALSE);
+		MessageBox(g_hwndMain,
+			"The selected disk is not available.", "Error",
+			MB_ICONEXCLAMATION | MB_OK);
+	}
+	else
+	{
+		EnableWindow(g_hwndButtonNext, TRUE);
+	}
 }
 
 static void _png_read(png_structp png_ptr, png_bytep data, png_size_t length)
@@ -326,6 +331,8 @@ int initGraphic(void)
 		return 1;
 		
 	}
+
+	return 0;
 }
 
 int try_cancel(void)
@@ -334,7 +341,7 @@ int try_cancel(void)
 	{
 		g_iWhichButton = ID_BTN_CANCEL;
 		QuitEventLoop();
-		return 0;
+		return 1;
 	}
 	else
 	{
@@ -380,7 +387,7 @@ int BrowseDir(HWND hwnd, char* pszDest)
     bi.pidlRoot = pidlPrograms; 
     bi.pszDisplayName = lpBuffer; 
     bi.lpszTitle = "Please choose a folder"; 
-    bi.ulFlags = 0; 
+    bi.ulFlags = BIF_RETURNONLYFSDIRS; 
     bi.lpfn = NULL; 
     bi.lParam = 0; 
 
@@ -391,6 +398,10 @@ int BrowseDir(HWND hwnd, char* pszDest)
         if (SHGetPathFromIDList(pidlBrowse, lpBuffer))
 		{
 			strcpy(pszDest, lpBuffer);
+
+			if( pszDest[ strlen( pszDest ) - 1 ] == '\\' )
+				pszDest[ strlen( pszDest ) - 1 ] = '\0';
+
 			result = 1;
 		}
  
@@ -422,7 +433,8 @@ LRESULT CALLBACK WndProc_Main(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
 		}
 		else if (wParam == 27)
 		{
-			return try_cancel();
+			try_cancel();
+			return 0;
 		}
 		else
 		{
@@ -439,7 +451,8 @@ LRESULT CALLBACK WndProc_Main(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
 		case BN_CLICKED:
 			if (wID == ID_BTN_CANCEL)
 			{
-				return try_cancel();
+				try_cancel();
+				return 0;
 			}
 			else if (wID == ID_BTN_NEXT)
 			{
@@ -467,6 +480,21 @@ LRESULT CALLBACK WndProc_Main(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
 		
 		return 0;
 	}
+
+	case WM_CLOSE:
+	{
+		if(s_bInstallationSuccessfull)
+		{
+			QuitEventLoop();
+			PostQuitMessage(0);
+			return 0;
+		}
+		else
+		{
+			return ! try_cancel();
+		}
+	}
+
 	case WM_DESTROY:
 	{
 		PostQuitMessage(0);
@@ -609,7 +637,7 @@ LRESULT CALLBACK WndProc_Graphic(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPar
 
 #define GRAPHIC_WIDTH		200
 
-int ASK_Win32_Init(HINSTANCE hInstance)
+int ASK_Win32_Init(HINSTANCE hInstance, long iIconId)
 {
 	int 		iScreenWidth;
 	int 		iScreenHeight;
@@ -633,7 +661,7 @@ int ASK_Win32_Init(HINSTANCE hInstance)
 	g_wndclassMain.cbClsExtra = 0;
 	g_wndclassMain.cbWndExtra = 0;
 	g_wndclassMain.hInstance = hInstance;
-	g_wndclassMain.hIcon = LoadIcon(NULL, IDI_APPLICATION);
+	g_wndclassMain.hIcon = LoadIcon(hInstance, MAKEINTRESOURCE(iIconId));
 	g_wndclassMain.hCursor = LoadCursor(NULL, IDC_ARROW);
 	g_wndclassMain.hbrBackground = GetSysColorBrush(COLOR_BTNFACE);
 	g_wndclassMain.lpszMenuName = NULL;
@@ -701,7 +729,7 @@ int ASK_Win32_Init(HINSTANCE hInstance)
 
 	g_hwndMain = CreateWindow("AbiSetup_MainWindow",
 						  "AbiSetup",
-						  WS_BORDER | WS_VISIBLE | WS_CLIPCHILDREN,
+						  WS_BORDER | WS_VISIBLE | WS_CLIPCHILDREN | WS_SYSMENU,
 						  (iScreenWidth - MAINWIN_WIDTH) / 2,
 						  (iScreenHeight - MAINWIN_HEIGHT) /2,
 						  MAINWIN_WIDTH,
@@ -957,6 +985,9 @@ int ASK_DoScreen_chooseDirForFileSet(ASK_FileSet* pSet)
 	int		iHeight;
 	RECT	r;
 
+	HCURSOR hcursorHourGlass;
+	HCURSOR	hcursorPrev;
+
 	if (pSet->bFixedPath)
 	{
 		// just do the fixed default path, but check for special names
@@ -973,6 +1004,9 @@ int ASK_DoScreen_chooseDirForFileSet(ASK_FileSet* pSet)
 		return 1;
 	}
 
+	hcursorHourGlass = LoadCursor(NULL, IDC_WAIT);
+	hcursorPrev = SetCursor(hcursorHourGlass);
+
 	sprintf(buf, "Choose directory for \"%s\"", pSet->pszName);
 	SetWindowText(g_hwndMain, buf);
 
@@ -1057,13 +1091,13 @@ int ASK_DoScreen_chooseDirForFileSet(ASK_FileSet* pSet)
 	SendMessage(hwndStatic_Dir, WM_SETFONT, (WPARAM) g_hfontMain, 0);
 	SendMessage(hwndStatic_Space, WM_SETFONT, (WPARAM) g_hfontMain, 0);
 
-	ASK_convertBytesToString(ASK_getFileSetTotalSizeInBytes(pSet), buf2);
+	ASK_convertBytesToString(ASK_getFileSetTotalSizeInBytesToCopy(pSet), buf2);
 	
 	sprintf(buf,
 			"\r\nPlease select a directory where '%s' will be installed.\r\n\r\nThe set '%s' contains %d files,\r\na total of %s.\r\n\r\n",
 			pSet->pszName,
 			pSet->pszName,
-			pSet->iNumFilesInSet,
+			ASK_getFileSetTotalFilesToCopy(pSet),
 			buf2);
 	SetWindowText(hwndStatic_Top, buf);
 
@@ -1071,6 +1105,8 @@ int ASK_DoScreen_chooseDirForFileSet(ASK_FileSet* pSet)
 	g_hwndStatic_BrowseDir = hwndStatic_Dir;
 	g_hwndStatic_DiskSpace = hwndStatic_Space;
 	
+	SetCursor(hcursorPrev);
+
 	_updateDirAndSpace(g_pSet_BrowseDir->pszDefaultPath);
 
 	result = DoEventLoop();
@@ -1105,8 +1141,8 @@ int ASK_DoScreen_readyToCopy(int iNumSets, ASK_FileSet** ppSets)
 	{
 		ASK_FileSet* pSet = ppSets[ndxSet];
 
-		iTotalBytes += ASK_getFileSetTotalSizeInBytes(pSet);
-		iTotalNumFiles += pSet->iNumFilesInSet;
+		iTotalBytes += ASK_getFileSetTotalSizeInBytesToCopy(pSet);
+		iTotalNumFiles += ASK_getFileSetTotalFilesToCopy(pSet);
 	}
 
 	ASK_convertBytesToString(iTotalBytes, buf2);
@@ -1151,7 +1187,8 @@ int ASK_DoScreen_copy(int iNumSets, ASK_FileSet** ppSets)
 	  etc.
 	*/
 
-	int 	result;
+	int 	bCopyCancelled = 0;
+	int		bErrorOccured = 0;
 	int 	ndxSet;
 	int 	ndxFile;
 
@@ -1177,8 +1214,8 @@ int ASK_DoScreen_copy(int iNumSets, ASK_FileSet** ppSets)
 	{
 		ASK_FileSet* pSet = ppSets[ndxSet];
 
-		iTotalBytes += ASK_getFileSetTotalSizeInBytes(pSet);
-		iTotalNumFiles += pSet->iNumFilesInSet;
+		iTotalBytes += ASK_getFileSetTotalSizeInBytesToCopy(pSet);
+		iTotalNumFiles += ASK_getFileSetTotalFilesToCopy(pSet);
 	}
 
 	GetClientRect(g_hwndPane, &r);
@@ -1258,7 +1295,7 @@ int ASK_DoScreen_copy(int iNumSets, ASK_FileSet** ppSets)
 			);
 	SetWindowText(hwndStatic_Top, buf);
 
-	for (ndxSet=0; ndxSet<iNumSets; ndxSet++)
+	for (ndxSet=0; ndxSet<iNumSets && bCopyCancelled == 0; ndxSet++)
 	{
 		ASK_FileSet* pSet = ppSets[ndxSet];
 
@@ -1269,81 +1306,117 @@ int ASK_DoScreen_copy(int iNumSets, ASK_FileSet** ppSets)
 		ASK_fixSlashes(pSet->szInstallPath);
 		ASK_verifyDirExists(pSet->szInstallPath);
 
-		for (ndxFile=0; ndxFile<pSet->iNumFilesInSet; ndxFile++)
+		for (ndxFile=0; ndxFile<pSet->iNumFilesInSet && bCopyCancelled == 0; ndxFile++)
 		{
-			ASK_DataFile* pFile = pSet->aFiles[ndxFile];
-			
-			iCurFile++;
-			
-			sprintf(buf, "Copying file %d of %d (%s)",
-					iCurFile,
-					iTotalNumFiles,
-					pFile->pszFileName
-					);
-			SetWindowText(hwndStatic_CurFile, buf);
-
+			if(pSet->aFiles[ndxFile]->bNoCopy == 0)
 			{
-				int err;
-
-				if (pFile->pszRelPath)
-				{
-					char szRelDir[ASK_MAX_PATH + 1];
-
-					sprintf(szRelDir, "%s/%s", pSet->szInstallPath, pFile->pszRelPath);
-					ASK_fixSlashes(szRelDir);
-					ASK_verifyDirExists(szRelDir);
-					
-					sprintf(pFile->szInstallPath, "%s/%s/%s", pSet->szInstallPath, pFile->pszRelPath, pFile->pszFileName);
-				}
-				else
-				{
-					sprintf(pFile->szInstallPath, "%s/%s", pSet->szInstallPath, pFile->pszFileName);
-				}
-
-				ASK_fixSlashes(pFile->szInstallPath);
+				ASK_DataFile* pFile = pSet->aFiles[ndxFile];
 				
-				// TODO check to see if the file and/or its directory exist.
+				iCurFile++;
 				
-				err = ASK_decompressAndWriteFile(pFile);
-				if (err < 0)
-				{
-					char szErr[ASK_MAX_PATH+1];
+				sprintf(buf, "Copying file %d of %d (%s)",
+						iCurFile,
+						iTotalNumFiles,
+						pFile->pszFileName
+						);
+				SetWindowText(hwndStatic_CurFile, buf);
 
-					sprintf(szErr, "Attempt to write file '%s' failed", pFile->szInstallPath);
-					
-					MessageBox(g_hwndMain, szErr, "Error", MB_ICONEXCLAMATION | MB_OK);
-				}
-
-				if (
-					!(pFile->bNoCopy)
-					&& !(pFile->bNoRemove))
 				{
-					ASK_addToRemoveFile(pFile->szInstallPath);
+					int err;
+					int bKeepExistingFile = 0;
+
+					if (pFile->pszRelPath)
+					{
+						char szRelDir[ASK_MAX_PATH + 1];
+
+						sprintf(szRelDir, "%s/%s", pSet->szInstallPath, pFile->pszRelPath);
+						ASK_fixSlashes(szRelDir);
+						ASK_verifyDirExists(szRelDir);
+
+						sprintf(pFile->szInstallPath, "%s/%s/%s", pSet->szInstallPath, pFile->pszRelPath, pFile->pszFileName);
+					}
+					else
+					{
+						sprintf(pFile->szInstallPath, "%s/%s", pSet->szInstallPath, pFile->pszFileName);
+					}
+
+					ASK_fixSlashes(pFile->szInstallPath);
+
+					if(ASK_isFileNewer(pFile->szInstallPath, pFile->iModTime))
+					{
+						sprintf(buf, "A file on your system is newer than the one supplied with this\r\n " \
+									"installation. It is recommended that you keep the existing file.\r\n\r\n " \
+									"File name:\t'%s'\r\n\r\n" \
+									"Do you want to keep this file?",
+							pFile->szInstallPath);
+
+						if(IDYES == MessageBox(g_hwndMain, buf, "Version Conflict",
+							MB_ICONEXCLAMATION | MB_YESNO))
+						{
+							bKeepExistingFile = 1;
+							iCurBytes += pFile->iOriginalLength;
+						}
+					}
+
+					if(bKeepExistingFile == 0)
+					{
+						err = ASK_decompressAndWriteFile(pFile);
+						if (err < 0)
+						{
+							sprintf(buf, "Attempt to write file '%s' failed", pFile->szInstallPath);
+
+							bErrorOccured = 1;
+							if(IDCANCEL == MessageBox(g_hwndMain, buf, "Error",
+								MB_ICONEXCLAMATION | MB_OKCANCEL))
+							{
+								if(bCopyCancelled = try_cancel())
+								{
+									break;
+								}
+							}
+						}
+						else
+						{
+							iCurBytes += pFile->iOriginalLength;
+						}
+					}
+
+					sprintf(buf, "Processed %d of %d bytes (%d%%)",iCurBytes, iTotalBytes, (iCurBytes * 100 / iTotalBytes));
+					SetWindowText(hwndStatic_CurBytes, buf);
+
+					SendMessage(hwndProgress, PBM_SETPOS, (WPARAM) (iCurBytes * 100 / iTotalBytes), (LPARAM) 0);
+
+					if (
+						!(pFile->bNoCopy)
+						&& !(pFile->bNoRemove))
+					{
+						ASK_addToRemoveFile(pFile->szInstallPath);
+					}
 				}
 			}
-			
-			iCurBytes += pFile->iOriginalLength;
-			
-			sprintf(buf, "Copied %d of %d bytes (%d%%)",iCurBytes, iTotalBytes, (iCurBytes * 100 / iTotalBytes));
-			SetWindowText(hwndStatic_CurBytes, buf);
-
-			SendMessage(hwndProgress, PBM_SETPOS, (WPARAM) (iCurBytes * 100 / iTotalBytes), (LPARAM) 0);
 		}
 	}
 
-	sprintf(buf,
-			"\r\nCopying is now complete.  Click the 'Next' button below to continue."
-			);
-	SetWindowText(hwndStatic_Top, buf);
+	if(bErrorOccured == 0)
+	{
+		sprintf(buf,
+				"\r\nCopying is now complete.  Click the 'Next' button below to continue."
+				);
+		SetWindowText(hwndStatic_Top, buf);
 
-	SetWindowText(g_hwndMain, "Done Copying Files");
-	EnableWindow(g_hwndButtonCancel, FALSE);
+		SetWindowText(g_hwndMain, "Done Copying Files");
+		EnableWindow(g_hwndButtonCancel, FALSE);
 
-	ShowWindow(hwndProgress, SW_HIDE);
-	
-	result = DoEventLoop();
+		ShowWindow(hwndProgress, SW_HIDE);
 
-	EnableWindow(g_hwndButtonCancel, TRUE);
+		s_bInstallationSuccessfull = 1;
+		DoEventLoop();
+	}
+	else
+	{
+		EnableWindow(g_hwndButtonNext, FALSE);
+		EnableWindow(g_hwndButtonCancel, FALSE);
+	}
 	
 	DestroyWindow(hwndStatic_Top);
 	DestroyWindow(hwndStatic_CurSet);
@@ -1351,7 +1424,7 @@ int ASK_DoScreen_copy(int iNumSets, ASK_FileSet** ppSets)
 	DestroyWindow(hwndStatic_CurBytes);
 	DestroyWindow(hwndProgress);
 	
-	return result;
+	return bErrorOccured;
 }
 
 int ASK_DoScreen_license(char* pszText)
@@ -1361,7 +1434,7 @@ int ASK_DoScreen_license(char* pszText)
 	HWND	hwndStatic;
 	RECT 	r;
 
-	SetWindowText(g_hwndMain, "You must accept this license in order to install the software.");
+	SetWindowText(g_hwndMain, "GNU General Public License.");
 	
 	GetClientRect(g_hwndPane, &r);
 	
@@ -1392,13 +1465,10 @@ int ASK_DoScreen_license(char* pszText)
 	SendMessage(hwndStatic, WM_SETFONT, (WPARAM) g_hfontMain, 0);
 	SendMessage(hwndText, WM_SETFONT, (WPARAM) g_hfontMain, 0);
 	
-	SetWindowText(hwndStatic, "Do you accept the terms of this license?");
+	SetWindowText(hwndStatic, "These are the terms of the GNU General Public License.");
 	
 	SetWindowText(hwndText, pszText);
 
-	SetWindowText(g_hwndButtonNext, "Yes");
-	SetWindowText(g_hwndButtonCancel, "No");
-
 	result = DoEventLoop();
 
 	SetWindowText(g_hwndButtonNext, "Next");
