@@ -305,7 +305,7 @@ HWND AP_Win32Frame::_createDocumentWindow(HWND hwndParent,
 
 UT_Bool AP_Win32Frame::loadDocument(const char * szFilename, int ieft)
 {
-	GR_Win32Graphics * pG = new GR_Win32Graphics(GetDC(m_hwndDocument), m_hwndDocument);
+	GR_Win32Graphics * pG = new GR_Win32Graphics(GetDC(m_hwndDocument), m_hwndDocument, getApp());
 	pG->setFont(pG->findFont("Times New Roman", "normal", NULL, "bold", NULL, "72pt"));
 	pG->setZoomPercentage(100);
 
@@ -612,11 +612,9 @@ LRESULT CALLBACK AP_Win32Frame::_DocumentWndProc(HWND hwnd, UINT iMsg, WPARAM wP
 
 	case WM_TIMER:
 	{
-		// Timers are handled differently on Win95 and WinNT.
 		TIMERPROC * pfn = (TIMERPROC *)lParam;
-		UT_Win32Timer * pTimer = (UT_Win32Timer *)wParam;
 		UT_ASSERT( (void *)(pfn) == (void *)(Global_Win32TimerProc) );
-		Global_Win32TimerProc(hwnd,WM_TIMER, pTimer->getIdentifier(),NULL);
+		Global_Win32TimerProc(hwnd,WM_TIMER,(UINT)wParam,NULL);
 		return 0;
 	}
 	
