@@ -139,21 +139,22 @@ UT_sint32 fb_LineBreaker::breakParagraph(fl_BlockLayout* pBlock)
 					  if it's a left tab, then add its width to the m_iWorkingLineWidth
 					*/
 
-					UT_sint32 iPos;
-					unsigned char iType;
+					UT_sint32	iPos;
+					eTabType	iType;
+					eTabLeader	iLeader;
 
 					// Subtract the width already added then work out new
 					// tab position.
 					
 					m_iWorkingLineWidth -= pCurrentRun->getWidthInLayoutUnits();
 
-					UT_Bool bRes = pLine->findNextTabStopInLayoutUnits(m_iWorkingLineWidth, iPos, iType);
+					UT_Bool bRes = pLine->findNextTabStopInLayoutUnits(m_iWorkingLineWidth, iPos, iType, iLeader);
 					if (bRes)
 					{
-
-						UT_DEBUGMSG(("%s(%d) tab run: type=%d height=%d width=%d offset=%d length=%d\n",
+						UT_DEBUGMSG(("%s:%d tab run: type=%d leader=%d height=%d width=%d offset=%d length=%d\n",
 								 __FILE__, __LINE__,
-								iType,
+								(int)iType,
+								(int)iLeader,
 								pCurrentRun->getHeight(),
 								pCurrentRun->getWidth(),
 								pCurrentRun->getBlockOffset(),
@@ -182,7 +183,7 @@ UT_sint32 fb_LineBreaker::breakParagraph(fl_BlockLayout* pBlock)
 								  pScanRun && pScanRun->getType() != FPRUN_TAB && (iScanWidth / 2 < iPos-m_iWorkingLineWidth); 
 								  pScanRun = pScanRun->getNext() )
 							{
-								iScanWidth += pScanRun->getWidth();
+								iScanWidth += pScanRun->getWidthInLayoutUnits();
 							}
 				
 							m_iWorkingLineWidth = iPos - iScanWidth/2;
@@ -199,7 +200,7 @@ UT_sint32 fb_LineBreaker::breakParagraph(fl_BlockLayout* pBlock)
 								  pScanRun && pScanRun->getType() != FPRUN_TAB && (iScanWidth < iPos-m_iWorkingLineWidth); 
 								  pScanRun = pScanRun->getNext() )
 							{
-								iScanWidth += pScanRun->getWidth();
+								iScanWidth += pScanRun->getWidthInLayoutUnits();
 							}
 				
 							m_iWorkingLineWidth = iPos - iScanWidth;
@@ -211,9 +212,8 @@ UT_sint32 fb_LineBreaker::breakParagraph(fl_BlockLayout* pBlock)
 						{
 							fp_Run *pScanRun;
 							int iScanWidth = 0;
-							UT_uint32   len = 0, runLen = 0;
+							UT_uint32 runLen = 0;
 
-							UT_UCSChar const *pSpanPtr, *pCh;
 							UT_UCSChar *pDecimalStr;
 							// the string to search for decimals
 							UT_UCS_cloneString_char(&pDecimalStr, ".");
@@ -224,33 +224,34 @@ UT_sint32 fb_LineBreaker::breakParagraph(fl_BlockLayout* pBlock)
 								  pScanRun = pScanRun->getNext() )
 							{
 								UT_Bool foundDecimal = UT_FALSE;
-								if ( pBlock->getSpanPtr( pScanRun->getBlockOffset(), &pSpanPtr, &len ) )
-								{
-									runLen = pScanRun->getLength();
-									UT_ASSERT(len >= runLen);
-									pCh = pSpanPtr;
 
-									while ( runLen )
+								if(pScanRun->getType() == FPRUN_TEXT)
+								{
+									UT_sint32 decimalBlockOffset = ((fp_TextRun *)pScanRun)->findCharacter(0, pDecimalStr[0]);
+
+									if(decimalBlockOffset != -1)
 									{
-										if ( *pCh++ == *pDecimalStr )		// TODO - FIXME
-										{
-											foundDecimal = UT_TRUE;	
-											break;
-										}
-										runLen--;
+										foundDecimal = UT_TRUE;
+
+										runLen = pScanRun->getBlockOffset() - decimalBlockOffset;
 									}
 								}
+
 								//UT_DEBUGMSG(("%s:%d  foundDecimal=%d len=%d iScanWidth=%d \n", 
 								//			__FILE__, __LINE__, foundDecimal, pScanRun->getLength()-runLen, iScanWidth));
 				
 								if ( foundDecimal )
 								{
-									iScanWidth += pBlock->getDocLayout()->getGraphics()->
-										measureString( pSpanPtr, 0, pScanRun->getLength() - runLen, NULL );
+									if(pScanRun->getType() == FPRUN_TEXT)
+									{
+										iScanWidth += ((fp_TextRun *)pScanRun)->simpleRecalcWidth(fp_TextRun::Width_type_layout_units, runLen);
+									}
 									break; // we found our decimal, don't search any further
 								}
 								else	
-									iScanWidth += pScanRun->getWidth();
+								{
+									iScanWidth += pScanRun->getWidthInLayoutUnits();
+								}
 							}
 							//UT_DEBUGMSG((" tabrun iX=%d iPos=%d iScanWidth=%d tabwidth=%d newX=%d\n", 
 							//					  iX,	iPos,   iScanWidth,   iPos-iX-iScanWidth,iPos-iScanWidth));	
@@ -285,6 +286,8 @@ UT_sint32 fb_LineBreaker::breakParagraph(fl_BlockLayout* pBlock)
 					
 				case FPRUN_FIELD:
 				case FPRUN_IMAGE:
+				case FPRUN_FIELDSTARTRUN:
+				case FPRUN_FIELDENDRUN:
 				case FPRUN_TEXT:
 				{
 
@@ -365,7 +368,6 @@ UT_sint32 fb_LineBreaker::_moveBackToFirstNonBlankData(fp_Run *pCurrentRun, fp_R
 	}
 	while(pCurrentRun);
 
-
 	*pOffendingRun = pCurrentRun;
 
 	return iTrailingBlank;
