@@ -28,11 +28,12 @@
 #include "ut_types.h"
 #include "ut_vector.h"
 #include "ut_growbuf.h"
-#include "xmlparse.h"
+#include "ut_xml.h"
 #include "pt_Types.h"
 #include "fl_Layout.h"
 #include "fl_DocLayout.h"		// FIXME: this is needed for the friend'ed function
 #include "fg_Graphic.h"
+#include "fl_AutoLists.h"
 
 // number of DocPositions occupied by the block strux
 #define fl_BLOCK_STRUX_OFFSET	1
@@ -55,6 +56,29 @@ class PX_ChangeRecord_SpanChange;
 class PX_ChangeRecord_Strux;
 class PX_ChangeRecord_StruxChange;
 class fl_PartOfBlock;
+class fl_AutoNum;
+
+// Tab types and leaders
+typedef enum {
+	FL_TAB_NONE = 0,
+	FL_TAB_LEFT,
+	FL_TAB_CENTER,
+	FL_TAB_RIGHT,
+	FL_TAB_DECIMAL,
+	FL_TAB_BAR,
+	__FL_TAB_MAX
+} eTabType;
+
+typedef enum {
+	FL_LEADER_NONE = 0,
+	FL_LEADER_DOT,
+	FL_LEADER_HYPHEN,
+	FL_LEADER_UNDERLINE,
+	FL_LEADER_THICKLINE,
+	FL_LEADER_EQUALSIGN,
+	__FL_LEADER_MAX
+} eTabLeader;
+
 
 class fl_CharWidths
 {
@@ -115,6 +139,9 @@ class fl_CharWidths
 	the normal flow, in order.
 */
 
+class fl_TabStop;
+void buildTabStops(GR_Graphics * pG, const char* pszTabStops, UT_Vector &m_vecTabs);
+
 class fl_BlockLayout : public fl_Layout
 {
 	friend class fl_DocListener;
@@ -140,8 +167,13 @@ public:
 	void		redrawUpdate();
 
 	fp_Line*	getNewLine(void);
+	FV_View *       getView(void);
 
-	const char*	getProperty(const XML_Char * pszName, UT_Bool bExpandStyles=UT_TRUE) const;
+// TODO: What I want to test is XML_Char != char
+#ifdef HAVE_LIBXML2
+	const char*	getProperty(const char * pszName, UT_Bool bExpandStyles = UT_TRUE) const;
+#endif
+	const char*	getProperty(const XML_Char * pszName, UT_Bool bExpandStyles = UT_TRUE) const;
 	void setAlignment(UT_uint32 iAlignCmd);
 
 	inline fl_BlockLayout* getNext(void) const { return m_pNext; }
@@ -161,6 +193,38 @@ public:
 
 	inline fp_Run* getFirstRun(void) const { return m_pFirstRun; }
 
+	inline UT_Bool isListItem(void) const { return m_bListItem; }
+	UT_Bool isFirstInList(void);
+//	inline fl_AutoNum * getAutoNum(void) const { return m_pAutoNum; }
+	void    getListAttributesVector( UT_Vector * va);
+	void  getListPropertyVector( UT_Vector * vp);
+
+	char *  getFormatFromListType(List_Type iListType);
+	void remItemFromList(void);
+	virtual void listUpdate(void); 
+	void resumeList( fl_BlockLayout * prevList);
+	void prependList( fl_BlockLayout * nextList);
+        List_Type decodeListType(char * listformat);
+	List_Type getListType(void);
+	XML_Char* getListStyleString( List_Type iListType);
+	List_Type getListTypeFromStyle( const XML_Char * style);
+	fl_BlockLayout * getNextList(UT_uint32 id);
+	UT_Bool isListLabelInBlock(void); 
+	void StartList( const XML_Char * style);
+
+	void StartList( List_Type lType, UT_uint32 start,const XML_Char * lDelim, const XML_Char * lDecimal, const XML_Char * fFont, float Align, float indent, UT_uint32 iParentID = 0, UT_uint32 level=0 );
+
+	void StopList(void);
+	void deleteListLabel(void);
+	XML_Char * getListLabel(void);
+	void transferListFlags(void);
+	UT_uint32 getLevel(void);
+	void setStarting( UT_Bool bValue);
+	void setStopping( UT_Bool bValue);
+        fl_BlockLayout * getPreviousList(UT_uint32 id);
+        fl_BlockLayout * getPreviousList(void);
+	inline fl_BlockLayout * getParentItem(void);
+	
 	void findSquigglesForRun(fp_Run* pRun);
 	UT_uint32 canSlurp(fp_Line* pLine) const;
 
@@ -203,8 +267,15 @@ public:
 	void checkForEndOnForcedBreak(void);
 
 	void checkSpelling(void);
-	UT_Bool	findNextTabStop(UT_sint32 iStartX, UT_sint32 iMaxX, UT_sint32& iPosition, unsigned char& iType);
-	UT_Bool	findNextTabStopInLayoutUnits(UT_sint32 iStartX, UT_sint32 iMaxX, UT_sint32& iPosition, unsigned char& iType);
+	void debugFlashing(void);
+ 	UT_Bool	findNextTabStop(UT_sint32 iStartX, UT_sint32 iMaxX, 
+				UT_sint32& iPosition, eTabType& iType, 
+				eTabLeader &iLeader );
+ 	UT_Bool	findNextTabStopInLayoutUnits(UT_sint32 iStartX, UT_sint32 iMaxX,
+					     UT_sint32& iPosition, 
+					     eTabType& iType, 
+					     eTabLeader &iLeader);
+
 	inline UT_sint32 getDefaultTabInterval(void) const { return m_iDefaultTabInterval; }
 	inline UT_sint32 getTabsCount(void) const { return (UT_sint32) m_vecTabs.getItemCount(); }
 
@@ -256,8 +327,16 @@ public:
 	fl_PartOfBlock*			getSquiggle(UT_uint32 iOffset) const;
 	void					recheckIgnoredWords();
 
-	static UT_Bool			s_EnumTabStops(void * myThis, UT_uint32 k, UT_sint32 & iPosition, unsigned char & iType, UT_uint32 & iOffset);
+	static UT_Bool			s_EnumTabStops(void * myThis, UT_uint32 k, fl_TabStop *pTabInfo);
 	
+	inline void			addBackgroundCheckReason(UT_uint32 reason) {m_uBackgroundCheckReasons |= reason;}
+	inline void			removeBackgroundCheckReason(UT_uint32 reason) {m_uBackgroundCheckReasons &= ~reason;}
+	inline UT_Bool		hasBackgroundCheckReason(UT_uint32 reason) const {return (m_uBackgroundCheckReasons & reason);}
+
+	// The following is a set of bit flags giving the reason this block is
+	// queued for background checking.  See specific values in fl_DocLayout.h
+	UT_uint32				m_uBackgroundCheckReasons;
+
 #ifdef FMT_TEST
 	void					__dump(FILE * fp) const;
 #endif
@@ -275,6 +354,8 @@ protected:
 
 	UT_Bool					_doInsertTextSpan(PT_BlockOffset blockOffset, UT_uint32 len);
 	UT_Bool					_doInsertForcedLineBreakRun(PT_BlockOffset blockOffset);
+	UT_Bool					_doInsertFieldStartRun(PT_BlockOffset blockOffset);
+	UT_Bool					_doInsertFieldEndRun(PT_BlockOffset blockOffset);
 	UT_Bool					_doInsertForcedColumnBreakRun(PT_BlockOffset blockOffset);
 	UT_Bool					_doInsertForcedPageBreakRun(PT_BlockOffset blockOffset);
 	UT_Bool					_doInsertTabRun(PT_BlockOffset blockOffset);
@@ -315,10 +396,16 @@ protected:
 
 	static void				_prefsListener(XAP_App * /*pApp*/, XAP_Prefs *pPrefs, UT_AlphaHashTable * /*phChanges*/, void * data);
 
+	void					_createListLabel(void);
+	void					_deleteListLabel(void);
+	inline void				_addBlockToPrevList( fl_BlockLayout * prevBlockInList, UT_uint32 level);	
+	inline void				_prependBlockToPrevList( fl_BlockLayout * nextBlockInList);	
+
 	UT_Bool					m_bNeedsReformat;
 	UT_Bool					m_bNeedsRedraw;
 	UT_Bool					m_bFixCharWidths;
-	
+        UT_Bool                                 m_bCursorErased;
+
 	fl_CharWidths			m_gbCharWidths;
 
 	FL_DocLayout*	       	m_pLayout;
@@ -359,14 +446,21 @@ protected:
 	UT_Bool					m_bKeepWithNext;
 	const XML_Char *		m_szStyle;
 
+	//fl_AutoNum *			m_pAutoNum;
+	UT_Bool					m_bListItem;
+	UT_Bool					m_bStartList;
+	UT_Bool					m_bStopList;
+	UT_Bool					m_bListLabelCreated;
+
 	// spell check stuff
 	UT_Vector				m_vecSquiggles;
+
 };
 
 /*
 	This class is used to represent a part of the block.  Pointers
 	to this class are the things contained in m_vecSquiggles and in 
-	FL_DocLayout::m_pPendingWord.
+	FL_DocLayout::m_pPendingWordForSpell
 */
 class fl_PartOfBlock
 {
@@ -383,22 +477,46 @@ public:
 protected:
 };
 
-// TODO make a typedef to type type rather than just using 'unsigned char'
-
-#define FL_TAB_LEFT				1
-#define FL_TAB_CENTER			2
-#define FL_TAB_RIGHT			3
-#define FL_TAB_DECIMAL			4
-#define FL_TAB_BAR				5
-
-struct fl_TabStop
+class fl_TabStop
 {
+public:
+	
 	fl_TabStop();
 	
+	UT_sint32		getPosition() { return iPosition;}
+	void			setPosition(UT_sint32 value) { iPosition = value;}
+	UT_sint32		getPositionLayoutUnits() { return iPositionLayoutUnits;}
+	void			setPositionLayoutUnits(UT_sint32 value) { iPositionLayoutUnits = value;}
+	eTabType		getType() { return iType;}
+	void			setType(eTabType type) { iType = type;}
+	eTabLeader		getLeader() { return iLeader;};
+	void			setLeader(eTabLeader leader) { iLeader = leader;}
+	UT_uint32		getOffset() { return iOffset;}
+	void			setOffset(UT_uint32 value) { iOffset = value;}
+
+	void operator = (const fl_TabStop &Other)
+	{
+		iPosition = Other.iPosition;
+		iPositionLayoutUnits = Other.iPositionLayoutUnits;
+		iType = Other.iType;
+		iLeader = Other.iLeader;
+		iOffset = Other.iOffset;
+	}
+
+protected:
+
 	UT_sint32		iPosition;
 	UT_sint32		iPositionLayoutUnits;
-	unsigned char	iType;
+	eTabType		iType;
+	eTabLeader		iLeader;
 	UT_uint32		iOffset;
 };
 
 #endif /* FL_BLOCKLAYOUT_H */
+
+
+
+
+
+
+
