@@ -82,13 +82,13 @@ fl_SectionLayout::~fl_SectionLayout()
 	}
 }
 
-const char*	fl_SectionLayout::getAttribute(const XML_Char * pszName) const
+const char*	fl_SectionLayout::getAttribute(const char * pszName) const
 {
 	const PP_AttrProp * pAP = NULL;
 	getAttrProp(&pAP);
 	
 	const XML_Char* pszAtt = NULL;
-	pAP->getAttribute(pszName, pszAtt);
+	pAP->getAttribute((XML_Char*)pszName, pszAtt);
 
 	return pszAtt;
 }
@@ -159,7 +159,6 @@ void fl_SectionLayout::addBlock(fl_BlockLayout* pBL)
 		m_pFirstBlock = pBL;
 		m_pLastBlock = m_pFirstBlock;
 	}
-
 	pBL->setSectionLayout(this);
 }
 
@@ -202,6 +201,7 @@ void fl_SectionLayout::removeBlock(fl_BlockLayout * pBL)
 	if (pBL->getNext())
 	{
 		pBL->getNext()->setPrev(pBL->getPrev());
+		pBL->transferListFlags();
 	}
 	
 	if (pBL == m_pFirstBlock)
@@ -243,13 +243,13 @@ fb_LineBreaker * fl_SectionLayout::_getLineBreaker(void)
 
 void fl_SectionLayout::_purgeLayout()
 {
-	fl_BlockLayout*	pBL = m_pFirstBlock;
+	fl_BlockLayout*	pBL = m_pLastBlock;
 
 	while (pBL)
 	{
 		fl_BlockLayout* pNuke = pBL;
 
-		pBL = pBL->getNext();
+		pBL = pBL->getPrev();
 
 		delete pNuke;
 	}
@@ -259,6 +259,13 @@ void fl_SectionLayout::_purgeLayout()
 
 UT_Bool fl_SectionLayout::bl_doclistener_populateSpan(fl_BlockLayout* pBL, const PX_ChangeRecord_Span * pcrs, PT_BlockOffset blockOffset, UT_uint32 len)
 {
+        if(pBL->getPrev()!= NULL && pBL->getPrev()->getLastLine()==NULL)
+        {
+	        UT_DEBUGMSG(("In bl_doclistner_pop no LastLine \n"));
+	        UT_DEBUGMSG(("getPrev = %d this = %d \n",pBL->getPrev(),pBL));
+		//  UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+	}
+
 	return pBL->doclistener_populateSpan(pcrs, blockOffset, len);
 }
 
@@ -361,7 +368,7 @@ UT_Bool fl_SectionLayout::bl_doclistener_changeFmtMark(fl_BlockLayout* pBL, cons
 #ifdef FMT_TEST
 void fl_SectionLayout::__dump(FILE * fp) const
 {
-	fprintf(fp,"Section: 0x%p [type %d]\n",this,getType());
+	fprintf(fp,"Section: 0x%p [type %d]\n",(void*)this,getType());
 	for (fl_BlockLayout * pBL=getFirstBlock(); (pBL); pBL=pBL->getNext())
 		pBL->__dump(fp);
 }
@@ -572,6 +579,18 @@ void fl_DocSectionLayout::format(void)
 	while (pBL)
 	{
 		pBL->format();
+		UT_sint32 count = 0;
+		while(pBL->getLastLine() == NULL || pBL->getFirstLine()==NULL)
+		{
+		       UT_DEBUGMSG(("Error formatting a block try again \n"));
+		       count = count + 1;
+		       pBL->format();
+		       if(count > 3)
+		       {
+		              UT_DEBUGMSG(("Give up trying to format. Hope for the best :-( \n"));
+		              break;
+		       }
+		}
 		pBL = pBL->getNext();
 	}
 
@@ -652,6 +671,20 @@ UT_Bool fl_DocSectionLayout::doclistener_changeStrux(const PX_ChangeRecord_Strux
 {
 	UT_ASSERT(pcrxc->getType()==PX_ChangeRecord::PXT_ChangeStrux);
 
+	// Sevior code here
+
+	PT_AttrPropIndex indexAP = pcrxc->getIndexAP();
+	const PP_AttrProp* pAP = NULL;
+			
+	UT_Bool bres = (m_pDoc->getAttrProp(indexAP, &pAP) && pAP);
+	UT_ASSERT(bres);
+	
+	const XML_Char* pszSectionType = NULL;
+	pAP->getAttribute((XML_Char*)"type", pszSectionType);
+	UT_DEBUGMSG(("SEVIOR: In sectionlayout change_strux:, type= %s \n",pszSectionType)); 
+
+	// end sevior code here
+
 	setAttrPropIndex(pcrxc->getIndexAP());
 
 	_lookupProperties();
@@ -733,7 +766,7 @@ void fl_DocSectionLayout::_lookupProperties(void)
 	*/
 	
 	const char* pszNumColumns = NULL;
-	pSectionAP->getProperty("columns", (const XML_Char *&)pszNumColumns);
+	pSectionAP->getProperty((XML_Char*)"columns", (const XML_Char *&)pszNumColumns);
 	if (pszNumColumns && pszNumColumns[0])
 	{
 		m_iNumColumns = atoi(pszNumColumns);
@@ -744,7 +777,7 @@ void fl_DocSectionLayout::_lookupProperties(void)
 	}
 
 	const char* pszColumnGap = NULL;
-	pSectionAP->getProperty("column-gap", (const XML_Char *&)pszColumnGap);
+	pSectionAP->getProperty((XML_Char*)"column-gap", (const XML_Char *&)pszColumnGap);
 	if (pszColumnGap && pszColumnGap[0])
 	{
 		m_iColumnGap = m_pLayout->getGraphics()->convertDimension(pszColumnGap);
@@ -756,8 +789,19 @@ void fl_DocSectionLayout::_lookupProperties(void)
 		m_iColumnGapLayoutUnits = UT_convertToLayoutUnits("0.25in");
 	}
 
+	const char* pszColumnLineBetween = NULL;
+	pSectionAP->getProperty((XML_Char*)"column-line", (const XML_Char *&)pszColumnLineBetween);
+	if (pszColumnLineBetween && pszColumnLineBetween[0])
+	{
+		m_bColumnLineBetween = (strcmp(pszColumnLineBetween, "on") == 0) ? UT_TRUE : UT_FALSE;
+	}
+	else
+	{
+		m_bColumnLineBetween = UT_FALSE;
+	}
+
 	const char* pszSpaceAfter = NULL;
-	pSectionAP->getProperty("section-space-after", (const XML_Char *&)pszSpaceAfter);
+	pSectionAP->getProperty((XML_Char*)"section-space-after", (const XML_Char *&)pszSpaceAfter);
 	if (pszSpaceAfter && pszSpaceAfter[0])
 	{
 		m_iSpaceAfter = m_pLayout->getGraphics()->convertDimension(pszSpaceAfter);
@@ -773,10 +817,10 @@ void fl_DocSectionLayout::_lookupProperties(void)
 	const char* pszTopMargin = NULL;
 	const char* pszRightMargin = NULL;
 	const char* pszBottomMargin = NULL;
-	pSectionAP->getProperty("page-margin-left", (const XML_Char *&)pszLeftMargin);
-	pSectionAP->getProperty("page-margin-top", (const XML_Char *&)pszTopMargin);
-	pSectionAP->getProperty("page-margin-right", (const XML_Char *&)pszRightMargin);
-	pSectionAP->getProperty("page-margin-bottom", (const XML_Char *&)pszBottomMargin);
+	pSectionAP->getProperty((XML_Char*)"page-margin-left", (const XML_Char *&)pszLeftMargin);
+	pSectionAP->getProperty((XML_Char*)"page-margin-top", (const XML_Char *&)pszTopMargin);
+	pSectionAP->getProperty((XML_Char*)"page-margin-right", (const XML_Char *&)pszRightMargin);
+	pSectionAP->getProperty((XML_Char*)"page-margin-bottom", (const XML_Char *&)pszBottomMargin);
 
 
 	const XML_Char * szRulerUnits;
@@ -805,7 +849,16 @@ void fl_DocSectionLayout::_lookupProperties(void)
 		strcpy(defaultMargin, "72.0pt");
 		break;
 
+		// TODO: MM, PX, and PERCENT
+		// let them fall through to the default now
+		// and we don't use them anyway
+#if 0
+	case DIM_MM:
+	case DIM_PX:
+	case DIM_PERCENT:
+#endif
 	case DIM_none:
+	default:
 		strcpy(defaultMargin, "1.0in");	// TODO: what to do with this.
 		break;
 
@@ -1097,7 +1150,6 @@ UT_sint32 fl_DocSectionLayout::breakSection(void)
 	fp_Line* pCurrentLine = pFirstBlock->getFirstLine();
 		
 	fp_Column* pCurColumn = (fp_Column*) getFirstContainer();
-	UT_ASSERT(pCurColumn);
 
 	while (pCurColumn)
 	{
@@ -1390,7 +1442,7 @@ UT_Bool fl_HdrFtrSectionLayout::recalculateFields(void)
 	for (UT_uint32 i=0; i<iCount; i++)
 	{
 		struct _PageHdrFtrShadowPair* pPair = (struct _PageHdrFtrShadowPair*) m_vecPages.getNthItem(i);
-
+		UT_ASSERT(pPair->pShadow);
 		bResult = pPair->pShadow->recalculateFields() || bResult;
 	}
 
@@ -1399,16 +1451,18 @@ UT_Bool fl_HdrFtrSectionLayout::recalculateFields(void)
 
 fp_Container* fl_HdrFtrSectionLayout::getFirstContainer()
 {
-	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
-
-	return NULL;
+	UT_uint32 iCount = m_vecPages.getItemCount();
+	UT_ASSERT(iCount);
+	struct _PageHdrFtrShadowPair* pPair = (struct _PageHdrFtrShadowPair*) m_vecPages.getNthItem(0);
+	return pPair->pShadow->getFirstContainer();
 }
 
 fp_Container* fl_HdrFtrSectionLayout::getLastContainer()
 {
-	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
-
-	return NULL;
+	UT_uint32 iCount = m_vecPages.getItemCount();
+	UT_ASSERT(iCount);
+	struct _PageHdrFtrShadowPair* pPair = (struct _PageHdrFtrShadowPair*) m_vecPages.getNthItem(iCount-1);
+	return pPair->pShadow->getFirstContainer();
 }
 
 fp_Container* fl_HdrFtrSectionLayout::getNewContainer(void)
@@ -1433,6 +1487,112 @@ UT_sint32 fl_HdrFtrSectionLayout::_findShadow(fp_Page* pPage)
 
 	return -1;
 }
+//
+// This method converts a previously existing section to this header/footer.
+// Code liberally stolen from fl_DocSectionLayout::doclistener_deleteStrux
+//
+void fl_HdrFtrSectionLayout::changeStrux( fl_DocSectionLayout * pSL)
+{
+        fl_DocSectionLayout* pPrevSL = pSL->getPrevDocSection();
+	UT_ASSERT(pPrevSL);
+	// clear all the columns
+	fp_Column* pCol =NULL;
+
+	pCol = (fp_Column *) pSL->getFirstContainer();
+	while (pCol)
+	{
+		pCol->clearScreen();
+
+		pCol = pCol->getNext();
+	}
+
+	// remove all the columns from their pages
+	pCol = (fp_Column *) pSL->getFirstContainer();
+	while (pCol)
+	{
+		if (pCol->getLeader() == pCol)
+		{
+			pCol->getPage()->removeColumnLeader(pCol);
+		}
+
+		pCol = pCol->getNext();
+	}
+
+
+	// get rid of all the layout information for every block
+	fl_BlockLayout*	pBL = pSL->getFirstBlock();
+	while (pBL)
+	{
+		pBL->collapse();
+
+		pBL = pBL->getNext();
+	}
+
+	// transfer the Sections' blocks into this header/footer
+
+	while (pSL->getFirstBlock())
+	{
+		pBL = pSL->getFirstBlock();
+		pSL->removeBlock(pBL);
+		addBlock(pBL);
+	}
+	
+	pPrevSL->setNext(pSL->getNext());
+							
+	if (pSL->getNext())
+	{
+		pSL->getNext()->setPrev(pPrevSL);
+	}
+
+	m_pLayout->removeSection(pSL);
+	DELETEP(pSL); // Old Section layout is totally gone
+
+	// OK now add all the pages this header/footer is associated with
+	// We have to extract this information from m_pDocSL
+	// Loop through all the columns in m_pDocSl and find the pages owned
+	// by m_pDocSL
+	//
+
+	pCol = (fp_Column *) m_pDocSL->getFirstContainer();
+	fp_Page * pOldPage = NULL;
+	fp_Page * pNewPage = NULL;
+	while(pCol)
+	{
+	       pNewPage = pCol->getPage();
+	       if(pNewPage != NULL && pNewPage != pOldPage)
+	       {
+		      fl_DocSectionLayout* pDocSec = pNewPage->getOwningSection();
+		      if(pDocSec == m_pDocSL && _findShadow(pNewPage) < 0)
+		      {
+			      addPage(pNewPage);
+		      }
+	       }
+	       pCol = pCol->getNext();
+	}
+	format();
+
+	UT_uint32 iCount = m_vecPages.getItemCount();
+	for (UT_uint32 i=0; i<iCount; i++)
+	  {
+	    struct _PageHdrFtrShadowPair* pPair = (struct _PageHdrFtrShadowPair*) m_vecPages.getNthItem(i);
+	    
+	    UT_DEBUGMSG(("SEVIOR: for i = %d pPair->pShadow = %x pPair->pPage = %x \n",i,pPair->pShadow,pPair->pPage));
+	  }
+
+	// OK set the insertion point at the beginning of the last block
+	pBL = getLastBlock();
+	//m_pLayout->updateLayout();
+	UT_ASSERT(pBL);
+
+	FV_View* pView = m_pLayout->getView();
+	if (pView)
+	{
+	        UT_uint32 pos = (UT_uint32) pBL->getPosition(UT_TRUE);
+		pView->setPoint(pos);
+	}
+
+	// Finished! we now have a header/footer
+}
 
 void fl_HdrFtrSectionLayout::addPage(fp_Page* pPage)
 {
@@ -1446,9 +1606,12 @@ void fl_HdrFtrSectionLayout::addPage(fp_Page* pPage)
 	
 	fl_ShadowListener* pShadowListener = new fl_ShadowListener(this, pPair->pShadow);
 	m_pDoc->tellListener(pShadowListener);
+
 	delete pShadowListener;
-	
 	m_vecPages.addItem(pPair);
+
+	UT_DEBUGMSG(("SEVIOR: Added page number %x page = %x \n",m_vecPages.getItemCount(),pPage));
+
 }
 
 void fl_HdrFtrSectionLayout::deletePage(fp_Page* pPage)
@@ -1469,6 +1632,17 @@ void fl_HdrFtrSectionLayout::deletePage(fp_Page* pPage)
 
 void fl_HdrFtrSectionLayout::format(void)
 {
+  //
+  // Format Blocks here, then blocks in the shadowlayouts
+  // we have to Maintain this overall layout structure plus the underlying
+  // shadow structure if we want to be able to edit headers/footers.
+  // 
+        fl_BlockLayout * pBL = m_pFirstBlock;
+	while(pBL)
+	{
+	        pBL->format();
+		pBL = pBL->getNext();
+  	}
 	UT_uint32 iCount = m_vecPages.getItemCount();
 	for (UT_uint32 i=0; i<iCount; i++)
 	{
@@ -1480,7 +1654,20 @@ void fl_HdrFtrSectionLayout::format(void)
 
 void fl_HdrFtrSectionLayout::updateLayout(void)
 {
-	UT_uint32 iCount = m_vecPages.getItemCount();
+  //
+  // update Blocks here, then blocks in the shadowlayouts
+  //
+	fl_BlockLayout*	pBL = m_pFirstBlock;
+	while (pBL)
+	{
+		if (pBL->needsReformat())
+		{
+			pBL->format();
+		}
+		
+		pBL = pBL->getNext();
+	}
+  	UT_uint32 iCount = m_vecPages.getItemCount();
 	for (UT_uint32 i=0; i<iCount; i++)
 	{
 		struct _PageHdrFtrShadowPair* pPair = (struct _PageHdrFtrShadowPair*) m_vecPages.getNthItem(i);
@@ -1491,6 +1678,9 @@ void fl_HdrFtrSectionLayout::updateLayout(void)
 
 void fl_HdrFtrSectionLayout::redrawUpdate(void)
 {
+  //
+  // Don't need to draw here since this is never displayed on the screen?
+  // 
 	UT_uint32 iCount = m_vecPages.getItemCount();
 	for (UT_uint32 i=0; i<iCount; i++)
 	{
@@ -1521,105 +1711,133 @@ void fl_HdrFtrSectionLayout::_lookupProperties(void)
 UT_Bool fl_HdrFtrSectionLayout::bl_doclistener_populateSpan(fl_BlockLayout* pBL, const PX_ChangeRecord_Span * pcrs, PT_BlockOffset blockOffset, UT_uint32 len)
 {
 	UT_Bool bResult = UT_TRUE;
+	fl_BlockLayout * pShadowBL = NULL;
 	UT_uint32 iCount = m_vecPages.getItemCount();
 	for (UT_uint32 i=0; i<iCount; i++)
 	{
 		struct _PageHdrFtrShadowPair* pPair = (struct _PageHdrFtrShadowPair*) m_vecPages.getNthItem(i);
-
-		bResult = pPair->pShadow->bl_doclistener_populateSpan(pBL, pcrs, blockOffset, len)
+		// Find matching block in this shadow.
+		pShadowBL = pPair->pShadow->findMatchingBlock(pBL);
+		bResult = pShadowBL->doclistener_populateSpan(pcrs,blockOffset,len)
 			&& bResult;
 	}
-
+	// Update the overall block too.
+        bResult = pBL->doclistener_populateSpan(pcrs,blockOffset,len)
+			&& bResult;
 	return bResult;
 }
 
 UT_Bool fl_HdrFtrSectionLayout::bl_doclistener_populateObject(fl_BlockLayout* pBL, PT_BlockOffset blockOffset, const PX_ChangeRecord_Object * pcro)
 {
 	UT_Bool bResult = UT_TRUE;
+	fl_BlockLayout * pShadowBL = NULL;
 	UT_uint32 iCount = m_vecPages.getItemCount();
 	for (UT_uint32 i=0; i<iCount; i++)
 	{
 		struct _PageHdrFtrShadowPair* pPair = (struct _PageHdrFtrShadowPair*) m_vecPages.getNthItem(i);
-
-		bResult = pPair->pShadow->bl_doclistener_populateObject(pBL, blockOffset, pcro)
+		// Find matching block in this shadow.
+		pShadowBL = pPair->pShadow->findMatchingBlock(pBL);
+		bResult = pShadowBL->doclistener_populateObject(blockOffset,pcro)
 			&& bResult;
 	}
-
+	// Update the overall block too.
+        bResult = pBL->doclistener_populateObject(blockOffset,pcro)
+			&& bResult;
 	return bResult;
 }
 	
 UT_Bool fl_HdrFtrSectionLayout::bl_doclistener_insertSpan(fl_BlockLayout* pBL, const PX_ChangeRecord_Span * pcrs)
 {
 	UT_Bool bResult = UT_TRUE;
+	fl_BlockLayout * pShadowBL = NULL;
 	UT_uint32 iCount = m_vecPages.getItemCount();
 	for (UT_uint32 i=0; i<iCount; i++)
 	{
 		struct _PageHdrFtrShadowPair* pPair = (struct _PageHdrFtrShadowPair*) m_vecPages.getNthItem(i);
-
-		bResult = pPair->pShadow->bl_doclistener_insertSpan(pBL, pcrs)
+		// Find matching block in this shadow.
+		pShadowBL = pPair->pShadow->findMatchingBlock(pBL);
+		bResult = pShadowBL->doclistener_insertSpan(pcrs)
 			&& bResult;
 	}
-
+	// Update the overall block too.
+        bResult = pBL->doclistener_insertSpan(pcrs)
+			&& bResult;
 	return bResult;
 }
 
 UT_Bool fl_HdrFtrSectionLayout::bl_doclistener_deleteSpan(fl_BlockLayout* pBL, const PX_ChangeRecord_Span * pcrs)
 {
 	UT_Bool bResult = UT_TRUE;
+	fl_BlockLayout * pShadowBL = NULL;
 	UT_uint32 iCount = m_vecPages.getItemCount();
 	for (UT_uint32 i=0; i<iCount; i++)
 	{
 		struct _PageHdrFtrShadowPair* pPair = (struct _PageHdrFtrShadowPair*) m_vecPages.getNthItem(i);
-
-		bResult = pPair->pShadow->bl_doclistener_deleteSpan(pBL, pcrs)
+		// Find matching block in this shadow.
+		pShadowBL = pPair->pShadow->findMatchingBlock(pBL);
+		bResult = pShadowBL->doclistener_deleteSpan(pcrs)
 			&& bResult;
 	}
-
+	// Update the overall block too.
+        bResult = pBL->doclistener_deleteSpan(pcrs)
+			&& bResult;
 	return bResult;
 }
 
 UT_Bool fl_HdrFtrSectionLayout::bl_doclistener_changeSpan(fl_BlockLayout* pBL, const PX_ChangeRecord_SpanChange * pcrsc)
 {
 	UT_Bool bResult = UT_TRUE;
+	fl_BlockLayout * pShadowBL = NULL;
 	UT_uint32 iCount = m_vecPages.getItemCount();
 	for (UT_uint32 i=0; i<iCount; i++)
 	{
 		struct _PageHdrFtrShadowPair* pPair = (struct _PageHdrFtrShadowPair*) m_vecPages.getNthItem(i);
-
-		bResult = pPair->pShadow->bl_doclistener_changeSpan(pBL, pcrsc)
+		// Find matching block in this shadow.
+		pShadowBL = pPair->pShadow->findMatchingBlock(pBL);
+		bResult = pShadowBL->doclistener_changeSpan(pcrsc)
 			&& bResult;
 	}
-
+	// Update the overall block too.
+        bResult = pBL->doclistener_changeSpan(pcrsc)
+			&& bResult;
 	return bResult;
 }
 
 UT_Bool fl_HdrFtrSectionLayout::bl_doclistener_deleteStrux(fl_BlockLayout* pBL, const PX_ChangeRecord_Strux * pcrx)
 {
 	UT_Bool bResult = UT_TRUE;
+	fl_BlockLayout * pShadowBL = NULL;
 	UT_uint32 iCount = m_vecPages.getItemCount();
 	for (UT_uint32 i=0; i<iCount; i++)
 	{
 		struct _PageHdrFtrShadowPair* pPair = (struct _PageHdrFtrShadowPair*) m_vecPages.getNthItem(i);
-
-		bResult = pPair->pShadow->bl_doclistener_deleteStrux(pBL, pcrx)
+		// Find matching block in this shadow.
+		pShadowBL = pPair->pShadow->findMatchingBlock(pBL);
+		bResult = pShadowBL->doclistener_deleteStrux(pcrx)
 			&& bResult;
 	}
-
+	// Update the overall block too.
+        bResult = pBL->doclistener_deleteStrux(pcrx)
+			&& bResult;
 	return bResult;
 }
 
 UT_Bool fl_HdrFtrSectionLayout::bl_doclistener_changeStrux(fl_BlockLayout* pBL, const PX_ChangeRecord_StruxChange * pcrxc)
 {
 	UT_Bool bResult = UT_TRUE;
+	fl_BlockLayout * pShadowBL = NULL;
 	UT_uint32 iCount = m_vecPages.getItemCount();
 	for (UT_uint32 i=0; i<iCount; i++)
 	{
 		struct _PageHdrFtrShadowPair* pPair = (struct _PageHdrFtrShadowPair*) m_vecPages.getNthItem(i);
-
-		bResult = pPair->pShadow->bl_doclistener_changeStrux(pBL, pcrxc)
+		// Find matching block in this shadow.
+		pShadowBL = pPair->pShadow->findMatchingBlock(pBL);
+		bResult = pShadowBL->doclistener_changeStrux(pcrxc)
 			&& bResult;
 	}
-
+	// Update the overall block too.
+        bResult = pBL->doclistener_changeStrux(pcrxc)
+			&& bResult;
 	return bResult;
 }
 
@@ -1631,15 +1849,19 @@ UT_Bool fl_HdrFtrSectionLayout::bl_doclistener_insertBlock(fl_BlockLayout* pBL,
 															PL_StruxFmtHandle sfhNew))
 {
 	UT_Bool bResult = UT_TRUE;
+	fl_BlockLayout * pShadowBL = NULL;
 	UT_uint32 iCount = m_vecPages.getItemCount();
 	for (UT_uint32 i=0; i<iCount; i++)
 	{
 		struct _PageHdrFtrShadowPair* pPair = (struct _PageHdrFtrShadowPair*) m_vecPages.getNthItem(i);
-
-		bResult = pPair->pShadow->bl_doclistener_insertBlock(pBL, pcrx, sdh, lid, pfnBindHandles)
+		// Find matching block in this shadow.
+		pShadowBL = pPair->pShadow->findMatchingBlock(pBL);
+		bResult = pShadowBL->doclistener_insertBlock(pcrx,sdh,lid,pfnBindHandles)
 			&& bResult;
 	}
-
+	// Update the overall block too.
+        bResult = pBL->doclistener_insertBlock(pcrx,sdh,lid,pfnBindHandles)
+			&& bResult;
 	return bResult;
 }
 
@@ -1668,90 +1890,114 @@ UT_Bool fl_HdrFtrSectionLayout::bl_doclistener_insertSection(fl_BlockLayout* pBL
 UT_Bool fl_HdrFtrSectionLayout::bl_doclistener_insertObject(fl_BlockLayout* pBL, const PX_ChangeRecord_Object * pcro)
 {
 	UT_Bool bResult = UT_TRUE;
+	fl_BlockLayout * pShadowBL = NULL;
 	UT_uint32 iCount = m_vecPages.getItemCount();
 	for (UT_uint32 i=0; i<iCount; i++)
 	{
 		struct _PageHdrFtrShadowPair* pPair = (struct _PageHdrFtrShadowPair*) m_vecPages.getNthItem(i);
-
-		bResult = pPair->pShadow->bl_doclistener_insertObject(pBL, pcro)
+		// Find matching block in this shadow.
+		pShadowBL = pPair->pShadow->findMatchingBlock(pBL);
+		bResult = pShadowBL->doclistener_insertObject(pcro)
 			&& bResult;
 	}
-
+	// Update the overall block too.
+        bResult = pBL->doclistener_insertObject(pcro)
+			&& bResult;
 	return bResult;
 }
 
 UT_Bool fl_HdrFtrSectionLayout::bl_doclistener_deleteObject(fl_BlockLayout* pBL, const PX_ChangeRecord_Object * pcro)
 {
 	UT_Bool bResult = UT_TRUE;
+	fl_BlockLayout * pShadowBL = NULL;
 	UT_uint32 iCount = m_vecPages.getItemCount();
 	for (UT_uint32 i=0; i<iCount; i++)
 	{
 		struct _PageHdrFtrShadowPair* pPair = (struct _PageHdrFtrShadowPair*) m_vecPages.getNthItem(i);
-
-		bResult = pPair->pShadow->bl_doclistener_deleteObject(pBL, pcro)
+		// Find matching block in this shadow.
+		pShadowBL = pPair->pShadow->findMatchingBlock(pBL);
+		bResult = pShadowBL->doclistener_deleteObject(pcro)
 			&& bResult;
 	}
-
+	// Update the overall block too.
+        bResult = pBL->doclistener_deleteObject(pcro)
+			&& bResult;
 	return bResult;
 }
 
 UT_Bool fl_HdrFtrSectionLayout::bl_doclistener_changeObject(fl_BlockLayout* pBL, const PX_ChangeRecord_ObjectChange * pcroc)
 {
 	UT_Bool bResult = UT_TRUE;
+	fl_BlockLayout * pShadowBL = NULL;
 	UT_uint32 iCount = m_vecPages.getItemCount();
 	for (UT_uint32 i=0; i<iCount; i++)
 	{
 		struct _PageHdrFtrShadowPair* pPair = (struct _PageHdrFtrShadowPair*) m_vecPages.getNthItem(i);
-
-		bResult = pPair->pShadow->bl_doclistener_changeObject(pBL, pcroc)
+		// Find matching block in this shadow.
+		pShadowBL = pPair->pShadow->findMatchingBlock(pBL);
+		bResult = pShadowBL->doclistener_changeObject(pcroc)
 			&& bResult;
 	}
-
+	// Update the overall block too.
+        bResult = pBL->doclistener_changeObject(pcroc)
+			&& bResult;
 	return bResult;
 }
 
 UT_Bool fl_HdrFtrSectionLayout::bl_doclistener_insertFmtMark(fl_BlockLayout* pBL, const PX_ChangeRecord_FmtMark * pcrfm)
 {
 	UT_Bool bResult = UT_TRUE;
+	fl_BlockLayout * pShadowBL = NULL;
 	UT_uint32 iCount = m_vecPages.getItemCount();
 	for (UT_uint32 i=0; i<iCount; i++)
 	{
 		struct _PageHdrFtrShadowPair* pPair = (struct _PageHdrFtrShadowPair*) m_vecPages.getNthItem(i);
-
-		bResult = pPair->pShadow->bl_doclistener_insertFmtMark(pBL, pcrfm)
+		// Find matching block in this shadow.
+		pShadowBL = pPair->pShadow->findMatchingBlock(pBL);
+		bResult = pShadowBL->doclistener_insertFmtMark(pcrfm)
 			&& bResult;
 	}
-
+	// Update the overall block too.
+        bResult = pBL->doclistener_insertFmtMark(pcrfm)
+			&& bResult;
 	return bResult;
 }
 
 UT_Bool fl_HdrFtrSectionLayout::bl_doclistener_deleteFmtMark(fl_BlockLayout* pBL, const PX_ChangeRecord_FmtMark * pcrfm)
 {
 	UT_Bool bResult = UT_TRUE;
+	fl_BlockLayout * pShadowBL = NULL;
 	UT_uint32 iCount = m_vecPages.getItemCount();
 	for (UT_uint32 i=0; i<iCount; i++)
 	{
 		struct _PageHdrFtrShadowPair* pPair = (struct _PageHdrFtrShadowPair*) m_vecPages.getNthItem(i);
-
-		bResult = pPair->pShadow->bl_doclistener_deleteFmtMark(pBL, pcrfm)
+		// Find matching block in this shadow.
+		pShadowBL = pPair->pShadow->findMatchingBlock(pBL);
+		bResult = pShadowBL->doclistener_deleteFmtMark(pcrfm)
 			&& bResult;
 	}
-
+	// Update the overall block too.
+        bResult = pBL->doclistener_deleteFmtMark(pcrfm)
+			&& bResult;
 	return bResult;
 }
 
 UT_Bool fl_HdrFtrSectionLayout::bl_doclistener_changeFmtMark(fl_BlockLayout* pBL, const PX_ChangeRecord_FmtMarkChange * pcrfmc)
 {
 	UT_Bool bResult = UT_TRUE;
+	fl_BlockLayout * pShadowBL = NULL;
 	UT_uint32 iCount = m_vecPages.getItemCount();
 	for (UT_uint32 i=0; i<iCount; i++)
 	{
 		struct _PageHdrFtrShadowPair* pPair = (struct _PageHdrFtrShadowPair*) m_vecPages.getNthItem(i);
-
-		bResult = pPair->pShadow->bl_doclistener_changeFmtMark(pBL, pcrfmc)
+		// Find matching block in this shadow.
+		pShadowBL = pPair->pShadow->findMatchingBlock(pBL);
+		bResult = pShadowBL->doclistener_changeFmtMark(pcrfmc)
 			&& bResult;
 	}
-
+	// Update the overall block too.
+        bResult = pBL->doclistener_changeFmtMark(pcrfmc)
+			&& bResult;
 	return bResult;
 }
 
@@ -1801,6 +2047,20 @@ fp_Container* fl_HdrFtrShadow::getNewContainer(void)
 	return NULL;
 }
 
+fl_BlockLayout* fl_HdrFtrShadow::findMatchingBlock(fl_BlockLayout* pBL)
+{
+  // This routine returns the matching block within this shadow of the
+  // hdrftrSectionlayout. 
+  //
+        fl_BlockLayout* ppBL = m_pFirstBlock;
+	while(ppBL && (ppBL->getStruxDocHandle() != pBL->getStruxDocHandle()))
+        {
+	      ppBL = ppBL->getNext();
+	}
+	UT_ASSERT(ppBL);
+	return ppBL;
+}
+	  
 void fl_HdrFtrShadow::format(void)
 {
 	fl_BlockLayout*	pBL = m_pFirstBlock;
@@ -1847,6 +2107,7 @@ UT_Bool fl_HdrFtrShadow::doclistener_changeStrux(const PX_ChangeRecord_StruxChan
 	return UT_FALSE;
 }
 
+
 void fl_HdrFtrShadow::_lookupProperties(void)
 {
 }
@@ -1921,8 +2182,22 @@ UT_Bool fl_ShadowListener::populate(PL_StruxFmtHandle sfh,
 			bResult = m_pCurrentBL->doclistener_populateObject(blockOffset,pcro);
 			goto finish_up;
 		}
+	
+	case PX_ChangeRecord::PXT_InsertFmtMark:
+                {
+		  //	const PX_ChangeRecord_FmtMark * pcrfm = static_cast<const PX_ChangeRecord_FmtMark *>(pcr);
+
+			{
+				fl_Layout * pL = (fl_Layout *)sfh;
+				UT_ASSERT(pL->getType() == PTX_Block);
+				UT_ASSERT(m_pCurrentBL == (static_cast<fl_BlockLayout *>(pL)));
+			}
+			bResult = m_pCurrentBL->doclistener_insertFmtMark( (const PX_ChangeRecord_FmtMark *) pcr);
+			goto finish_up;
+		}
 
 	default:
+	        UT_DEBUGMSG(("SEVIOR: Unknown Change record = %d \n",pcr->getType())); 
 		UT_ASSERT(0);
 		return UT_FALSE;
 	}
@@ -1937,7 +2212,7 @@ UT_Bool fl_ShadowListener::populateStrux(PL_StruxDocHandle sdh,
 										 PL_StruxFmtHandle * psfh)
 {
 	UT_ASSERT(m_pShadow);
-	UT_DEBUGMSG(("fl_ShadowListener::populateStrux\n"));
+	//	UT_DEBUGMSG(("fl_ShadowListener::populateStrux\n"));
 
 	UT_ASSERT(pcr->getType() == PX_ChangeRecord::PXT_InsertStrux);
 	const PX_ChangeRecord_Strux * pcrx = static_cast<const PX_ChangeRecord_Strux *> (pcr);
@@ -1948,11 +2223,10 @@ UT_Bool fl_ShadowListener::populateStrux(PL_StruxDocHandle sdh,
 	{
 		PT_AttrPropIndex indexAP = pcr->getIndexAP();
 		const PP_AttrProp* pAP = NULL;
-			
 		if (m_pDoc->getAttrProp(indexAP, &pAP) && pAP)
 		{
 			const XML_Char* pszSectionType = NULL;
-			pAP->getAttribute("type", pszSectionType);
+			pAP->getAttribute((XML_Char*)"type", pszSectionType);
 			if (
 				!pszSectionType
 				|| (0 == UT_stricmp(pszSectionType, "doc"))
@@ -1996,16 +2270,16 @@ UT_Bool fl_ShadowListener::populateStrux(PL_StruxDocHandle sdh,
 				UT_DEBUGMSG(("no memory for BlockLayout"));
 				return UT_FALSE;
 			}
-
+			UT_DEBUGMSG(("SEVIOR: Appending block %x to this header/footer \n",pBL));
 			m_pCurrentBL = pBL;	
 			*psfh = (PL_StruxFmtHandle)pBL;
 		}
-
+		
 #if 0		// TODO are we spell-checking headers and footers?
 		
 		// BUGBUG: this is *not* thread-safe, but should work for now
 		if (m_bScreen)
-			m_pLayout->queueBlockForSpell(pBL);
+			m_pLayout->queueBlockForBackgroundCheck(bgcrSpelling, pBL);
 #endif
 
 	}
@@ -2047,3 +2321,6 @@ UT_Bool fl_ShadowListener::signal(UT_uint32 /*iSignal*/)
 	return UT_FALSE;
 }
 
+
+
+
