@@ -187,7 +187,7 @@ void fp_Line::mapXYToPosition(UT_sint32 x, UT_sint32 y, PT_DocPosition& pos, UT_
 	{
 		fp_Run* pRun2 = (fp_Run*) m_vecRuns.getNthItem(i);
 
-		if (pRun2->canContainPoint())
+		if (pRun2->canContainPoint()  || pRun2->isField())
 		{
 			UT_sint32 y2 = y - pRun2->getY() - m_iAscent + pRun2->getAscent();
 			if ((x >= (UT_sint32) pRun2->getX()) && (x < (UT_sint32) (pRun2->getX() + pRun2->getWidth())))
@@ -266,8 +266,15 @@ void fp_Line::mapXYToPosition(UT_sint32 x, UT_sint32 y, PT_DocPosition& pos, UT_
 	UT_ASSERT(pClosestRun);
 	
 	UT_sint32 y2 = y - pClosestRun->getY() - m_iAscent + pClosestRun->getAscent();
-	pClosestRun->mapXYToPosition(x - pClosestRun->getX(), y2, pos, bBOL, bEOL);
-
+	if(pClosestRun->isField())
+	{
+	       UT_uint32 width = pClosestRun->getWidth() + 1;
+	       pClosestRun->mapXYToPosition(width , y2, pos, bBOL, bEOL);
+	}
+	else
+	{
+	       pClosestRun->mapXYToPosition(x - pClosestRun->getX(), y2, pos, bBOL, bEOL);
+	}
 	UT_ASSERT(bEOL == UT_TRUE || bEOL == UT_FALSE);
 	UT_ASSERT(bBOL == UT_TRUE || bBOL == UT_FALSE);
 
@@ -425,8 +432,8 @@ void fp_Line::clearScreen(void)
 
 			m_pContainer->getScreenOffsets(this, xoffLine, yoffLine);
 
-			UT_sint32 columnGap = getColumnGap();
-			pRun->getGraphics()->clearArea(xoffLine - columnGap / 2, yoffLine, m_iMaxWidth + columnGap, m_iHeight);
+			pRun->getGraphics()->clearArea(xoffLine, yoffLine, m_iMaxWidth, m_iHeight);
+
 		}
 	}
 	
@@ -464,17 +471,7 @@ void fp_Line::clearScreenFromRunToEnd(UT_uint32 runIndex)
 		UT_sint32 xoffLine, yoffLine;
 
 		m_pContainer->getScreenOffsets(this, xoffLine, yoffLine);
-
-		if(runIndex == 0)
-		{
-			// Erasing the whole line so include 1/2 column gap at start and end.
-		        pRun->getGraphics()->clearArea(xoff - getColumnGap() / 2, yoff, m_iMaxWidth + getColumnGap() - (xoff - xoffLine), m_iHeight);
-		}
-		else
-		{
-			// Erasing to the end so include 1/2 column gap at end.
-			pRun->getGraphics()->clearArea(xoff, yoff, m_iMaxWidth + getColumnGap() / 2 - (xoff - xoffLine), m_iHeight);
-		}
+		pRun->getGraphics()->clearArea(xoff, yoff, m_iMaxWidth - (xoff - xoffLine), m_iHeight);
 		
 		for (i = runIndex; i < count; i++)
 		{
@@ -550,13 +547,16 @@ void fp_Line::layout(void)
 	pAlignment->initialize(this);
 
 
+	UT_sint32 Screen_resolution = getBlock()->getDocLayout()->getGraphics()->getResolution();
+
 	UT_uint32 iCountRuns = m_vecRuns.getItemCount();
-	UT_sint32 iX = 0;
-	UT_sint32 iStartX;
+	UT_sint32 iX, iXLayoutUnits;
+	UT_sint32 iStartX, iStartXLayoutUnits;
 	UT_uint32 i;
 	UT_Bool bLineErased = UT_FALSE;
 
 	iStartX = iX = pAlignment->getStartPosition();
+	iStartXLayoutUnits = iXLayoutUnits = pAlignment->getStartPositionInLayoutUnits();
 	
 	for (i=0; i<iCountRuns; i++)		// TODO do we need to do this if iMoveOver is zero ??
 	{
@@ -575,22 +575,27 @@ void fp_Line::layout(void)
 		
 		if (pRun->getType() == FPRUN_TAB)
 		{
-			UT_sint32 iPos;
-			unsigned char iTabType;
+			UT_sint32 iPosLayoutUnits;
+			eTabType	iTabType;
+			eTabLeader	iTabLeader;
 
-			UT_Bool bRes = findNextTabStop(iX - iStartX, iPos, iTabType);
+			UT_Bool bRes = findNextTabStopInLayoutUnits(iXLayoutUnits - iStartXLayoutUnits, iPosLayoutUnits, iTabType, iTabLeader);
 			UT_ASSERT(bRes);
 
 			fp_TabRun* pTabRun = static_cast<fp_TabRun*>(pRun);
-			fp_Run *pScanRun;
+			fp_Run *pScanRun = NULL;
 			int iScanWidth = 0;
+			int iScanWidthLayoutUnits = 0;
 
 			// for everybody except the left tab, we need to know how much text is to follow
 			switch ( iTabType )
 			{
 			case FL_TAB_LEFT:
-				pTabRun->setWidth(iPos - (iX - iStartX));
-				iX = iPos + iStartX;
+				{
+				iXLayoutUnits = (iPosLayoutUnits + iStartXLayoutUnits);
+				iX = iXLayoutUnits * Screen_resolution / UT_LAYOUT_UNITS;
+				pTabRun->setWidth(iX - pTabRun->getX());
+				}
 				break;
 
 			case FL_TAB_CENTER:
@@ -599,43 +604,45 @@ void fp_Line::layout(void)
 					  pScanRun = pScanRun->getNext() )
 				{
 					iScanWidth += pScanRun->getWidth();
+					iScanWidthLayoutUnits += pScanRun->getWidthInLayoutUnits();
 				}
 	
-				if ( iScanWidth / 2 > iPos - (iX - iStartX) )
+				if ( iScanWidthLayoutUnits / 2 > iPosLayoutUnits - (iXLayoutUnits - iStartXLayoutUnits) )
 					pTabRun->setWidth(0);
 				else
 				{
-					int tabWidth = iPos - (iX - iStartX) - iScanWidth / 2 ;
-					pTabRun->setWidth( tabWidth );
-					iX += tabWidth;
+					iXLayoutUnits += iPosLayoutUnits - (iXLayoutUnits - iStartXLayoutUnits) - iScanWidthLayoutUnits / 2;
+					iX += iPosLayoutUnits * Screen_resolution / UT_LAYOUT_UNITS - (iX - iStartX) - iScanWidth / 2;
+					pTabRun->setWidth(iX - pTabRun->getX());
 				}
 		
 				break;
 
 			case FL_TAB_RIGHT:
+			{
 				for ( pScanRun = pRun->getNext();	
 					  pScanRun && pScanRun->getType() != FPRUN_TAB; 
 					  pScanRun = pScanRun->getNext() )
 				{
 					iScanWidth += pScanRun->getWidth();
+					iScanWidthLayoutUnits += pScanRun->getWidthInLayoutUnits();
 				}
 		
-				if ( iScanWidth > iPos - (iX - iStartX) )
+				if ( iScanWidthLayoutUnits > iPosLayoutUnits - (iXLayoutUnits - iStartXLayoutUnits) )
 					pTabRun->setWidth(0);
 				else
 				{
-					int tabWidth = iPos - (iX - iStartX) - iScanWidth ;
-					pTabRun->setWidth( tabWidth );
-					iX += tabWidth;
+					iXLayoutUnits += iPosLayoutUnits - (iXLayoutUnits - iStartXLayoutUnits) - iScanWidthLayoutUnits;
+					iX += iPosLayoutUnits * Screen_resolution / UT_LAYOUT_UNITS - (iX - iStartX) - iScanWidth;
+					pTabRun->setWidth(iX - pTabRun->getX());
 				}
 				break;
+			}
 
 			case FL_TAB_DECIMAL:
 			{
-				UT_UCSChar const *pSpanPtr, *pCh;
 				UT_UCSChar *pDecimalStr;
-				UT_uint32   len, runLen;
-				iScanWidth=0;
+				UT_uint32	runLen = 0;
 
 				// the string to search for decimals
 				UT_UCS_cloneString_char(&pDecimalStr, ".");
@@ -645,47 +652,41 @@ void fp_Line::layout(void)
 					  pScanRun = pScanRun->getNext() )
 				{
 					UT_Bool foundDecimal = UT_FALSE;
-					if ( getBlock()->getSpanPtr( pScanRun->getBlockOffset(), &pSpanPtr, &len ) )
+
+					if(pScanRun->getType() == FPRUN_TEXT)
 					{
-						runLen = pScanRun->getLength();
-						UT_ASSERT(len >= runLen);
-						pCh = pSpanPtr;
+						UT_sint32 decimalBlockOffset = ((fp_TextRun *)pScanRun)->findCharacter(0, pDecimalStr[0]);
 
-						#if defined(UT_DEBUG) && 1
-							char szbuffer[0x400];
-							UT_UCS_strcpy_to_char(szbuffer, pSpanPtr);
-							szbuffer[runLen] = 0;
-							UT_DEBUGMSG(("%s:%d  searching span [%s] (%d chars) (iScanWidth=%d)\n", 
-										  __FILE__, __LINE__, szbuffer, runLen, iScanWidth));
-						#endif
-
-						while ( runLen )
+						if(decimalBlockOffset != -1)
 						{
-							if ( *pCh++ == *pDecimalStr )		// TODO - FIXME
-							{
-								foundDecimal = UT_TRUE;	
-								break;
-							}
-							runLen--;
+							foundDecimal = UT_TRUE;
+
+							runLen = pScanRun->getBlockOffset() - decimalBlockOffset;
 						}
 					}
+
 					UT_DEBUGMSG(("%s:%d  foundDecimal=%d len=%d iScanWidth=%d \n", 
 								__FILE__, __LINE__, foundDecimal, pScanRun->getLength()-runLen, iScanWidth));
 					if ( foundDecimal )
 					{
-						iScanWidth += getBlock()->getDocLayout()->getGraphics()->
-							measureString( pSpanPtr, 0, pScanRun->getLength() - runLen, NULL );
+						if(pScanRun->getType() == FPRUN_TEXT)
+						{
+							iScanWidth += ((fp_TextRun *)pScanRun)->simpleRecalcWidth(fp_TextRun::Width_type_display, runLen);
+							iScanWidthLayoutUnits += ((fp_TextRun *)pScanRun)->simpleRecalcWidth(fp_TextRun::Width_type_layout_units, runLen);
+						}
 						break; // we found our decimal, don't search any further
 					}
-					else	
+					else
+					{
 						iScanWidth += pScanRun->getWidth();
+						iScanWidthLayoutUnits += pScanRun->getWidthInLayoutUnits();
+					}
 				}
 			
-				UT_DEBUGMSG((" tabrun iX=%d iPos=%d iScanWidth=%d tabwidth=%d newX=%d\n", 
-									  iX,	iPos,   iScanWidth,   iPos-(iX-iStartX)-iScanWidth,iPos-iScanWidth));	
-				pTabRun->setWidth(iPos - (iX - iStartX) - iScanWidth);
-				iX = iPos - iScanWidth + iStartX;
-			
+				iXLayoutUnits = iPosLayoutUnits - iScanWidthLayoutUnits + iStartXLayoutUnits;
+				iX = iPosLayoutUnits * Screen_resolution / UT_LAYOUT_UNITS - iScanWidth + iStartX;
+				pTabRun->setWidth(iX - pTabRun->getX());
+
 				FREEP(pDecimalStr);	
 				break;
 			}
@@ -699,7 +700,8 @@ void fp_Line::layout(void)
 		}
 		else
 		{
-			iX += pAlignment->getMove(pRun);
+			iXLayoutUnits += pRun->getWidthInLayoutUnits();
+			iX += pRun->getWidth();
 		}
 	}
 }
@@ -825,12 +827,27 @@ UT_Bool fp_Line::recalculateFields(void)
 	return bResult;
 }
 
-UT_Bool	fp_Line::findNextTabStop(UT_sint32 iStartX, UT_sint32& iPosition, unsigned char& iType)
-{
-	UT_sint32		iTabStopPosition = 0;
-	unsigned char	iTabStopType;
+fp_Run* fp_Line::getLastRun(void) const
+{ 
+	UT_sint32 i = m_vecRuns.getItemCount();
+	if( i <= 0 )
+	{    
+		fp_Run * pRun = getBlock()->getFirstRun();
+		return pRun;
+	}
+	else
+	{
+		return ((fp_Run*) m_vecRuns.getLastItem()); 
+	}
+}
 
-	UT_Bool bRes = m_pBlock->findNextTabStop(iStartX + getX(), getX() + getMaxWidth(), iTabStopPosition, iTabStopType);
+UT_Bool	fp_Line::findNextTabStop(UT_sint32 iStartX, UT_sint32& iPosition, eTabType & iType, eTabLeader & iLeader )
+{
+	UT_sint32	iTabStopPosition = 0;
+	eTabType	iTabStopType = FL_TAB_NONE;
+	eTabLeader	iTabStopLeader = FL_LEADER_NONE;
+
+	UT_Bool bRes = m_pBlock->findNextTabStop(iStartX + getX(), getX() + getMaxWidth(), iTabStopPosition, iTabStopType, iTabStopLeader);
 	UT_ASSERT(bRes);
 
 	iTabStopPosition -= getX();
@@ -839,6 +856,7 @@ UT_Bool	fp_Line::findNextTabStop(UT_sint32 iStartX, UT_sint32& iPosition, unsign
 	{
 		iPosition = iTabStopPosition;
 		iType = iTabStopType;
+		iLeader = iTabStopLeader;
 
 		return UT_TRUE;
 	}
@@ -848,12 +866,15 @@ UT_Bool	fp_Line::findNextTabStop(UT_sint32 iStartX, UT_sint32& iPosition, unsign
 	}
 }
 
-UT_Bool	fp_Line::findNextTabStopInLayoutUnits(UT_sint32 iStartX, UT_sint32& iPosition, unsigned char& iType)
+UT_Bool	fp_Line::findNextTabStopInLayoutUnits(UT_sint32 iStartX, UT_sint32& iPosition, eTabType& iType, eTabLeader& iLeader )
 {
-	UT_sint32		iTabStopPosition = 0;
-	unsigned char	iTabStopType;
+	UT_sint32	iTabStopPosition = 0;
+	eTabType	iTabStopType = FL_TAB_NONE;
+	eTabLeader	iTabStopLeader = FL_LEADER_NONE;
 
-	UT_Bool bRes = m_pBlock->findNextTabStopInLayoutUnits(iStartX + getXInLayoutUnits(), getXInLayoutUnits() + getMaxWidthInLayoutUnits(), iTabStopPosition, iTabStopType);
+	UT_Bool bRes = m_pBlock->findNextTabStopInLayoutUnits( iStartX + getXInLayoutUnits(), 
+														   getXInLayoutUnits() + getMaxWidthInLayoutUnits(), 
+														   iTabStopPosition, iTabStopType, iTabStopLeader);
 	UT_ASSERT(bRes);
 
 	iTabStopPosition -= getXInLayoutUnits();
@@ -862,6 +883,7 @@ UT_Bool	fp_Line::findNextTabStopInLayoutUnits(UT_sint32 iStartX, UT_sint32& iPos
 	{
 		iPosition = iTabStopPosition;
 		iType = iTabStopType;
+		iLeader = iTabStopLeader;
 
 		return UT_TRUE;
 	}
@@ -947,7 +969,7 @@ fp_Line*	fp_Line::getPrevLineInSection(void) const
 	return NULL;
 }
 
-UT_Bool	fp_Line::containsForcedColumnBreak(void) const
+UT_Bool	fp_Line::containsForcedColumnBreak(void) const 
 {
 	if(!isEmpty())
 	{
@@ -971,7 +993,6 @@ UT_Bool fp_Line::containsForcedPageBreak(void) const
 			return UT_TRUE;
 		}
 	}
-
 	return UT_FALSE;
 }
 
@@ -1006,13 +1027,16 @@ UT_sint32 fp_Line::calculateWidthOfLine(void)
 		
 		if (pRun->getType() == FPRUN_TAB)
 		{
-			UT_sint32 iPos;
-			unsigned char iTabType;
+			UT_sint32	iPos;
+			eTabType	iTabType;
+			eTabLeader	iTabLeader;
 
-			UT_Bool bRes = findNextTabStop(iX, iPos, iTabType);
+			UT_Bool bRes = findNextTabStop(iX, iPos, iTabType, iTabLeader);
 			UT_ASSERT(bRes);
 			UT_ASSERT(iTabType == FL_TAB_LEFT);
 
+			// TODO -- support all the tabs  shack@uiuc.edu
+
 			fp_TabRun* pTabRun = static_cast<fp_TabRun*>(pRun);
 			pTabRun->setWidth(iPos - iX);
 			
@@ -1045,9 +1069,10 @@ UT_sint32 fp_Line::calculateWidthOfLineInLayoutUnits(void)
 		if (pRun->getType() == FPRUN_TAB)
 		{
 			UT_sint32 iPos;
-			unsigned char iTabType;
+			eTabType iTabType;
+			eTabLeader iTabLeader;
 
-			UT_Bool bRes = findNextTabStopInLayoutUnits(iX, iPos, iTabType);
+			UT_Bool bRes = findNextTabStopInLayoutUnits(iX, iPos, iTabType, iTabLeader);
 			UT_ASSERT(bRes);
 			UT_ASSERT(iTabType == FL_TAB_LEFT);
 
@@ -1149,8 +1174,8 @@ UT_uint32 fp_Line::countJustificationPoints(void) const
 		
 		if (pRun->getType() == FPRUN_TAB)
 		{
-			UT_ASSERT(UT_FALSE);
-			
+		  //			UT_ASSERT(UT_FALSE);
+		  UT_DEBUGMSG(("TODO - decide if tab is a space \n"));
 			// TODO: decide if a tab is a space.
 
 		}
