@@ -207,7 +207,20 @@ void fp_Page::draw(dg_DrawArgs* pDA)
 			da.yoff += pTmpCol->getY();
 			pTmpCol->draw(&da);
 
-			pTmpCol = pTmpCol->getFollower();
+			fp_Column *pNextCol = pTmpCol->getFollower();
+
+			if(pNextCol && pTmpCol->getDocSectionLayout()->getColumnLineBetween())
+			{
+				// draw line between columns if required.
+
+				UT_sint32 x = pDA->xoff + (pTmpCol->getX() + pTmpCol->getWidth() + pNextCol->getX()) / 2;
+				UT_sint32 y = pDA->yoff + pTmpCol->getY();
+				UT_RGBColor Line_color(0, 0, 0);
+				pDA->pG->setColor(Line_color);
+				pDA->pG->drawLine(x, y, x, y + pTmpCol->getHeight());
+			}
+
+			pTmpCol = pNextCol;
 		}
 	}
 
@@ -238,7 +251,7 @@ UT_Bool fp_Page::needsRedraw(void) const
 #ifdef FMT_TEST
 void fp_Page::__dump(FILE * fp) const
 {
-	fprintf(fp,"\tPage: 0x%p\n",this);
+	fprintf(fp,"\tPage: 0x%p\n",(void*)this);
 }
 #endif
 
@@ -511,6 +524,11 @@ void fp_Page::setView(FV_View* pView)
 	m_pView = pView;
 }
 
+const fp_PageSize&	fp_Page::getPageSize(void) const
+{
+	return m_pageSize;
+}
+
 fp_HdrFtrContainer* fp_Page::getHeaderContainer(fl_HdrFtrSectionLayout* pHFSL)
 {
 	if (m_pHeader)
@@ -554,3 +572,4 @@ fp_HdrFtrContainer* fp_Page::getFooterContainer(fl_HdrFtrSectionLayout* pHFSL)
 
 	return m_pFooter;
 }
+
