@@ -34,6 +34,7 @@
 #include "gr_DrawArgs.h"
 #include "fv_View.h"
 #include "pp_AttrProp.h"
+#include "fd_Field.h"
 
 #include "ut_debugmsg.h"
 #include "ut_assert.h"
@@ -90,6 +91,7 @@ fp_Run::fp_Run(fl_BlockLayout* pBL,
 	m_iDescent = 0;
 	m_iAscentLayoutUnits = 0;
 	m_iDescentLayoutUnits = 0;
+        m_pField = NULL;
 }
 
 fp_Run::~fp_Run()
@@ -107,8 +109,8 @@ void fp_Run::__dump(FILE * fp) const
 	const char * szName = (((m_iType >= FPRUN__FIRST__) && (m_iType <= FPRUN__LAST__)) ? s_names[m_iType-1] : "Unknown");
 
 	fprintf(fp,"    Run: %p T=%s Off=%d Len=%d D=%c Line=%p [x %d y %d w %d h %d]\n",
-			this, szName, m_iOffsetFirst, m_iLen, 
-			((m_bDirty) ? 'y' : 'n'), m_pLine,
+			(void*)this, szName, m_iOffsetFirst, m_iLen, 
+			((m_bDirty) ? 'y' : 'n'), (void*)m_pLine,
 			m_iX, m_iY, m_iWidth, m_iHeight);
 }
 #endif
@@ -247,6 +249,7 @@ void fp_Run::setBlockOffset(UT_uint32 offset)
 
 void fp_Run::clearScreen(UT_Bool bFullLineHeightRect)
 {
+
 	if (m_bDirty)
 	{
 		// no need to clear if we've already done so.
@@ -269,6 +272,7 @@ void fp_Run::clearScreen(UT_Bool bFullLineHeightRect)
 
 void fp_Run::draw(dg_DrawArgs* pDA)
 {
+
 	if (pDA->bDirtyRunsOnly)
 	{
 		if (!m_bDirty)
@@ -325,6 +329,7 @@ const PP_AttrProp* fp_Run::getAP(void) const
 
 void fp_Run::_drawTextLine(UT_sint32 xoff,UT_sint32 yoff,UT_uint32 iWidth,UT_uint32 iHeight,UT_UCSChar *pText)
 {
+
     m_pG->setFont(m_pG->getGUIFont());
 
     UT_uint32 iTextLen = UT_UCS_strlen(pText);
@@ -358,6 +363,8 @@ void fp_TabRun::lookupProperties(void)
 	
 	m_pBL->getSpanAttrProp(m_iOffsetFirst,UT_FALSE,&pSpanAP);
 	m_pBL->getAttrProp(&pBlockAP);
+        m_pBL->getField(m_iOffsetFirst,m_pField);
+
 
 	// look for fonts in this DocLayout's font cache
 	FL_DocLayout * pLayout = m_pBL->getDocLayout();
@@ -449,7 +456,6 @@ void fp_TabRun::_clearScreen(UT_Bool /* bFullLineHeightRect */)
 	
 	// need to clear full height of line, in case we had a selection
 	m_pLine->getScreenOffsets(this, xoff, yoff);
-	
 	m_pG->clearArea(xoff, yoff, m_iWidth, m_pLine->getHeight());
 }
 
@@ -459,49 +465,35 @@ void fp_TabRun::_drawArrow(UT_uint32 iLeft,UT_uint32 iTop,UT_uint32 iWidth, UT_u
         return;
     }
 
-    #define NPOINTS 8
+    #define NPOINTS 4
 
-    UT_Point * points = (UT_Point *)calloc(NPOINTS,sizeof(UT_Point));
-    UT_ASSERT(points);
+    UT_Point points[NPOINTS];
 
     UT_sint32 cur_linewidth = 1 + (UT_MAX(10,m_iAscent) - 10) / 8;
     UT_uint32 iyAxis = iTop + m_pLine->getAscent() * 2 / 3;
     UT_uint32 iMaxWidth = iWidth / 10 * 6;
     UT_uint32 ixGap = (iWidth - iMaxWidth) / 2;
 
-    points[0].x = iLeft + ixGap;
-    points[0].y = iyAxis - cur_linewidth / 2;
+    points[0].x = iLeft + ixGap + iMaxWidth - cur_linewidth * 4;
+    points[0].y = iyAxis - cur_linewidth * 2;
 
-    points[1].x = iLeft + ixGap + iMaxWidth - cur_linewidth * 4;
-    points[1].y = points[0].y;
+    points[1].x = iLeft + iWidth - ixGap;
+    points[1].y = iyAxis;
 
-    points[2].x = points[1].x;
-    points[2].y = points[0].y - cur_linewidth * 2;
+    points[2].x = points[0].x;
+    points[2].y = iyAxis + cur_linewidth * 2;
 
-    points[3].x = iLeft + iWidth - ixGap;
-    points[3].y = iyAxis;
+    points[3].x = points[0].x;
+    points[3].y = points[0].y;
 
-    points[4].x = points[1].x;
-    points[4].y = iyAxis + cur_linewidth * 2;
-
-    points[5].x = points[1].x;
-    points[5].y = iyAxis + cur_linewidth / 2;
-
-    points[6].x = points[0].x;
-    points[6].y = points[5].y;
-
-    points[7].x = points[0].x;
-    points[7].y = points[0].y;
-
-    m_pG->setColor(m_colorFG);
-    m_pG->polyLine(points,NPOINTS);
-    FREEP(points);
+    m_pG->polygon(m_colorFG,points,NPOINTS);
+    m_pG->fillRect(m_colorFG,iLeft + ixGap,iyAxis - cur_linewidth / 2,iMaxWidth - cur_linewidth * 4,cur_linewidth);
 }
 
 void fp_TabRun::_draw(dg_DrawArgs* pDA)
 {
 	UT_ASSERT(pDA->pG == m_pG);
-	
+
 	UT_RGBColor clrSelBackground(192, 192, 192);
 	UT_RGBColor clrNormalBackground(255,255,255);
 
@@ -546,10 +538,13 @@ void fp_TabRun::_draw(dg_DrawArgs* pDA)
 
 fp_ForcedLineBreakRun::fp_ForcedLineBreakRun(fl_BlockLayout* pBL, GR_Graphics* pG, UT_uint32 iOffsetFirst, UT_uint32 iLen) : fp_Run(pBL, pG, iOffsetFirst, iLen, FPRUN_FORCEDLINEBREAK)
 {
+        lookupProperties();
 }
 
 void fp_ForcedLineBreakRun::lookupProperties(void)
 {
+        m_pBL->getField(m_iOffsetFirst,m_pField);
+
 }
 
 UT_Bool fp_ForcedLineBreakRun::canContainPoint(void) const
@@ -613,6 +608,145 @@ void fp_ForcedLineBreakRun::_draw(dg_DrawArgs* pDA)
 	UT_ASSERT(pDA->pG == m_pG);
 }
 
+
+//////////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////////
+
+fp_FieldStartRun::fp_FieldStartRun(fl_BlockLayout* pBL, GR_Graphics* pG, UT_uint32 iOffsetFirst, UT_uint32 iLen) : fp_Run(pBL, pG, iOffsetFirst, iLen, FPRUN_FIELDSTARTRUN)
+{
+        lookupProperties();
+}
+
+void fp_FieldStartRun::lookupProperties(void)
+{
+        m_pBL->getField(m_iOffsetFirst,m_pField);
+	m_iWidth = 0;
+}
+
+UT_Bool fp_FieldStartRun::canContainPoint(void) const
+{
+	return UT_FALSE;
+}
+
+UT_uint32 fp_FieldStartRun::containsOffset(UT_uint32 /* iOffset */)
+{
+	return FP_RUN_NOT;
+}
+
+UT_Bool fp_FieldStartRun::canBreakAfter(void) const
+{
+	return UT_TRUE;
+}
+
+UT_Bool fp_FieldStartRun::canBreakBefore(void) const
+{
+	return UT_TRUE;
+}
+
+UT_Bool fp_FieldStartRun::letPointPass(void) const
+{
+	return UT_TRUE;
+}
+
+UT_Bool	fp_FieldStartRun::findMaxLeftFitSplitPointInLayoutUnits(UT_sint32 /* iMaxLeftWidth */, fp_RunSplitInfo& /* si */, UT_Bool /* bForce */)
+{
+	return UT_FALSE;
+}
+
+void fp_FieldStartRun::mapXYToPosition(UT_sint32 /* x */, UT_sint32 /*y*/, PT_DocPosition& pos, UT_Bool& bBOL, UT_Bool& bEOL)
+{
+	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+	
+	pos = m_pBL->getPosition() + m_iOffsetFirst;
+	bBOL = UT_FALSE;
+	bEOL = UT_FALSE;
+}
+
+void fp_FieldStartRun::findPointCoords(UT_uint32 iOffset, UT_sint32& x, UT_sint32& y, UT_sint32& height)
+{
+	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+}
+
+void fp_FieldStartRun::_clearScreen(UT_Bool /* bFullLineHeightRect */)
+{
+	UT_ASSERT(!m_bDirty);
+	UT_ASSERT(m_pG->queryProperties(GR_Graphics::DGP_SCREEN));
+}
+
+void fp_FieldStartRun::_draw(dg_DrawArgs* pDA)
+{
+	UT_ASSERT(pDA->pG == m_pG);
+}
+
+//////////////////////////////////////////////////////////////////
+//////////////////////////////////////////////////////////////////
+
+fp_FieldEndRun::fp_FieldEndRun(fl_BlockLayout* pBL, GR_Graphics* pG, UT_uint32 iOffsetFirst, UT_uint32 iLen) : fp_Run(pBL, pG, iOffsetFirst, iLen, FPRUN_FIELDENDRUN)
+{
+        lookupProperties();
+}
+
+void fp_FieldEndRun::lookupProperties(void)
+{
+        m_pBL->getField(m_iOffsetFirst,m_pField);
+	m_iWidth = 0;
+}
+
+UT_Bool fp_FieldEndRun::canContainPoint(void) const
+{
+	return UT_FALSE;
+}
+
+UT_uint32 fp_FieldEndRun::containsOffset(UT_uint32 /* iOffset */)
+{
+	return FP_RUN_NOT;
+}
+
+UT_Bool fp_FieldEndRun::canBreakAfter(void) const
+{
+	return UT_TRUE;
+}
+
+UT_Bool fp_FieldEndRun::canBreakBefore(void) const
+{
+	return UT_TRUE;
+}
+
+UT_Bool fp_FieldEndRun::letPointPass(void) const
+{
+	return UT_TRUE;
+}
+
+UT_Bool	fp_FieldEndRun::findMaxLeftFitSplitPointInLayoutUnits(UT_sint32 /* iMaxLeftWidth */, fp_RunSplitInfo& /* si */, UT_Bool /* bForce */)
+{
+	return UT_FALSE;
+}
+
+void fp_FieldEndRun::mapXYToPosition(UT_sint32 /* x */, UT_sint32 /*y*/, PT_DocPosition& pos, UT_Bool& bBOL, UT_Bool& bEOL)
+{
+	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+	
+	pos = m_pBL->getPosition() + m_iOffsetFirst;
+	bBOL = UT_FALSE;
+	bEOL = UT_FALSE;
+}
+
+void fp_FieldEndRun::findPointCoords(UT_uint32 iOffset, UT_sint32& x, UT_sint32& y, UT_sint32& height)
+{
+	UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+}
+
+void fp_FieldEndRun::_clearScreen(UT_Bool /* bFullLineHeightRect */)
+{
+	UT_ASSERT(!m_bDirty);
+	UT_ASSERT(m_pG->queryProperties(GR_Graphics::DGP_SCREEN));
+}
+
+void fp_FieldEndRun::_draw(dg_DrawArgs* pDA)
+{
+	UT_ASSERT(pDA->pG == m_pG);
+}
+
 //////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////
 
@@ -637,6 +771,7 @@ fp_ImageRun::~fp_ImageRun()
 
 void fp_ImageRun::lookupProperties(void)
 {
+        m_pBL->getField(m_iOffsetFirst,m_pField);
 	if (m_pImage)
 	{
 		m_iWidth = m_pImage->getDisplayWidth();
@@ -719,6 +854,7 @@ void fp_ImageRun::_clearScreen(UT_Bool /* bFullLineHeightRect */)
 	UT_ASSERT(!m_bDirty);
 	
 	UT_ASSERT(m_pG->queryProperties(GR_Graphics::DGP_SCREEN));
+
 	UT_sint32 xoff = 0, yoff = 0;
 	
 	// need to clear full height of line, in case we had a selection
@@ -806,13 +942,13 @@ void fp_ImageRun::_draw(dg_DrawArgs* pDA)
 //////////////////////////////////////////////////////////////////
 
 #define  _FIELD(type,desc,tag)  /*nothing*/
-#define  _FIELDTYPE(type,desc)  { FPFIELDTYPE_##type, desc },
+#define  _FIELDTYPE(type,desc)  {FPFIELDTYPE_##type, NULL, desc},
 
-const fp_FieldTypeData fp_FieldTypes[] = {
+fp_FieldTypeData fp_FieldTypes[] = {
 
 #include "fp_Fields.h"
 
-{ FPFIELDTYPE_END, NULL } };
+{FPFIELDTYPE_END, NULL, 0} };
 
 #undef  _FIELD
 #undef  _FIELDTYPE
@@ -820,122 +956,43 @@ const fp_FieldTypeData fp_FieldTypes[] = {
 // The way to turn macro argument into string constant
 #define xstr2(x) #x
 #define xstr(x) xstr2(x)
-#define _FIELD(type,desc,tag)  { FPFIELDTYPE_##type, FPFIELD_##tag, desc, xstr(tag) },
+#define _FIELD(type,desc,tag)  {FPFIELDTYPE_##type, FPFIELD_##tag, NULL, xstr(tag), desc},
 #define _FIELDTYPE(type,desc)  /*nothing*/
 
-const fp_FieldData fp_FieldFmts[] = {
+fp_FieldData fp_FieldFmts[] = {
 
 #include "fp_Fields.h"
 
-{ FPFIELDTYPE_END, FPFIELD_end, NULL, NULL } };
+{FPFIELDTYPE_END, FPFIELD_end, NULL, NULL, 0} };
 
 #undef  xstr2
 #undef  xstr
 #undef  _FIELD
 #undef  _FIELDTYPE
 
-fp_FieldRun::fp_FieldRun(fl_BlockLayout* pBL, GR_Graphics* pG, UT_uint32 iOffsetFirst, UT_uint32 iLen) : fp_Run(pBL, pG, iOffsetFirst, iLen, FPRUN_FIELD)
+fp_FieldRun::fp_FieldRun(fl_BlockLayout* pBL, GR_Graphics* pG, UT_uint32 iOffsetFirst, UT_uint32 iLen)
+:	fp_Run(pBL, pG, iOffsetFirst, iLen, FPRUN_FIELD),
+	m_pFont(0),
+	m_pFontLayout(0),
+	m_iFieldType(FPFIELD_start)
 {
-	m_pFont = NULL;
-	m_pFontLayout = NULL;
-
 	m_sFieldValue[0] = 0;
-	
-	lookupProperties();
-
-	calculateValue();
 }
 
-UT_Bool fp_FieldRun::calculateValue(void)
+UT_Bool fp_FieldRun::_setValue(UT_UCSChar *p_new_value)
 {
-	UT_UCSChar sz_ucs_FieldValue[FPFIELD_MAX_LENGTH + 1];
-	sz_ucs_FieldValue[0] = 0;
-	
-	switch (m_iFieldType)
-	{
-	case FPFIELD_time:
-	{
-		char szFieldValue[FPFIELD_MAX_LENGTH + 1];
 
-		time_t	tim = time(NULL);
-		struct tm *pTime = localtime(&tim);
-	
-		strftime(szFieldValue, FPFIELD_MAX_LENGTH, "%I:%M:%S %p", pTime);
-
-		UT_UCS_strcpy_char(sz_ucs_FieldValue, szFieldValue);
-		break;
-	}
-	case FPFIELD_page_number:
-	{
-		char szFieldValue[FPFIELD_MAX_LENGTH + 1];
-
-		if (m_pLine && m_pLine->getContainer() && m_pLine->getContainer()->getPage())
-		{
-			fp_Page* pPage = m_pLine->getContainer()->getPage();
-			FL_DocLayout* pDL = pPage->getDocLayout();
-
-			UT_sint32 iPageNum = 0;
-			UT_uint32 iNumPages = pDL->countPages();
-			for (UT_uint32 i=0; i<iNumPages; i++)
-			{
-				fp_Page* pPg = pDL->getNthPage(i);
-
-				if (pPg == pPage)
-				{
-					iPageNum = i + 1;
-					break;
-				}
-			}
-
-			UT_ASSERT(iPageNum > 0);
-
-			sprintf(szFieldValue, "%d", iPageNum);
-		}
-		else
-		{
-			strcpy(szFieldValue, "?");
-		}
-
-		UT_UCS_strcpy_char(sz_ucs_FieldValue, szFieldValue);
-		break;
-	}
-	case FPFIELD_page_count:
-	{
-		char szFieldValue[FPFIELD_MAX_LENGTH + 1];
-		
-		if (m_pLine && m_pLine->getContainer() && m_pLine->getContainer()->getPage())
-		{
-
-			fp_Page* pPage = m_pLine->getContainer()->getPage();
-			FL_DocLayout* pDL = pPage->getDocLayout();
-
-			sprintf(szFieldValue, "%d", pDL->countPages());
-		}
-		else
-		{
-			strcpy(szFieldValue, "?");
-		}
-
-		UT_UCS_strcpy_char(sz_ucs_FieldValue, szFieldValue);
-		break;
-	}
-	default:
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
-		return UT_FALSE;
-	}
-	
-	if (0 != UT_UCS_strcmp(sz_ucs_FieldValue, m_sFieldValue))
+	if (0 != UT_UCS_strcmp(p_new_value, m_sFieldValue))
 	{
 		clearScreen();
-		
-		UT_UCS_strcpy(m_sFieldValue, sz_ucs_FieldValue);
+
+		UT_UCS_strcpy(m_sFieldValue, p_new_value);
 
 		{
 			unsigned short aCharWidths[FPFIELD_MAX_LENGTH];
-	
+			lookupProperties();
 			m_pG->setFont(m_pFont);
 			UT_sint32 iNewWidth = m_pG->measureString(m_sFieldValue, 0, UT_UCS_strlen(m_sFieldValue), aCharWidths);
-
 			if (iNewWidth != m_iWidth)
 			{
 				clearScreen();
@@ -961,16 +1018,30 @@ void fp_FieldRun::lookupProperties(void)
 	const PP_AttrProp * pSectionAP = NULL; // TODO do we care about section-level inheritance?
 	
 	m_pBL->getSpanAttrProp(m_iOffsetFirst,UT_FALSE,&pSpanAP);
-	
+	//	UT_DEBUGMSG(("SEVIOR: Doing Lookupprops for block %x run %x  offset =%d \n ",m_pBL,this,m_iOffsetFirst));
+	UT_ASSERT(pSpanAP);
 	PD_Document * pDoc = m_pBL->getDocument();
 	m_pBL->getAttrProp(&pBlockAP);
-
+        m_pBL->getField(m_iOffsetFirst+1,m_pField); // Next Pos?
+	if(m_pField != NULL)
+	{
+	         m_pField->setBlock(m_pBL);
+	}
 	// look for fonts in this DocLayout's font cache
 	FL_DocLayout * pLayout = m_pBL->getDocLayout();
-	m_pFont = pLayout->findFont(pSpanAP,pBlockAP,pSectionAP, FL_DocLayout::FIND_FONT_AT_SCREEN_RESOLUTION),
-	m_pFontLayout = pLayout->findFont(pSpanAP,pBlockAP,pSectionAP, FL_DocLayout::FIND_FONT_AT_LAYOUT_RESOLUTION),
+	if(m_iFieldType == FPFIELD_list_label)
+	{
+		m_pFont = pLayout->findFont(pSpanAP,pBlockAP,pSectionAP, FL_DocLayout::FIND_FONT_AT_SCREEN_RESOLUTION,UT_TRUE);
+		m_pFontLayout = pLayout->findFont(pSpanAP,pBlockAP,pSectionAP, FL_DocLayout::FIND_FONT_AT_LAYOUT_RESOLUTION,UT_TRUE);
+	}
+	else
+	{
+		m_pFont = pLayout->findFont(pSpanAP,pBlockAP,pSectionAP, FL_DocLayout::FIND_FONT_AT_SCREEN_RESOLUTION);
+		m_pFontLayout = pLayout->findFont(pSpanAP,pBlockAP,pSectionAP, FL_DocLayout::FIND_FONT_AT_LAYOUT_RESOLUTION);
+	}
 
 	UT_parseColor(PP_evalProperty("color",pSpanAP,pBlockAP,pSectionAP, m_pBL->getDocument(), UT_TRUE), m_colorFG);
+	UT_parseColor(PP_evalProperty("field-color",pSpanAP,pBlockAP,pSectionAP, m_pBL->getDocument(), UT_TRUE), m_colorBG);
 
 	m_pG->setFont(m_pFont);
 
@@ -997,9 +1068,12 @@ void fp_FieldRun::lookupProperties(void)
 	{
 		m_fPosition = TEXT_POSITION_SUBSCRIPT;
 	} 
-	else m_fPosition = TEXT_POSITION_NORMAL;
+	else
+	{
+		 m_fPosition = TEXT_POSITION_NORMAL;
+	} 
 
-	pSpanAP->getAttribute("type", pszType);
+	pSpanAP->getAttribute((XML_Char*)"type", pszType);
 	UT_ASSERT(pszType);
 
 	int i;
@@ -1013,10 +1087,17 @@ void fp_FieldRun::lookupProperties(void)
 	}
 	if( fp_FieldFmts[i].m_Tag == NULL )
 	{
-		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+        // probably new type of field
+        //		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 	}
 }
 
+
+fp_FieldsEnum fp_FieldRun::getFieldType(void) const
+{
+        return m_iFieldType;
+}
+
 UT_Bool fp_FieldRun::canBreakAfter(void) const
 {
 	return UT_TRUE;
@@ -1067,6 +1148,8 @@ void fp_FieldRun::findPointCoords(UT_uint32 iOffset, UT_sint32& x, UT_sint32& y,
 	UT_sint32 yoff;
 
 	UT_ASSERT(m_pLine);
+
+	lookupProperties();
 	
 	m_pLine->getOffsets(this, xoff, yoff);
 
@@ -1082,31 +1165,70 @@ void fp_FieldRun::findPointCoords(UT_uint32 iOffset, UT_sint32& x, UT_sint32& y,
 	{
 		yoff += m_iDescent /* * 3/2 */;
 	}
-
         x = xoff;
 	y = yoff;
 	height = m_iHeight;
 }
 
+UT_Bool fp_FieldRun::calculateValue(void)
+{
+  //
+  // Code for the Piece Table Fields Calculation
+  // Get size of the field from the following runs
+  //
+  //      return m_pField->update();
+  //        UT_ASSERT(m_pField);
+
+  /*        UT_sint32 count = 0;
+        fp_Run* pNext = getNext();
+	while(pNext != NULL && pNext->getField() != NULL )
+	{
+	        if(m_pField == NULL)
+		{
+		        m_pField = pNext->getField();
+		}
+	        pNext = getNext();
+		count++;
+	}
+	if( count == 0)
+	{
+	        m_iWidth = 0;
+		m_iHeight = 0;
+		m_iWidthLayoutUnits = 0;
+		m_iHeightLayoutUnits = 0;
+	}
+	else
+        {
+	        pNext = getPrev();
+	        m_iWidth = pNext->getWidth();
+		m_iHeight = pNext->getHeight();
+		m_iWidthLayoutUnits = pNext->getWidthInLayoutUnits();
+		m_iHeightLayoutUnits = pNext->getHeightInLayoutUnits();
+	}
+	if(m_pField != NULL)
+	m_pField->update(); 
+*/
+	return UT_TRUE;
+}
+
 void fp_FieldRun::_clearScreen(UT_Bool /* bFullLineHeightRect */)
 {
 	UT_ASSERT(!m_bDirty);
-	
+
 	UT_ASSERT(m_pG->queryProperties(GR_Graphics::DGP_SCREEN));
 	UT_sint32 xoff = 0, yoff = 0;
 	
 	// need to clear full height of line, in case we had a selection
 	m_pLine->getScreenOffsets(this, xoff, yoff);
 	UT_sint32 iLineHeight = m_pLine->getHeight();
-	m_pG->clearArea(xoff, yoff, m_iWidth, iLineHeight);
+	m_pG->clearArea(xoff, yoff-1, m_iWidth, iLineHeight);
 }
 
-void fp_FieldRun::_draw(dg_DrawArgs* pDA)
+void fp_FieldRun::_defaultDraw(dg_DrawArgs* pDA)
 {
 	UT_ASSERT(pDA->pG == m_pG);
 
-
-
+	lookupProperties();
 	UT_sint32 iYdraw =  pDA->yoff - getAscent()-1;
 	
 	if (m_fPosition == TEXT_POSITION_SUPERSCRIPT)
@@ -1131,7 +1253,7 @@ void fp_FieldRun::_draw(dg_DrawArgs* pDA)
 		  surrounding text.
 		*/
 		UT_RGBColor clrSelBackground(112, 112, 112);
-		UT_RGBColor clrNormalBackground(220, 220, 220);
+		//		UT_RGBColor clrNormalBackground(220, 220, 220);
 		
 		UT_sint32 iFillTop = iYdraw;
 		UT_sint32 iFillHeight = getAscent() + getDescent();
@@ -1155,16 +1277,110 @@ void fp_FieldRun::_draw(dg_DrawArgs* pDA)
 		}
 		else
 		{
-			m_pG->fillRect(clrNormalBackground, pDA->xoff, iFillTop, m_iWidth, iFillHeight);
+			m_pG->fillRect(m_colorBG, pDA->xoff, iFillTop, m_iWidth, iFillHeight);
 		}
 	}
-
 	m_pG->setFont(m_pFont);
 	m_pG->setColor(m_colorFG);
 	
 	m_pG->drawChars(m_sFieldValue, 0, UT_UCS_strlen(m_sFieldValue), pDA->xoff,iYdraw);
 }
 
+fp_FieldTimeRun::fp_FieldTimeRun(fl_BlockLayout* pBL, GR_Graphics* pG, UT_uint32 iOffsetFirst, UT_uint32 iLen) : fp_FieldRun(pBL, pG, iOffsetFirst, iLen)
+{
+}
+
+UT_Bool fp_FieldTimeRun::calculateValue(void)
+{
+	UT_UCSChar sz_ucs_FieldValue[FPFIELD_MAX_LENGTH + 1];
+	sz_ucs_FieldValue[0] = 0;
+	
+	char szFieldValue[FPFIELD_MAX_LENGTH + 1];
+
+	time_t	tim = time(NULL);
+	struct tm *pTime = localtime(&tim);
+
+	strftime(szFieldValue, FPFIELD_MAX_LENGTH, "%I:%M:%S %p", pTime);
+
+	UT_UCS_strcpy_char(sz_ucs_FieldValue, szFieldValue);
+
+	return _setValue(sz_ucs_FieldValue);
+}
+
+fp_FieldPageNumberRun::fp_FieldPageNumberRun(fl_BlockLayout* pBL, GR_Graphics* pG, UT_uint32 iOffsetFirst, UT_uint32 iLen) : fp_FieldRun(pBL, pG, iOffsetFirst, iLen)
+{
+}
+
+UT_Bool fp_FieldPageNumberRun::calculateValue(void)
+{
+	UT_UCSChar sz_ucs_FieldValue[FPFIELD_MAX_LENGTH + 1];
+	sz_ucs_FieldValue[0] = 0;
+	
+	char szFieldValue[FPFIELD_MAX_LENGTH + 1];
+
+	if (m_pLine && m_pLine->getContainer() && m_pLine->getContainer()->getPage())
+	{
+		fp_Page* pPage = m_pLine->getContainer()->getPage();
+		FL_DocLayout* pDL = pPage->getDocLayout();
+
+		UT_sint32 iPageNum = 0;
+		UT_uint32 iNumPages = pDL->countPages();
+		for (UT_uint32 i=0; i<iNumPages; i++)
+		{
+			fp_Page* pPg = pDL->getNthPage(i);
+
+			if (pPg == pPage)
+			{
+				iPageNum = i + 1;
+				break;
+			}
+		}
+
+		UT_ASSERT(iPageNum > 0);
+
+		sprintf(szFieldValue, "%d", iPageNum);
+	}
+	else
+	{
+		strcpy(szFieldValue, "?");
+	}
+
+	UT_UCS_strcpy_char(sz_ucs_FieldValue, szFieldValue);
+
+	return _setValue(sz_ucs_FieldValue);
+}
+
+fp_FieldPageCountRun::fp_FieldPageCountRun(fl_BlockLayout* pBL, GR_Graphics* pG, UT_uint32 iOffsetFirst, UT_uint32 iLen) : fp_FieldRun(pBL, pG, iOffsetFirst, iLen)
+{
+}
+
+UT_Bool fp_FieldPageCountRun::calculateValue(void)
+{
+	UT_UCSChar sz_ucs_FieldValue[FPFIELD_MAX_LENGTH + 1];
+	sz_ucs_FieldValue[0] = 0;
+	
+	char szFieldValue[FPFIELD_MAX_LENGTH + 1];
+	
+	if (m_pLine && m_pLine->getContainer() && m_pLine->getContainer()->getPage())
+	{
+
+		fp_Page* pPage = m_pLine->getContainer()->getPage();
+		FL_DocLayout* pDL = pPage->getDocLayout();
+
+		sprintf(szFieldValue, "%d", pDL->countPages());
+	}
+	else
+	{
+		strcpy(szFieldValue, "?");
+	}
+
+	UT_UCS_strcpy_char(sz_ucs_FieldValue, szFieldValue);
+
+	return _setValue(sz_ucs_FieldValue);
+}
+
+
+
 //////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////
 
@@ -1174,6 +1390,9 @@ fp_ForcedColumnBreakRun::fp_ForcedColumnBreakRun(fl_BlockLayout* pBL, GR_Graphic
 
 void fp_ForcedColumnBreakRun::lookupProperties(void)
 {
+
+        m_pBL->getField(m_iOffsetFirst,m_pField);
+
 }
 
 UT_Bool fp_ForcedColumnBreakRun::canContainPoint(void) const
@@ -1267,6 +1486,7 @@ fp_ForcedPageBreakRun::fp_ForcedPageBreakRun(fl_BlockLayout* pBL, GR_Graphics* p
 
 void fp_ForcedPageBreakRun::lookupProperties(void)
 {
+        m_pBL->getField(m_iOffsetFirst,m_pField);
 }
 
 UT_Bool fp_ForcedPageBreakRun::canContainPoint(void) const
@@ -1344,3 +1564,11 @@ void fp_ForcedPageBreakRun::_draw(dg_DrawArgs* pDA)
     FREEP(pPageBreak);
 }
 
+
+
+
+
+
+
+
+
