@@ -30,6 +30,7 @@
 #include "ut_misc.h"
 #include "pt_Types.h"
 #include "ut_assert.h"
+#include "ap_Strings.h"
 
 
 class UT_GrowBuf;
@@ -42,6 +43,7 @@ class PD_Document;
 class PP_AttrProp;
 struct dg_DrawArgs;
 class fl_CharWidths;
+class fd_Field;
 
 struct fp_RunSplitInfo
 {
@@ -77,7 +79,9 @@ enum FP_RUN_TYPE
 	FPRUN_FORCEDPAGEBREAK			= 6,
 	FPRUN_FIELD						= 7,
 	FPRUN_FMTMARK					= 8,
-	FPRUN__LAST__					= 8
+	FPRUN_FIELDSTARTRUN					= 9,
+	FPRUN_FIELDENDRUN					= 10,
+	FPRUN__LAST__					= 11
 };
 
 /*
@@ -91,6 +95,8 @@ enum FP_RUN_TYPE
 		fp_ImageRun
 		fp_FieldRun
 		fp_FmtMarkRun
+		fp_FieldStartRun
+		fp_FieldEndRun
 
 	As far as the formatter's concerned, each subclass behaves somewhat 
 	differently, but they can all be treated like rectangular blocks to 
@@ -110,6 +116,7 @@ public:
 	inline UT_sint32		getX(void) const 				{ return m_iX; }
 	inline UT_sint32		getY(void) const 				{ return m_iY; }
 	inline UT_sint32		getHeight(void) const     		{ return m_iHeight; }
+	inline UT_sint32		getHeightInLayoutUnits(void) const     		{ return m_iHeightLayoutUnits; }
 	inline UT_sint32		getWidth(void) const     		{ return m_iWidth; }
 	inline UT_sint32		getWidthInLayoutUnits(void) const     { return m_iWidthLayoutUnits; }
 	inline fp_Run* 			getNext(void) const     		{ return m_pNext; }
@@ -123,6 +130,8 @@ public:
 	inline UT_uint32		getDescentInLayoutUnits(void) const 	{ return m_iDescentLayoutUnits; }
 	void					insertIntoRunListBeforeThis(fp_Run& newRun);
 	void					insertIntoRunListAfterThis(fp_Run& newRun);
+	fd_Field *                      getField(void) { return   m_pField;}
+	UT_Bool                         isField(void) { return (UT_Bool) (m_pField != NULL) ;}
 	void					unlinkFromRunList();
 	
 	void					setLine(fp_Line*);
@@ -159,7 +168,7 @@ public:
 	virtual UT_Bool			findMaxLeftFitSplitPointInLayoutUnits(UT_sint32 iMaxLeftWidth, fp_RunSplitInfo& si, UT_Bool bForce=UT_FALSE) = 0;
 	virtual UT_sint32		findTrailingSpaceDistance(void) const { return 0; }	
 	virtual UT_sint32		findTrailingSpaceDistanceInLayoutUnits(void) const { return 0; }	
-	virtual UT_Bool			findFirstNonBlankSplitPoint(fp_RunSplitInfo& si) { return UT_FALSE; }
+	virtual UT_Bool			findFirstNonBlankSplitPoint(fp_RunSplitInfo& /*si*/) { return UT_FALSE; }
 	virtual void			mapXYToPosition(UT_sint32 xPos, UT_sint32 yPos, PT_DocPosition& pos, UT_Bool& bBOL, UT_Bool& bEOL) = 0;
 	virtual void 			findPointCoords(UT_uint32 iOffset, UT_sint32& x, UT_sint32& y, UT_sint32& height) = 0;
 	virtual void			lookupProperties(void) = 0;
@@ -204,6 +213,7 @@ protected:
 	UT_uint32				m_iDescentLayoutUnits;
 	GR_Graphics*            m_pG;
 	UT_Bool					m_bDirty;		// run erased @ old coords, needs to be redrawn
+	fd_Field *                 m_pField;
 
 private:
 	fp_Run(const fp_Run&);			// no impl.
@@ -252,6 +262,46 @@ protected:
 	virtual void       		_clearScreen(UT_Bool bFullLineHeightRect);
 };
 
+class fp_FieldStartRun : public fp_Run
+{
+ public:
+	fp_FieldStartRun(fl_BlockLayout* pBL, GR_Graphics* pG, UT_uint32 iOffsetFirst, UT_uint32 iLen);
+
+	virtual UT_Bool			canContainPoint(void) const;
+	virtual UT_uint32 		containsOffset(UT_uint32 iOffset);
+	virtual void			lookupProperties(void);
+	virtual void			mapXYToPosition(UT_sint32 xPos, UT_sint32 yPos, PT_DocPosition& pos, UT_Bool& bBOL, UT_Bool& bEOL);
+	virtual void 			findPointCoords(UT_uint32 iOffset, UT_sint32& x, UT_sint32& y, UT_sint32& height);
+	virtual UT_Bool			canBreakAfter(void) const;
+	virtual UT_Bool			canBreakBefore(void) const;
+	virtual UT_Bool			letPointPass(void) const;
+	virtual UT_Bool			isForcedBreak(void) const { return UT_TRUE; }
+	virtual UT_Bool			findMaxLeftFitSplitPointInLayoutUnits(UT_sint32 iMaxLeftWidth, fp_RunSplitInfo& si, UT_Bool bForce=UT_FALSE);
+protected:
+	virtual void			_draw(dg_DrawArgs*);
+	virtual void       		_clearScreen(UT_Bool bFullLineHeightRect);
+};
+
+class fp_FieldEndRun : public fp_Run
+{
+ public:
+	fp_FieldEndRun(fl_BlockLayout* pBL, GR_Graphics* pG, UT_uint32 iOffsetFirst, UT_uint32 iLen);
+
+	virtual UT_Bool			canContainPoint(void) const;
+	virtual UT_uint32 		containsOffset(UT_uint32 iOffset);
+	virtual void			lookupProperties(void);
+	virtual void			mapXYToPosition(UT_sint32 xPos, UT_sint32 yPos, PT_DocPosition& pos, UT_Bool& bBOL, UT_Bool& bEOL);
+	virtual void 			findPointCoords(UT_uint32 iOffset, UT_sint32& x, UT_sint32& y, UT_sint32& height);
+	virtual UT_Bool			canBreakAfter(void) const;
+	virtual UT_Bool			canBreakBefore(void) const;
+	virtual UT_Bool			letPointPass(void) const;
+	virtual UT_Bool			isForcedBreak(void) const { return UT_TRUE; }
+	virtual UT_Bool			findMaxLeftFitSplitPointInLayoutUnits(UT_sint32 iMaxLeftWidth, fp_RunSplitInfo& si, UT_Bool bForce=UT_FALSE);
+protected:
+	virtual void			_draw(dg_DrawArgs*);
+	virtual void       		_clearScreen(UT_Bool bFullLineHeightRect);
+};
+
 class fp_ForcedColumnBreakRun : public fp_Run
 {
  public:
@@ -320,6 +370,8 @@ protected:
 #define FPFIELD_TIME		1
 #define FPFIELD_PAGE_NUMBER	2
 #define FPFIELD_PAGE_COUNT	3
+#define FPFIELD_LIST_LABEL	4
+=======
 */
 
 #define  _FIELD(type,desc,tag)  /*nothing*/
@@ -346,58 +398,100 @@ FPFIELD_end };
 #undef  _FIELD
 #undef  _FIELDTYPE
 
+
 struct fp_FieldTypeData
 {
     fp_FieldTypesEnum  m_Type;
-	char*              m_Desc;
+    const char*        m_Desc;
+    XAP_String_Id      m_DescId;
 };
 
 struct fp_FieldData
 {
     fp_FieldTypesEnum  m_Type;
     fp_FieldsEnum      m_Num;
-	char*              m_Desc;
-	char*              m_Tag;
+    const char*        m_Desc;
+    const char*        m_Tag;
+    XAP_String_Id      m_DescId;
 };
 
-extern const fp_FieldTypeData fp_FieldTypes[];
-extern const fp_FieldData fp_FieldFmts[];
+
+extern fp_FieldTypeData fp_FieldTypes[];
+extern fp_FieldData fp_FieldFmts[];
 
 class fp_FieldRun : public fp_Run
 {
 public:
 	fp_FieldRun(fl_BlockLayout* pBL, GR_Graphics* pG, UT_uint32 iOffsetFirst, UT_uint32 iLen);
+	virtual ~fp_FieldRun() {};
 
 	virtual void			lookupProperties(void);
 	virtual void			mapXYToPosition(UT_sint32 xPos, UT_sint32 yPos, PT_DocPosition& pos, UT_Bool& bBOL, UT_Bool& bEOL);
 	virtual void 			findPointCoords(UT_uint32 iOffset, UT_sint32& x, UT_sint32& y, UT_sint32& height);
 	virtual UT_Bool			canBreakAfter(void) const;
+	virtual fp_FieldsEnum	getFieldType(void) const;
 	virtual UT_Bool			canBreakBefore(void) const;
 	virtual UT_Bool			letPointPass(void) const;
 	virtual UT_Bool			findMaxLeftFitSplitPointInLayoutUnits(UT_sint32 iMaxLeftWidth, fp_RunSplitInfo& si, UT_Bool bForce=UT_FALSE);
 
-	UT_Bool					calculateValue(void);
-        virtual UT_Bool                 isSuperscript(void) const;
-        virtual UT_Bool                 isSubscript(void) const;
+	virtual UT_Bool			isSuperscript(void) const;
+	virtual UT_Bool			isSubscript(void) const;
+
+	UT_Bool					_setValue(UT_UCSChar *p_new_value);					
+
+	virtual UT_Bool			calculateValue(void);
 	
 protected:
-	virtual void			_draw(dg_DrawArgs*);
+	virtual void			_draw(dg_DrawArgs*) {};
+	virtual void			_defaultDraw(dg_DrawArgs*);
 	virtual void       		_clearScreen(UT_Bool bFullLineHeightRect);
 
 	GR_Font*				m_pFont;
 	GR_Font*				m_pFontLayout;
 	UT_RGBColor				m_colorFG;
+	UT_RGBColor				m_colorBG;
 	UT_UCSChar				m_sFieldValue[FPFIELD_MAX_LENGTH];
 	fp_FieldsEnum			m_iFieldType;
         enum
         {
-                TEXT_POSITION_NORMAL,
-                TEXT_POSITION_SUPERSCRIPT,
-                TEXT_POSITION_SUBSCRIPT
+			TEXT_POSITION_NORMAL,
+			TEXT_POSITION_SUPERSCRIPT,
+			TEXT_POSITION_SUBSCRIPT
         };
-        UT_Byte                         m_fPosition;
+	UT_Byte                         m_fPosition;
 };
 
+class fp_FieldTimeRun : public fp_FieldRun
+{
+public:
+	
+	fp_FieldTimeRun(fl_BlockLayout* pBL, GR_Graphics* pG, UT_uint32 iOffsetFirst, UT_uint32 iLen);
+
+	virtual UT_Bool			calculateValue(void);
+	virtual void			_draw(dg_DrawArgs* pDA) { _defaultDraw(pDA); }
+};
+
+class fp_FieldPageNumberRun : public fp_FieldRun
+{
+public:
+	
+	fp_FieldPageNumberRun(fl_BlockLayout* pBL, GR_Graphics* pG, UT_uint32 iOffsetFirst, UT_uint32 iLen);
+
+	virtual UT_Bool			calculateValue(void);
+	virtual void			_draw(dg_DrawArgs* pDA) { _defaultDraw(pDA); }
+};
+
+class fp_FieldPageCountRun : public fp_FieldRun
+{
+public:
+	
+	fp_FieldPageCountRun(fl_BlockLayout* pBL, GR_Graphics* pG, UT_uint32 iOffsetFirst, UT_uint32 iLen);
+
+	virtual UT_Bool			calculateValue(void);
+	virtual void			_draw(dg_DrawArgs* pDA) { _defaultDraw(pDA); }
+};
+
+
 class fp_FmtMarkRun : public fp_Run
 {
 public:
@@ -420,9 +514,9 @@ protected:
 	virtual void       		_clearScreen(UT_Bool bFullLineHeightRect);
         enum
         {
-                TEXT_POSITION_NORMAL,
-                TEXT_POSITION_SUPERSCRIPT,
-                TEXT_POSITION_SUBSCRIPT
+            TEXT_POSITION_NORMAL,
+            TEXT_POSITION_SUPERSCRIPT,
+            TEXT_POSITION_SUBSCRIPT
         };
         UT_Byte                         m_fPosition;
 };
