@@ -37,6 +37,7 @@
 #include "ut_growbuf.h"
 #include "ut_units.h"
 
+#include "xap_EncodingManager.h"
 /*****************************************************************/
 
 fp_TextRun::fp_TextRun(fl_BlockLayout* pBL,
@@ -53,7 +54,7 @@ fp_TextRun::fp_TextRun(fl_BlockLayout* pBL,
 	m_iLineWidth = 0;
 	m_bSquiggled = UT_FALSE;
 	m_iSpaceWidthBeforeJustification = JUSTIFICATION_NOT_USED;
-
+        m_pField = NULL;
 	if (bLookupProperties)
 	{
 		lookupProperties();
@@ -71,10 +72,9 @@ void fp_TextRun::lookupProperties(void)
 	const PP_AttrProp * pSpanAP = NULL;
 	const PP_AttrProp * pBlockAP = NULL;
 	const PP_AttrProp * pSectionAP = NULL; // TODO do we care about section-level inheritance?
-	
 	m_pBL->getSpanAttrProp(m_iOffsetFirst,UT_FALSE,&pSpanAP);
 	m_pBL->getAttrProp(&pBlockAP);
-
+        static_cast<fl_Layout *>(m_pBL)->getField(m_iOffsetFirst,m_pField);
 	// look for fonts in this DocLayout's font cache
 	FL_DocLayout * pLayout = m_pBL->getDocLayout();
 	m_pFont = pLayout->findFont(pSpanAP,pBlockAP,pSectionAP, FL_DocLayout::FIND_FONT_AT_SCREEN_RESOLUTION);
@@ -166,7 +166,7 @@ UT_Bool fp_TextRun::canBreakAfter(void) const
 			{
 				UT_ASSERT(len>0);
 
-				if (pSpan[len-1] == UCS_SPACE)
+				if (XAP_EncodingManager::instance->can_break_at(pSpan[len-1]))
 				{
 					return UT_TRUE;
 				}
@@ -204,7 +204,7 @@ UT_Bool fp_TextRun::canBreakBefore(void) const
 		{
 			UT_ASSERT(lenSpan>0);
 
-			if (pSpan[0] == UCS_SPACE)
+			if (XAP_EncodingManager::instance->can_break_at(pSpan[0]))
 			{
 				return UT_TRUE;
 			}
@@ -286,10 +286,18 @@ UT_Bool	fp_TextRun::findMaxLeftFitSplitPointInLayoutUnits(UT_sint32 iMaxLeftWidt
 		{
 			iLeftWidth += pCharWidths[i + offset];
 			iRightWidth -= pCharWidths[i + offset];
-
-			if (
+#if 0
+    /*
+	FIXME: this is a direct equivalent to HJ's patch, but other branch
+	could be more correct than this one. - VH
+    */
+			if ( (XAP_EncodingManager::instance->can_break_at(pSpan[i]) && pSpan[i]!=UCS_SPACE) ||
 				(
 					(UCS_SPACE == pSpan[i])
+#else
+			if (
+				(XAP_EncodingManager::instance->can_break_at(pSpan[i]) 
+#endif					
 					&& ((i + offset) != (m_iOffsetFirst + m_iLen - 1))
 					)
 				|| bForce
@@ -445,6 +453,7 @@ UT_Bool fp_TextRun::canMergeWithNext(void)
 		|| (m_iHeight != pNext->m_iHeight)
 		|| (m_iSpaceWidthBeforeJustification != JUSTIFICATION_NOT_USED)
 		|| (pNext->m_iSpaceWidthBeforeJustification != JUSTIFICATION_NOT_USED)
+                || (pNext->m_pField != m_pField)
 		)
 	{
 		return UT_FALSE;
@@ -471,7 +480,7 @@ void fp_TextRun::mergeWithNext(void)
 	UT_ASSERT(m_iLineWidth == pNext->m_iLineWidth);
 //	UT_ASSERT(m_iSpaceWidthBeforeJustification == pNext->m_iSpaceWidthBeforeJustification);
 
-
+        m_pField = pNext->m_pField;
 	m_iWidth += pNext->m_iWidth;
 	m_iWidthLayoutUnits += pNext->m_iWidthLayoutUnits;
 	m_iLen += pNext->m_iLen;
@@ -498,6 +507,7 @@ UT_Bool fp_TextRun::split(UT_uint32 iSplitOffset)
 	pNew->m_pFontLayout = this->m_pFontLayout;
 	pNew->m_fDecorations = this->m_fDecorations;
 	pNew->m_colorFG = this->m_colorFG;
+	pNew->m_pField = this->m_pField;
         pNew->m_fPosition = this->m_fPosition;
 
 	pNew->m_iAscent = this->m_iAscent;
@@ -523,11 +533,11 @@ UT_Bool fp_TextRun::split(UT_uint32 iSplitOffset)
 
 	m_pLine->insertRunAfter(pNew, this);
 
-	m_iWidth = simpleRecalcWidth();
+	m_iWidth = simpleRecalcWidth(Width_type_display);
 	m_iWidthLayoutUnits = simpleRecalcWidth(Width_type_layout_units);
 	pNew->m_iX = m_iX + m_iWidth;
 	pNew->m_iY = m_iY;
-	pNew->m_iWidth = pNew->simpleRecalcWidth();
+	pNew->m_iWidth = pNew->simpleRecalcWidth(Width_type_display);
 	pNew->m_iWidthLayoutUnits = pNew->simpleRecalcWidth(Width_type_layout_units);
 	
 	return UT_TRUE;
@@ -583,8 +593,22 @@ void fp_TextRun::fetchCharWidths(fl_CharWidths * pgbCharWidths)
 
 }
 
-UT_sint32 fp_TextRun::simpleRecalcWidth(UT_sint32 iWidthType) const
+UT_sint32 fp_TextRun::simpleRecalcWidth(UT_sint32 iWidthType, UT_sint32 iLength) const
 {
+
+	if(iLength == Calculate_full_width)
+	{
+		iLength = m_iLen;
+	}
+	UT_ASSERT(iLength >= 0);
+
+	UT_ASSERT((UT_uint32)iLength <= m_iLen);
+	if((UT_uint32)iLength > m_iLen)
+		iLength = (UT_sint32)m_iLen;
+
+	if (iLength == 0)
+		return 0;
+
 	UT_GrowBuf * pgbCharWidths;
 	switch(iWidthType)
 	{
@@ -605,14 +629,12 @@ UT_sint32 fp_TextRun::simpleRecalcWidth(UT_sint32 iWidthType) const
 	UT_uint16* pCharWidths = pgbCharWidths->getPointer(0);
 	
 	UT_sint32 iWidth = 0;
-	if (m_iLen == 0)
-		return 0;
 
 	{
 		const UT_UCSChar* pSpan;
 		UT_uint32 lenSpan;
 		UT_uint32 offset = m_iOffsetFirst;
-		UT_uint32 len = m_iLen;
+		UT_uint32 len = iLength;
 		UT_Bool bContinue = UT_TRUE;
 
 		while (bContinue)
@@ -692,23 +714,15 @@ void fp_TextRun::_clearScreen(UT_Bool /* bFullLineHeightRect */)
 		  changed, for things such as table cells.
 		*/
 		UT_RGBColor clrNormalBackground(255,255,255);
+		if (m_pField)
+		{
+		        UT_setColor(clrNormalBackground,220, 220, 220);
+		}
 		m_pG->setColor(clrNormalBackground);
 		
 		UT_sint32 xoff = 0, yoff = 0;
 		m_pLine->getScreenOffsets(this, xoff, yoff);
-		
-		if(!m_pLine->isEmpty() && m_pLine->getFirstRun() == this)
-		{
-			// First run on the line so add extra at start to clear any glyph before margin.
-			// Left margin only.
-			UT_sint32 halfcolumnGap = m_pLine->getColumnGap() / 2;
-
-			m_pG->clearArea(xoff - halfcolumnGap, yoff, m_iWidth + halfcolumnGap, m_pLine->getHeight());
-		}
-		else
-		{
-		        m_pG->clearArea(xoff, yoff, m_iWidth, m_pLine->getHeight());
-		}
+		m_pG->clearArea(xoff, yoff, m_iWidth, m_pLine->getHeight());
 	}
 
 }
@@ -719,8 +733,12 @@ void fp_TextRun::_draw(dg_DrawArgs* pDA)
 	  Upon entry to this function, pDA->yoff is the BASELINE of this run, NOT
 	  the top.
 	*/
+#ifndef NDEBUG
+	FV_View* ppView = m_pBL->getDocLayout()->getView();
+	if(ppView) UT_ASSERT(ppView && ppView->isCursorOn()==UT_FALSE);
+#endif
 
-        UT_sint32 yTopOfRun = pDA->yoff - m_iAscent-1; // Hack to remove
+	UT_sint32 yTopOfRun = pDA->yoff - m_iAscent-1; // Hack to remove
 	UT_sint32 yTopOfSel = yTopOfRun+1; // final character dirt
 
 	if (m_fPosition == TEXT_POSITION_SUPERSCRIPT)
@@ -744,8 +762,14 @@ void fp_TextRun::_draw(dg_DrawArgs* pDA)
 	  appropriate selection background color based on the color
 	  of the foreground text, probably.
 	*/
+	UT_RGBColor clrNormalBackground(255,255,255);
 	UT_RGBColor clrSelBackground(192, 192, 192);
-
+	if (m_pField)
+	{
+	         UT_setColor(clrNormalBackground,220, 220, 220);
+	         UT_setColor(clrSelBackground,112, 112, 112);
+	}
+    
 	UT_uint32 iRunBase = iBase + m_iOffsetFirst;
 
 	FV_View* pView = m_pBL->getDocLayout()->getView();
@@ -760,6 +784,7 @@ void fp_TextRun::_draw(dg_DrawArgs* pDA)
 	if (pView->getFocus()==AV_FOCUS_NONE || iSel1 == iSel2)
 	{
 		// nothing in this run is selected
+                _fillRect(clrNormalBackground, pDA->xoff, yTopOfSel, m_iOffsetFirst, m_iLen, pgbCharWidths);
 		_drawPart(pDA->xoff, yTopOfRun, m_iOffsetFirst, m_iLen, pgbCharWidths);
 	}
 	else if (iSel1 <= iRunBase)
@@ -767,6 +792,7 @@ void fp_TextRun::_draw(dg_DrawArgs* pDA)
 		if (iSel2 <= iRunBase)
 		{
 			// nothing in this run is selected
+                        _fillRect(clrNormalBackground, pDA->xoff, yTopOfSel, m_iOffsetFirst, m_iLen, pgbCharWidths);
 			_drawPart(pDA->xoff, yTopOfRun, m_iOffsetFirst, m_iLen, pgbCharWidths);
 		}
 		else if (iSel2 >= (iRunBase + m_iLen))
@@ -782,17 +808,19 @@ void fp_TextRun::_draw(dg_DrawArgs* pDA)
 
 			_fillRect(clrSelBackground, pDA->xoff, yTopOfSel, m_iOffsetFirst, iSel2 - iRunBase, pgbCharWidths);
 			_drawPart(pDA->xoff, yTopOfRun, m_iOffsetFirst, iSel2 - iRunBase, pgbCharWidths);
-
+            _fillRect(clrNormalBackground, pDA->xoff, yTopOfSel, iSel2 - iBase, m_iLen - (iSel2 - iRunBase), pgbCharWidths);
 			_drawPart(pDA->xoff, yTopOfRun, iSel2 - iBase, m_iLen - (iSel2 - iRunBase), pgbCharWidths);
 		}
 	}
 	else if (iSel1 >= (iRunBase + m_iLen))
 	{
 		// nothing in this run is selected
+        _fillRect(clrNormalBackground, pDA->xoff, yTopOfSel, m_iOffsetFirst, m_iLen, pgbCharWidths);
 		_drawPart(pDA->xoff, yTopOfRun, m_iOffsetFirst, m_iLen, pgbCharWidths);
 	}
 	else
 	{
+        _fillRect(clrNormalBackground, pDA->xoff, yTopOfSel, m_iOffsetFirst, iSel1 - iRunBase, pgbCharWidths);
 		_drawPart(pDA->xoff, yTopOfRun, m_iOffsetFirst, iSel1 - iRunBase, pgbCharWidths);
 		
 		if (iSel2 >= (iRunBase + m_iLen))
@@ -804,15 +832,16 @@ void fp_TextRun::_draw(dg_DrawArgs* pDA)
 		{
 			_fillRect(clrSelBackground, pDA->xoff, yTopOfSel, iSel1 - iBase, iSel2 - iSel1, pgbCharWidths);
 			_drawPart(pDA->xoff, yTopOfRun, iSel1 - iBase, iSel2 - iSel1, pgbCharWidths);
-
+            _fillRect(clrNormalBackground, pDA->xoff, yTopOfSel, iSel2 - iBase, m_iLen - (iSel2 - iRunBase), pgbCharWidths);
 			_drawPart(pDA->xoff, yTopOfRun, iSel2 - iBase, m_iLen - (iSel2 - iRunBase), pgbCharWidths);
 		}
 	}
 
 	_drawDecors(pDA->xoff, yTopOfRun);
-    if(pView->getShowPara()){
-        _drawInvisibles(pDA->xoff, yTopOfRun);
-    }
+	if(pView->getShowPara())
+	{
+                _drawInvisibles(pDA->xoff, yTopOfRun);
+	}
 
 	// TODO: draw this underneath (ie, before) the text and decorations
 	m_bSquiggled = UT_FALSE;
@@ -830,7 +859,11 @@ void fp_TextRun::_fillRect(UT_RGBColor& clr,
 	  Upon entry to this function, yoff is the TOP of the run,
 	  NOT the baseline.
 	*/
-	
+#ifndef NDEBUG
+	FV_View* ppView = m_pBL->getDocLayout()->getView();
+	if(ppView) UT_ASSERT(ppView && ppView->isCursorOn()==UT_FALSE);
+#endif
+
 	if (m_pG->queryProperties(GR_Graphics::DGP_SCREEN))
 	{
 		UT_Rect r;
@@ -894,8 +927,8 @@ void fp_TextRun::_drawPart(UT_sint32 xoff,
 	  NOT the baseline.
 	*/
 	
-	const UT_UCSChar* pSpan;
-	UT_uint32 lenSpan;
+	const UT_UCSChar* pSpan = NULL;
+	UT_uint32 lenSpan = 0;
 	UT_uint32 offset = iStart;
 	UT_uint32 len = iLen;
 	UT_Bool bContinue = UT_TRUE;
@@ -1125,8 +1158,8 @@ void fp_TextRun::_drawInvisibleSpaces(UT_sint32 xoff, UT_sint32 yoff)
 {
     UT_GrowBuf * pgbCharWidths = m_pBL->getCharWidths()->getCharWidths();
     UT_uint16* pCharWidths = pgbCharWidths->getPointer(0);
-    const UT_UCSChar* pSpan;
-    UT_uint32 lenSpan;
+    const UT_UCSChar* pSpan = NULL;
+    UT_uint32 lenSpan = 0;
     UT_uint32 len = m_iLen;
     UT_sint32 iWidth = 0;
     UT_sint32 cur_linewidth = 1+ (UT_MAX(10,m_iAscent)-10)/8;
@@ -1134,6 +1167,11 @@ void fp_TextRun::_drawInvisibleSpaces(UT_sint32 xoff, UT_sint32 yoff)
     UT_Bool bContinue = UT_TRUE;
     UT_uint32 offset = m_iOffsetFirst;
 
+#ifndef NDEBUG
+    FV_View* ppView = m_pBL->getDocLayout()->getView();
+    if(ppView) UT_ASSERT(ppView && ppView->isCursorOn()==UT_FALSE);
+#endif
+
     if(findCharacter(0, UCS_SPACE) > 0){
         while(bContinue){
             bContinue = m_pBL->getSpanPtr(offset,&pSpan,&lenSpan);
@@ -1165,10 +1203,15 @@ void fp_TextRun::_drawInvisibleSpaces(UT_sint32 xoff, UT_sint32 yoff)
 
 void fp_TextRun::_drawInvisibles(UT_sint32 xoff, UT_sint32 yoff)
 {
-    if (!(m_pG->queryProperties(GR_Graphics::DGP_SCREEN))){
-        return;
-    }
-    _drawInvisibleSpaces(xoff,yoff);
+#ifndef NDEBUG
+	FV_View* ppView = m_pBL->getDocLayout()->getView();
+	if(ppView) UT_ASSERT(ppView && ppView->isCursorOn()==UT_FALSE);
+#endif
+
+        if (!(m_pG->queryProperties(GR_Graphics::DGP_SCREEN))){
+                return;
+        }
+        _drawInvisibleSpaces(xoff,yoff);
 }
 
 void fp_TextRun::_drawSquiggle(UT_sint32 top, UT_sint32 left, UT_sint32 right)
@@ -1188,7 +1231,15 @@ void fp_TextRun::_drawSquiggle(UT_sint32 top, UT_sint32 left, UT_sint32 right)
 			to move the coordinates into a platform-specific point 
 			structure.  They're all x, y but different widths.  Bummer. 
 	*/
-	UT_Point * points = (UT_Point *)calloc(nPoints, sizeof(UT_Point));
+	UT_Point * points, scratchpoints[100];
+	if ((unsigned)nPoints < (sizeof(scratchpoints)/sizeof(scratchpoints[0])))
+	{
+		points = scratchpoints;
+	}
+	else
+	{
+		points = new UT_Point[nPoints];
+	}
 	UT_ASSERT(points);
 
 	points[0].x = left;
@@ -1210,7 +1261,7 @@ void fp_TextRun::_drawSquiggle(UT_sint32 top, UT_sint32 left, UT_sint32 right)
 
 	m_pG->polyLine(points, nPoints);
 
-	FREEP(points);
+	if (points != scratchpoints) delete points;
 }
 
 void fp_TextRun::drawSquiggle(UT_uint32 iOffset, UT_uint32 iLen)
@@ -1246,8 +1297,8 @@ UT_sint32 fp_TextRun::findCharacter(UT_uint32 startPosition, UT_UCSChar Characte
 {
 	// NOTE: startPosition is run-relative
 	// NOTE: return value is block-relative (don't ask me why)
-	const UT_UCSChar* pSpan;
-	UT_uint32 lenSpan;
+	const UT_UCSChar* pSpan = NULL;
+	UT_uint32 lenSpan = 0;
 	UT_uint32 offset = m_iOffsetFirst + startPosition;
 	UT_uint32 len = m_iLen - startPosition;
 	UT_Bool bContinue = UT_TRUE;
@@ -1295,8 +1346,8 @@ UT_sint32 fp_TextRun::findCharacter(UT_uint32 startPosition, UT_UCSChar Characte
 
 UT_Bool fp_TextRun::getCharacter(UT_uint32 run_offset, UT_UCSChar &Character) const
 {
-	const UT_UCSChar* pSpan;
-	UT_uint32 lenSpan;
+	const UT_UCSChar* pSpan = NULL;
+	UT_uint32 lenSpan = 0;
 
 	UT_ASSERT(run_offset < m_iLen);
 
@@ -1345,8 +1396,8 @@ UT_Bool	fp_TextRun::doesContainNonBlankData(void) const
 {
 	if(m_iLen > 0)
 	{
-		const UT_UCSChar* pSpan;
-		UT_uint32 lenSpan;
+		const UT_UCSChar* pSpan = NULL;
+		UT_uint32 lenSpan = 0;
 		UT_uint32 offset = m_iOffsetFirst;
 		UT_uint32 len = m_iLen;
 		UT_Bool bContinue = UT_TRUE;
@@ -1630,6 +1681,18 @@ UT_uint32 fp_TextRun::countJustificationPoints(void) const
 	return iCount;
 }
 
+UT_Bool fp_TextRun::canContainPoint(void) const
+{
+	if (m_pField) 
+	{
+		return UT_FALSE;
+	}
+	else
+	{
+		return UT_TRUE;
+	}
+}
+
 //////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////
 
@@ -1643,8 +1706,8 @@ void fp_TextRun::__dump(FILE * fp) const
 
 	fprintf(fp,"      [");
 	{
-		const UT_UCSChar* pSpan;
-		UT_uint32 lenSpan;
+		const UT_UCSChar* pSpan = NULL;
+		UT_uint32 lenSpan = 0;
 
 		UT_uint32 koff=m_iOffsetFirst;
 		UT_uint32 klen=m_iLen;
