@@ -63,7 +63,11 @@ class fp_TextRun : public fp_Run
 		Width_type_display,
 		Width_type_layout_units
 	};
-	UT_sint32				simpleRecalcWidth(UT_sint32 iWidthType = Width_type_display) const;
+	enum
+	{
+		Calculate_full_width = -1
+	};
+	UT_sint32				simpleRecalcWidth(UT_sint32 iWidthType, UT_sint32 iLength = Calculate_full_width) const;
 
 	void					resetJustification();
 	void					distributeJustificationAmongstSpaces(UT_sint32 iAmount, UT_uint32 iSpacesInRun);
@@ -90,7 +94,7 @@ class fp_TextRun : public fp_Run
 	virtual void                            setMinOverline(UT_sint32 xoff);
 	virtual UT_sint32                       getMinOverline(void);
 	UT_uint32				countTrailingSpaces(void) const;
-
+    UT_Bool                 canContainPoint(void) const;
 #ifdef FMT_TEST
 	virtual void			__dump(FILE * fp) const;
 #endif	
