@@ -31,6 +31,7 @@
 #include "ie_types.h"
 #include "xap_Prefs.h"
 #include "ap_Dialog_Goto.h"
+#include "fl_AutoLists.h"
 
 // number of milliseconds between cursor blinks
 const int AUTO_DRAW_POINT = 600;
@@ -51,6 +52,7 @@ class XAP_App;
 class XAP_Prefs;
 class UT_AlphaHashTable;
 class PP_AttrProp;
+class fl_AutoNum;
 
 typedef enum _FVDocPos
 {
@@ -62,6 +64,14 @@ typedef enum _FVDocPos
 	FV_DOCPOS_BOW, FV_DOCPOS_EOW_MOVE, FV_DOCPOS_EOW_SELECT	// word
 } FV_DocPos;
 
+typedef enum
+{
+        BreakSectionContinuous,
+	BreakSectionNextPage,
+	BreakSectionEvenPage,
+        BreakSectionOddPage
+} BreakSectionType;
+
 struct fv_ChangeState
 {
 	UT_Bool				bUndo;
@@ -124,7 +134,7 @@ public:
 
 	virtual void	getTopRulerInfo(AP_TopRulerInfo * pInfo);
 	virtual void	getLeftRulerInfo(AP_LeftRulerInfo * pInfo);
-	
+
 	virtual EV_EditMouseContext getMouseContext(UT_sint32 xPos, UT_sint32 yPos);
 	virtual EV_EditMouseContext getInsertionPointContext(UT_sint32 * pxPos, UT_sint32 * pyPos);
 
@@ -136,6 +146,7 @@ public:
 	FL_DocLayout* 	getLayout() const;
 	UT_uint32		getCurrentPageNumForStatusBar(void) const;
 	fp_Page*		getCurrentPage(void) const;
+	fl_BlockLayout *        getCurrentBlock(void);
 
 	void draw(int page, dg_DrawArgs* da);
 
@@ -149,23 +160,54 @@ public:
 	UT_Bool setSectionFormat(const XML_Char * properties[]);
 	UT_Bool getSectionFormat(const XML_Char *** properties);
 
+	UT_Bool isCursorOn(void);
+	void    eraseInsertionPoint(void);
+	void    drawInsertionPoint(void);
+
 	UT_Bool setBlockFormat(const XML_Char * properties[]);
 	UT_Bool getBlockFormat(const XML_Char *** properties,UT_Bool bExpandStyles=UT_TRUE);
 
+
+	UT_Bool isTabListBehindPoint(void);
+	UT_Bool isTabListAheadPoint(void);
+	void    processSelectedBlocks(List_Type listType);
+	void    getListBlocksInSelection( UT_Vector * vBlock);
+	UT_Bool isPointBeforeListLabel(void);
+	UT_Bool isCurrentListBlockEmpty(void);
+	UT_Bool cmdStartList(const XML_Char * style);
+	UT_Bool cmdStopList(void);
+	void    changeListStyle(fl_AutoNum* pAuto,
+							List_Type lType,
+							UT_uint32 startv,
+							const XML_Char* pszDelim,
+							const XML_Char* pszDecimal,
+							const XML_Char* pszFormat,
+							float Aligm,
+							float Indent);
+
+	void    setDontChangeInsPoint(void);
+	void    allowChangeInsPoint(void);
+	UT_Bool isDontChangeInsPoint(void);
+
+
 	UT_Bool setCharFormat(const XML_Char * properties[]);
 	UT_Bool getCharFormat(const XML_Char *** properties,UT_Bool bExpandStyles=UT_TRUE);
 
 	UT_Bool setStyle(const XML_Char * style);
 	UT_Bool getStyle(const XML_Char ** style);
 
+	UT_uint32		getCurrentPageNumber(void);
+
 	void insertParagraphBreak(void);
+	void insertParagraphBreaknoListUpdate(void);
+	void insertSectionBreak( BreakSectionType type);
 	void insertSectionBreak(void);
-	void insertSymbol(UT_UCSChar c, XML_Char * symfont, XML_Char * currentfont);
+	void insertSymbol(UT_UCSChar c, XML_Char * symfont);
 // ----------------------
 	UT_Bool			isLeftMargin(UT_sint32 xPos, UT_sint32 yPos);
 	void			cmdSelect(UT_sint32 xPos, UT_sint32 yPos, FV_DocPos dpBeg, FV_DocPos dpEnd);
 	void			cmdCharMotion(UT_Bool bForward, UT_uint32 count);
-    UT_Bool         cmdCharInsert(UT_UCSChar * text, UT_uint32 count, UT_Bool bForce = UT_FALSE);
+    UT_Bool			cmdCharInsert(UT_UCSChar * text, UT_uint32 count, UT_Bool bForce = UT_FALSE);
 	void			cmdCharDelete(UT_Bool bForward, UT_uint32 count);
 	void			delTo(FV_DocPos dp);
 	UT_UCSChar * 	getSelectionText(void);
@@ -214,10 +256,13 @@ public:
 	// finds the next "find" and selects it, filling bool when done the entire document
 	UT_Bool			findNext(const UT_UCSChar * find, UT_Bool matchCase = UT_TRUE, UT_Bool * bDoneEntireDocument = NULL);
 	UT_Bool			_findNext(const UT_UCSChar * find, UT_Bool matchCase = UT_TRUE, UT_Bool * bDoneEntireDocument = NULL);
+	UT_Bool			_findNext(const UT_UCSChar * find, UT_uint32 *prefix, UT_Bool matchCase = UT_TRUE, UT_Bool * bDoneEntireDocument = NULL);
 	// replaces the selection of "find" with "replace" and selects the next, filling
 	// bool when done the entire document
 	UT_Bool			_findReplace(const UT_UCSChar * find, const UT_UCSChar * replace,
 								 UT_Bool matchCase = UT_FALSE, UT_Bool * bDoneEntireDocument = NULL);
+	UT_Bool			_findReplace(const UT_UCSChar * find, const UT_UCSChar * replace, UT_uint32 *prefix,
+								 UT_Bool matchCase = UT_FALSE, UT_Bool * bDoneEntireDocument = NULL);
 	UT_Bool			findReplace(const UT_UCSChar * find, const UT_UCSChar * replace,
 								UT_Bool matchCase = UT_FALSE, UT_Bool * bDoneEntireDocument = NULL);
 	// replaces every occurance of "find" with "replace" without stopping for anything
@@ -234,8 +279,21 @@ public:
 
 	FV_DocCount                     countWords(void);
 
-    void            setShowPara(UT_Bool);    
-    inline UT_Bool  getShowPara(void) const { return m_bShowPara; };
+// -----------------------
+
+	UT_Bool                         insertPageNum(const XML_Char ** props, UT_Bool ftr);
+	void				setPoint(UT_uint32 pt);
+
+// -----------------------
+
+    void				setShowPara(UT_Bool);    
+    inline UT_Bool		getShowPara(void) const { return m_bShowPara; };
+	
+	const fp_PageSize&	getPageSize(void) const;
+	UT_uint32			calculateZoomPercentForPageWidth();
+   UT_uint32           calculateZoomPercentForPageHeight();
+	UT_uint32			calculateZoomPercentForWholePage();
+
 protected:
 	void				_generalUpdate(void);
 	
@@ -250,6 +308,7 @@ protected:
 	fp_Page *           _getCurrentPage(void);
 	void                _moveInsPtNthPage(UT_uint32 n);
 	void                _moveInsPtToPage(fp_Page *page);
+	void                _insertSectionBreak(void);
 
 	PT_DocPosition		_getDocPosFromPoint(PT_DocPosition iPoint, FV_DocPos dp, UT_Bool bKeepLooking=UT_TRUE);
 	PT_DocPosition		_getDocPos(FV_DocPos dp, UT_Bool bKeepLooking=UT_TRUE);
@@ -300,9 +359,11 @@ protected:
 	void				_doPaste(UT_Bool bUseClipboard);
 	void				_clearIfAtFmtMark(PT_DocPosition dpos);
 
-	void				_checkPendingWord(void);
+	void				_checkPendingWordForSpell(void);
 
 
+	UT_Bool				_insertHeaderFooter(const XML_Char ** props, UT_Bool ftr);
+
 
 	PT_DocPosition		m_iInsPoint;
 	UT_sint32			m_xPoint;
@@ -315,7 +376,7 @@ protected:
 
 	UT_Bool				m_bPointVisible;
 	UT_Bool				m_bPointEOL;
-
+	UT_Bool                         m_bDontChangeInsPoint;
 	FL_DocLayout*		m_pLayout;
 	PD_Document*		m_pDoc;
 	GR_Graphics*		m_pG;
@@ -335,7 +396,8 @@ protected:
 	UT_Bool				m_bCursorIsOn;
 	UT_Bool				m_bEraseSaysStopBlinking;
 	UT_Bool				m_bCursorBlink;
-	
+
+	UT_Bool                         m_bdontSpellCheckRightNow;
 	fv_ChangeState		m_chg;
 
 	// find and replace stuff
