@@ -36,6 +36,14 @@
 #include "ie_exp.h"
 #include "pf_Frag_Strux.h"
 #include "pd_Style.h"
+#include "pf_Frag_Object.h"
+#include "pf_Frag.h"
+#include "fd_Field.h"
+#include "fl_BlockLayout.h"
+#include "fl_Layout.h"
+#include "fl_DocLayout.h"
+#include "fv_View.h"
+#include "fl_AutoNum.h"
 
 
 //////////////////////////////////////////////////////////////////
@@ -51,7 +59,7 @@ struct _dataItemPair
 //////////////////////////////////////////////////////////////////
 
 PD_Document::PD_Document()
-	: AD_Document(), m_hashDataItems(11)
+  : AD_Document(), m_hashDataItems(11)
 {
 	m_pPieceTable = NULL;
 
@@ -59,6 +67,8 @@ PD_Document::PD_Document()
 	// but now we just depend on save() never being called without
 	// a previous saveAs() (which specifies a type)
 	m_lastSavedAsType = IEFT_AbiWord_1;
+	m_ballowListUpdates = UT_FALSE;
+        m_bPieceTableChanging = UT_FALSE;
 }
 
 PD_Document::~PD_Document()
@@ -95,7 +105,7 @@ UT_Error PD_Document::readFromFile(const char * szFilename, int ieft)
 	IE_Imp * pie = NULL;
 	UT_Error errorCode;
 
-	errorCode = IE_Imp::constructImporter(this,szFilename,(IEFileType) ieft,&pie);
+	errorCode = IE_Imp::constructImporter(this, szFilename, (IEFileType) ieft, &pie, &m_lastSavedAsType);
 	if (errorCode)
 	{
 		UT_DEBUGMSG(("PD_Document::readFromFile -- could not construct importer\n"));
@@ -119,10 +129,8 @@ UT_Error PD_Document::readFromFile(const char * szFilename, int ieft)
 		return UT_IE_NOMEMORY;
 	}
 
-	// save out file type so future saves know the type imported as
-	m_lastSavedAsType = (IEFileType) ieft;
-	
 	m_pPieceTable->setPieceTableState(PTS_Editing);
+	updateFields();
 	_setClean();							// mark the document as not-dirty
 	return UT_OK;
 }
@@ -156,7 +164,7 @@ UT_Error PD_Document::saveAs(const char * szFilename, int ieft)
 	IE_Exp * pie = NULL;
 	UT_Error errorCode;
 
-	errorCode = IE_Exp::constructExporter(this,szFilename,(IEFileType) ieft,&pie);
+	errorCode = IE_Exp::constructExporter(this, szFilename, (IEFileType) ieft, &pie, &m_lastSavedAsType);
 	if (errorCode)
 	{
 		UT_DEBUGMSG(("PD_Document::Save -- could not construct exporter\n"));
@@ -181,13 +189,10 @@ UT_Error PD_Document::saveAs(const char * szFilename, int ieft)
 	}
 	
 	char * szFilenameCopy = NULL;
-    if (!UT_cloneString(szFilenameCopy,szFilename))
+	if (!UT_cloneString(szFilenameCopy,szFilename))
 		return UT_SAVE_OTHERERROR;
 	m_szFilename = szFilenameCopy;
 
-	// save the type we just saved as
-	m_lastSavedAsType = (IEFileType) ieft;
-	
 	_setClean();
 	return UT_OK;
 }
@@ -196,6 +201,8 @@ UT_Error PD_Document::save(void)
 {
 	if (!m_szFilename || !*m_szFilename)
 		return UT_SAVE_NAMEERROR;
+	if (m_lastSavedAsType == IEFT_Unknown)
+		return UT_EXTENSIONERROR;
 
 	IE_Exp * pie = NULL;
 	UT_Error errorCode;
@@ -253,6 +260,20 @@ UT_Bool	PD_Document::insertObject(PT_DocPosition dpos,
 	return m_pPieceTable->insertObject(dpos, pto, attributes, properties);
 }
 
+
+
+UT_Bool	PD_Document::insertObject(PT_DocPosition dpos,
+										 PTObjectType pto,
+										 const XML_Char ** attributes,
+										 const XML_Char ** properties, fd_Field ** pField)
+{
+	pf_Frag_Object * pfo = NULL;
+	UT_Bool bres =  m_pPieceTable->insertObject(dpos, pto, attributes, properties, &pfo);
+	*pField = pfo->getField();
+	return bres;
+}
+
+
 UT_Bool PD_Document::insertSpan(PT_DocPosition dpos,
 								  const UT_UCSChar * p,
 								  UT_uint32 length,
@@ -557,6 +578,39 @@ UT_Bool PD_Document::getSpanAttrProp(PL_StruxDocHandle sdh, UT_uint32 offset, UT
 	return m_pPieceTable->getSpanAttrProp(sdh,offset,bLeftSide,ppAP);
 }
 
+UT_Bool PD_Document::getField(PL_StruxDocHandle sdh, UT_uint32 offset,
+                               fd_Field * & pField)
+{
+
+	pf_Frag * pf = (pf_Frag *)sdh;
+	UT_ASSERT(pf->getType() == pf_Frag::PFT_Strux);
+	pf_Frag_Strux * pfsBlock = static_cast<pf_Frag_Strux *> (pf);
+	UT_ASSERT(pfsBlock->getStruxType() == PTX_Block);
+
+	UT_uint32 cumOffset = 0;
+	pf_Frag_Text * pft = NULL;
+	for (pf_Frag * pfTemp=pfsBlock->getNext(); (pfTemp); pfTemp=pfTemp->getNext())
+	{
+		cumOffset += pfTemp->getLength();
+		if (offset < cumOffset)
+		{
+			switch (pfTemp->getType()) 
+			{
+			case pf_Frag::PFT_Text:
+				pft = static_cast<pf_Frag_Text *> (pfTemp);
+				pField = pft->getField();
+				return UT_TRUE; // break out of loop
+				break;
+			default:
+				return UT_FALSE;
+				break;
+			}
+		}
+
+	}
+	return UT_FALSE;
+}
+
 UT_Bool PD_Document::getStruxFromPosition(PL_ListenerId listenerId,
 										  PT_DocPosition docPos,
 										  PL_StruxFmtHandle * psfh) const
@@ -798,6 +852,7 @@ void PD_Document::_destroyDataItemData(void)
 		UT_ASSERT(pPair);
 
 		delete pPair->pBuf;
+		FREEP(pPair->pToken);
 		delete pPair;
 
 		pHE->pData = NULL;
@@ -826,3 +881,350 @@ void PD_Document::clearIfAtFmtMark(PT_DocPosition dpos)
 	m_pPieceTable->clearIfAtFmtMark(dpos);
 }
 
+UT_Bool PD_Document::updateFields(void)
+{
+	//
+	// Turn off Insertion point motion during this general update
+	//
+	setDontChangeInsPoint();
+	pf_Frag * currentFrag = m_pPieceTable->getFragments().getFirst();
+	UT_ASSERT(currentFrag);
+	while (currentFrag!=m_pPieceTable->getFragments().getLast())
+	{
+		if (currentFrag->getType()==pf_Frag::PFT_Object)
+		{
+			pf_Frag_Object * pfo = static_cast<pf_Frag_Object *>
+				(currentFrag);
+			if (pfo->getObjectType()==PTO_Field)
+			{
+				UT_ASSERT (pfo->getField());
+				pfo->getField()->update();
+			}
+		}
+		currentFrag = currentFrag->getNext();
+	}
+	//
+	// Restore insertion point motion
+	//
+	allowChangeInsPoint();
+	return true;
+}
+
+void PD_Document::setDontChangeInsPoint(void)
+{
+	//
+	// Notify all views that they cannot change the insertion point
+	//
+	pf_Frag * currentFrag = m_pPieceTable->getFragments().getFirst();
+	//
+	//Look for the first strux so we can notify the views via a strux listner
+	//
+	while (currentFrag!=m_pPieceTable->getFragments().getLast())
+	{
+		if (currentFrag->getType()==pf_Frag::PFT_Strux)
+		{
+			break;
+		}
+		currentFrag = currentFrag->getNext();
+	}
+	if(currentFrag!=m_pPieceTable->getFragments().getLast())
+	{
+		pf_Frag_Strux * pfs = ( pf_Frag_Strux *) currentFrag;
+		PT_AttrPropIndex pAppIndex = pfs->getIndexAP();
+		PT_DocPosition pos = getStruxPosition(pfs);
+		const PX_ChangeRecord * pcr = new PX_ChangeRecord(PX_ChangeRecord::PXT_DontChangeInsPoint,pos,pAppIndex);
+		notifyListeners(pfs, pcr);
+		delete pcr;		
+	}
+	else
+	{
+		UT_DEBUGMSG(("setDontChangeInsPoint: No strux in document!! \n"));
+		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+	}
+}
+
+void PD_Document::allowChangeInsPoint(void)
+{
+  //
+  // Notify all views that they can change the insertion point
+  //
+	pf_Frag * currentFrag = m_pPieceTable->getFragments().getFirst();
+	//
+	//Look for the first strux so we can notify the views via a strux listner
+	//
+	while (currentFrag!=m_pPieceTable->getFragments().getLast())
+	{
+		if (currentFrag->getType()==pf_Frag::PFT_Strux)
+		{
+			break;
+		}
+		currentFrag = currentFrag->getNext();
+	}
+	if(currentFrag!=m_pPieceTable->getFragments().getLast())
+	{
+		pf_Frag_Strux * pfs = ( pf_Frag_Strux *) currentFrag;
+		PT_AttrPropIndex pAppIndex = pfs->getIndexAP();
+		PT_DocPosition pos = getStruxPosition(pfs);
+		const PX_ChangeRecord * pcr = new PX_ChangeRecord(PX_ChangeRecord::PXT_AllowChangeInsPoint,pos,pAppIndex);
+		notifyListeners(pfs, pcr);
+		delete pcr;
+	}
+	else
+	{
+		UT_DEBUGMSG(("allowChangeInsPoint: No strux in document!! \n"));
+		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+	}
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// Step towards full thread safety
+
+void PD_Document::notifyPieceTableChangeStart(void)
+{
+        m_bPieceTableChanging = UT_TRUE;
+}
+
+
+void PD_Document::notifyPieceTableChangeEnd(void)
+{
+        m_bPieceTableChanging = UT_FALSE;
+}
+
+UT_Bool PD_Document::isPieceTableChanging(void)
+{
+        return m_bPieceTableChanging;
+}
+
+////////////////////////////////////////////////////////////////
+// List Vector Functions
+
+
+
+fl_AutoNum * PD_Document::getListByID(UT_uint32 id) const
+{
+	UT_uint16 i = 0;
+	UT_sint32 cnt = 0;
+	fl_AutoNum * pAutoNum;
+
+	cnt = m_vecLists.getItemCount();
+	if ( cnt <= 0)
+		return (fl_AutoNum *) NULL;
+	UT_ASSERT(m_vecLists.getFirstItem());
+
+	while (i<cnt)
+	{
+		pAutoNum = (fl_AutoNum *)m_vecLists[i];
+		if (pAutoNum->getID() == id)
+			return pAutoNum;
+		i++;
+	}
+	
+	return (fl_AutoNum *) NULL;
+}
+
+UT_Bool PD_Document::enumLists(UT_uint32 k, fl_AutoNum ** pAutoNum) 
+{
+	UT_uint32 kLimit = m_vecLists.getItemCount();
+	if (k >= kLimit)
+		return UT_FALSE;
+	
+	if (pAutoNum)
+		*pAutoNum = (fl_AutoNum *)m_vecLists[k];
+	
+	return UT_TRUE;
+}
+
+fl_AutoNum * PD_Document::getNthList(UT_uint32 i) const
+{
+	UT_ASSERT(i >= 0);
+	return (fl_AutoNum *)m_vecLists[i];
+}
+
+UT_uint32 PD_Document::getListsCount(void) const
+{
+	return m_vecLists.getItemCount();
+}
+
+void PD_Document::addList(fl_AutoNum * pAutoNum)
+{
+	UT_uint32 id = pAutoNum->getID();
+	UT_uint32 i;
+	UT_uint32 numlists = m_vecLists.getItemCount();
+	for(i=0; i < numlists; i++)
+	{
+		fl_AutoNum * pAuto = (fl_AutoNum *) m_vecLists.getNthItem(i);
+		if(pAuto->getID() == id)
+			break;
+	}
+	if(i >= numlists)
+		m_vecLists.addItem(pAutoNum);
+}
+
+void PD_Document::listUpdate(PL_StruxDocHandle sdh )
+{
+	//
+	// Notify all views of a listupdate
+	//
+	UT_ASSERT(sdh);
+	pf_Frag_Strux * pfs = (pf_Frag_Strux *) sdh;
+	PT_AttrPropIndex pAppIndex = pfs->getIndexAP();
+	//PT_DocPosition pos = getStruxPosition(sdh) + fl_BLOCK_STRUX_OFFSET;
+	PT_DocPosition pos = getStruxPosition(sdh);
+		const PX_ChangeRecord * pcr = new PX_ChangeRecord(PX_ChangeRecord::PXT_ListUpdate,pos,pAppIndex);
+	notifyListeners(pfs, pcr);
+	delete pcr;
+}
+
+
+void PD_Document::StopList(PL_StruxDocHandle sdh )
+{
+	//
+	// Notify all views of a stoplist
+	//
+	pf_Frag_Strux * pfs = (pf_Frag_Strux *) sdh;
+	PT_AttrPropIndex pAppIndex = pfs->getIndexAP();
+	PT_DocPosition pos = getStruxPosition(sdh);
+	const PX_ChangeRecord * pcr = new PX_ChangeRecord(PX_ChangeRecord::PXT_StopList,pos,pAppIndex);
+	notifyListeners(pfs, pcr);
+	delete pcr;
+}
+
+
+UT_Bool PD_Document::appendList(const XML_Char ** attributes)
+{
+	const XML_Char * szID=NULL, * szPid=NULL, * szType=NULL, * szStart=NULL, * szDelim=NULL, *szDec=NULL;
+	UT_uint32 id, parent_id, start;
+	List_Type type;
+	
+	for (const XML_Char ** a = attributes; (*a); a++)
+	{
+		if (UT_XML_stricmp(a[0],"id") == 0)
+			szID = a[1];
+		else if (UT_XML_stricmp(a[0], "parentid") == 0)
+			szPid = a[1];
+		else if (UT_XML_stricmp(a[0], "type") == 0)
+			szType = a[1];
+		else if (UT_XML_stricmp(a[0], "start-value") == 0)
+			szStart = a[1];
+		else if (UT_XML_stricmp(a[0], "list-delim") == 0)
+			szDelim = a[1];
+		else if (UT_XML_stricmp(a[0], "list-decimal") == 0)
+			szDec = a[1];
+	}
+
+	if(!szID)
+		return UT_FALSE;
+	if(!szPid)
+		return UT_FALSE;
+	if(!szType)
+		return UT_FALSE;
+	if(!szStart)
+		return UT_FALSE;
+	if(!szDelim)
+		return UT_FALSE;
+	if(!szDec)
+		szDec = (const XML_Char *) ".";
+	id = atoi(szID);
+	UT_uint32 i;
+	UT_uint32 numlists = m_vecLists.getItemCount();
+	for(i=0; i < numlists; i++)
+	{
+		fl_AutoNum * pAuto = (fl_AutoNum *) m_vecLists.getNthItem(i);
+		if(pAuto->getID() == id)
+			break;
+	}
+	if(i < numlists)
+		return UT_TRUE; // List is already present
+	parent_id = atoi(szPid);
+	type = (List_Type)atoi(szType);
+	start = atoi(szStart);
+
+	fl_AutoNum * pAutoNum = new fl_AutoNum(id, parent_id, type, start, szDelim,szDec,this);
+	addList(pAutoNum);
+	
+	return UT_TRUE;
+}
+
+UT_Bool PD_Document::areListUpdatesAllowed(void)
+{
+        return m_ballowListUpdates;
+}
+ 
+void PD_Document::disableListUpdates(void)
+{
+        m_ballowListUpdates = UT_FALSE;
+}
+   
+void PD_Document::enableListUpdates(void)
+{
+        m_ballowListUpdates = UT_TRUE;
+}
+  
+void PD_Document::updateDirtyLists(void)
+{
+	UT_uint32 iNumLists = m_vecLists.getItemCount();
+	UT_uint32 i;
+	fl_AutoNum * pAutoNum;
+	for(i=0; i< iNumLists; i++)
+	{
+		pAutoNum = (fl_AutoNum *) m_vecLists.getNthItem(i);
+		if(pAutoNum->isDirty() == UT_TRUE)
+		{
+			pAutoNum->update(0);
+		}
+	}
+	for(i=0; i< iNumLists; i++)
+	{
+		pAutoNum = (fl_AutoNum *) m_vecLists.getNthItem(i);
+		pAutoNum->findAndSetParentItem();
+	}
+}
+
+
+UT_Bool PD_Document::fixListHierarchy(void)
+{
+	UT_uint32 iNumLists = m_vecLists.getItemCount();
+	fl_AutoNum * pAutoNum;
+
+	if (iNumLists == 0)
+	{
+		return UT_FALSE;
+	}
+	else
+	{
+		for (UT_uint32 i = 0; i < iNumLists; i++)
+		{
+			pAutoNum = (fl_AutoNum *)m_vecLists.getNthItem(i);
+			pAutoNum->fixHierarchy(this);
+		}
+		return UT_TRUE;
+	}
+}
+
+void PD_Document::removeList(fl_AutoNum * pAutoNum, PL_StruxDocHandle sdh )
+{
+	UT_ASSERT(pAutoNum);
+	UT_sint32 ndx = m_vecLists.findItem(pAutoNum);
+	UT_ASSERT(ndx >= 0);
+	if (ndx != -1)
+	{
+		//
+		// Notify all views of a remove List
+		//
+		pf_Frag_Strux * pfs = (pf_Frag_Strux *) sdh;
+		PT_AttrPropIndex pAppIndex = pfs->getIndexAP();
+		PT_DocPosition pos = getStruxPosition(sdh);
+		const PX_ChangeRecord * pcr = new PX_ChangeRecord(PX_ChangeRecord::PXT_RemoveList,pos,pAppIndex);
+		notifyListeners(pfs, pcr);
+		delete pcr;						  
+		m_vecLists.deleteNthItem(ndx);
+	}
+}
+
+
+
+
+
+
+
+
