@@ -28,7 +28,7 @@
 #include "ut_vector.h"
 #include "ut_alphahash.h"
 #include "xad_Document.h"
-#include "xmlparse.h"
+#include "ut_xml.h"
 #include "pt_Types.h"
 #include "pl_Listener.h"
 #include "ie_types.h"
@@ -40,6 +40,9 @@ class PP_AttrProp;
 class pf_Frag_Strux;
 class PX_ChangeRecord;
 class PD_Style;
+class fd_Field;
+class fl_AutoNum;
+
 
 #ifdef PT_TEST
 #include "ut_test.h"
@@ -77,6 +80,10 @@ public:
 										 PTObjectType pto,
 										 const XML_Char ** attributes,
 										 const XML_Char ** properties);
+	UT_Bool					insertObject(PT_DocPosition dpos,
+										 PTObjectType pto,
+										 const XML_Char ** attributes,
+										 const XML_Char ** properties, fd_Field ** pField );
 
 	UT_Bool					insertSpan(PT_DocPosition dpos,
 									   const UT_UCSChar * p,
@@ -112,7 +119,7 @@ public:
 	UT_Bool					appendSpan(UT_UCSChar * p, UT_uint32 length);
 	UT_Bool					appendObject(PTObjectType pto, const XML_Char ** attributes);
 	UT_Bool					appendStyle(const XML_Char ** attributes);
-
+	
 	UT_Bool					tellListener(PL_Listener * pListener);
 	UT_Bool					tellListenerSubset(PL_Listener * pListener,
 											   PD_DocumentRange * pDocRange);
@@ -161,7 +168,35 @@ public:
 										  const char ** pszName, const PD_Style ** ppStyle) const;
 
 	void					clearIfAtFmtMark(PT_DocPosition dpos);
+
+	const char *                            getFileName() { return m_szFilename; }
+	UT_uint32                               getLastType() { return m_lastSavedAsType; }
+        UT_Bool                 updateFields(void);
+        UT_Bool                 getField(PL_StruxDocHandle sdh, 
+					 UT_uint32 offset,
+                                     fd_Field * &pField);
+	void                    setDontChangeInsPoint(void);
+	void                    allowChangeInsPoint(void);
+	UT_Bool                 isPieceTableChanging(void);
+        void                    notifyPieceTableChangeStart(void);
+        void                    notifyPieceTableChangeEnd(void);
 	
+	// List Functions
+	fl_AutoNum *    getListByID(UT_uint32 id) const;
+	fl_AutoNum *    getNthList(UT_uint32 i) const; 
+	UT_Bool		enumLists(UT_uint32 k, fl_AutoNum ** pAutoNum);
+	UT_uint32       getListsCount(void) const; 
+	void            addList(fl_AutoNum * pAutoNum);
+	UT_Bool		appendList(const XML_Char ** attributes);
+	UT_Bool		fixListHierarchy(void);
+	void	 	removeList(fl_AutoNum * pAutoNum,PL_StruxDocHandle sdh );
+	void            listUpdate(PL_StruxDocHandle sdh);
+	void            StopList(PL_StruxDocHandle sdh);
+	void            disableListUpdates(void);
+	void            enableListUpdates(void);
+	void            updateDirtyLists(void);
+	UT_Bool         areListUpdatesAllowed(void);
+ 
 #ifdef PT_TEST
 	void					__dump(FILE * fp) const;
 #endif
@@ -171,14 +206,29 @@ protected:
 
 	void					_setClean(void);
 	void					_destroyDataItemData(void);
-
-	pt_PieceTable *			m_pPieceTable;
+	UT_Bool                                 m_ballowListUpdates;
+	pt_PieceTable *			        m_pPieceTable;
 	UT_Vector				m_vecListeners;
-
-	UT_AlphaHashTable		m_hashDataItems;
+	UT_Vector				m_vecLists;
+	
+	UT_AlphaHashTable		        m_hashDataItems;
 
 	IEFileType				m_lastSavedAsType;
+	UT_Bool                                 m_bPieceTableChanging;
 };
 
 
 #endif /* PD_DOCUMENT_H */
+
+
+
+
+
+
+
+
+
+
+
+
+
