@@ -24,7 +24,8 @@
 #include "ut_types.h"
 #include "ut_alphahash.h"
 #include "ut_vector.h"
-#include "xmlparse.h"
+#include "ut_xml.h"
+
 
 // PP_AttrProp captures the complete set of XML and CSS
 // Attributes/Properties for a piece of the document.
