@@ -37,6 +37,9 @@
 #include "ap_BeOSFrame.h"
 #include "ap_BeOSApp.h"
 
+#include "fv_View.h"
+#include "fp_Run.h"
+
 // HACK: set this symbol to 0 to turn off command-line conversion
 // TODO: if it works, remove it entirely
 #ifndef CONVERT
@@ -65,6 +68,10 @@
 
 #include "ap_Clipboard.h"
 
+// Signal handling
+#include <signal.h>
+void signalWrapper(int sig_num);
+
 /*****************************************************************/
 /*
  Splash Window Related Stuff
@@ -311,6 +318,22 @@ UT_Bool AP_BeOSApp::initialize(void)
 			free(szPathname);
 		}
 	}
+	// Now we have the strings loaded we can populate the field names correctly
+	// CHECK THIS: the following was added by a Linux developer who can't test
+	// on BeOS, or even compile, so someone with a BeOS box needs to check it
+	int i;
+	
+	for (i = 0; fp_FieldTypes[i].m_Type != FPFIELDTYPE_END; i++)
+	{
+	    (&fp_FieldTypes[i])->m_Desc = m_pStringSet->getValue(fp_FieldTypes[i].m_DescId);
+	    UT_DEBUGMSG(("Setting field type desc for type %d, desc=%s\n", fp_FieldTypes[i].m_Type, fp_FieldTypes[i].m_Desc));
+	}
+
+	for (i = 0; fp_FieldFmts[i].m_Tag != NULL; i++)
+	{
+	    (&fp_FieldFmts[i])->m_Desc = m_pStringSet->getValue(fp_FieldFmts[i].m_DescId);
+	    UT_DEBUGMSG(("Setting field desc for field %s, desc=%s\n", fp_FieldFmts[i].m_Tag, fp_FieldFmts[i].m_Desc));
+	}
 
 	//////////////////////////////////////////////////////////////////
 
@@ -503,6 +526,25 @@ int AP_BeOSApp::local_main(const char * szAppName, int argc, char ** argv) {
 	//Show the splash screen perhaps
   	_showSplash(&Args, szAppName);        
 
+	// Setup signal handlers, primarily for segfault
+	// If we segfaulted before here, we *really* blew it
+
+	struct sigaction sa;
+
+	sa.sa_handler = signalWrapper;
+
+	sigfillset(&sa.sa_mask);  // We don't want to hear about other signals
+	sigdelset(&sa.sa_mask, SIGABRT); // But we will call abort(), so we can't ignore that
+
+	sa.sa_flags = 0; // Unsupported under BeOS -- SA_NODEFER | SA_RESETHAND; // Don't handle nested signals
+	
+	sigaction(SIGSEGV, &sa, NULL);
+	sigaction(SIGBUS, &sa, NULL);
+	sigaction(SIGILL, &sa, NULL);
+	sigaction(SIGQUIT, &sa, NULL);
+	sigaction(SIGFPE, &sa, NULL);
+	// TODO: handle SIGABRT
+
 	// if the initialize fails, we don't have icons, fonts, etc.
 	if (!pMyBeOSApp->initialize())
 	{
@@ -676,3 +718,44 @@ void AP_BeOSApp::ParseCommandLine(void)
 	return;
 }
 
+void signalWrapper(int sig_num)
+{
+	AP_BeOSApp *pApp = (AP_BeOSApp *) XAP_App::getApp();
+	pApp->catchSignals(sig_num);
+}
+
+
+
+
+static int s_signal_count = 0;
+
+void AP_BeOSApp::catchSignals(int sig_num)
+{
+	// Reset the signal handler (not that it matters - this is mostly for race conditions)
+	signal(SIGSEGV, signalWrapper);
+
+        s_signal_count = s_signal_count + 1;
+        if(s_signal_count > 1)
+        {
+                UT_DEBUGMSG(("Segfault during filesave - no file saved  \n"));
+                fflush(stdout);
+                abort();
+        }
+
+
+	UT_DEBUGMSG(("Oh no - we just segfaulted!\n"));
+
+	UT_uint32 i = 0;
+	for(;i<m_vecFrames.getItemCount();i++)
+	{
+		AP_BeOSFrame * curFrame = (AP_BeOSFrame*) m_vecFrames[i];
+		UT_ASSERT(curFrame);
+		curFrame->backup();
+	}
+
+	fflush(stdout);
+
+	// Abort and dump core
+	abort();
+}
+
