@@ -29,6 +29,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <signal.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 
@@ -37,12 +38,16 @@
 #include "ut_misc.h"
 
 #include "xap_Args.h"
+#include "ap_Convert.h"
 #include "ap_QNXFrame.h"
 #include "ap_QNXApp.h"
 #include "sp_spell.h"
 #include "ap_Strings.h"
 #include "xap_EditMethods.h"
 #include "ap_LoadBindings.h"
+#include "xap_DialogFactory.h"
+#include "xap_Dlg_MessageBox.h"
+#include "xap_Dialog_Id.h"
 #include "xap_Menu_ActionSet.h"
 #include "xap_Toolbar_ActionSet.h"
 #include "xav_View.h"
@@ -65,6 +70,9 @@
 #include "ut_qnxHelper.h"
 
 #include "fv_View.h"
+#include "fp_Run.h"
+
+void signalWrapper(int sig_num);
 
 /*****************************************************************/
 
@@ -225,6 +233,23 @@ UT_Bool AP_QNXApp::initialize(void)
 		}
 	}
 
+	// Now we have the strings loaded we can populate the field names correctly
+	// CHECK THIS: the following was added by a Linux developer who can't test
+	// on QNX, or even compile, so someone with a QNX box (or fridge? ;-)) needs to check it
+	int i;
+	
+	for (i = 0; fp_FieldTypes[i].m_Type != FPFIELDTYPE_END; i++)
+	{
+	    (&fp_FieldTypes[i])->m_Desc = m_pStringSet->getValue(fp_FieldTypes[i].m_DescId);
+	    UT_DEBUGMSG(("Setting field type desc for type %d, desc=%s\n", fp_FieldTypes[i].m_Type, fp_FieldTypes[i].m_Desc));
+	}
+
+	for (i = 0; fp_FieldFmts[i].m_Tag != NULL; i++)
+	{
+	    (&fp_FieldFmts[i])->m_Desc = m_pStringSet->getValue(fp_FieldFmts[i].m_DescId);
+	    UT_DEBUGMSG(("Setting field desc for field %s, desc=%s\n", fp_FieldFmts[i].m_Tag, fp_FieldFmts[i].m_Desc));
+	}
+
 	//////////////////////////////////////////////////////////////////
 
 	return UT_TRUE;
@@ -292,26 +317,39 @@ const XAP_StringSet * AP_QNXApp::getStringSet(void) const
 
 //////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////
+#define CLIP_FORMAT_COUNT 2
+#define Ph_CLIPBOARD_TYPE_RTF "RTF"
 
 void AP_QNXApp::copyToClipboard(PD_DocumentRange * pDocRange)
 {
+	PhClipHeader clip[CLIP_FORMAT_COUNT];		//RTF and 8 bit text
+	UT_ByteBuf rtfbuf, txtbuf;
+	int			format_count;
+
 	// copy the given subset of the given document to the
 	// system clipboard in a variety of formats.
 	//
 	m_pClipboard->clearClipboard();
 	
 	// also put RTF on the clipboard
+	format_count = 0;
 	
 	IE_Exp_RTF * pExpRtf = new IE_Exp_RTF(pDocRange->m_pDoc);
 	if (pExpRtf)
 	{
-		UT_ByteBuf buf;
-		UT_Error status = pExpRtf->copyToBuffer(pDocRange,&buf);
+		UT_Error status = pExpRtf->copyToBuffer(pDocRange,&rtfbuf);
 		UT_Byte b = 0;
-		buf.append(&b,1);			// null terminate string
-		m_pClipboard->addData(AP_CLIPBOARD_RTF,(UT_Byte *)buf.getPointer(0),buf.getLength());
+		rtfbuf.append(&b,1);			// null terminate string
+		UT_DEBUGMSG(("CopyToClipboard: copying %d bytes in RTF format.",rtfbuf.getLength()));
+#if defined(USE_ABI_CLIPBOARD)
+		m_pClipboard->addData(AP_CLIPBOARD_RTF,(UT_Byte *)rtfbuf.getPointer(0),rtfbuf.getLength());
+#else
+		strcpy(clip[format_count].type, Ph_CLIPBOARD_TYPE_RTF);
+		clip[format_count].length = rtfbuf.getLength();
+		clip[format_count].data = (void *)rtfbuf.getPointer(0);
+		format_count++;
+#endif
 		DELETEP(pExpRtf);
-		UT_DEBUGMSG(("CopyToClipboard: copying %d bytes in RTF format.\n",buf.getLength()));
 	}
 
 	// put raw 8bit text on the clipboard
@@ -319,23 +357,56 @@ void AP_QNXApp::copyToClipboard(PD_DocumentRange * pDocRange)
 	IE_Exp_Text * pExpText = new IE_Exp_Text(pDocRange->m_pDoc);
 	if (pExpText)
 	{
-		UT_ByteBuf buf;
-		UT_Error status = pExpText->copyToBuffer(pDocRange,&buf);
+		UT_Error status = pExpText->copyToBuffer(pDocRange,&txtbuf);
 		UT_Byte b = 0;
-		buf.append(&b,1);			// null terminate string
-		m_pClipboard->addData(AP_CLIPBOARD_TEXTPLAIN_8BIT,(UT_Byte *)buf.getPointer(0),buf.getLength());
+		txtbuf.append(&b,1);			// null terminate string
+		UT_DEBUGMSG(("CopyToClipboard: copying %d bytes in TEXTPLAIN format.\n",txtbuf.getLength()));
+#if defined(USE_ABI_CLIPBOARD)
+		m_pClipboard->addData(AP_CLIPBOARD_TEXTPLAIN_8BIT,(UT_Byte *)txtbuf.getPointer(0),txtbuf.getLength());
+#else
+		strcpy(clip[format_count].type, Ph_CLIPBOARD_TYPE_TEXT);
+		clip[format_count].length = txtbuf.getLength();
+		clip[format_count].data = (void *)txtbuf.getPointer(0);
+		format_count++;
+#endif
 		DELETEP(pExpText);
-		UT_DEBUGMSG(("CopyToClipboard: copying %d bytes in TEXTPLAIN format.\n",buf.getLength()));
 	}
 
+#if !defined(USE_ABI_CLIPBOARD)
+	PhClipboardCopy(PhInputGroup(NULL), format_count, clip);
+#endif
 }
 
 void AP_QNXApp::pasteFromClipboard(PD_DocumentRange * pDocRange, UT_Bool bUseClipboard)
 {
+	PhClipHeader *clip;
+	void *handle;
+
 	// paste from the system clipboard using the best-for-us format
 	// that is present.
-	
-	if (m_pClipboard->hasFormat(AP_CLIPBOARD_RTF))
+
+
+	handle = PhClipboardPasteStart(PhInputGroup(NULL));
+
+	if (handle && (clip = PhClipboardPasteType(handle, Ph_CLIPBOARD_TYPE_RTF))) {
+		unsigned char * pData = (unsigned char *)clip->data;
+		UT_uint32 iLen = clip->length;
+		iLen = MyMin(iLen,strlen((const char *) pData));
+		UT_DEBUGMSG(("PasteFromClipboard: pasting %d bytes in RTF format.\n",iLen));
+		IE_Imp_RTF * pImpRTF = new IE_Imp_RTF(pDocRange->m_pDoc);
+		pImpRTF->pasteFromBuffer(pDocRange,pData,iLen);
+		DELETEP(pImpRTF);
+	}
+	else if (handle && (clip = PhClipboardPasteType(handle, Ph_CLIPBOARD_TYPE_TEXT))) {
+		unsigned char * pData = (unsigned char *)clip->data;
+		UT_uint32 iLen = clip->length;
+		iLen = MyMin(iLen,strlen((const char *) pData));
+		UT_DEBUGMSG(("PasteFromClipboard: pasting %d bytes in TEXTPLAIN format.\n",iLen));
+		IE_Imp_Text * pImpText = new IE_Imp_Text(pDocRange->m_pDoc);
+		pImpText->pasteFromBuffer(pDocRange,pData,iLen);
+		DELETEP(pImpText);
+	}
+	else if (m_pClipboard->hasFormat(AP_CLIPBOARD_RTF))
 	{
 		unsigned char * pData = NULL;
 		UT_uint32 iLen = 0;
@@ -363,12 +434,28 @@ void AP_QNXApp::pasteFromClipboard(PD_DocumentRange * pDocRange, UT_Bool bUseCli
 		// TODO figure out what to do with an image....
 		UT_DEBUGMSG(("PasteFromClipboard: TODO support this format..."));
 	}
+	
+	if (handle) {
+		PhClipboardPasteFinish(handle);
+	}
 
 	return;
 }
 
 UT_Bool AP_QNXApp::canPasteFromClipboard(void)
 {
+	void *handle;
+
+	handle = PhClipboardPasteStart(PhInputGroup(NULL));
+	if (handle) {
+		if (PhClipboardPasteType(handle, Ph_CLIPBOARD_TYPE_RTF) ||
+		    PhClipboardPasteType(handle, Ph_CLIPBOARD_TYPE_TEXT)) {
+			PhClipboardPasteFinish(handle);
+			return UT_TRUE;
+		}
+		PhClipboardPasteFinish(handle);
+	}
+
 	if (m_pClipboard->hasFormat(AP_CLIPBOARD_RTF))
 		return UT_TRUE;
 	if (m_pClipboard->hasFormat(AP_CLIPBOARD_TEXTPLAIN_8BIT))
@@ -715,7 +802,8 @@ static GR_Image * _showSplash(PtWidget_t *spwin, UT_uint32 delay)
 		PtAddEventHandler(da, Ph_EV_BUT_RELEASE, s_hideSplash, NULL);
 
 		// create image context
-		pQNXGraphics = new GR_QNXGraphics(spwin, da);
+		// TODO: find an XAP_App pointer for the following call:
+		pQNXGraphics = new GR_QNXGraphics(spwin, da, 0);
 		pSplashImage = pQNXGraphics->createNewImage("splash", pBB, iSplashWidth, iSplashHeight);
 
 		PtRealizeWidget(spwin);
@@ -792,6 +880,25 @@ int AP_QNXApp::main(const char * szAppName, int argc, char ** argv)
 	AP_QNXApp * pMyQNXApp;
 	gQNXApp = pMyQNXApp = new AP_QNXApp(&Args, szAppName);
 
+    // Setup signal handlers, primarily for segfault
+    // If we segfaulted before here, we *really* blew it
+
+    struct sigaction sa;
+
+    sa.sa_handler = signalWrapper;
+
+    sigfillset(&sa.sa_mask);  // We don't want to hear about other signals
+    sigdelset(&sa.sa_mask, SIGABRT); // But we will call abort(), so we can't ignore that
+
+    sa.sa_flags = SA_NODEFER | SA_RESETHAND; // Don't handle nested signals
+
+    sigaction(SIGSEGV, &sa, NULL);
+    sigaction(SIGBUS, &sa, NULL);
+    sigaction(SIGILL, &sa, NULL);
+    sigaction(SIGQUIT, &sa, NULL);
+    sigaction(SIGFPE, &sa, NULL);
+    // TODO: handle SIGABRT
+
 	//This is used by all the timer classes, and should probably be in the XAP contructor
 	PtArg_t args[2];
 	PtSetArg(&args[0], Pt_ARG_REGION_FIELDS, Ph_REGION_EV_SENSE, Ph_REGION_EV_SENSE);
@@ -837,6 +944,9 @@ UT_Bool AP_QNXApp::parseCommandLine(void)
 	int nFirstArg = 1;
 	int k;
 	int kWindowsOpened = 0;
+	char *to = NULL;
+	int verbose = 1;
+	UT_Bool show = UT_FALSE;
         
 	for (k=nFirstArg; (k<m_pArgs->m_argc); k++)
 	{
@@ -900,6 +1010,21 @@ UT_Bool AP_QNXApp::parseCommandLine(void)
 				// set the xap-level geometry for future frame use
 				setGeometry(x, y, width, height, f);
 			}
+			else if (UT_stricmp (m_pArgs->m_argv[k],"-to") == 0)
+			{
+				k++;
+				to = m_pArgs->m_argv[k];
+			}
+			else if (UT_stricmp (m_pArgs->m_argv[k], "-show") == 0)
+			{
+				show = UT_TRUE;
+			}
+			else if (UT_stricmp (m_pArgs->m_argv[k], "-verbose") == 0)
+			{
+				k++;
+				verbose = atoi (m_pArgs->m_argv[k]);
+			}
+
 			else
 			{
 				UT_DEBUGMSG(("Unknown command line option [%s]\n",m_pArgs->m_argv[k]));
@@ -912,33 +1037,103 @@ UT_Bool AP_QNXApp::parseCommandLine(void)
 		else
 		{
 			// [filename]
+			if (to) {
+ 				AP_Convert * conv = new AP_Convert();
+ 				conv->setVerbose(verbose);
+ 				conv->convertTo(m_pArgs->m_argv[k], to);
+ 				delete conv;
+  			} 
+			else {
                         
-			AP_QNXFrame * pFirstQNXFrame = new AP_QNXFrame(this);
-			pFirstQNXFrame->initialize();
-
-			UT_Error error = pFirstQNXFrame->loadDocument(m_pArgs->m_argv[k], IEFT_Unknown);
-			if (!error)
-			{
-				kWindowsOpened++;
-			}
-			else
-			{
-				// TODO: warn user that we couldn't open that file
+				AP_QNXFrame * pFirstQNXFrame = new AP_QNXFrame(this);
+				pFirstQNXFrame->initialize();
+				UT_Error error = pFirstQNXFrame->loadDocument(m_pArgs->m_argv[k], IEFT_Unknown);
+				if (!error)
+				{
+					kWindowsOpened++;
+				}
+				else
+				{
+					// TODO: warn user that we couldn't open that file
 
 #if 1
-				// TODO we crash if we just delete this without putting something
-				// TODO in it, so let's go ahead and open an untitled document
-				// TODO for now.  this would cause us to get 2 untitled documents
-				// TODO if the user gave us 2 bogus pathnames....
-				kWindowsOpened++;
-				pFirstQNXFrame->loadDocument(NULL, IEFT_Unknown);
+					// TODO we crash if we just delete this without putting something
+					// TODO in it, so let's go ahead and open an untitled document
+					// TODO for now.  this would cause us to get 2 untitled documents
+					// TODO if the user gave us 2 bogus pathnames....
+					kWindowsOpened++;
+					pFirstQNXFrame->loadDocument(NULL, IEFT_Unknown);
+
+					pFirstQNXFrame->raise();
+					
+					XAP_DialogFactory * pDialogFactory
+						= (XAP_DialogFactory *)(pFirstQNXFrame->getDialogFactory());
+					
+					XAP_Dialog_MessageBox * pDialog
+						= (XAP_Dialog_MessageBox *)(pDialogFactory->requestDialog(XAP_DIALOG_ID_MESSAGE_BOX));
+					UT_ASSERT(pDialog);
+					
+					const XAP_StringSet * pSS = pFirstQNXFrame->getApp()->getStringSet();
+					
+					switch (error)
+					{
+					case -301:
+						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_IE_FileNotFound),m_pArgs->m_argv[k]);
+						break;
+						
+					case -302:
+						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_IE_NoMemory),m_pArgs->m_argv[k]);
+						break;
+						
+					case -303:
+						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_IE_UnknownType),m_pArgs->m_argv[k]);
+						break;
+						
+					case -304:
+						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_IE_BogusDocument),m_pArgs->m_argv[k]);
+						break;
+						
+					case -305:
+						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_IE_CouldNotOpen),m_pArgs->m_argv[k]);
+						break;
+							
+					case -306:
+						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_IE_CouldNotWrite),m_pArgs->m_argv[k]);
+						break;
+						
+					case -307:
+						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_IE_FakeType),m_pArgs->m_argv[k]);
+						break;
+						
+					case -311:
+						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_IE_UnsupportedType),m_pArgs->m_argv[k]);
+						break;
+						
+					default:
+						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_ImportError),m_pArgs->m_argv[k]);
+					}
+					pDialog->setButtons(XAP_Dialog_MessageBox::b_O);
+					pDialog->setDefaultAnswer(XAP_Dialog_MessageBox::a_OK);
+					
+					pDialog->runModal(pFirstQNXFrame);
+					
+					XAP_Dialog_MessageBox::tAnswer ans = pDialog->getAnswer();
+					
+					pDialogFactory->releaseDialog(pDialog);
 #else
-				delete pFirstQNXFrame;
+					delete pFirstQNXFrame;
 #endif
+				}
 			}
 		}
 	}
 
+	// command-line conversion may not open any windows at all
+	if (to && !show) 
+	{
+		return UT_TRUE;
+	}
+
 	if (kWindowsOpened == 0)
 	{
 		// no documents specified or were able to be opened, open an untitled one
@@ -960,7 +1155,11 @@ void AP_QNXApp::_printUsage(void)
 {
 	// just print to stdout, not stderr
 	printf("\nUsage: %s [option]... [file]...\n\n", m_pArgs->m_argv[0]);
-
+	printf("  -to               The target format of the file\n");
+        printf("                    (abw, zabw, rtf, txt, utf8, html, latex)\n");
+	printf("  -verbose          The verbosity level (0, 1, 2)\n");
+	printf("  -show             If you really want to start the GUI\n");
+        printf("                    (even if you use the --to option)\n");
 #ifdef DEBUG
 	printf("  -dumpstrings      dump strings strings to file\n");
 #endif
@@ -974,3 +1173,42 @@ void AP_QNXApp::_printUsage(void)
 	printf("\n");
 }
 
+/*** Signal handling functionality ***/
+void signalWrapper(int sig_num)
+{
+	AP_QNXApp *pApp = (AP_QNXApp *) XAP_App::getApp();
+	pApp->catchSignals(sig_num);
+}
+
+static int s_signal_count = 0;
+
+void AP_QNXApp::catchSignals(int sig_num)
+{
+        // Reset the signal handler 
+  // (not that it matters - this is mostly for race conditions)
+        signal(SIGSEGV, signalWrapper);
+
+        s_signal_count = s_signal_count + 1;
+        if(s_signal_count > 1)
+        {
+                UT_DEBUGMSG(("Segfault during filesave - no file saved  \n"));
+                fflush(stdout);
+                abort();
+        }
+
+	UT_DEBUGMSG(("Oh no - we just segfaulted!\n"));
+
+	UT_uint32 i = 0;
+	for(;i<m_vecFrames.getItemCount();i++)
+	{
+		AP_QNXFrame * curFrame = (AP_QNXFrame*) m_vecFrames[i];
+		UT_ASSERT(curFrame);
+		curFrame->backup();
+	}
+
+	fflush(stdout);
+
+	// Abort and dump core
+	abort();
+}
+
