@@ -45,11 +45,16 @@
 
 #	include "ap_QNXDialog_Replace.h"
 #	include "ap_QNXDialog_Break.h"
-#	include "ap_QNXDialog_Spell.h"
+#	include "ap_QNXDialog_Goto.h"
 #	include "ap_QNXDialog_Paragraph.h"
 #	include "ap_QNXDialog_Options.h"
+#	include "ap_QNXDialog_Spell.h"
+#	include "ap_QNXDialog_Tab.h"
 #	include "ap_QNXDialog_Insert_DateTime.h"
 #	include "ap_QNXDialog_WordCount.h"
+#	include "ap_QNXDialog_Field.h"
+#	include "ap_QNXDialog_Lists.h"
+#	include "ap_QNXDialog_Columns.h"
 
 	// ... add new dialogs here ...
 
@@ -63,17 +68,24 @@
 	DeclareDialog(XAP_DIALOG_ID_FONT,			XAP_QNXDialog_FontChooser)
 	DeclareDialog(XAP_DIALOG_ID_WINDOWMORE,		XAP_QNXDialog_WindowMore)
 	DeclareDialog(XAP_DIALOG_ID_ZOOM,			XAP_QNXDialog_Zoom)
-	DeclareDialog(XAP_DIALOG_ID_ABOUT,			XAP_QNXDialog_About)
 	DeclareDialog(XAP_DIALOG_ID_INSERT_SYMBOL,	XAP_QNXDialog_Insert_Symbol)
+	DeclareDialog(XAP_DIALOG_ID_INSERT_PICTURE,	XAP_QNXDialog_FileOpenSaveAs)
+
+	DeclareDialog(XAP_DIALOG_ID_ABOUT,			XAP_QNXDialog_About)
 
 	DeclareDialog(AP_DIALOG_ID_REPLACE,			AP_QNXDialog_Replace)
 	DeclareDialog(AP_DIALOG_ID_FIND,			AP_QNXDialog_Replace)
+	DeclareDialog(AP_DIALOG_ID_GOTO,			AP_QNXDialog_Goto)
 	DeclareDialog(AP_DIALOG_ID_BREAK,			AP_QNXDialog_Break)
 	DeclareDialog(AP_DIALOG_ID_SPELL,			AP_QNXDialog_Spell)
 	DeclareDialog(AP_DIALOG_ID_PARAGRAPH,		AP_QNXDialog_Paragraph)	
 	DeclareDialog(AP_DIALOG_ID_OPTIONS,			AP_QNXDialog_Options)	
+	DeclareDialog(AP_DIALOG_ID_TAB,				AP_QNXDialog_Tab)
 	DeclareDialog(AP_DIALOG_ID_INSERT_DATETIME, AP_QNXDialog_Insert_DateTime)
 	DeclareDialog(AP_DIALOG_ID_WORDCOUNT,		AP_QNXDialog_WordCount)
+	DeclareDialog(AP_DIALOG_ID_FIELD,			AP_QNXDialog_Field)
+	DeclareDialog(AP_DIALOG_ID_LISTS,			AP_QNXDialog_Lists)
+	DeclareDialog(AP_DIALOG_ID_COLUMNS,			AP_QNXDialog_Columns)
 
 	// ... also add new dialogs here ...
 
