@@ -313,7 +313,10 @@ PtWidget_t* AP_QNXDialog_Options::_constructWindow ()
 	PtWidget_t *listViewRulerUnit;
 	PtWidget_t *listViewRulerUnit_menu;
 	PtWidget_t *checkbuttonViewCursorBlink;
-	PtWidget_t *checkbuttonViewToolbars;
+	PtWidget_t *checkbuttonViewStandard;
+	PtWidget_t *checkbuttonViewFormat;
+	PtWidget_t *checkbuttonViewExtra;
+	PtWidget_t *checkbuttonViewStatusBar;
 	PtWidget_t *frameViewStuff;
 	PtWidget_t *vbox6;
 	PtWidget_t *checkbuttonViewAll;
@@ -513,8 +516,20 @@ PtWidget_t* AP_QNXDialog_Options::_constructWindow ()
 	UT_QNXComboSetPos(listViewRulerUnit, 1);
 	
 	n = 0;
-	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, TR(pSS->getValue(AP_STRING_ID_DLG_Options_Label_ViewToolbars)), 0);
-	checkbuttonViewToolbars = PtCreateWidget(PtToggleButton, vshowgroup, n, args);
+	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, TR(pSS->getValue(AP_STRING_ID_DLG_Options_Label_ViewStandardTB)), 0);
+	checkbuttonViewStandard = PtCreateWidget(PtToggleButton, vshowgroup, n, args);
+
+	n = 0;
+	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, TR(pSS->getValue(AP_STRING_ID_DLG_Options_Label_ViewFormatTB)), 0);
+	checkbuttonViewFormat = PtCreateWidget(PtToggleButton, vshowgroup, n, args);
+
+	n = 0;
+	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, TR(pSS->getValue(AP_STRING_ID_DLG_Options_Label_ViewExtraTB)), 0);
+	checkbuttonViewExtra = PtCreateWidget(PtToggleButton, vshowgroup, n, args);
+
+	n = 0;
+	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, TR(pSS->getValue(AP_STRING_ID_DLG_Options_Label_ViewStatusBar)), 0);
+	checkbuttonViewStatusBar = PtCreateWidget(PtToggleButton, vshowgroup, n, args);
 
 	n = 0;
 	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, TR(pSS->getValue(AP_STRING_ID_DLG_Options_Label_ViewCursorBlink)), 0);
@@ -542,7 +557,28 @@ PtWidget_t* AP_QNXDialog_Options::_constructWindow ()
 	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, TR(pSS->getValue(AP_STRING_ID_DLG_Options_Label_ViewUnprintable)), 0);
 	checkbuttonViewUnprintable = PtCreateWidget(PtToggleButton, vviewviewgroup, n, args);
 
-	/* Create the horizontal button group */
+	/*** Other/Misc Tab (smart quotes) ***/
+	n = 0;
+	PtSetArg(&args[n++], Pt_ARG_TITLE, TR(pSS->getValue(AP_STRING_ID_DLG_Options_TabLabel_Other)), 0);
+	PtSetArg(&args[n++], Pt_ARG_WIDTH, TAB_WIDTH, 0);
+	PtSetArg(&args[n++], Pt_ARG_HEIGHT, TAB_HEIGHT, 0);
+	PtWidget_t *miscTab = PtCreateWidget(PtPane, panelGroup, n, args);
+
+	n = 0;
+	PtSetArg(&args[n++], Pt_ARG_GROUP_ORIENTATION, Pt_GROUP_VERTICAL, 0);
+	PtSetArg(&args[n++], Pt_ARG_MARGIN_WIDTH, ABI_MODAL_MARGIN_SIZE, 0);
+	PtSetArg(&args[n++], Pt_ARG_MARGIN_HEIGHT, ABI_MODAL_MARGIN_SIZE, 0);
+	PtWidget_t *vmiscgroup = PtCreateWidget(PtGroup, miscTab, n, args);
+
+	/** SmartQuotes Enable **/
+	n = 0;
+	PtSetArg(&args[n++], Pt_ARG_TEXT_STRING, 
+		TR(pSS->getValue(AP_STRING_ID_DLG_Options_Label_SmartQuotesEnable)), 0);
+	PtSetArg(&args[n++], Pt_ARG_WIDTH,  2 * ABI_DEFAULT_BUTTON_WIDTH, 0);
+	m_checkbuttonSmartQuotesEnable = PtCreateWidget(PtToggleButton, vmiscgroup, n, args);
+
+
+	/*** Create the horizontal button group ***/
 	n = 0;
 	PtSetArg(&args[n++], Pt_ARG_GROUP_SPACING_X, 5, 0);
 	PtWidget_t *hbuttongroup = PtCreateWidget (PtGroup, vallgroup, n, args);
@@ -604,7 +640,10 @@ PtWidget_t* AP_QNXDialog_Options::_constructWindow ()
     m_checkbuttonViewShowRuler		= checkbuttonViewRuler;
     m_listViewRulerUnits			= listViewRulerUnit;
     m_checkbuttonViewCursorBlink	= checkbuttonViewCursorBlink;
-    m_checkbuttonViewShowToolbars	= checkbuttonViewToolbars;
+    m_checkbuttonViewShowStandardBar	= checkbuttonViewStandard;
+    m_checkbuttonViewShowFormatBar	= checkbuttonViewFormat;
+    m_checkbuttonViewShowExtraBar	= checkbuttonViewExtra;
+    m_checkbuttonViewShowStatusBar	= checkbuttonViewStatusBar;
     m_checkbuttonViewAll			= checkbuttonViewAll;
     m_checkbuttonViewHiddenText		= checkbuttonViewHidden;
     m_checkbuttonViewUnprintable	= checkbuttonViewUnprintable;
@@ -668,6 +707,12 @@ PtWidget_t *AP_QNXDialog_Options::_lookupWidget ( tControl id )
 		return m_buttonSpellIgnoreEdit;
 		break;
 
+	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+	// smart quotes
+	case id_CHECK_SMART_QUOTES_ENABLE:
+		return m_checkbuttonSmartQuotesEnable;
+		break;
+
 	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 	// prefs
 	case id_CHECK_PREFS_AUTO_SAVE:
@@ -692,8 +737,20 @@ PtWidget_t *AP_QNXDialog_Options::_lookupWidget ( tControl id )
 		return m_checkbuttonViewCursorBlink;
 		break;
 
-	case id_CHECK_VIEW_SHOW_TOOLBARS:
-		return m_checkbuttonViewShowToolbars;
+	case id_CHECK_VIEW_SHOW_STANDARD_TOOLBAR:
+		return m_checkbuttonViewShowStandardBar;
+		break;
+
+	case id_CHECK_VIEW_SHOW_FORMAT_TOOLBAR:
+		return m_checkbuttonViewShowFormatBar;
+		break;
+
+	case id_CHECK_VIEW_SHOW_EXTRA_TOOLBAR:
+		return m_checkbuttonViewShowExtraBar;
+		break;
+
+	case id_CHECK_VIEW_SHOW_STATUS_BAR:
+		return m_checkbuttonViewShowStatusBar;
 		break;
 
 	case id_CHECK_VIEW_ALL:
@@ -777,9 +834,15 @@ DEFINE_GET_SET_BOOL(SpellUppercase);
 DEFINE_GET_SET_BOOL(SpellNumbers);
 DEFINE_GET_SET_BOOL(SpellInternet);
 
+DEFINE_GET_SET_BOOL(SmartQuotesEnable);
+
 DEFINE_GET_SET_BOOL(PrefsAutoSave);
 
-DEFINE_GET_SET_BOOL	(ViewShowRuler);
+DEFINE_GET_SET_BOOL(ViewShowRuler);
+DEFINE_GET_SET_BOOL(ViewShowStandardBar);
+DEFINE_GET_SET_BOOL(ViewShowFormatBar);
+DEFINE_GET_SET_BOOL(ViewShowExtraBar);
+DEFINE_GET_SET_BOOL(ViewShowStatusBar);
 
 UT_Dimension AP_QNXDialog_Options::_gatherViewRulerUnits(void) 
 {				
@@ -828,7 +891,6 @@ void    AP_QNXDialog_Options::_setViewRulerUnits(UT_Dimension dim)
 }
 
 DEFINE_GET_SET_BOOL	(ViewCursorBlink);
-DEFINE_GET_SET_BOOL	(ViewShowToolbars);
 
 DEFINE_GET_SET_BOOL	(ViewAll);
 DEFINE_GET_SET_BOOL	(ViewHiddenText);
