@@ -97,7 +97,7 @@ UT_Error AP_QNXFrame::_showDocument(UT_uint32 iZoom)
 	UT_uint32 nrToolbars;
 	UT_uint32 point = 0;
 
-	pG = new GR_QNXGraphics(m_wTopLevelWindow, m_dArea);
+	pG = new GR_QNXGraphics(m_wTopLevelWindow, m_dArea, getApp());
 	ENSUREP(pG);
 	pG->setZoomPercentage(iZoom);
 	
@@ -398,12 +398,39 @@ UT_Bool AP_QNXFrame::initialize(void)
 		return UT_FALSE;
 
 	_createTopLevelWindow();
+	_showOrHideToolbars();
+	_showOrHideStatusbar();
 	PtRealizeWidget(m_wTopLevelWindow);
 	PtDamageWidget(m_wTopLevelWindow);
 
 	return UT_TRUE;
 }
 
+// Does the initial show/hide of toolbars (based on the user prefs).
+// This is needed because toggleBar is called only when the user
+// (un)checks the show {Stantandard,Format,Extra} toolbar checkbox,
+// and thus we have to manually call this function at startup.
+void AP_QNXFrame::_showOrHideToolbars(void)
+{
+    UT_Bool *bShowBar = static_cast<AP_FrameData*> (m_pData)->m_bShowBar;
+
+    for (UT_uint32 i = 0; i < m_vecToolbarLayoutNames.getItemCount(); i++)
+    {
+        // TODO: The two next lines are here to bind the EV_Toolbar to the
+        // AP_FrameData, but their correct place are next to the toolbar creation (JCA)
+        EV_QNXToolbar * pQNXToolbar = static_cast<EV_QNXToolbar *> (m_vecToolbars.getNthItem(i));
+        static_cast<AP_FrameData*> (m_pData)->m_pToolbar[i] = pQNXToolbar;
+        toggleBar(i, bShowBar[i]);
+    }
+}
+
+// Does the initial show/hide of toolbars (based on the user prefs).
+void AP_QNXFrame::_showOrHideStatusbar(void)
+{
+    UT_Bool bShowStatusBar = static_cast<AP_FrameData*> (m_pData)->m_bShowStatusBar;
+    //  toggleStatusBar(bShowStatusBar);
+}
+
 /*****************************************************************/
 
 UT_Bool AP_QNXFrame::initFrameData(void)
@@ -648,9 +675,6 @@ PtWidget_t * AP_QNXFrame::_createDocumentWindow(void)
 #define _VS_STRETCH_ (Pt_GROUP_STRETCH_HORIZONTAL | Pt_GROUP_STRETCH_VERTICAL)
 	PtSetArg(&args[n], Pt_ARG_GROUP_FLAGS, _VS_STRETCH_, _VS_STRETCH_); n++;
 	group = PtCreateWidget(PtGroup, getTopLevelWindow(), n, args);
-	if (!group) {
-		printf("Can't get the VSB group \n");
-	}
 
 	n = 0;
 	PtSetArg(&args[n++], Pt_ARG_SCROLLBAR_FLAGS, Pt_SCROLLBAR_FOCUSED | 0 /*Vertical*/, 
@@ -669,7 +693,7 @@ PtWidget_t * AP_QNXFrame::_createDocumentWindow(void)
 	m_AvailableArea.size.h -= area.size.h;
 #else
 	area.size.h = SCROLLBAR_WIDTHHEIGHT;
-	area.size.w = savedarea.size.w;
+	area.size.w = savedarea.size.w - SCROLLBAR_WIDTHHEIGHT;
 	area.pos.y = savedarea.pos.y + savedarea.size.h - area.size.h;
 	area.pos.x = savedarea.pos.x;
 #endif
@@ -680,9 +704,6 @@ PtWidget_t * AP_QNXFrame::_createDocumentWindow(void)
 #define _HS_STRETCH_ (Pt_GROUP_STRETCH_HORIZONTAL | Pt_GROUP_STRETCH_VERTICAL)
 	PtSetArg(&args[n], Pt_ARG_GROUP_FLAGS, _HS_STRETCH_, _HS_STRETCH_); n++;
 	group = PtCreateWidget(PtGroup, getTopLevelWindow(), n, args);
-	if (!group) {
-		printf("Can't get the HSB group \n");
-	}
 	
 	n = 0;
 	PtSetArg(&args[n++], Pt_ARG_SCROLLBAR_FLAGS, Pt_SCROLLBAR_FOCUSED | 1 /*Horizontal*/,
@@ -759,6 +780,7 @@ void AP_QNXFrame::setStatusMessage(const char * szMsg)
 
 void AP_QNXFrame::_setWindowIcon(void)
 {
+#if 0
 	UT_DEBUGMSG(("TODO: Fix the setting of the ICON "));
 
 	// attach program icon to window
@@ -786,6 +808,7 @@ void AP_QNXFrame::_setWindowIcon(void)
 	PtSetArg(&args[n++], Pt_ARG_ICON_WINDOW, icon, sizeof(*icon));
 	PtSetArg(&args[n++], Pt_ARG_WINDOW_RENDER_FLAGS, Ph_WM_RENDER_ASICON, Ph_WM_RENDER_ASICON);
 	PtSetResources(window, n, args);
+#endif
 }
 
 UT_Error AP_QNXFrame::_replaceDocument(AD_Document * pDoc)
@@ -796,6 +819,34 @@ UT_Error AP_QNXFrame::_replaceDocument(AD_Document * pDoc)
 	return _showDocument();
 }
 
+void AP_QNXFrame::toggleBar(UT_uint32 iBarNb, UT_Bool bBarOn) {
+    AP_FrameData *pFrameData = static_cast<AP_FrameData *> (getFrameData());
+    UT_ASSERT(pFrameData);
+
+    if (bBarOn) {
+        pFrameData->m_pToolbar[iBarNb]->show();
+    }
+    else {
+        pFrameData->m_pToolbar[iBarNb]->hide();
+    }
+}
+
 void AP_QNXFrame::toggleRuler(UT_Bool bRulerOn) {
 	UT_DEBUGMSG(("TODO: Toggle ruler code "));
 }
+
+void AP_QNXFrame::toggleStatusBar(UT_Bool bStatusBarOn) {
+    AP_FrameData *pFrameData = static_cast<AP_FrameData *> (getFrameData());
+    UT_ASSERT(pFrameData);
+
+    if (bStatusBarOn) {
+        pFrameData->m_pStatusBar->show();
+    }
+    else {
+        pFrameData->m_pStatusBar->hide();
+    }
+}
+
+void AP_QNXFrame::setDocumentFocus() {
+	PtContainerGiveFocus(m_dArea, NULL);
+}
