@@ -47,7 +47,11 @@ public:
 	virtual void				setZoomPercentage(UT_uint32 iZoom);
 	virtual UT_uint32			getZoomPercentage(void);
 	virtual void				setStatusMessage(const char * szMsg);
+
 	virtual void 				toggleRuler(UT_Bool bRulerOn);
+	virtual void 				toggleBar(UT_uint32 iBarNb, UT_Bool bRulerOn);
+	virtual void 				toggleStatusBar(UT_Bool bStatusBarOn);
+	virtual void				setDocumentFocus();
 
 protected:
 	virtual PtWidget_t *		_createDocumentWindow(void);
@@ -58,6 +62,8 @@ protected:
 	static void					_scrollFuncX(void * pData, UT_sint32 xoff, UT_sint32 xlimit);
 	static void					_scrollFuncY(void * pData, UT_sint32 yoff, UT_sint32 ylimit);
 	UT_Error					_replaceDocument(AD_Document * pDoc);
+    virtual void                _showOrHideToolbars();
+    virtual void                _showOrHideStatusbar();
 	
 
 	PtWidget_t *				m_dArea;
