@@ -31,6 +31,7 @@
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <signal.h>
 
 #include "ut_debugmsg.h"
 #include "ut_string.h"
@@ -69,6 +70,7 @@
 #include "ut_debugmsg.h"
 
 #include "fv_View.h"
+#include "fp_Run.h"
 
 /*****************************************************************/
 
@@ -153,11 +155,14 @@ UT_Bool AP_UnixApp::initialize(void)
 	
 	{
 		const char * szISpellDirectory = NULL;
-		getPrefsValueDirectory(UT_FALSE,AP_PREF_KEY_SpellDirectory,&szISpellDirectory);
+		getPrefsValueDirectory(UT_FALSE,
+				       (const XML_Char*)AP_PREF_KEY_SpellDirectory,
+				       (const XML_Char**)&szISpellDirectory);
 		UT_ASSERT((szISpellDirectory) && (*szISpellDirectory));
 
 		const char * szSpellCheckWordList = NULL;
-		getPrefsValue(AP_PREF_KEY_SpellCheckWordList,&szSpellCheckWordList);
+		getPrefsValue(AP_PREF_KEY_SpellCheckWordList,
+			      (const XML_Char**)&szSpellCheckWordList);
 		UT_ASSERT((szSpellCheckWordList) && (*szSpellCheckWordList));
 		
 		char * szPathname = (char *)calloc(sizeof(char),strlen(szISpellDirectory)+strlen(szSpellCheckWordList)+2);
@@ -183,21 +188,24 @@ UT_Bool AP_UnixApp::initialize(void)
 		// assume we will be using the builtin set (either as the main
 		// set or as the fallback set).
 		
-		AP_BuiltinStringSet * pBuiltinStringSet = new AP_BuiltinStringSet(this,AP_PREF_DEFAULT_StringSet);
+		AP_BuiltinStringSet * pBuiltinStringSet = new AP_BuiltinStringSet(this,(XML_Char*)AP_PREF_DEFAULT_StringSet);
 		UT_ASSERT(pBuiltinStringSet);
 		m_pStringSet = pBuiltinStringSet;
 
-		// see if we should load an alternate set from the disk
+		// see if we should load an alternative set from the disk
 		
 		const char * szDirectory = NULL;
 		const char * szStringSet = NULL;
 
-		if (   (getPrefsValue(AP_PREF_KEY_StringSet,&szStringSet))
+		if (   (getPrefsValue(AP_PREF_KEY_StringSet,
+				      (const XML_Char**)&szStringSet))
 			&& (szStringSet)
 			&& (*szStringSet)
 			&& (UT_stricmp(szStringSet,AP_PREF_DEFAULT_StringSet) != 0))
 		{
-			getPrefsValueDirectory(UT_TRUE,AP_PREF_KEY_StringSetDirectory,&szDirectory);
+			getPrefsValueDirectory(UT_TRUE,
+					       (const XML_Char*)AP_PREF_KEY_StringSetDirectory,
+					       (const XML_Char**)&szDirectory);
 			UT_ASSERT((szDirectory) && (*szDirectory));
 
 			char * szPathname = (char *)calloc(sizeof(char),strlen(szDirectory)+strlen(szStringSet)+100);
@@ -226,6 +234,21 @@ UT_Bool AP_UnixApp::initialize(void)
 			free(szPathname);
 		}
 	}
+	
+	// Now we have the strings loaded we can populate the field names correctly
+	int i;
+	
+	for (i = 0; fp_FieldTypes[i].m_Type != FPFIELDTYPE_END; i++)
+	{
+	    (&fp_FieldTypes[i])->m_Desc = m_pStringSet->getValue(fp_FieldTypes[i].m_DescId);
+	    UT_DEBUGMSG(("Setting field type desc for type %d, desc=%s\n", fp_FieldTypes[i].m_Type, fp_FieldTypes[i].m_Desc));
+	}
+
+	for (i = 0; fp_FieldFmts[i].m_Tag != NULL; i++)
+	{
+	    (&fp_FieldFmts[i])->m_Desc = m_pStringSet->getValue(fp_FieldFmts[i].m_DescId);
+	    UT_DEBUGMSG(("Setting field desc for field %s, desc=%s\n", fp_FieldFmts[i].m_Tag, fp_FieldFmts[i].m_Desc));
+	}
 
 	//////////////////////////////////////////////////////////////////
 
@@ -347,9 +370,15 @@ void AP_UnixApp::pasteFromClipboard(PD_DocumentRange * pDocRange, UT_Bool bUseCl
 	// paste from the system clipboard using the best-for-us format
 	// that is present.  try to get the content in the order listed.
 
+	/*
+	    I've reordered AP_CLIPBOARD_STRING and AP_CLIPBOARD_TEXTPLAIN_8BIT
+	    since for non-Latin1 text the data in AP_CLIPBOARD_TEXTPLAIN_8BIT
+	    format has name of encoding as prefix, and AP_CLIPBOARD_STRING
+	    doesn't - hvv.
+	*/
 	static const char * aszFormatsAccepted[] = { AP_CLIPBOARD_RTF,
-												 AP_CLIPBOARD_TEXTPLAIN_8BIT,
 												 AP_CLIPBOARD_STRING,
+												 AP_CLIPBOARD_TEXTPLAIN_8BIT,
 												 0 /* must be last */ };
 
 	// TODO currently i have this set so that a ^v or Menu[Edit/Paste] will
@@ -763,7 +792,7 @@ GR_Image * AP_UnixApp::_showSplash(UT_uint32 delay)
 		gtk_widget_show(wSplash);
 		
 		// create image context
-		pUnixGraphics = new GR_UnixGraphics(da->window, NULL);
+		pUnixGraphics = new GR_UnixGraphics(da->window, NULL, m_pApp);
 		pSplashImage = pUnixGraphics->createNewImage("splash", pBB, iSplashWidth, iSplashHeight);
 
 		// another for luck (to bring it up forward and paint)
@@ -837,6 +866,29 @@ int AP_UnixApp::main(const char * szAppName, int argc, char ** argv)
 
 	AP_UnixApp * pMyUnixApp = new AP_UnixApp(&Args, szAppName);
 
+	// Setup signal handlers, primarily for segfault
+	// If we segfaulted before here, we *really* blew it
+
+	struct sigaction sa;
+
+	sa.sa_handler = signalWrapper;
+
+	sigfillset(&sa.sa_mask);  // We don't want to hear about other signals
+	sigdelset(&sa.sa_mask, SIGABRT); // But we will call abort(), so we can't ignore that
+
+#ifndef AIX
+	sa.sa_flags = SA_NODEFER | SA_RESETHAND; // Don't handle nested signals
+#else
+	sa.sa_flags = 0;
+#endif
+
+	sigaction(SIGSEGV, &sa, NULL);
+	sigaction(SIGBUS, &sa, NULL);
+	sigaction(SIGILL, &sa, NULL);
+	sigaction(SIGQUIT, &sa, NULL);
+	sigaction(SIGFPE, &sa, NULL);
+	// TODO: handle SIGABRT
+	
 	// if the initialize fails, we don't have icons, fonts, etc.
 	if (!pMyUnixApp->initialize())
 	{
@@ -900,7 +952,7 @@ UT_Bool AP_UnixApp::parseCommandLine(void)
 #ifdef DEBUG
 				// dump the string table in english as a template for translators.
 				// see abi/docs/AbiSource_Localization.abw for details.
-				AP_BuiltinStringSet * pBuiltinStringSet = new AP_BuiltinStringSet(this,AP_PREF_DEFAULT_StringSet);
+				AP_BuiltinStringSet * pBuiltinStringSet = new AP_BuiltinStringSet(this,(XML_Char*)AP_PREF_DEFAULT_StringSet);
 				pBuiltinStringSet->dumpBuiltinSet("en-US.strings");
 				delete pBuiltinStringSet;
 #endif
@@ -1009,39 +1061,39 @@ UT_Bool AP_UnixApp::parseCommandLine(void)
 					switch (error)
 					{
 					case -301:
-						pDialog->setMessage(pSS->getValue(AP_STRING_ID_MSG_IE_FileNotFound),m_pArgs->m_argv[k]);
+						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_IE_FileNotFound),m_pArgs->m_argv[k]);
 						break;
 						
 					case -302:
-						pDialog->setMessage(pSS->getValue(AP_STRING_ID_MSG_IE_NoMemory),m_pArgs->m_argv[k]);
+						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_IE_NoMemory),m_pArgs->m_argv[k]);
 						break;
 						
 					case -303:
-						pDialog->setMessage(pSS->getValue(AP_STRING_ID_MSG_IE_UnknownType),m_pArgs->m_argv[k]);
+						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_IE_UnknownType),m_pArgs->m_argv[k]);
 						break;
 						
 					case -304:
-						pDialog->setMessage(pSS->getValue(AP_STRING_ID_MSG_IE_BogusDocument),m_pArgs->m_argv[k]);
+						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_IE_BogusDocument),m_pArgs->m_argv[k]);
 						break;
 						
 					case -305:
-						pDialog->setMessage(pSS->getValue(AP_STRING_ID_MSG_IE_CouldNotOpen),m_pArgs->m_argv[k]);
+						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_IE_CouldNotOpen),m_pArgs->m_argv[k]);
 						break;
 							
 					case -306:
-						pDialog->setMessage(pSS->getValue(AP_STRING_ID_MSG_IE_CouldNotWrite),m_pArgs->m_argv[k]);
+						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_IE_CouldNotWrite),m_pArgs->m_argv[k]);
 						break;
 						
 					case -307:
-						pDialog->setMessage(pSS->getValue(AP_STRING_ID_MSG_IE_FakeType),m_pArgs->m_argv[k]);
+						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_IE_FakeType),m_pArgs->m_argv[k]);
 						break;
 						
 					case -311:
-						pDialog->setMessage(pSS->getValue(AP_STRING_ID_MSG_IE_UnsupportedType),m_pArgs->m_argv[k]);
+						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_IE_UnsupportedType),m_pArgs->m_argv[k]);
 						break;
 						
 					default:
-						pDialog->setMessage(pSS->getValue(AP_STRING_ID_MSG_ImportError),m_pArgs->m_argv[k]);
+						pDialog->setMessage((char*)pSS->getValue(AP_STRING_ID_MSG_ImportError),m_pArgs->m_argv[k]);
 					}
 					pDialog->setButtons(XAP_Dialog_MessageBox::b_O);
 					pDialog->setDefaultAnswer(XAP_Dialog_MessageBox::a_OK);
@@ -1085,9 +1137,11 @@ void AP_UnixApp::_printUsage(void)
 {
 	// just print to stdout, not stderr
 	printf("\nUsage: %s [option]... [file]...\n\n", m_pArgs->m_argv[0]);
-	printf("  -to               The target format of the file (abw, zabw, rtf, txt, utf8, html, latex)");
-	printf("  -verbose          The verbosity level (0, 1, 2)");
-	printf("  -show             If you really want to start the GUI (even if you use the --to option)");
+	printf("  -to               The target format of the file\n");
+        printf("                    (abw, zabw, rtf, txt, utf8, html, latex)\n");
+	printf("  -verbose          The verbosity level (0, 1, 2)\n");
+	printf("  -show             If you really want to start the GUI\n");
+        printf("                    (even if you use the --to option)\n");
 #ifdef DEBUG
 	printf("  -dumpstrings      dump strings strings to file\n");
 #endif
@@ -1101,3 +1155,41 @@ void AP_UnixApp::_printUsage(void)
 	printf("\n");
 }
 
+void signalWrapper(int sig_num)
+{
+	AP_UnixApp *pApp = (AP_UnixApp *) XAP_App::getApp();
+	pApp->catchSignals(sig_num);
+}
+
+static gint s_signal_count = 0;
+
+void AP_UnixApp::catchSignals(int sig_num)
+{
+        // Reset the signal handler 
+  // (not that it matters - this is mostly for race conditions)
+        signal(SIGSEGV, signalWrapper);
+
+        s_signal_count = s_signal_count + 1;
+        if(s_signal_count > 1)
+        {
+                UT_DEBUGMSG(("Segfault during filesave - no file saved  \n"));
+                fflush(stdout);
+                abort();
+        }
+
+	UT_DEBUGMSG(("Oh no - we just segfaulted!\n"));
+
+	UT_uint32 i = 0;
+	for(;i<m_vecFrames.getItemCount();i++)
+	{
+		AP_UnixFrame * curFrame = (AP_UnixFrame*) m_vecFrames[i];
+		UT_ASSERT(curFrame);
+		curFrame->backup();
+	}
+
+	fflush(stdout);
+
+	// Abort and dump core
+	abort();
+}
+	
