@@ -37,6 +37,7 @@
 #	include "xap_UnixDlg_MessageBox.h"
 #	include "xap_UnixDlg_FileOpenSaveAs.h"
 #	include "xap_UnixDlg_Print.h"
+#       include "xap_UnixDlg_PrintPreview.h"
 #	include "xap_UnixDlg_WindowMore.h"
 #	include "xap_UnixDlg_FontChooser.h"
 #	include "xap_UnixDlg_About.h"
@@ -49,24 +50,35 @@
 #	include "ap_UnixDialog_Paragraph.h"
 #	include "ap_UnixDialog_Options.h"
 #	include "ap_UnixDialog_Spell.h"
+#	include "ap_UnixDialog_Tab.h"
 #	include "ap_UnixDialog_Insert_DateTime.h"
 #	include "ap_UnixDialog_WordCount.h"
 #	include "ap_UnixDialog_Field.h"
+#	include "ap_UnixDialog_Lists.h"
+#	include "ap_UnixDialog_Columns.h"
+#	include "ap_UnixDialog_Tab.h"
 
 #ifdef HAVE_GNOME
 #	include "xap_UnixGnomeDlg_About.h"
 #	include "xap_UnixGnomeDlg_FontChooser.h"
 #	include "xap_UnixGnomeDlg_Insert_Symbol.h"
 #	include "xap_UnixGnomeDlg_MessageBox.h"
+#       include "xap_UnixGnomeDlg_Print.h"
+#       include "xap_UnixGnomeDlg_PrintPreview.h"
 #	include "xap_UnixGnomeDlg_Zoom.h"
 
 #	include "ap_UnixGnomeDialog_Replace.h"
 #	include "ap_UnixGnomeDialog_Break.h"
 #	include "ap_UnixGnomeDialog_Goto.h"
+#       include "ap_UnixGnomeDialog_Insert_DateTime.h"
 #	include "ap_UnixGnomeDialog_Paragraph.h"
 #	include "ap_UnixGnomeDialog_Options.h"
 #	include "ap_UnixGnomeDialog_WordCount.h"
 #	include "ap_UnixGnomeDialog_Field.h"
+#       include "ap_UnixGnomeDialog_Lists.h"
+#       include "ap_UnixGnomeDialog_Columns.h"
+#       include "ap_UnixGnomeDialog_Spell.h"
+#       include "ap_UnixGnomeDialog_Tab.h"
 
 #endif
 
@@ -78,24 +90,30 @@
 	DeclareDialog(XAP_DIALOG_ID_MESSAGE_BOX,	XAP_UnixGnomeDialog_MessageBox)
 	DeclareDialog(XAP_DIALOG_ID_FILE_OPEN,		XAP_UnixDialog_FileOpenSaveAs)
 	DeclareDialog(XAP_DIALOG_ID_FILE_SAVEAS,	XAP_UnixDialog_FileOpenSaveAs)
-	DeclareDialog(XAP_DIALOG_ID_PRINT,			XAP_UnixDialog_Print)
+	DeclareDialog(XAP_DIALOG_ID_PRINT,			XAP_UnixGnomeDialog_Print)
+        DeclareDialog(XAP_DIALOG_ID_PRINTPREVIEW, XAP_UnixGnomeDialog_PrintPreview)
 	DeclareDialog(XAP_DIALOG_ID_PRINTTOFILE,	XAP_UnixDialog_FileOpenSaveAs)
 	DeclareDialog(XAP_DIALOG_ID_FONT,			XAP_UnixGnomeDialog_FontChooser)
 	DeclareDialog(XAP_DIALOG_ID_WINDOWMORE,		XAP_UnixDialog_WindowMore)
 	DeclareDialog(XAP_DIALOG_ID_ZOOM,			XAP_UnixGnomeDialog_Zoom)
 	DeclareDialog(XAP_DIALOG_ID_ABOUT,			XAP_UnixGnomeDialog_About)
 	DeclareDialog(XAP_DIALOG_ID_INSERT_SYMBOL,  XAP_UnixGnomeDialog_Insert_Symbol)
+	DeclareDialog(XAP_DIALOG_ID_INSERT_PICTURE,	XAP_UnixDialog_FileOpenSaveAs)
 
 	DeclareDialog(AP_DIALOG_ID_REPLACE,			AP_UnixGnomeDialog_Replace)
 	DeclareDialog(AP_DIALOG_ID_FIND,			AP_UnixGnomeDialog_Replace)
 	DeclareDialog(AP_DIALOG_ID_GOTO,			AP_UnixGnomeDialog_Goto)
 	DeclareDialog(AP_DIALOG_ID_BREAK,			AP_UnixGnomeDialog_Break)
-	DeclareDialog(AP_DIALOG_ID_SPELL,			AP_UnixDialog_Spell)
-	DeclareDialog(AP_DIALOG_ID_PARAGRAPH,		AP_UnixGnomeDialog_Paragraph)	
+	DeclareDialog(AP_DIALOG_ID_SPELL,			AP_UnixGnomeDialog_Spell)
+	DeclareDialog(AP_DIALOG_ID_TAB,				AP_UnixGnomeDialog_Tab)
+	DeclareDialog(AP_DIALOG_ID_PARAGRAPH,		AP_UnixGnomeDialog_Paragraph)
 	DeclareDialog(AP_DIALOG_ID_OPTIONS,			AP_UnixGnomeDialog_Options)
-	DeclareDialog(AP_DIALOG_ID_INSERT_DATETIME,	AP_UnixDialog_Insert_DateTime)			
+	DeclareDialog(AP_DIALOG_ID_INSERT_DATETIME,	AP_UnixGnomeDialog_Insert_DateTime)
 	DeclareDialog(AP_DIALOG_ID_WORDCOUNT,       AP_UnixGnomeDialog_WordCount)
 	DeclareDialog(AP_DIALOG_ID_FIELD,			AP_UnixGnomeDialog_Field)
+	DeclareDialog(AP_DIALOG_ID_LISTS,		     AP_UnixGnomeDialog_Lists)
+	DeclareDialog(AP_DIALOG_ID_COLUMNS,			AP_UnixGnomeDialog_Columns)
+	DeclareDialog(AP_DIALOG_ID_TAB,				AP_UnixGnomeDialog_Tab)
 
 	// ... also add new dialogs here ...
 #   else
@@ -108,6 +126,7 @@
 	DeclareDialog(XAP_DIALOG_ID_WINDOWMORE,		XAP_UnixDialog_WindowMore)
 	DeclareDialog(XAP_DIALOG_ID_ZOOM,			XAP_UnixDialog_Zoom)
 	DeclareDialog(XAP_DIALOG_ID_INSERT_SYMBOL,  XAP_UnixDialog_Insert_Symbol)
+	DeclareDialog(XAP_DIALOG_ID_INSERT_PICTURE,	XAP_UnixDialog_FileOpenSaveAs)
 
 	DeclareDialog(XAP_DIALOG_ID_ABOUT,			XAP_UnixDialog_About)
 
@@ -117,14 +136,16 @@
 	DeclareDialog(AP_DIALOG_ID_BREAK,			AP_UnixDialog_Break)
 	DeclareDialog(AP_DIALOG_ID_SPELL,			AP_UnixDialog_Spell)
 	DeclareDialog(AP_DIALOG_ID_PARAGRAPH,		AP_UnixDialog_Paragraph)	
-	DeclareDialog(AP_DIALOG_ID_OPTIONS,			AP_UnixDialog_Options)
+	DeclareDialog(AP_DIALOG_ID_OPTIONS,			AP_UnixDialog_Options)	
+	DeclareDialog(AP_DIALOG_ID_TAB,				AP_UnixDialog_Tab)
 	DeclareDialog(AP_DIALOG_ID_INSERT_DATETIME,	AP_UnixDialog_Insert_DateTime)		
 	DeclareDialog(AP_DIALOG_ID_WORDCOUNT,		AP_UnixDialog_WordCount)
 	DeclareDialog(AP_DIALOG_ID_FIELD,			AP_UnixDialog_Field)
+	DeclareDialog(AP_DIALOG_ID_LISTS,			AP_UnixDialog_Lists)
+	DeclareDialog(AP_DIALOG_ID_COLUMNS,			AP_UnixDialog_Columns)
+	DeclareDialog(AP_DIALOG_ID_TAB,				AP_UnixDialog_Tab)
+
 	// ... also add new dialogs here ...
 #   endif
 
 #endif
-
-
-
