@@ -114,7 +114,6 @@ void AP_UnixDialog_Options::runModal(XAP_Frame * pFrame)
     // Center our new dialog in its parent and make it a transient
     // so it won't get lost underneath
     centerDialog(parentWindow, mainWindow);
-    gtk_window_set_transient_for(GTK_WINDOW(mainWindow), GTK_WINDOW(parentWindow));
 
     // Show the top level dialog,
     gtk_widget_show(mainWindow);
@@ -188,6 +187,7 @@ GtkWidget* AP_UnixDialog_Options::_constructWindowContents ()
 
 	GtkWidget *windowOptions;
 	GtkWidget *notebook1;
+
 	GtkWidget *tableSpell;
 	GtkWidget *checkbuttonSpellHideErrors;
 	GtkWidget *checkbuttonSpellSuggest;
@@ -205,12 +205,18 @@ GtkWidget* AP_UnixDialog_Options::_constructWindowContents ()
 	GtkWidget *listSpellDicts_menu;
 	GtkWidget *glade_menuitem;
 	GtkWidget *labelSpell;
+
+	GtkWidget *tableOther;
+	GtkWidget *checkbuttonSmartQuotesEnable;
+	GtkWidget *labelSmartQuotes;
+
 	GtkWidget *tablePreferences;
 	GtkWidget *checkbuttonPrefsAutoSave;
 	GtkWidget *label6;
 	GtkWidget *comboPrefsSchemes;
 	GtkWidget *comboPrefsSchemesEdit;
 	GtkWidget *labelPreferences;
+
 	GtkWidget *hboxView;
 	GtkWidget *vbox4;
 	GtkWidget *frame2;
@@ -221,7 +227,10 @@ GtkWidget* AP_UnixDialog_Options::_constructWindowContents ()
 	GtkWidget *listViewRulerUnit;
 	GtkWidget *listViewRulerUnit_menu;
 	GtkWidget *checkbuttonViewCursorBlink;
-	GtkWidget *checkbuttonViewToolbars;
+	GtkWidget *checkbuttonViewStandard;
+	GtkWidget *checkbuttonViewFormat;
+	GtkWidget *checkbuttonViewExtra;
+	GtkWidget *checkbuttonViewStatusBar;
 	GtkWidget *frameViewStuff;
 	GtkWidget *vbox6;
 	GtkWidget *checkbuttonViewAll;
@@ -236,6 +245,7 @@ GtkWidget* AP_UnixDialog_Options::_constructWindowContents ()
 	                          (GtkDestroyNotify) gtk_widget_unref);
 	gtk_widget_show (notebook1);
 
+	// SPELL ////////////////////////////////////////////////////////////////
 	tableSpell = gtk_table_new (9, 3, FALSE);
 	gtk_widget_ref (tableSpell);
 	gtk_object_set_data_full (GTK_OBJECT (windowOptions), "tableSpell", tableSpell,
@@ -376,6 +386,7 @@ GtkWidget* AP_UnixDialog_Options::_constructWindowContents ()
 	gtk_widget_show (labelSpell);
 	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 0), labelSpell);
 
+	// PREFERENCES ////////////////////////////////////////////////////////////////
 	tablePreferences = gtk_table_new (2, 3, FALSE);
 	gtk_widget_ref (tablePreferences);
 	gtk_object_set_data_full (GTK_OBJECT (windowOptions), "tablePreferences", tablePreferences,
@@ -482,7 +493,7 @@ GtkWidget* AP_UnixDialog_Options::_constructWindowContents ()
 	gtk_widget_show (listViewRulerUnit);
 	gtk_box_pack_start (GTK_BOX (hbox10), listViewRulerUnit, FALSE, FALSE, 0);
 	listViewRulerUnit_menu = gtk_menu_new ();
-	glade_menuitem = gtk_menu_item_new_with_label (_("inch"));		// TODO
+	glade_menuitem = gtk_menu_item_new_with_label (pSS->getValue(XAP_STRING_ID_DLG_Unit_inch));
  	/**/ gtk_object_set_data(GTK_OBJECT(glade_menuitem), WIDGET_MENU_OPTION_PTR, (gpointer) listViewRulerUnit );
  	/**/ gtk_object_set_data(GTK_OBJECT(glade_menuitem), WIDGET_MENU_VALUE_TAG,  (gpointer) DIM_IN );	
  	CONNECT_MENU_ITEM_SIGNAL_ACTIVATE(glade_menuitem);
@@ -496,7 +507,7 @@ GtkWidget* AP_UnixDialog_Options::_constructWindowContents ()
  	// gtk_widget_show (glade_menuitem);
  	// gtk_menu_append (GTK_MENU (listViewRulerUnit_menu), glade_menuitem);
  
-	glade_menuitem = gtk_menu_item_new_with_label (_("cm"));		// TODO
+	glade_menuitem = gtk_menu_item_new_with_label (pSS->getValue(XAP_STRING_ID_DLG_Unit_cm));
  	/**/ gtk_object_set_data(GTK_OBJECT(glade_menuitem), WIDGET_MENU_OPTION_PTR, (gpointer) listViewRulerUnit );
  	/**/ gtk_object_set_data(GTK_OBJECT(glade_menuitem), WIDGET_MENU_VALUE_TAG,  (gpointer) DIM_CM );	
  	CONNECT_MENU_ITEM_SIGNAL_ACTIVATE(glade_menuitem);
@@ -510,14 +521,14 @@ GtkWidget* AP_UnixDialog_Options::_constructWindowContents ()
  	// CONNECT_MENU_ITEM_SIGNAL_ACTIVATE(glade_menuitem);
  	// gtk_menu_append (GTK_MENU (listViewRulerUnit_menu), glade_menuitem);
  
-	glade_menuitem = gtk_menu_item_new_with_label (_("points"));	// TODO
+	glade_menuitem = gtk_menu_item_new_with_label (pSS->getValue(XAP_STRING_ID_DLG_Unit_points));
  	/**/ gtk_object_set_data(GTK_OBJECT(glade_menuitem), WIDGET_MENU_OPTION_PTR, (gpointer) listViewRulerUnit );
   	/**/ gtk_object_set_data(GTK_OBJECT(glade_menuitem), WIDGET_MENU_VALUE_TAG,  (gpointer) DIM_PT );	
   	CONNECT_MENU_ITEM_SIGNAL_ACTIVATE(glade_menuitem);
 	gtk_widget_show (glade_menuitem);
 	gtk_menu_append (GTK_MENU (listViewRulerUnit_menu), glade_menuitem);
   
-  	glade_menuitem = gtk_menu_item_new_with_label (_("pico"));
+  	glade_menuitem = gtk_menu_item_new_with_label (pSS->getValue(XAP_STRING_ID_DLG_Unit_pico));
   	/**/ gtk_object_set_data(GTK_OBJECT(glade_menuitem), WIDGET_MENU_OPTION_PTR, (gpointer) listViewRulerUnit );
   	/**/ gtk_object_set_data(GTK_OBJECT(glade_menuitem), WIDGET_MENU_VALUE_TAG,  (gpointer) DIM_PI  );	
   	CONNECT_MENU_ITEM_SIGNAL_ACTIVATE(glade_menuitem);
@@ -526,12 +537,21 @@ GtkWidget* AP_UnixDialog_Options::_constructWindowContents ()
   
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (listViewRulerUnit), listViewRulerUnit_menu);
 
-	checkbuttonViewToolbars = gtk_check_button_new_with_label (pSS->getValue(AP_STRING_ID_DLG_Options_Label_ViewToolbars));
-	gtk_widget_ref (checkbuttonViewToolbars);
-	gtk_object_set_data_full (GTK_OBJECT (windowOptions), "checkbuttonViewToolbars", checkbuttonViewToolbars,
-	                          (GtkDestroyNotify) gtk_widget_unref);
-	gtk_widget_show (checkbuttonViewToolbars);
-	gtk_box_pack_start (GTK_BOX (vbox7), checkbuttonViewToolbars, FALSE, FALSE, 0);
+	checkbuttonViewStandard = gtk_check_button_new_with_label (pSS->getValue(AP_STRING_ID_DLG_Options_Label_ViewStandardTB));
+	gtk_widget_show (checkbuttonViewStandard);
+	gtk_box_pack_start (GTK_BOX (vbox7), checkbuttonViewStandard, FALSE, FALSE, 0);
+
+	checkbuttonViewFormat = gtk_check_button_new_with_label (pSS->getValue(AP_STRING_ID_DLG_Options_Label_ViewFormatTB));
+	gtk_widget_show (checkbuttonViewFormat);
+	gtk_box_pack_start (GTK_BOX (vbox7), checkbuttonViewFormat, FALSE, FALSE, 0);
+
+	checkbuttonViewExtra = gtk_check_button_new_with_label (pSS->getValue(AP_STRING_ID_DLG_Options_Label_ViewExtraTB));
+	gtk_widget_show (checkbuttonViewExtra);
+	gtk_box_pack_start (GTK_BOX (vbox7), checkbuttonViewExtra, FALSE, FALSE, 0);
+
+	checkbuttonViewStatusBar = gtk_check_button_new_with_label (pSS->getValue(AP_STRING_ID_DLG_Options_Label_ViewStatusBar));
+	gtk_widget_show (checkbuttonViewStatusBar);
+	gtk_box_pack_start (GTK_BOX (vbox7), checkbuttonViewStatusBar, FALSE, FALSE, 0);
 
 	checkbuttonViewCursorBlink = gtk_check_button_new_with_label (pSS->getValue(AP_STRING_ID_DLG_Options_Label_ViewCursorBlink));
 	gtk_widget_ref (checkbuttonViewCursorBlink);
@@ -585,6 +605,34 @@ GtkWidget* AP_UnixDialog_Options::_constructWindowContents ()
     //////////////////////////////////////////////////////////////////////
 	// END: glade stuff
 
+
+	// OTHER STUFF (including SMART QUOTES) //////////////////////////////
+	tableOther = gtk_table_new (2, 3, FALSE);
+	gtk_widget_ref (tableOther);
+	gtk_object_set_data_full (GTK_OBJECT (windowOptions), "tableOther", tableOther,
+	                          (GtkDestroyNotify) gtk_widget_unref);
+	gtk_widget_show (tableOther);
+	gtk_container_add (GTK_CONTAINER (notebook1), tableOther);
+	gtk_container_set_border_width (GTK_CONTAINER (tableOther), 10);
+
+	checkbuttonSmartQuotesEnable = gtk_check_button_new_with_label (pSS->getValue(AP_STRING_ID_DLG_Options_Label_SmartQuotesEnable));
+	gtk_widget_ref (checkbuttonSmartQuotesEnable);
+	gtk_object_set_data_full (GTK_OBJECT (windowOptions), "checkbuttonSmartQuotesEnable", checkbuttonSmartQuotesEnable,
+	                          (GtkDestroyNotify) gtk_widget_unref);
+	gtk_widget_show (checkbuttonSmartQuotesEnable);
+	gtk_table_attach (GTK_TABLE (tableOther), checkbuttonSmartQuotesEnable, 0, 3, 0, 1,
+	                  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
+	                  (GtkAttachOptions) (0), 0, 0);
+
+	labelSmartQuotes = gtk_label_new (pSS->getValue(AP_STRING_ID_DLG_Options_TabLabel_Other));
+	gtk_widget_ref (labelSmartQuotes);
+	gtk_object_set_data_full (GTK_OBJECT (windowOptions), "labelSmartQuotes", labelSmartQuotes,
+	                          (GtkDestroyNotify) gtk_widget_unref);
+	gtk_widget_show (labelSmartQuotes);
+	gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 3), labelSmartQuotes);
+
+	//////////////////////////////////////////////////////////////////
+
 	m_notebook = notebook1;
 
     m_checkbuttonSpellCheckAsType	= checkbuttonSpellCheckAsType;
@@ -600,13 +648,18 @@ GtkWidget* AP_UnixDialog_Options::_constructWindowContents ()
 	m_buttonSpellIgnoreEdit			= buttonSpellIgnoreEdit;
 	m_buttonSpellIgnoreReset		= buttonSpellIgnoreReset;
 
+    m_checkbuttonSmartQuotesEnable	= checkbuttonSmartQuotesEnable;
+
     m_checkbuttonPrefsAutoSave		= checkbuttonPrefsAutoSave;
 	m_comboPrefsScheme				= comboPrefsSchemes;
 
     m_checkbuttonViewShowRuler		= checkbuttonViewRuler;
     m_listViewRulerUnits			= listViewRulerUnit;
     m_checkbuttonViewCursorBlink	= checkbuttonViewCursorBlink;
-    m_checkbuttonViewShowToolbars	= checkbuttonViewToolbars;
+    m_checkbuttonViewShowStandardBar	= checkbuttonViewStandard;
+    m_checkbuttonViewShowFormatBar	= checkbuttonViewFormat;
+    m_checkbuttonViewShowExtraBar	= checkbuttonViewExtra;
+    m_checkbuttonViewShowStatusBar	= checkbuttonViewStatusBar;
     m_checkbuttonViewAll			= checkbuttonViewAll;
     m_checkbuttonViewHiddenText		= checkbuttonViewHidden;
     m_checkbuttonViewUnprintable	= checkbuttonViewUnprintable;
@@ -839,6 +892,12 @@ GtkWidget *AP_UnixDialog_Options::_lookupWidget ( tControl id )
 		return m_buttonSpellIgnoreEdit;
 		break;
 
+	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
+	// smart quotes
+	case id_CHECK_SMART_QUOTES_ENABLE:
+		return m_checkbuttonSmartQuotesEnable;
+		break;
+
 	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 	// prefs
 	case id_CHECK_PREFS_AUTO_SAVE:
@@ -863,8 +922,20 @@ GtkWidget *AP_UnixDialog_Options::_lookupWidget ( tControl id )
 		return m_checkbuttonViewCursorBlink;
 		break;
 
-	case id_CHECK_VIEW_SHOW_TOOLBARS:
-		return m_checkbuttonViewShowToolbars;
+	case id_CHECK_VIEW_SHOW_STANDARD_TOOLBAR:
+		return m_checkbuttonViewShowStandardBar;
+		break;
+
+	case id_CHECK_VIEW_SHOW_FORMAT_TOOLBAR:
+		return m_checkbuttonViewShowFormatBar;
+		break;
+
+	case id_CHECK_VIEW_SHOW_EXTRA_TOOLBAR:
+		return m_checkbuttonViewShowExtraBar;
+		break;
+
+	case id_CHECK_VIEW_SHOW_STATUS_BAR:
+		return m_checkbuttonViewShowStatusBar;
 		break;
 
 	case id_CHECK_VIEW_ALL:
@@ -934,9 +1005,15 @@ DEFINE_GET_SET_BOOL(SpellUppercase);
 DEFINE_GET_SET_BOOL(SpellNumbers);
 DEFINE_GET_SET_BOOL(SpellInternet);
 
+DEFINE_GET_SET_BOOL(SmartQuotesEnable);
+
 DEFINE_GET_SET_BOOL(PrefsAutoSave);
 
-DEFINE_GET_SET_BOOL	(ViewShowRuler);
+DEFINE_GET_SET_BOOL(ViewShowRuler);
+DEFINE_GET_SET_BOOL(ViewShowStandardBar);
+DEFINE_GET_SET_BOOL(ViewShowFormatBar);
+DEFINE_GET_SET_BOOL(ViewShowExtraBar);
+DEFINE_GET_SET_BOOL(ViewShowStatusBar);
 
 UT_Dimension AP_UnixDialog_Options::_gatherViewRulerUnits(void) 
 {				
@@ -1009,7 +1086,6 @@ void    AP_UnixDialog_Options::_setViewRulerUnits(UT_Dimension dim)
 }
 
 DEFINE_GET_SET_BOOL	(ViewCursorBlink);
-DEFINE_GET_SET_BOOL	(ViewShowToolbars);
 
 DEFINE_GET_SET_BOOL	(ViewAll);
 DEFINE_GET_SET_BOOL	(ViewHiddenText);
