@@ -130,44 +130,45 @@ static void s_replace_entry_activate(GtkWidget * widget, AP_UnixDialog_Replace *
 
 /*****************************************************************/
 
-void AP_UnixDialog_Replace::runModal(XAP_Frame * pFrame)
+void AP_UnixDialog_Replace::activate(void)
 {
+        UT_ASSERT(m_windowMain);
+	ConstructWindowName();
+	gtk_window_set_title (GTK_WINDOW (m_windowMain), m_WindowName);
+        gdk_window_raise(m_windowMain->window);
+}
+
+
+void AP_UnixDialog_Replace::notifyActiveFrame(XAP_Frame *pFrame)
+{
+        UT_ASSERT(m_windowMain);
+	ConstructWindowName();
+	gtk_window_set_title (GTK_WINDOW (m_windowMain), m_WindowName);
+}
+
+void AP_UnixDialog_Replace::runModeless(XAP_Frame * pFrame)
+{
+	// get the Dialog Id number
+	UT_sint32 sid =(UT_sint32)  getDialogId();
+
 	// Build the window's widgets and arrange them
 	GtkWidget * mainWindow = _constructWindow();
 	UT_ASSERT(mainWindow);
 
-	connectFocus(GTK_WIDGET(mainWindow),pFrame);
+	// Save dialog the ID number and pointer to the Dialog
+	m_pApp->rememberModelessId( sid,  (XAP_Dialog_Modeless *) m_pDialog);
+
+	// This magic command displays the frame where strings will be found
+	connectFocusModeless(GTK_WIDGET(mainWindow),m_pApp);
+
 	// Populate the window's data items
 	_populateWindowData();
 	
-	// To center the dialog, we need the frame of its parent.
-	XAP_UnixFrame * pUnixFrame = static_cast<XAP_UnixFrame *>(pFrame);
-	UT_ASSERT(pUnixFrame);
-	
-	// Get the GtkWindow of the parent frame
-	GtkWidget * parentWindow = pUnixFrame->getTopLevelWindow();
-	UT_ASSERT(parentWindow);
-	
-	// Center our new dialog in its parent and make it a transient
-	// so it won't get lost underneath
-    centerDialog(parentWindow, mainWindow);
-	gtk_window_set_transient_for(GTK_WINDOW(mainWindow), GTK_WINDOW(parentWindow));
-
 	// Show the top level dialog,
 	gtk_widget_show(mainWindow);
 
-	// Make it modal, and stick it up top
-	gtk_grab_add(mainWindow);
-
-	// this dialogs needs this
-	setView(static_cast<FV_View *> (pFrame->getCurrentView()));
-
-	// Run into the GTK event loop for this window.
-	gtk_main();
-
-	_storeWindowData();
-	
-	gtk_widget_destroy(mainWindow);
+	// this dialog needs this
+	setView(static_cast<FV_View *> (getActiveFrame()->getCurrentView()));
 }
 
 void AP_UnixDialog_Replace::event_Find(void)
@@ -241,13 +242,21 @@ void AP_UnixDialog_Replace::event_MatchCaseToggled(void)
 void AP_UnixDialog_Replace::event_Cancel(void)
 {
 	m_answer = AP_Dialog_Replace::a_CANCEL;
-	gtk_main_quit();
+	destroy();
+}
+
+void AP_UnixDialog_Replace::destroy(void)
+{
+	_storeWindowData();
+        modeless_cleanup();
+        gtk_widget_destroy(m_windowMain);
+        m_windowMain = NULL;
 }
 
 void AP_UnixDialog_Replace::event_WindowDelete(void)
 {
 	m_answer = AP_Dialog_Replace::a_CANCEL;	
-	gtk_main_quit();
+	destroy();
 }
 
 /*****************************************************************/
@@ -258,38 +267,29 @@ GtkWidget * AP_UnixDialog_Replace::_constructWindow(void)
 	GtkWidget *vboxReplace;
 	GtkWidget *tableReplace;
 	GtkWidget *entryFind;
-	GtkWidget *entryReplace;
+	GtkWidget *entryReplace = 0;
 	GtkWidget *checkbuttonMatchCase;
 	GtkWidget *labelFind;
 	GtkWidget *labelReplace;
+	GtkWidget *separator;
 	GtkWidget *hbuttonbox1;
 	GtkWidget *buttonFindNext;
-	GtkWidget *buttonReplace;
-	GtkWidget *buttonReplaceAll;
+	GtkWidget *buttonReplace = 0;
+	GtkWidget *buttonReplaceAll = 0;
 	GtkWidget *buttonCancel;
 
 
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
 	XML_Char * unixstr = NULL;	// used for conversions
 
-	// conditionally set title
-	if (m_id == AP_DIALOG_ID_FIND)
-		UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_FR_FindTitle));
-	else
-		UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_FR_ReplaceTitle));		
-	windowReplace = gtk_window_new (GTK_WINDOW_DIALOG);
+
+	windowReplace = gtk_window_new(GTK_WINDOW_DIALOG);
 	gtk_object_set_data (GTK_OBJECT (windowReplace), "windowReplace", windowReplace);
-	FREEP(unixstr);
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_Break_Insert));
-	gtk_window_set_title (GTK_WINDOW (windowReplace), "REPLACE THIS STRING");
-	// find is smaller
-	if (m_id == AP_DIALOG_ID_FIND)
-		gtk_widget_set_usize (windowReplace, 390, 102);
-	else
-		gtk_widget_set_usize (windowReplace, 390, 123);
-	
-	FREEP(unixstr);
-	gtk_window_set_policy (GTK_WINDOW (windowReplace), FALSE, FALSE, FALSE);
+
+	ConstructWindowName();
+	gtk_window_set_title (GTK_WINDOW (windowReplace),  m_WindowName);
+	gtk_window_set_default_size(GTK_WINDOW (windowReplace), 400, 100);
+
 
 	// top level vbox
 	vboxReplace = gtk_vbox_new (FALSE, 12);
@@ -311,9 +311,8 @@ GtkWidget * AP_UnixDialog_Replace::_constructWindow(void)
 	gtk_object_set_data (GTK_OBJECT (windowReplace), "labelFind", labelFind);
 	gtk_widget_show (labelFind);
 	gtk_table_attach (GTK_TABLE (tableReplace), labelFind, 0, 1, 0, 1,
-					  (GtkAttachOptions) (GTK_EXPAND | GTK_SHRINK | GTK_FILL),
-					  (GtkAttachOptions) (GTK_EXPAND | GTK_SHRINK | GTK_FILL), 0, 0);
-	gtk_label_set_justify (GTK_LABEL (labelFind), GTK_JUSTIFY_LEFT);
+			  GtkAttachOptions(GTK_FILL),
+			  GtkAttachOptions(0), 5, 0);
 	gtk_misc_set_alignment (GTK_MISC (labelFind), 0, 0.5);
 
 	// find entry is always here
@@ -321,8 +320,8 @@ GtkWidget * AP_UnixDialog_Replace::_constructWindow(void)
 	gtk_object_set_data (GTK_OBJECT (windowReplace), "entryFind", entryFind);
 	gtk_widget_show (entryFind);
 	gtk_table_attach (GTK_TABLE (tableReplace), entryFind, 1, 2, 0, 1,
-					  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
-					  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+			  GtkAttachOptions(GTK_EXPAND | GTK_FILL),
+			  GtkAttachOptions(GTK_EXPAND | GTK_FILL), 0, 0);
 
 	// match case is always here
 	UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_FR_MatchCase));	
@@ -331,8 +330,8 @@ GtkWidget * AP_UnixDialog_Replace::_constructWindow(void)
 	gtk_object_set_data (GTK_OBJECT (windowReplace), "checkbuttonMatchCase", checkbuttonMatchCase);
 	gtk_widget_show (checkbuttonMatchCase);
 	gtk_table_attach (GTK_TABLE (tableReplace), checkbuttonMatchCase, 1, 2, 1, 2,
-					  (GtkAttachOptions) (GTK_SHRINK | GTK_FILL),
-					  (GtkAttachOptions) (GTK_EXPAND | GTK_SHRINK | GTK_FILL), 0, 0);
+			  GtkAttachOptions(GTK_FILL),
+			  GtkAttachOptions(0), 0, 0);
 
 	// the replace label and field are only visible if we're a "replace" dialog
 	if (m_id == AP_DIALOG_ID_REPLACE)
@@ -344,9 +343,7 @@ GtkWidget * AP_UnixDialog_Replace::_constructWindow(void)
 		gtk_object_set_data (GTK_OBJECT (windowReplace), "labelReplace", labelReplace);
 		gtk_widget_show (labelReplace);
 		gtk_table_attach (GTK_TABLE (tableReplace), labelReplace, 0, 1, 2, 3,
-						  (GtkAttachOptions) (GTK_EXPAND | GTK_SHRINK | GTK_FILL),
-						  (GtkAttachOptions) (GTK_EXPAND | GTK_SHRINK | GTK_FILL), 0, 0);
-		gtk_label_set_justify (GTK_LABEL (labelReplace), GTK_JUSTIFY_LEFT);
+				  GtkAttachOptions(0), GtkAttachOptions(0), 5, 0);
 		gtk_misc_set_alignment (GTK_MISC (labelReplace), 0, 0.5);
 
 		// create replace entry
@@ -354,11 +351,16 @@ GtkWidget * AP_UnixDialog_Replace::_constructWindow(void)
 		gtk_object_set_data (GTK_OBJECT (windowReplace), "entryReplace", entryReplace);
 		gtk_widget_show (entryReplace);
 		gtk_table_attach (GTK_TABLE (tableReplace), entryReplace, 1, 2, 2, 3,
-						  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
-						  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
+				  GtkAttachOptions(GTK_EXPAND | GTK_FILL),
+				  GtkAttachOptions(GTK_EXPAND | GTK_FILL), 0, 0);
 
 	}
 	
+	// Horizontal separator above button box
+	separator = GTK_WIDGET (gtk_hseparator_new());
+	gtk_box_pack_start (GTK_BOX (vboxReplace), separator, FALSE, FALSE, 0);
+	gtk_widget_show (separator);
+
 	// button box at the bottom
 	hbuttonbox1 = gtk_hbutton_box_new ();
 	gtk_object_set_data (GTK_OBJECT (windowReplace), "hbuttonbox1", hbuttonbox1);
@@ -368,13 +370,6 @@ GtkWidget * AP_UnixDialog_Replace::_constructWindow(void)
 	gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbuttonbox1), 10);
 	gtk_button_box_set_child_size (GTK_BUTTON_BOX (hbuttonbox1), 85, 24);
 
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_FR_FindNextButton));	
-	buttonFindNext = gtk_button_new_with_label (unixstr);
-	FREEP(unixstr);
-	gtk_object_set_data (GTK_OBJECT (windowReplace), "buttonFindNext", buttonFindNext);
-	gtk_widget_show (buttonFindNext);
-	gtk_container_add (GTK_CONTAINER (hbuttonbox1), buttonFindNext);
-
 	if (m_id == AP_DIALOG_ID_REPLACE)
 	{
 		UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_FR_ReplaceButton));	
@@ -392,17 +387,18 @@ GtkWidget * AP_UnixDialog_Replace::_constructWindow(void)
 		gtk_container_add (GTK_CONTAINER (hbuttonbox1), buttonReplaceAll);
 	}
 
+	UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_FR_FindNextButton));	
+	buttonFindNext = gtk_button_new_with_label (unixstr);
+	FREEP(unixstr);
+	gtk_object_set_data (GTK_OBJECT (windowReplace), "buttonFindNext", buttonFindNext);
+	gtk_widget_show (buttonFindNext);
+	gtk_container_add (GTK_CONTAINER (hbuttonbox1), buttonFindNext);
+
 	buttonCancel = gtk_button_new_with_label (pSS->getValue(XAP_STRING_ID_DLG_Cancel));
 	gtk_object_set_data (GTK_OBJECT (windowReplace), "buttonCancel", buttonCancel);
 	gtk_widget_show (buttonCancel);
 	gtk_container_add (GTK_CONTAINER (hbuttonbox1), buttonCancel);
 
-	// Configuration of the title depending on the ID
-	if (m_id == AP_DIALOG_ID_FIND)
-		gtk_window_set_title(GTK_WINDOW(windowReplace), pSS->getValue(AP_STRING_ID_DLG_FR_FindTitle));
-	else
-		gtk_window_set_title(GTK_WINDOW(windowReplace), pSS->getValue(AP_STRING_ID_DLG_FR_ReplaceTitle));
-
 	// attach generic signals
 	gtk_signal_connect(GTK_OBJECT(checkbuttonMatchCase),
 					   "toggled",
