@@ -69,7 +69,7 @@ void AP_UnixDialog_Spell::runModal(XAP_Frame * pFrame)
       GtkWidget * mainWindow = _constructWindow();
       UT_ASSERT(mainWindow);
       
-	  connectFocus(GTK_WIDGET(mainWindow),pFrame);
+      connectFocus(GTK_WIDGET(mainWindow),pFrame);
       // Populate the window's data items
       _populateWindowData();
       
@@ -80,15 +80,14 @@ void AP_UnixDialog_Spell::runModal(XAP_Frame * pFrame)
       // Get the GtkWindow of the parent frame
       GtkWidget * parentWindow = pUnixFrame->getTopLevelWindow();
       UT_ASSERT(parentWindow);
-      
+
       // Center our new dialog in its parent and make it a transient
-      // soo it won't get lost underneath
+      // so it won't get lost underneath
       centerDialog(parentWindow, mainWindow);
-      gtk_window_set_transient_for(GTK_WINDOW(mainWindow), GTK_WINDOW(parentWindow));
       
       // Show the top level dialog
-      gtk_widget_show(mainWindow);
-      
+      gtk_widget_show_all(mainWindow);
+
       // Make it modal, and stick it up top
       gtk_grab_add(mainWindow);
       
@@ -185,267 +184,204 @@ static void s_delete_clicked(GtkWidget * /* widget */,
 GtkWidget * AP_UnixDialog_Spell::_constructWindow(void)
 {
    GtkWidget *windowSpell;
+   GtkWidget *vbox;
+
+   const XAP_StringSet * pSS = m_pApp->getStringSet();
+   
+   //windowSpell = gtk_window_new (GTK_WINDOW_DIALOG);
+   windowSpell = gtk_dialog_new();
+   gtk_window_set_title (GTK_WINDOW (windowSpell),  pSS->getValue(AP_STRING_ID_DLG_Spell_SpellTitle));
+   gtk_window_set_policy (GTK_WINDOW (windowSpell), TRUE, TRUE, FALSE);
+
+   // *very* important to add the vbox
+   // to the window so that it gets a
+   // gdkwindow associated with it
+   //
+   m_windowMain = windowSpell;
+   vbox = GTK_DIALOG(m_windowMain)->vbox;
+
+   _constructWindowContents(vbox);
+   _connectSignals();
+
+   gtk_widget_show_all(windowSpell);
+   return windowSpell;
+}
+
+void AP_UnixDialog_Spell::_constructWindowContents(GtkWidget *box)
+{
    GtkWidget *tableMain;
    
    GtkWidget *label1;
    GtkWidget *scroll2;
-   GtkWidget *textWord;
    GtkWidget *label2;
-   GtkWidget *entryChange;
    GtkWidget *scroll1;
-   GtkWidget *clistSuggestions;
-   GtkWidget *buttonChange;
-   GtkWidget *buttonChangeAll;
-   GtkWidget *buttonIgnore;
-   GtkWidget *buttonIgnoreAll;
-   GtkWidget *buttonAddToDict;
-   GtkWidget *buttonCancel;
 
    const XAP_StringSet * pSS = m_pApp->getStringSet();
    XML_Char * unixstr = NULL;      // used for conversions
-   
-   windowSpell = gtk_window_new (GTK_WINDOW_DIALOG);
-   gtk_object_set_data (GTK_OBJECT (windowSpell), "windowSpell", windowSpell);
-   gtk_window_set_title (GTK_WINDOW (windowSpell),  pSS->getValue(AP_STRING_ID_DLG_Spell_SpellTitle));
-   gtk_window_set_policy (GTK_WINDOW (windowSpell), TRUE, TRUE, FALSE);
+
+   // create the buttons right away
+   _createButtons();
 
    tableMain = gtk_table_new (10, 3, FALSE);
-   gtk_widget_ref (tableMain);
-   gtk_object_set_data_full (GTK_OBJECT (windowSpell), "tableMain", tableMain,
-			     (GtkDestroyNotify) gtk_widget_unref);
-   gtk_widget_show (tableMain);
-   gtk_container_add (GTK_CONTAINER (windowSpell), tableMain);
+   gtk_container_add (GTK_CONTAINER (box), tableMain);
    gtk_container_set_border_width (GTK_CONTAINER(tableMain), 5);
    gtk_table_set_row_spacings (GTK_TABLE(tableMain), 2);
    gtk_table_set_row_spacing (GTK_TABLE(tableMain), 4, 7);
    gtk_table_set_row_spacing (GTK_TABLE(tableMain), 8, 7);
    gtk_table_set_col_spacings (GTK_TABLE(tableMain), 2);
-   
+
    GtkWidget * alignmentLabel = gtk_alignment_new (0, 1, 0, 0);
-   gtk_widget_ref (alignmentLabel);
-   gtk_object_set_data_full (GTK_OBJECT (windowSpell), "alignmentLabel", alignmentLabel,
-			     (GtkDestroyNotify) gtk_widget_unref);
-   gtk_widget_show (alignmentLabel);
    gtk_table_attach (GTK_TABLE(tableMain), alignmentLabel, 0, 2, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
 
    UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_Spell_UnknownWord));
    label1 = gtk_label_new (unixstr);
    FREEP(unixstr);
-   gtk_widget_ref (label1);
-   gtk_object_set_data_full (GTK_OBJECT (windowSpell), "label1", label1,
-			     (GtkDestroyNotify) gtk_widget_unref);
-   gtk_widget_show (label1);
    gtk_label_set_justify (GTK_LABEL(label1), GTK_JUSTIFY_LEFT);
    gtk_container_add (GTK_CONTAINER(alignmentLabel), label1);
-   
+
    scroll2 = gtk_scrolled_window_new (NULL, NULL);
-   gtk_widget_ref (scroll2);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll2),
 				   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-   gtk_object_set_data_full (GTK_OBJECT (windowSpell), "scroll2", scroll2,
-			     (GtkDestroyNotify) gtk_widget_unref);
-   gtk_widget_show (scroll2);
-   gtk_table_attach_defaults (GTK_TABLE(tableMain), scroll2, 0, 2, 1, 4);   
+   gtk_table_attach_defaults (GTK_TABLE(tableMain), scroll2, 0, 2, 1, 4);
 
-   textWord = gtk_text_new (NULL, NULL);
-   gtk_widget_ref (textWord);
-   gtk_object_set_data_full (GTK_OBJECT (windowSpell), "textWord", textWord,
-			     (GtkDestroyNotify) gtk_widget_unref);
-   gtk_widget_show (textWord);
-   gtk_container_add (GTK_CONTAINER (scroll2), textWord);
-   gtk_text_set_word_wrap(GTK_TEXT(textWord), TRUE);
-   gtk_widget_set_usize (textWord, 350, 80);
-
-   gtk_widget_realize (textWord);
+   m_textWord = gtk_text_new (NULL, NULL);
+   gtk_widget_ref (m_textWord);
+   gtk_container_add (GTK_CONTAINER (scroll2), m_textWord);
+   gtk_text_set_word_wrap(GTK_TEXT(m_textWord), TRUE);
+   gtk_widget_set_usize (m_textWord, 350, 80);
+   gtk_widget_realize (m_textWord);
 
    // ignore button set
    GtkWidget * vboxIgnoreButtons = gtk_vbox_new(FALSE, 5);
-   gtk_widget_ref (vboxIgnoreButtons);
-   gtk_object_set_data_full (GTK_OBJECT (windowSpell), "vboxIgnoreButtons", vboxIgnoreButtons,
-			     (GtkDestroyNotify) gtk_widget_unref);
-   gtk_widget_show (vboxIgnoreButtons);
    gtk_table_attach_defaults (GTK_TABLE(tableMain), vboxIgnoreButtons, 2, 3, 1, 4);
    
-   UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_Spell_Ignore));
-   buttonIgnore = gtk_button_new_with_label (unixstr);
-   FREEP(unixstr);
-   gtk_widget_ref (buttonIgnore);
-   gtk_object_set_data_full (GTK_OBJECT (windowSpell), "buttonIgnore", buttonIgnore,
-			     (GtkDestroyNotify) gtk_widget_unref);
-   gtk_widget_show (buttonIgnore);
-   gtk_box_pack_start (GTK_BOX(vboxIgnoreButtons), buttonIgnore, FALSE, FALSE, 5);
-//   gtk_table_attach (GTK_TABLE(tableMain), buttonIgnore, 2, 3, 1, 2, GTK_FILL, GTK_EXPAND, 2, 0);
-   
-   UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_Spell_IgnoreAll));
-   buttonIgnoreAll = gtk_button_new_with_label (unixstr);
-   FREEP(unixstr);
-   gtk_widget_ref (buttonIgnoreAll);
-   gtk_object_set_data_full (GTK_OBJECT (windowSpell), "buttonIgnoreAll", buttonIgnoreAll,
-			     (GtkDestroyNotify) gtk_widget_unref);
-   gtk_widget_show (buttonIgnoreAll);
-   gtk_box_pack_start (GTK_BOX(vboxIgnoreButtons), buttonIgnoreAll, FALSE, FALSE, 5);
-//   gtk_table_attach (GTK_TABLE(tableMain), buttonIgnoreAll, 2, 3, 2, 3, GTK_FILL, GTK_EXPAND, 2, 5);
-   
-   UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_Spell_AddToDict));
-   buttonAddToDict = gtk_button_new_with_label (unixstr);
-   FREEP(unixstr);
-   gtk_widget_ref (buttonAddToDict);
-   gtk_object_set_data_full (GTK_OBJECT (windowSpell), "buttonAddToDict", buttonAddToDict,
-			     (GtkDestroyNotify) gtk_widget_unref);
-   gtk_widget_show (buttonAddToDict);
-   gtk_box_pack_start (GTK_BOX(vboxIgnoreButtons), buttonAddToDict, FALSE, FALSE, 5);
-//   gtk_table_attach (GTK_TABLE(tableMain), buttonAddToDict, 2, 3, 3, 4, GTK_FILL, GTK_EXPAND, 2, 0);
-
+   gtk_box_pack_start (GTK_BOX(vboxIgnoreButtons), m_buttonIgnore, FALSE, FALSE, 5);
+   gtk_box_pack_start (GTK_BOX(vboxIgnoreButtons), m_buttonIgnoreAll, FALSE, FALSE, 5);
+   gtk_box_pack_start (GTK_BOX(vboxIgnoreButtons), m_buttonAddToDict, FALSE, FALSE, 5);
 
    // suggestion half
    GtkWidget * hboxChangeTo = gtk_hbox_new(FALSE, 5);
-   gtk_widget_ref (hboxChangeTo);
-   gtk_object_set_data_full (GTK_OBJECT (windowSpell), "hboxChangeTo", hboxChangeTo,
-			     (GtkDestroyNotify) gtk_widget_unref);
-   gtk_widget_show (hboxChangeTo);
    gtk_table_attach_defaults (GTK_TABLE(tableMain), hboxChangeTo, 0, 2, 5, 6);
 
    UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_Spell_ChangeTo));
    label2 = gtk_label_new (unixstr);
    FREEP(unixstr);
-   gtk_widget_ref (label2);
-   gtk_object_set_data_full (GTK_OBJECT (windowSpell), "label2", label2,
-			     (GtkDestroyNotify) gtk_widget_unref);
-   gtk_widget_show (label2);
    gtk_box_pack_start (GTK_BOX(hboxChangeTo), label2, FALSE, FALSE, 5);
-//   gtk_table_attach (GTK_TABLE(tableMain), label2, 0, 1, 5, 6, GTK_SHRINK, GTK_SHRINK, 0, 0);
    gtk_label_set_justify( GTK_LABEL(label2), GTK_JUSTIFY_LEFT);
-//   gtk_misc_set_padding (GTK_MISC (label2), 5, 0);
 
-   entryChange = gtk_entry_new ();
-   gtk_widget_ref (entryChange);
-   gtk_object_set_data_full (GTK_OBJECT (windowSpell), "entryChange", entryChange,
-                            (GtkDestroyNotify) gtk_widget_unref);
-   gtk_widget_show (entryChange);
-   gtk_box_pack_start (GTK_BOX(hboxChangeTo), entryChange, TRUE, TRUE, 0);
-//   gtk_table_attach_defaults (GTK_TABLE(tableMain), entryChange, 1, 2, 5, 6);
+   m_entryChange = gtk_entry_new ();
+   gtk_box_pack_start (GTK_BOX(hboxChangeTo), m_entryChange, TRUE, TRUE, 0);
 
    scroll1 = gtk_scrolled_window_new (NULL, NULL);
-   gtk_widget_ref (scroll1);
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll1),
-				   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-   gtk_object_set_data_full (GTK_OBJECT (windowSpell), "scroll1", scroll1,
-			     (GtkDestroyNotify) gtk_widget_unref);
-   gtk_widget_show (scroll1);
+                                   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
    gtk_table_attach_defaults (GTK_TABLE(tableMain), scroll1, 0, 2, 6, 9);
 
-   clistSuggestions = gtk_clist_new (1);
-   gtk_widget_ref (clistSuggestions);
-   gtk_object_set_data_full (GTK_OBJECT (windowSpell), "clistSuggestions", 
-			     clistSuggestions, (GtkDestroyNotify) gtk_widget_unref);
-   gtk_widget_show (clistSuggestions);
-   gtk_container_add (GTK_CONTAINER (scroll1), clistSuggestions);
-   gtk_widget_set_usize (clistSuggestions, -2, 100);
-   gtk_clist_set_column_width (GTK_CLIST (clistSuggestions), 0, 80);
-   gtk_clist_column_titles_hide (GTK_CLIST (clistSuggestions));
-
+   m_clistSuggestions = gtk_clist_new (1);
+   gtk_container_add (GTK_CONTAINER (scroll1), m_clistSuggestions);
+   gtk_widget_set_usize (m_clistSuggestions, -2, 100);
+   gtk_clist_set_column_width (GTK_CLIST (m_clistSuggestions), 0, 80);
+   gtk_clist_column_titles_hide (GTK_CLIST (m_clistSuggestions));
    
-   // change buttons
+   // change button set
    GtkWidget * vboxChangeButtons = gtk_vbox_new(FALSE, 5);
-   gtk_widget_ref (vboxChangeButtons);
-   gtk_object_set_data_full (GTK_OBJECT (windowSpell), "vboxChangeButtons", vboxChangeButtons,
-			     (GtkDestroyNotify) gtk_widget_unref);
-   gtk_widget_show (vboxChangeButtons);
    gtk_table_attach_defaults (GTK_TABLE(tableMain), vboxChangeButtons, 2, 3, 6, 9);
 
-   UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_Spell_Change));
-   buttonChange = gtk_button_new_with_label (unixstr);
-   FREEP(unixstr);
-   gtk_widget_ref (buttonChange);
-   gtk_object_set_data_full (GTK_OBJECT (windowSpell), "buttonChange", buttonChange,
-			     (GtkDestroyNotify) gtk_widget_unref);
-   gtk_widget_show (buttonChange);
-   gtk_box_pack_start (GTK_BOX(vboxChangeButtons), buttonChange, FALSE, FALSE, 5);
-//   gtk_table_attach (GTK_TABLE(tableMain), buttonChange, 2, 3, 6, 7, GTK_FILL, GTK_EXPAND, 2, 0);
-   
-   UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_Spell_ChangeAll));
-   buttonChangeAll = gtk_button_new_with_label (unixstr);
-   FREEP(unixstr);
-   gtk_widget_ref (buttonChangeAll);
-   gtk_object_set_data_full (GTK_OBJECT (windowSpell), "buttonChangeAll", buttonChangeAll,
-			     (GtkDestroyNotify) gtk_widget_unref);
-   gtk_widget_show (buttonChangeAll);
-   gtk_box_pack_start (GTK_BOX(vboxChangeButtons), buttonChangeAll, FALSE, FALSE, 5);
-//   gtk_table_attach (GTK_TABLE(tableMain), buttonChangeAll, 2, 3, 7, 8, GTK_FILL, GTK_EXPAND, 2, 5);
-   
-   
-   
-   buttonCancel = gtk_button_new_with_label (pSS->getValue(XAP_STRING_ID_DLG_Cancel));
-   gtk_widget_ref (buttonCancel);
-   gtk_object_set_data_full (GTK_OBJECT (windowSpell), "buttonCancel", buttonCancel,
-			     (GtkDestroyNotify) gtk_widget_unref);
-   gtk_widget_show (buttonCancel);
-   gtk_table_attach (GTK_TABLE(tableMain), buttonCancel, 2, 3, 9, 10, GTK_FILL, GTK_EXPAND, 2, 5);
-   
-   // connect signals to handlers
+   gtk_box_pack_start (GTK_BOX(vboxChangeButtons), m_buttonChange, FALSE, FALSE, 5);
+   gtk_box_pack_start (GTK_BOX(vboxChangeButtons), m_buttonChangeAll, FALSE, FALSE, 5);
+   gtk_box_pack_start (GTK_BOX(vboxChangeButtons), m_buttonCancel, FALSE, FALSE, 5);
+   // gtk_table_attach (GTK_TABLE(tableMain), buttonCancel, 2, 3, 9, 10, GTK_FILL, GTK_EXPAND, 2, 5);
 
-   // buttons
-   gtk_signal_connect(GTK_OBJECT(buttonChange), "clicked",
-		      GTK_SIGNAL_FUNC(s_change_clicked),
-		      (gpointer) this);
-   gtk_signal_connect(GTK_OBJECT(buttonChangeAll), "clicked",
-		      GTK_SIGNAL_FUNC(s_change_all_clicked),
-		      (gpointer) this);
-   gtk_signal_connect(GTK_OBJECT(buttonIgnore), "clicked",
-		      GTK_SIGNAL_FUNC(s_ignore_clicked),
-		      (gpointer) this);
-   gtk_signal_connect(GTK_OBJECT(buttonIgnoreAll), "clicked",
-		      GTK_SIGNAL_FUNC(s_ignore_all_clicked),
-		      (gpointer) this);
-   gtk_signal_connect(GTK_OBJECT(buttonAddToDict), "clicked",
-		      GTK_SIGNAL_FUNC(s_add_to_dict_clicked),
-		      (gpointer) this);
-   gtk_signal_connect(GTK_OBJECT(buttonCancel), "clicked",
-		      GTK_SIGNAL_FUNC(s_cancel_clicked),
-		      (gpointer) this);
-
-   // suggestion list
-   m_listHandlerID = gtk_signal_connect(GTK_OBJECT(clistSuggestions), "select-row",
-		      GTK_SIGNAL_FUNC(s_suggestion_selected),
-		      (gpointer) this);
-   
-   // replacement edited
-   m_replaceHandlerID = gtk_signal_connect(GTK_OBJECT(entryChange), "changed",
-		      GTK_SIGNAL_FUNC(s_replacement_changed),
-		      (gpointer) this);
-   
-   // the catch-alls
-   gtk_signal_connect_after(GTK_OBJECT(windowSpell),
-			    "delete_event",
-			    GTK_SIGNAL_FUNC(s_delete_clicked),
-			    (gpointer) this);
-         
-   gtk_signal_connect_after(GTK_OBJECT(windowSpell),
-			    "destroy",
-			    NULL,
-			    NULL);
-   
-   // update member variables with the important widgets 
-   // that we'll interact with later
-   
-   m_buttonChange = buttonChange;
-   m_buttonChangeAll = buttonChangeAll;
-   m_buttonIgnore = buttonIgnore;
-   m_buttonIgnoreAll = buttonIgnoreAll;
-   m_buttonAddToDict = buttonAddToDict;
-   m_buttonCancel = buttonCancel;
-   
-   m_textWord = textWord;
-   m_entryChange = entryChange;
-   m_clistSuggestions = clistSuggestions;
-   
+   // highlight our misspelled word in red
    GdkColormap * cm = gdk_colormap_get_system();
    m_highlight.red = 0xffff;
    m_highlight.green = 0x0000;
    m_highlight.blue = 0x0000;
    gdk_colormap_alloc_color(cm, &m_highlight, FALSE, TRUE);
    
-   return windowSpell;
+   gtk_widget_show_all(tableMain);
+}
+
+// create normal gtk buttons
+// override for gnome buttons
+void AP_UnixDialog_Spell::_createButtons(void)
+{
+  const XAP_StringSet * pSS = m_pApp->getStringSet();
+  XML_Char * unixstr = NULL;      // used for conversions
+
+  UT_XML_cloneNoAmpersands(unixstr,
+			   pSS->getValue(AP_STRING_ID_DLG_Spell_Change));
+  m_buttonChange = gtk_button_new_with_label (unixstr);
+  FREEP(unixstr);
+  
+  UT_XML_cloneNoAmpersands(unixstr, 
+			   pSS->getValue(AP_STRING_ID_DLG_Spell_ChangeAll));
+  m_buttonChangeAll = gtk_button_new_with_label (unixstr);
+  FREEP(unixstr);
+
+  UT_XML_cloneNoAmpersands(unixstr, 
+			   pSS->getValue(AP_STRING_ID_DLG_Spell_Ignore));
+  m_buttonIgnore = gtk_button_new_with_label (unixstr);
+  FREEP(unixstr);
+
+  UT_XML_cloneNoAmpersands(unixstr, 
+			   pSS->getValue(AP_STRING_ID_DLG_Spell_IgnoreAll));
+  m_buttonIgnoreAll = gtk_button_new_with_label (unixstr);
+  FREEP(unixstr);
+
+  UT_XML_cloneNoAmpersands(unixstr, 
+			   pSS->getValue(AP_STRING_ID_DLG_Spell_AddToDict));
+  m_buttonAddToDict = gtk_button_new_with_label (unixstr);
+  FREEP(unixstr);
+
+   // add the cancel button
+   m_buttonCancel = gtk_button_new_with_label (pSS->getValue(XAP_STRING_ID_DLG_Cancel));
+}
+
+void AP_UnixDialog_Spell::_connectSignals(void)
+{
+   // connect signals to handlers
+
+   // buttons
+   gtk_signal_connect(GTK_OBJECT(m_buttonChange), "clicked",
+		      GTK_SIGNAL_FUNC(s_change_clicked),
+		      (gpointer) this);
+   gtk_signal_connect(GTK_OBJECT(m_buttonChangeAll), "clicked",
+		      GTK_SIGNAL_FUNC(s_change_all_clicked),
+		      (gpointer) this);
+   gtk_signal_connect(GTK_OBJECT(m_buttonIgnore), "clicked",
+		      GTK_SIGNAL_FUNC(s_ignore_clicked),
+		      (gpointer) this);
+   gtk_signal_connect(GTK_OBJECT(m_buttonIgnoreAll), "clicked",
+		      GTK_SIGNAL_FUNC(s_ignore_all_clicked),
+		      (gpointer) this);
+   gtk_signal_connect(GTK_OBJECT(m_buttonAddToDict), "clicked",
+		      GTK_SIGNAL_FUNC(s_add_to_dict_clicked),
+		      (gpointer) this);
+   gtk_signal_connect(GTK_OBJECT(m_buttonCancel), "clicked",
+		      GTK_SIGNAL_FUNC(s_cancel_clicked),
+		      (gpointer) this);
+
+   // suggestion list
+   m_listHandlerID = gtk_signal_connect(GTK_OBJECT(m_clistSuggestions), "select-row",
+					GTK_SIGNAL_FUNC(s_suggestion_selected),
+					(gpointer) this);
+   
+   // replacement edited
+   m_replaceHandlerID = gtk_signal_connect(GTK_OBJECT(m_entryChange), "changed",
+					   GTK_SIGNAL_FUNC(s_replacement_changed),
+					   (gpointer) this);
+   
+   // the catch-alls
+   gtk_signal_connect_after(GTK_OBJECT(m_windowMain),
+			    "delete_event",
+			    GTK_SIGNAL_FUNC(s_delete_clicked),
+			    (gpointer) this);
+         
+   gtk_signal_connect_after(GTK_OBJECT(m_windowMain),
+			    "destroy",
+			    NULL,
+			    NULL);
 }
 
 void AP_UnixDialog_Spell::_showMisspelledWord(void)
@@ -464,10 +400,12 @@ void AP_UnixDialog_Spell::_showMisspelledWord(void)
    FREEP(p);
    gtk_text_insert(GTK_TEXT(m_textWord), NULL, NULL, NULL,
 		   preword, strlen(preword));
+
    // insert misspelled word (in highlight color)
    p = _getCurrentWord();
    gchar * word = (gchar*) _convertToMB(p);
    FREEP(p);
+
    gtk_text_insert(GTK_TEXT(m_textWord) , NULL, &m_highlight, NULL,
 		   word, strlen(word));
    
@@ -477,13 +415,11 @@ void AP_UnixDialog_Spell::_showMisspelledWord(void)
    FREEP(p);
    gtk_text_insert(GTK_TEXT(m_textWord), NULL, NULL, NULL,
 		   postword, strlen(postword));
-   
+
    // TODO: set scroll position so misspelled word is centered
 
-   gtk_text_thaw( GTK_TEXT(m_textWord) );
-   
-   gtk_clist_freeze( GTK_CLIST(m_clistSuggestions) );
-   
+   gtk_text_thaw( GTK_TEXT(m_textWord) );   
+   gtk_clist_freeze( GTK_CLIST(m_clistSuggestions) );   
    gtk_clist_clear(GTK_CLIST(m_clistSuggestions));
    
    gchar *suggest[2] = {NULL, NULL};
@@ -499,7 +435,7 @@ void AP_UnixDialog_Spell::_showMisspelledWord(void)
       UT_XML_cloneNoAmpersands(suggest[0], pSS->getValue(AP_STRING_ID_DLG_Spell_NoSuggestions));
       gtk_clist_append( GTK_CLIST(m_clistSuggestions), suggest);
       FREEP(suggest[0]);
-      gtk_clist_set_selectable( GTK_CLIST(m_clistSuggestions), 0, FALSE);
+      gtk_clist_set_selectable(GTK_CLIST(m_clistSuggestions), 0, FALSE);
 
       gtk_signal_handler_block(GTK_OBJECT(m_entryChange), m_replaceHandlerID);
       gtk_entry_set_text(GTK_ENTRY(m_entryChange), "");
@@ -510,11 +446,11 @@ void AP_UnixDialog_Spell::_showMisspelledWord(void)
    } else {
 
       // select first on the list; signal handler should update our entry box
-      gtk_clist_select_row( GTK_CLIST(m_clistSuggestions), 0, 0);
+      gtk_clist_select_row(GTK_CLIST(m_clistSuggestions), 0, 0);
 
    }
    
-   gtk_clist_thaw( GTK_CLIST(m_clistSuggestions) );
+   gtk_clist_thaw(GTK_CLIST(m_clistSuggestions) );
    
 }
 
@@ -532,24 +468,28 @@ void AP_UnixDialog_Spell::_storeWindowData(void)
 
 void AP_UnixDialog_Spell::event_Change()
 {
-   UT_UCSChar * replace = NULL;
-   if (m_iSelectedRow != -1)
-     {
-	replace = (UT_UCSChar*) m_Suggestions.word[m_iSelectedRow];
-	changeWordWith(replace);
-     }
-   else
-     {
-	replace = _convertFromMB((char*)gtk_entry_get_text(GTK_ENTRY(m_entryChange)));
-	if (!UT_UCS_strlen(replace)) {
-	   FREEP(replace);
-	   return;
+	UT_UCSChar * replace = NULL;
+	UT_DEBUGMSG(("m_iSelectedRow is %i\n", m_iSelectedRow));
+	if (m_iSelectedRow != -1)
+	{
+		replace = (UT_UCSChar*) m_Suggestions.word[m_iSelectedRow];
+		UT_DEBUGMSG(("Replacing with %s\n", (char*) replace));
+		//fprintf(stderr, "Replacing with %s\n", replace);
+		changeWordWith(replace);		
 	}
-	changeWordWith(replace);
-	FREEP(replace);
-     }
-
-   gtk_main_quit();
+	else
+	  {
+	        replace = _convertFromMB((char*)gtk_entry_get_text(GTK_ENTRY(m_entryChange)));
+		if (!UT_UCS_strlen(replace)) {
+		  UT_DEBUGMSG(("replace is 0 length\n"));
+		  FREEP(replace);
+		  return;
+		}
+		changeWordWith(replace);
+		FREEP(replace);
+	  }
+	
+	gtk_main_quit();
 }
 
 void AP_UnixDialog_Spell::event_ChangeAll()
@@ -650,45 +590,15 @@ void AP_UnixDialog_Spell::event_ReplacementChanged()
 // make a multibyte encoded version of a string
 char * AP_UnixDialog_Spell::_convertToMB(UT_UCSChar *wword)
 {
-   static char word[1024];
-   UT_UCS_strcpy_to_char(word,wword);
-   return word;
+	char *word = (char *) malloc (UT_UCS_strlen(wword)*2);
+	UT_UCS_strcpy_to_char(word,wword);
+	return word;
 }
 
 // make a wide string from a multibyte string
 UT_UCSChar * AP_UnixDialog_Spell::_convertFromMB(char *word)
 {
-   int wcindex = 0, mbindex = 0;
-   int wclength = 0;
-   int mblength = strlen(word);
-   while (mbindex < mblength) {
-      int len = mblen(word+mbindex, mblength-mbindex);
-      //      UT_ASSERT(len >= 0);
-      if(len >= 0)
-	{
-	  mblength += len;
-	}
-      mbindex++;
-   }
-   mbindex = 0;
-
-   wchar_t wch; // we only support two bytes, so this is a temp Unicode char holder
-   UT_UCSChar * wword = (UT_UCSChar*) calloc(wclength + 1, sizeof(UT_UCSChar));
-   if (wword == NULL) return NULL;
-     
-   while (wcindex < wclength) {
-      int len = mbtowc(&wch, word+mbindex, mblength-mbindex);
-      //      UT_ASSERT(len >= 0);
-      if(len >= 0)
-	{
-	  mblength += len;
-	}
-      wcindex++;
-   }
-   wword[wclength] = 0;
-   
-   UT_DEBUGMSG(("mb2wc: mb %i/%i - wc %i/%i\n", mbindex, mblength, wcindex, wclength));
-   UT_ASSERT(wclength >= wcindex);
-   
-   return (UT_UCSChar*)wword;
+	UT_UCSChar *wword = (UT_UCSChar *) malloc (strlen(word)*2);
+	UT_UCS_strcpy_char(wword,word);
+	return wword;
 }
