@@ -19,6 +19,7 @@
 
 #include "ut_types.h"
 #include "ut_string.h"
+#include "ut_units.h"
 #include "ut_assert.h"
 #include "ut_debugmsg.h"
 
@@ -27,6 +28,7 @@
 #include "ut_dialogHelper.h"
 
 #include "gr_UnixGraphics.h"
+#include "fl_BlockLayout.h"
 
 #include "xap_App.h"
 #include "xap_UnixApp.h"
@@ -58,8 +60,8 @@ AP_UnixDialog_Tab::AP_UnixDialog_Tab(XAP_DialogFactory * pDlgFactory,
                                                  XAP_Dialog_Id id)
     : AP_Dialog_Tab(pDlgFactory,id)
 {
-	m_current_alignment = align_LEFT;
-	m_current_leader	= leader_NONE;
+	m_current_alignment = FL_TAB_LEFT;
+	m_current_leader	= FL_LEADER_NONE;
 	m_bInSetCall		= UT_FALSE;
 
 }
@@ -95,7 +97,6 @@ void AP_UnixDialog_Tab::runModal(XAP_Frame * pFrame)
     // Center our new dialog in its parent and make it a transient
     // so it won't get lost underneath
     centerDialog(parentWindow, mainWindow);
-    gtk_window_set_transient_for(GTK_WINDOW(mainWindow), GTK_WINDOW(parentWindow));
 
     // Show the top level dialog,
     gtk_widget_show(mainWindow);
@@ -165,31 +166,88 @@ void AP_UnixDialog_Tab::event_WindowDelete(void)
                 GTK_SIGNAL_FUNC(s_menu_item_activate),		\
                 (gpointer) this);							\
         } while (0)
-GtkWidget* AP_UnixDialog_Tab::_constructWindow ()
+GtkWidget* AP_UnixDialog_Tab::_constructWindow (void )
 {
-    //////////////////////////////////////////////////////////////////////
-	// BEGIN: glade stuff (interface.c)
-
-	// for the internationalization	
-	const XAP_StringSet * pSS = m_pApp->getStringSet();
 	
-	//checkbuttonSpellHideErrors = gtk_check_button_new_with_label
-	//	(pSS->getValue( AP_STRING_ID_DLG_Tab_Label_SpellHideErrors ));
+	GtkWidget *windowTabs;
+	GtkAccelGroup *accel_group;
 
-    //////////////////////////////////////////////////////////////////////
+	accel_group = gtk_accel_group_new ();
+	const XAP_StringSet * pSS = m_pApp->getStringSet();
+
+	windowTabs = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+	//windowTabs = gtk_window_new (GTK_WINDOW_DIALOG);
+	gtk_object_set_data (GTK_OBJECT (windowTabs), "windowTabs", windowTabs);
+	gtk_window_set_title (GTK_WINDOW (windowTabs), pSS->getValue( AP_STRING_ID_DLG_Tab_TabTitle));
+
+	_constructWindowContents(windowTabs);
+	gtk_window_add_accel_group (GTK_WINDOW (windowTabs), accel_group);
+
+	return windowTabs;
+}
+
+void    AP_UnixDialog_Tab::_constructGnomeButtons( GtkWidget * windowTabs)
+{
+	GtkWidget *buttonOK;
+	GtkWidget *buttonCancel;
+	GtkWidget *buttonApply;
+	GtkWidget *hseparator5;
+
+	const XAP_StringSet * pSS = m_pApp->getStringSet();
+
+	//
+	// Gnome buttons
+	//
+	
+	buttonApply = gtk_button_new_with_label (pSS->getValue( AP_STRING_ID_DLG_Options_Btn_Apply));
+	gtk_widget_ref (buttonApply);
+	gtk_object_set_data_full (GTK_OBJECT (windowTabs), "buttonApply", buttonApply,
+							(GtkDestroyNotify) gtk_widget_unref);
+	gtk_widget_show (buttonApply);
+	gtk_container_add (GTK_CONTAINER (m_GnomeButtons), buttonApply);
+	GTK_WIDGET_SET_FLAGS (buttonApply, GTK_CAN_DEFAULT);
+
+	buttonOK = gtk_button_new_with_label (pSS->getValue( XAP_STRING_ID_DLG_OK));
+	gtk_widget_ref (buttonOK);
+	gtk_object_set_data_full (GTK_OBJECT (windowTabs), "buttonOK", buttonOK,
+							  (GtkDestroyNotify) gtk_widget_unref);
+	gtk_widget_show (buttonOK);
+	gtk_container_add (GTK_CONTAINER (m_GnomeButtons), buttonOK);
+	GTK_WIDGET_SET_FLAGS (buttonOK, GTK_CAN_DEFAULT);
+
+	buttonCancel = gtk_button_new_with_label (pSS->getValue( XAP_STRING_ID_DLG_Cancel));
+	gtk_widget_ref (buttonCancel);
+	gtk_object_set_data_full (GTK_OBJECT (windowTabs), "buttonCancel", buttonCancel,
+							  (GtkDestroyNotify) gtk_widget_unref);
+	gtk_widget_show (buttonCancel);
+	gtk_container_add (GTK_CONTAINER (m_GnomeButtons), buttonCancel);
+	GTK_WIDGET_SET_FLAGS (buttonCancel, GTK_CAN_DEFAULT);
+	m_buttonApply = buttonApply;
+	m_buttonOK = buttonOK;
+	m_buttonCancel = buttonCancel;
+
+	hseparator5 = gtk_hseparator_new ();
+	gtk_widget_ref (hseparator5);
+	gtk_object_set_data_full (GTK_OBJECT (windowTabs), "hseparator5", hseparator5,
+							  (GtkDestroyNotify) gtk_widget_unref);
+	gtk_widget_show (hseparator5);
+	gtk_table_attach (GTK_TABLE (m_wTable), hseparator5, 0, 1, 3, 4,
+					  (GtkAttachOptions) (GTK_FILL),
+					  (GtkAttachOptions) (GTK_FILL), 0, 0);
+
+}
+
+void    AP_UnixDialog_Tab::_constructWindowContents( GtkWidget * windowTabs )
+{
+   //////////////////////////////////////////////////////////////////////
 	// BEGIN: glade stuff
 
-	GtkWidget *windowTabs;
 	GtkWidget *table13;
 	GtkWidget *hbuttonbox4;
 	GtkWidget *buttonSet;
 	GtkWidget *buttonClear;
 	GtkWidget *buttonClearAll;
 	GtkWidget *hbuttonbox3;
-	GtkWidget *buttonOK;
-	GtkWidget *buttonCancel;
-	GtkWidget *buttonApply;
-	GtkWidget *hseparator5;
 	GtkWidget *hbox10;
 	GtkWidget *label8;
 	GtkWidget *hbox11;
@@ -223,14 +281,8 @@ GtkWidget* AP_UnixDialog_Tab::_constructWindow ()
 	GtkWidget *label9;
 	GtkObject *spinbuttonTabstop_adj;
 	GtkWidget *spinbuttonTabstop;
-	GtkAccelGroup *accel_group;
 
-	accel_group = gtk_accel_group_new ();
-
-	windowTabs = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-	gtk_object_set_data (GTK_OBJECT (windowTabs), "windowTabs", windowTabs);
-	//	(pSS->getValue( AP_STRING_ID_DLG_Tab_Label_SpellHideErrors ));
-	gtk_window_set_title (GTK_WINDOW (windowTabs), pSS->getValue( AP_STRING_ID_DLG_Tab_TabTitle));
+	const XAP_StringSet * pSS = m_pApp->getStringSet();
 
 	table13 = gtk_table_new (5, 1, FALSE);
 	gtk_widget_ref (table13);
@@ -276,9 +328,10 @@ GtkWidget* AP_UnixDialog_Tab::_constructWindow ()
 	GTK_WIDGET_SET_FLAGS (buttonClearAll, GTK_CAN_DEFAULT);
 
 	hbuttonbox3 = gtk_hbutton_box_new ();
+	m_GnomeButtons = hbuttonbox3;
 	gtk_widget_ref (hbuttonbox3);
 	gtk_object_set_data_full (GTK_OBJECT (windowTabs), "hbuttonbox3", hbuttonbox3,
-							  (GtkDestroyNotify) gtk_widget_unref);
+				  (GtkDestroyNotify) gtk_widget_unref);
 	gtk_widget_show (hbuttonbox3);
 	gtk_table_attach (GTK_TABLE (table13), hbuttonbox3, 0, 1, 4, 5,
 					  (GtkAttachOptions) (GTK_FILL),
@@ -287,38 +340,12 @@ GtkWidget* AP_UnixDialog_Tab::_constructWindow ()
 	gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbuttonbox3), 5);
 	gtk_button_box_set_child_ipadding (GTK_BUTTON_BOX (hbuttonbox3), 0, 0);
 
-	buttonApply = gtk_button_new_with_label (pSS->getValue( AP_STRING_ID_DLG_Options_Btn_Apply));
-	gtk_widget_ref (buttonApply);
-	gtk_object_set_data_full (GTK_OBJECT (windowTabs), "buttonApply", buttonApply,
-							(GtkDestroyNotify) gtk_widget_unref);
-	gtk_widget_show (buttonApply);
-	gtk_container_add (GTK_CONTAINER (hbuttonbox3), buttonApply);
-	GTK_WIDGET_SET_FLAGS (buttonApply, GTK_CAN_DEFAULT);
+	//
+	// Construct the buttons to be gnomified
+	//
+        m_wTable = table13;
+	_constructGnomeButtons( windowTabs);
 
-	buttonOK = gtk_button_new_with_label (pSS->getValue( XAP_STRING_ID_DLG_OK));
-	gtk_widget_ref (buttonOK);
-	gtk_object_set_data_full (GTK_OBJECT (windowTabs), "buttonOK", buttonOK,
-							  (GtkDestroyNotify) gtk_widget_unref);
-	gtk_widget_show (buttonOK);
-	gtk_container_add (GTK_CONTAINER (hbuttonbox3), buttonOK);
-	GTK_WIDGET_SET_FLAGS (buttonOK, GTK_CAN_DEFAULT);
-
-	buttonCancel = gtk_button_new_with_label (pSS->getValue( XAP_STRING_ID_DLG_Cancel));
-	gtk_widget_ref (buttonCancel);
-	gtk_object_set_data_full (GTK_OBJECT (windowTabs), "buttonCancel", buttonCancel,
-							  (GtkDestroyNotify) gtk_widget_unref);
-	gtk_widget_show (buttonCancel);
-	gtk_container_add (GTK_CONTAINER (hbuttonbox3), buttonCancel);
-	GTK_WIDGET_SET_FLAGS (buttonCancel, GTK_CAN_DEFAULT);
-
-	hseparator5 = gtk_hseparator_new ();
-	gtk_widget_ref (hseparator5);
-	gtk_object_set_data_full (GTK_OBJECT (windowTabs), "hseparator5", hseparator5,
-							  (GtkDestroyNotify) gtk_widget_unref);
-	gtk_widget_show (hseparator5);
-	gtk_table_attach (GTK_TABLE (table13), hseparator5, 0, 1, 3, 4,
-					  (GtkAttachOptions) (GTK_FILL),
-					  (GtkAttachOptions) (GTK_FILL), 0, 0);
 
 	hbox10 = gtk_hbox_new (FALSE, 0);
 	gtk_widget_ref (hbox10);
@@ -574,17 +601,18 @@ GtkWidget* AP_UnixDialog_Tab::_constructWindow ()
 	gtk_object_set_data_full (GTK_OBJECT (windowTabs), "label9", label9,
 							  (GtkDestroyNotify) gtk_widget_unref);
 	gtk_widget_show (label9);
-	gtk_box_pack_start (GTK_BOX (hbox12), label9, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox12), label9, FALSE, FALSE, 1);
 
-	spinbuttonTabstop_adj = gtk_adjustment_new (1, 0, 100, 1, 10, 10);
-	spinbuttonTabstop = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonTabstop_adj), 1, 0);
-	gtk_widget_ref (spinbuttonTabstop);
-	gtk_object_set_data_full (GTK_OBJECT (windowTabs), "spinbuttonTabstop", spinbuttonTabstop,
-							  (GtkDestroyNotify) gtk_widget_unref);
+	spinbuttonTabstop_adj = gtk_adjustment_new (1, -1000, 1000, 3, 3, 10);
+        spinbuttonTabstop = gtk_entry_new();
 	gtk_widget_show (spinbuttonTabstop);
 	gtk_box_pack_start (GTK_BOX (hbox12), spinbuttonTabstop, TRUE, TRUE, 0);
+	gtk_entry_set_editable( GTK_ENTRY( spinbuttonTabstop),FALSE);
+	GtkWidget * spinbuttonTabstop_dum = gtk_spin_button_new (GTK_ADJUSTMENT (spinbuttonTabstop_adj), 1, 0);
+	gtk_widget_show (spinbuttonTabstop_dum);
+	gtk_widget_set_usize(spinbuttonTabstop_dum,10,-2);
+	gtk_box_pack_start (GTK_BOX (hbox12), spinbuttonTabstop_dum, FALSE,FALSE, 0);
 
-	gtk_window_add_accel_group (GTK_WINDOW (windowTabs), accel_group);
 
     //////////////////////////////////////////////////////////////////////
 	// END: glade stuff
@@ -603,17 +631,17 @@ GtkWidget* AP_UnixDialog_Tab::_constructWindow ()
 
     //////////////////////////////////////////////////////////////////////
     // the control buttons
-    gtk_signal_connect(GTK_OBJECT(buttonOK),
+    gtk_signal_connect(GTK_OBJECT(m_buttonOK),
                        "clicked",
                        GTK_SIGNAL_FUNC(s_ok_clicked),
                        (gpointer) this);
     
-    gtk_signal_connect(GTK_OBJECT(buttonCancel),
+    gtk_signal_connect(GTK_OBJECT(m_buttonCancel),
                        "clicked",
                        GTK_SIGNAL_FUNC(s_cancel_clicked),
                        (gpointer) this);
 
-    gtk_signal_connect(GTK_OBJECT(buttonApply),
+    gtk_signal_connect(GTK_OBJECT(m_buttonApply),
                        "clicked",
                        GTK_SIGNAL_FUNC(s_apply_clicked),
                        (gpointer) this);
@@ -638,9 +666,17 @@ GtkWidget* AP_UnixDialog_Tab::_constructWindow ()
                        GTK_SIGNAL_FUNC(s_edit_change),
                        (gpointer) this);
 
+    gtk_signal_connect(spinbuttonTabstop_adj,
+                       "value_changed",
+                       GTK_SIGNAL_FUNC(s_spin_default_changed),
+                       (gpointer) this);
+
     // Update member variables with the important widgets that
     // might need to be queried or altered later.
 
+
+        m_iDefaultSpin =  (UT_sint32) GTK_ADJUSTMENT(spinbuttonTabstop_adj)->value;
+        m_oDefaultSpin_adj = spinbuttonTabstop_adj;
 	m_Widgets.setNthItem( id_EDIT_TAB,				entryTabEntry,		NULL);
 	m_Widgets.setNthItem( id_LIST_TAB,				listTabs,			NULL);
 	m_Widgets.setNthItem( id_SPIN_DEFAULT_TAB_STOP,	spinbuttonTabstop,	NULL);
@@ -660,9 +696,9 @@ GtkWidget* AP_UnixDialog_Tab::_constructWindow ()
 	m_Widgets.setNthItem( id_BUTTON_CLEAR,			buttonClear,				NULL);
 	m_Widgets.setNthItem( id_BUTTON_CLEAR_ALL,		buttonClearAll,				NULL);
 
-	m_Widgets.setNthItem( id_BUTTON_OK,				buttonOK,					NULL);
-	m_Widgets.setNthItem( id_BUTTON_CANCEL,			buttonCancel,				NULL);
-	m_Widgets.setNthItem( id_BUTTON_APPLY,			buttonApply,				NULL);
+	m_Widgets.setNthItem( id_BUTTON_OK,			m_buttonOK,					NULL);
+	m_Widgets.setNthItem( id_BUTTON_CANCEL,			m_buttonCancel,				NULL);
+	m_Widgets.setNthItem( id_BUTTON_APPLY,			m_buttonApply,				NULL);
 
 
 	// some lists of signals to set
@@ -676,7 +712,7 @@ GtkWidget* AP_UnixDialog_Tab::_constructWindow ()
 						   (gpointer) this);
 
 		// set the "userdata" to be the tALignment
-		gtk_object_set_user_data( GTK_OBJECT(w), (gpointer)((UT_uint32)id - (UT_uint32)id_ALIGN_LEFT));
+		gtk_object_set_user_data( GTK_OBJECT(w), (gpointer)((UT_uint32)id - (UT_uint32)id_ALIGN_LEFT + (UT_uint32)FL_TAB_LEFT));
 	}
 
 	for ( id = id_LEADER_NONE; id <= id_LEADER_UNDERLINE; id = (tControl)((UT_uint32)id + 1))
@@ -688,7 +724,7 @@ GtkWidget* AP_UnixDialog_Tab::_constructWindow ()
 						   (gpointer) this);
 
 		// set the "userdata" to be the tALignment
-		gtk_object_set_user_data( GTK_OBJECT(w), (gpointer)((UT_uint32)id - (UT_uint32)id_ALIGN_LEFT));
+		gtk_object_set_user_data( GTK_OBJECT(w), (gpointer)((UT_uint32)id - (UT_uint32)id_LEADER_NONE + (UT_uint32)FL_LEADER_NONE));
 	}
 
 	// create the accelerators from &'s
@@ -707,8 +743,6 @@ GtkWidget* AP_UnixDialog_Tab::_constructWindow ()
 
 		gtk_object_set_data( GTK_OBJECT(w), "tControl", (gpointer) i );
 	}
-
-    return windowTabs;
 }
 
 GtkWidget *AP_UnixDialog_Tab::_lookupWidget ( tControl id )
@@ -728,6 +762,20 @@ void AP_UnixDialog_Tab::_controlEnable( tControl id, UT_Bool value )
 	gtk_widget_set_sensitive( w, value );
 }
 
+
+void AP_UnixDialog_Tab::_spinChanged(void)
+{
+        UT_sint32 i =  (UT_sint32) GTK_ADJUSTMENT(m_oDefaultSpin_adj)->value;
+	UT_sint32 amt = i - m_iDefaultSpin;
+	if(amt < 0)
+	      amt = -1;
+	else if(amt > 0)
+	      amt = 1;
+	_doSpin(id_SPIN_DEFAULT_TAB_STOP, amt);
+	m_iDefaultSpin = i;
+}
+
+
 /*****************************************************************/
 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
@@ -760,6 +808,13 @@ void AP_UnixDialog_Tab::_controlEnable( tControl id, UT_Bool value )
 // WP level events
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
  
+/*static*/ void AP_UnixDialog_Tab::s_spin_default_changed(GtkWidget * widget, gpointer data )
+{ 
+	AP_UnixDialog_Tab * dlg = (AP_UnixDialog_Tab *)data;
+	dlg->_spinChanged();
+}
+
+
 /*static*/ void AP_UnixDialog_Tab::s_set_clicked(GtkWidget * widget, gpointer data )
 { 
 	AP_UnixDialog_Tab * dlg = (AP_UnixDialog_Tab *)data;
@@ -794,16 +849,10 @@ void AP_UnixDialog_Tab::_controlEnable( tControl id, UT_Bool value )
 	UT_ASSERT(dlg); 
 	UT_ASSERT(widget && GTK_IS_LIST_ITEM(widget));
 
-	tTabInfo *pTabInfo = (tTabInfo *) gtk_object_get_user_data(GTK_OBJECT(widget));
-	UT_ASSERT(pTabInfo);
-	
-	UT_DEBUGMSG(("AP_UnixDialog_Tab::s_list_select [%s]\n", pTabInfo->pszTab ));
-	
 	// get the -1, 0.. (n-1) index
 	dlg->m_iGtkListIndex = gtk_list_child_position(GTK_LIST(dlg->_lookupWidget(id_LIST_TAB)), widget);
 
-	// to make it easier, pass in the actual data and not the index
-	dlg->_event_TabSelected( pTabInfo );
+	dlg->_event_TabSelected( dlg->m_iGtkListIndex );
 }
 
 /*static*/ void AP_UnixDialog_Tab::s_list_deselect(GtkWidget * widget, gpointer data )
@@ -822,7 +871,7 @@ void AP_UnixDialog_Tab::_controlEnable( tControl id, UT_Bool value )
 	UT_ASSERT(widget && dlg); 
 	UT_DEBUGMSG(("AP_UnixDialog_Tab::s_edit_change\n"));
 
-	dlg->_event_somethingChanged();
+	dlg->_event_TabChange();
 }
 
 /*static*/ void AP_UnixDialog_Tab::s_alignment_change( GtkWidget *widget, gpointer data )
@@ -834,10 +883,10 @@ void AP_UnixDialog_Tab::_controlEnable( tControl id, UT_Bool value )
 	if ( dlg->m_bInSetCall || gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(widget)) == FALSE ) 
 		return;
 
-	dlg->m_current_alignment = (tAlignment)((UT_uint32)gtk_object_get_user_data(GTK_OBJECT(widget)));
+	dlg->m_current_alignment = (eTabType)((UT_uint32)gtk_object_get_user_data(GTK_OBJECT(widget)));
 
 	UT_DEBUGMSG(("AP_UnixDialog_Tab::s_alignment_change [%c]\n", AlignmentToChar(dlg->m_current_alignment)));
-	dlg->_event_somethingChanged();
+	dlg->_event_AlignmentChange();
 }
 
 /*static*/ void AP_UnixDialog_Tab::s_leader_change( GtkWidget *widget, gpointer data )
@@ -849,7 +898,7 @@ void AP_UnixDialog_Tab::_controlEnable( tControl id, UT_Bool value )
 	if ( dlg->m_bInSetCall || gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON(widget)) == FALSE ) 
 		return;
 	
-	dlg->m_current_leader = (tLeader)((UT_uint32)gtk_object_get_user_data(GTK_OBJECT(widget)));
+	dlg->m_current_leader = (eTabLeader)((UT_uint32)gtk_object_get_user_data(GTK_OBJECT(widget)));
 	
 	UT_DEBUGMSG(("AP_UnixDialog_Tab::s_leader_change\n"));
 	dlg->_event_somethingChanged();
@@ -868,26 +917,44 @@ void AP_UnixDialog_Tab::_controlEnable( tControl id, UT_Bool value )
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 
-AP_UnixDialog_Tab::tAlignment AP_UnixDialog_Tab::_gatherAlignment()
+eTabType AP_UnixDialog_Tab::_gatherAlignment()
 {
-	// for ( UT_uint32 i = (UT_uint32)id_ALIGN_LEFT; 
-	// 	  i <= (UT_uint32)id_ALIGN_BAR;
-	// 	  i++ )
-
 	return m_current_alignment;
 }
 
-void AP_UnixDialog_Tab::_setAlignment( AP_UnixDialog_Tab::tAlignment a )
+void AP_UnixDialog_Tab::_setAlignment( eTabType a )
 {
-	// NOTE - tControl id_ALIGN_LEFT .. id_ALIGN_BAR must be in the same order
-	// as the tAlignment enums.
+	tControl id = id_ALIGN_LEFT;
+	
+	
+	switch(a)
+		{
+		case FL_TAB_LEFT:
+			id = id_ALIGN_LEFT;
+			break;
 
-	// magic noted above
-	tControl id = (tControl)((UT_uint32)id_ALIGN_LEFT + (UT_uint32)a);	
-	UT_ASSERT( id >= id_ALIGN_LEFT && id <= id_ALIGN_BAR );
+		case FL_TAB_CENTER:
+			id = id_ALIGN_CENTER;
+			break;
 
+		case FL_TAB_RIGHT:
+			id = id_ALIGN_RIGHT;
+			break;
+
+		case FL_TAB_DECIMAL:
+			id = id_ALIGN_DECIMAL;
+			break;
+
+		case FL_TAB_BAR:
+			id = id_ALIGN_BAR;
+			break;
+
+			// FL_TAB_NONE, __FL_TAB_MAX
+		default:
+		  return;
+		}
 	// time to set the alignment radiobutton widget
-	GtkWidget *w = _lookupWidget( id );
+	GtkWidget *w = _lookupWidget( id); 
 	UT_ASSERT(w && GTK_IS_RADIO_BUTTON(w));
 
 	// tell the change routines to ignore this message
@@ -895,15 +962,17 @@ void AP_UnixDialog_Tab::_setAlignment( AP_UnixDialog_Tab::tAlignment a )
 	gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON(w), TRUE );
 	m_bInSetCall = UT_FALSE;
 
+	m_current_alignment = a;
+
 }
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 
-AP_UnixDialog_Tab::tLeader AP_UnixDialog_Tab::_gatherLeader()
+eTabLeader AP_UnixDialog_Tab::_gatherLeader()
 {
-	return leader_NONE;
+	return FL_LEADER_NONE;
 }
 
-void AP_UnixDialog_Tab::_setLeader( AP_UnixDialog_Tab::tLeader a )
+void AP_UnixDialog_Tab::_setLeader( eTabLeader a )
 {
 	// NOTE - tControl id_LEADER_NONE .. id_ALIGN_BAR must be in the same order
 	// as the tAlignment enums.
@@ -924,39 +993,33 @@ void AP_UnixDialog_Tab::_setLeader( AP_UnixDialog_Tab::tLeader a )
 }
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 
-UT_sint32 AP_UnixDialog_Tab::_gatherDefaultTabStop()
+const XML_Char* AP_UnixDialog_Tab::_gatherDefaultTabStop()
 {
-	return 5;
+        return gtk_entry_get_text( GTK_ENTRY( _lookupWidget( id_SPIN_DEFAULT_TAB_STOP ) ) );
 }
 
-void AP_UnixDialog_Tab::_setDefaultTabStop( UT_sint32 a )
+void AP_UnixDialog_Tab::_setDefaultTabStop( const XML_Char* defaultTabStop )
 {
-	UT_UNUSED(a);
+	GtkWidget *w = _lookupWidget( id_SPIN_DEFAULT_TAB_STOP );
+
+	// then set the text
+
+	gtk_entry_set_text( GTK_ENTRY(w), defaultTabStop );
 }
 // - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 
-const UT_Vector& AP_UnixDialog_Tab::_gatherTabList()
-{
-	return m_tabInfo;
-}
-
-void AP_UnixDialog_Tab::_setTabList( const UT_Vector &v )
+void AP_UnixDialog_Tab::_setTabList(UT_uint32 count)
 {
 	GList *gList = NULL;
 	GtkList *wList = GTK_LIST(_lookupWidget( id_LIST_TAB ));
 	UT_uint32 i;
-	tTabInfo *pTabInfo;
 
 	// clear all the items from the list
 	gtk_list_clear_items( wList, 0, -1 );
 
-	for ( i = 0; i < v.getItemCount(); i++ )
+	for ( i = 0; i < count; i++ )
 	{
-		pTabInfo = (tTabInfo *)v.getNthItem(i);
-
-		// this will do for the time being, but if we want 
-		GtkWidget *li = gtk_list_item_new_with_label( pTabInfo->pszTab );
-		gtk_object_set_user_data( GTK_OBJECT(li), (gpointer) pTabInfo );
+		GtkWidget *li = gtk_list_item_new_with_label( _getTabDimensionString(i));
 
 		// we want to DO stuff
 		gtk_signal_connect(GTK_OBJECT(li),
@@ -964,14 +1027,6 @@ void AP_UnixDialog_Tab::_setTabList( const UT_Vector &v )
 						   GTK_SIGNAL_FUNC(s_list_select),
 						   (gpointer) this);
 
-	/*
-		gtk_signal_connect(GTK_OBJECT(li),
-						   "deselect",
-						   GTK_SIGNAL_FUNC(s_list_deselect),
-						   (gpointer) this);
-	*/
-
-
 		// show this baby
 		gtk_widget_show(li);
 
@@ -998,7 +1053,8 @@ void AP_UnixDialog_Tab::_setSelectTab( UT_sint32 v )
 	}
 	else
 	{
-		UT_ASSERT(UT_NOT_IMPLEMENTED);
+		GtkList *wList = GTK_LIST(_lookupWidget( id_LIST_TAB ));
+		gtk_list_select_item( wList, m_iGtkListIndex);
 	}
 }
 
@@ -1032,3 +1088,7 @@ void AP_UnixDialog_Tab::_clearList()
 	// clear all the items from the list
 	gtk_list_clear_items( wList, 0, -1 );
 }
+
+
+
+
