@@ -74,7 +74,7 @@ static void s_ok_clicked(GtkWidget * widget, AP_UnixDialog_WordCount * dlg)
 }
 
 
-static void s_Update_clicked(GtkWidget * widget, AP_UnixDialog_WordCount * dlg)
+static void s_update_clicked(GtkWidget * widget, AP_UnixDialog_WordCount * dlg)
 {
 	UT_ASSERT(widget && dlg);
 	dlg->event_Update();
@@ -108,11 +108,14 @@ void  AP_UnixDialog_WordCount::activate(void)
 {
 	UT_ASSERT (m_windowMain);
         
+	ConstructWindowName();
+	gtk_window_set_title (GTK_WINDOW (m_windowMain), m_WindowName);
 	setCountFromActiveFrame ();
 	_updateWindowData ();
 	gdk_window_raise (m_windowMain->window);
 }
 
+
 void AP_UnixDialog_WordCount::runModeless(XAP_Frame * pFrame)
 {
 	// Build the window's widgets and arrange them
@@ -136,11 +139,6 @@ void AP_UnixDialog_WordCount::runModeless(XAP_Frame * pFrame)
 	// Get the GtkWindow of the parent frame
 	GtkWidget * parentWindow = pUnixFrame->getTopLevelWindow();
 	UT_ASSERT(parentWindow);
-	
-	// Center our new dialog in its parent and make it a transient
-	// so it won't get lost underneath
-	// centerDialog(parentWindow, mainWindow);
-	//	gtk_window_set_transient_for(GTK_WINDOW(mainWindow), GTK_WINDOW(parentWindow));
 
 	// Show the top level dialog,
 	gtk_widget_show(mainWindow);
@@ -179,7 +177,8 @@ void    AP_UnixDialog_WordCount::setUpdateCounter( void )
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (m_pAutocheck), FALSE);
 	}
 	set_sensitivity();
-}         
+}
+         
 void    AP_UnixDialog_WordCount::autoupdateWC(UT_Timer * pTimer)
 {
 	UT_ASSERT(pTimer);
@@ -267,6 +266,9 @@ void AP_UnixDialog_WordCount::event_WindowDelete(void)
 
 void AP_UnixDialog_WordCount::notifyActiveFrame(XAP_Frame *pFrame)
 {
+        UT_ASSERT(m_windowMain);
+	ConstructWindowName();
+	gtk_window_set_title (GTK_WINDOW (m_windowMain), m_WindowName);
 	event_Update();
 }
 
@@ -294,7 +296,8 @@ GtkWidget * AP_UnixDialog_WordCount::_constructWindow(void)
 	m_windowMain = gtk_window_new (GTK_WINDOW_DIALOG);
 
 	gtk_container_set_border_width(GTK_CONTAINER (m_windowMain), 20);
-	gtk_window_set_title (GTK_WINDOW (m_windowMain), pSS->getValue(AP_STRING_ID_DLG_WordCount_WordCountTitle));
+        ConstructWindowName();
+	gtk_window_set_title (GTK_WINDOW (m_windowMain), m_WindowName);
 	gtk_window_set_policy (GTK_WINDOW (m_windowMain), FALSE, FALSE, FALSE);
 
 	vbox = gtk_vbox_new (FALSE, 4);
@@ -541,7 +544,7 @@ void AP_UnixDialog_WordCount::_connectSignals(void)
 
 	gtk_signal_connect(GTK_OBJECT(m_buttonUpdate),
 					   "clicked",
-					   GTK_SIGNAL_FUNC(s_Update_clicked),
+					   GTK_SIGNAL_FUNC(s_update_clicked),
 					   (gpointer) this);
 
 	gtk_signal_connect (GTK_OBJECT (m_Spinrange), "value_changed",
@@ -554,7 +557,8 @@ void AP_UnixDialog_WordCount::_connectSignals(void)
 					   (gpointer) this);
 
 	// the catch-alls
-	gtk_signal_connect_after(GTK_OBJECT(m_windowMain),
+	// Don't use gtk_signal_connect_after for Modeless dialogs
+	gtk_signal_connect(GTK_OBJECT(m_windowMain),
 							 "delete_event",
 							 GTK_SIGNAL_FUNC(s_delete_clicked),
 							 (gpointer) this);
