@@ -96,7 +96,7 @@ UT_Error AP_UnixFrame::_showDocument(UT_uint32 iZoom)
 	gboolean bFocus;
 	XAP_UnixFontManager * fontManager = ((XAP_UnixApp *) getApp())->getFontManager();
 	
-	pG = new GR_UnixGraphics(m_dArea->window, fontManager);
+	pG = new GR_UnixGraphics(m_dArea->window, fontManager, getApp());
 	ENSUREP(pG);
 	pG->setZoomPercentage(iZoom);
 	
@@ -285,7 +285,7 @@ void AP_UnixFrame::setXScrollRange(void)
 		newvalue = newmax;
 
 	UT_Bool bDifferentPosition = (newvalue != (int)m_pHadj->value);
-	UT_Bool bDifferentLimits = ((width-windowWidth) != (m_pHadj->upper-m_pHadj->page_size));
+	UT_Bool bDifferentLimits = ((width-windowWidth) != (int)(m_pHadj->upper-m_pHadj->page_size));
 	
 	m_pHadj->value = newvalue;
 	m_pHadj->lower = 0.0;
@@ -296,7 +296,7 @@ void AP_UnixFrame::setXScrollRange(void)
 	gtk_signal_emit_by_name(GTK_OBJECT(m_pHadj), "changed");
 
 	if (m_pView && (bDifferentPosition || bDifferentLimits))
-		m_pView->sendHorizontalScrollEvent(newvalue, (long) m_pHadj->upper-m_pHadj->page_size);
+		m_pView->sendHorizontalScrollEvent(newvalue, (int)(m_pHadj->upper-m_pHadj->page_size));
 }
 
 void AP_UnixFrame::setYScrollRange(void)
@@ -312,7 +312,7 @@ void AP_UnixFrame::setYScrollRange(void)
 		newvalue = newmax;
 
 	UT_Bool bDifferentPosition = (newvalue != (int)m_pVadj->value);
-	UT_Bool bDifferentLimits ((height-windowHeight) != (m_pVadj->upper-m_pVadj->page_size));
+	UT_Bool bDifferentLimits ((height-windowHeight) != (int)(m_pVadj->upper-m_pVadj->page_size));
 	
 	m_pVadj->value = newvalue;
 	m_pVadj->lower = 0.0;
@@ -323,7 +323,7 @@ void AP_UnixFrame::setYScrollRange(void)
 	gtk_signal_emit_by_name(GTK_OBJECT(m_pVadj), "changed");
 
 	if (m_pView && (bDifferentPosition || bDifferentLimits))
-		m_pView->sendVerticalScrollEvent(newvalue, (long) m_pVadj->upper-m_pVadj->page_size);
+		m_pView->sendVerticalScrollEvent(newvalue, (int)(m_pVadj->upper-m_pVadj->page_size));
 }
 
 
@@ -348,6 +348,7 @@ AP_UnixFrame::~AP_UnixFrame(void)
 
 UT_Bool AP_UnixFrame::initialize(void)
 {
+	UT_DEBUGMSG(("AP_UnixFrame::initialize"));
 	if (!initFrameData())
 		return UT_FALSE;
 
@@ -359,11 +360,40 @@ UT_Bool AP_UnixFrame::initialize(void)
 		return UT_FALSE;
 
 	_createTopLevelWindow();
+	_showOrHideToolbars();
+	_showOrHideStatusbar();
 	gtk_widget_show(m_wTopLevelWindow);
 
 	return UT_TRUE;
 }
 
+// Does the initial show/hide of toolbars (based on the user prefs).
+// This is needed because toggleBar is called only when the user
+// (un)checks the show {Stantandard,Format,Extra} toolbar checkbox,
+// and thus we have to manually call this function at startup.
+void AP_UnixFrame::_showOrHideToolbars(void)
+{
+	UT_Bool *bShowBar = static_cast<AP_FrameData*> (m_pData)->m_bShowBar;
+
+	for (UT_uint32 i = 0; i < m_vecToolbarLayoutNames.getItemCount(); i++)
+	{
+		// TODO: The two next lines are here to bind the EV_Toolbar to the
+		// AP_FrameData, but their correct place are next to the toolbar creation (JCA)
+		EV_UnixToolbar * pUnixToolbar = static_cast<EV_UnixToolbar *> (m_vecToolbars.getNthItem(i));
+		static_cast<AP_FrameData*> (m_pData)->m_pToolbar[i] = pUnixToolbar;
+		toggleBar(i, bShowBar[i]);
+	}
+}
+
+// Does the initial show/hide of toolbars (based on the user prefs).
+// Idem.
+void AP_UnixFrame::_showOrHideStatusbar(void)
+{
+        //UT_Bool bShowStatusBar = static_cast<AP_FrameData*> (m_pData)->m_bShowStatusBar;
+	// I don't know why it doesn't work...
+	//	toggleStatusBar(bShowStatusBar);
+}
+
 /*****************************************************************/
 
 UT_Bool AP_UnixFrame::initFrameData(void)
@@ -532,9 +562,7 @@ void AP_UnixFrame::_scrollFuncX(void * pData, UT_sint32 xoff, UT_sint32 /*xrange
 
 GtkWidget * AP_UnixFrame::_createDocumentWindow(void)
 {
-	GtkWidget * wSunkenBox;
-
-	UT_Bool bShowRulers = ((AP_FrameData*)m_pData)->m_bShowRuler;
+	UT_Bool bShowRulers = static_cast<AP_FrameData*> (m_pData)->m_bShowRuler;
 
 	// create the rulers
 	AP_UnixTopRuler * pUnixTopRuler = NULL;
@@ -666,9 +694,9 @@ GtkWidget * AP_UnixFrame::_createDocumentWindow(void)
 
 	// create a 3d box and put the table in it, so that we
 	// get a sunken in look.
-	wSunkenBox = gtk_frame_new(NULL);
-	gtk_frame_set_shadow_type(GTK_FRAME(wSunkenBox), GTK_SHADOW_IN);
-	gtk_container_add(GTK_CONTAINER(wSunkenBox), m_table);
+	m_wSunkenBox = gtk_frame_new(NULL);
+	gtk_frame_set_shadow_type(GTK_FRAME(m_wSunkenBox), GTK_SHADOW_IN);
+	gtk_container_add(GTK_CONTAINER(m_wSunkenBox), m_table);
 
 	gtk_widget_show(m_hScroll);
 	gtk_widget_show(m_vScroll);
@@ -676,7 +704,7 @@ GtkWidget * AP_UnixFrame::_createDocumentWindow(void)
 	gtk_widget_show(m_innertable);
 	gtk_widget_show(m_table);
 
-	return wSunkenBox;
+	return m_wSunkenBox;
 }
 
 void AP_UnixFrame::translateDocumentToScreen(UT_sint32 &x, UT_sint32 &y)
@@ -796,3 +824,29 @@ void AP_UnixFrame::toggleRuler(UT_Bool bRulerOn)
 	((AP_FrameData*)m_pData)->m_pTopRuler = pUnixTopRuler;
 	((AP_FrameData*)m_pData)->m_pLeftRuler = pUnixLeftRuler;
 }
+
+void AP_UnixFrame::toggleBar(UT_uint32 iBarNb, UT_Bool bBarOn)
+{
+	UT_DEBUGMSG(("AP_UnixFrame::toggleBar %d, %d\n", iBarNb, bBarOn));	
+
+	AP_FrameData *pFrameData = static_cast<AP_FrameData *> (getFrameData());
+	UT_ASSERT(pFrameData);
+	
+	if (bBarOn)
+		pFrameData->m_pToolbar[iBarNb]->show();
+	else	// turning toolbar off
+		pFrameData->m_pToolbar[iBarNb]->hide();
+}
+
+void AP_UnixFrame::toggleStatusBar(UT_Bool bStatusBarOn)
+{
+	UT_DEBUGMSG(("AP_UnixFrame::toggleStatusBar %d\n", bStatusBarOn));	
+
+	AP_FrameData *pFrameData = static_cast<AP_FrameData *> (getFrameData());
+	UT_ASSERT(pFrameData);
+	
+	if (bStatusBarOn)
+		pFrameData->m_pStatusBar->show();
+	else	// turning status bar off
+		pFrameData->m_pStatusBar->hide();
+}
