@@ -62,11 +62,18 @@ AP_UnixGnomeDialog_WordCount::~AP_UnixGnomeDialog_WordCount(void)
 GtkWidget * AP_UnixGnomeDialog_WordCount::_constructWindow(void)
 {
 	const XAP_StringSet * pSS = m_pApp->getStringSet();
-	XML_Char * unixstr;	// used for conversions
 
-	UT_XML_cloneNoAmpersands(unixstr, pSS->getValue(AP_STRING_ID_DLG_WordCount_WordCountTitle));
-	m_windowMain = gnome_dialog_new (unixstr, pSS->getValue(XAP_STRING_ID_DLG_Update),
-									 GNOME_STOCK_BUTTON_CLOSE, NULL);
+	ConstructWindowName();
+	m_windowMain = gnome_dialog_new (m_WindowName, NULL);
+
+	// update button
+	gnome_dialog_append_button_with_pixmap(GNOME_DIALOG(m_windowMain), 
+					       pSS->getValue(XAP_STRING_ID_DLG_Update),
+					       GNOME_STOCK_PIXMAP_REFRESH);
+
+	// close button
+	gnome_dialog_append_button(GNOME_DIALOG(m_windowMain),
+				   GNOME_STOCK_BUTTON_CLOSE);
 
 	_constructWindowContents();
 	gtk_container_add (GTK_CONTAINER (GNOME_DIALOG (m_windowMain)->vbox), m_wContent);
