@@ -55,6 +55,8 @@
 #include "ie_imp_RTF.h"
 #include "ap_EditMethods.h"
 
+#include "fp_Run.h"
+
 /*****************************************************************/
 
 AP_Win32App::AP_Win32App(HINSTANCE hInstance, XAP_Args * pArgs, const char * szAppName)
@@ -210,6 +212,22 @@ UT_Bool AP_Win32App::initialize(void)
 			free(szPathname);
 		}
 	}
+	// Now we have the strings loaded we can populate the field names correctly
+	// CHECK THIS: the following was added by a Linux developer who can't test
+	// on Win32, or even compile, so someone with a Windows box needs to check it
+	int i;
+	
+	for (i = 0; fp_FieldTypes[i].m_Type != FPFIELDTYPE_END; i++)
+	{
+	    (&fp_FieldTypes[i])->m_Desc = m_pStringSet->getValue(fp_FieldTypes[i].m_DescId);
+	    UT_DEBUGMSG(("Setting field type desc for type %d, desc=%s\n", fp_FieldTypes[i].m_Type, fp_FieldTypes[i].m_Desc));
+	}
+
+	for (i = 0; fp_FieldFmts[i].m_Tag != NULL; i++)
+	{
+	    (&fp_FieldFmts[i])->m_Desc = m_pStringSet->getValue(fp_FieldFmts[i].m_DescId);
+	    UT_DEBUGMSG(("Setting field desc for field %s, desc=%s\n", fp_FieldFmts[i].m_Tag, fp_FieldFmts[i].m_Desc));
+	}
 
 	//////////////////////////////////////////////////////////////////
 
@@ -507,7 +525,8 @@ static LRESULT CALLBACK _SplashWndProc(HWND hWnd, UINT message, WPARAM wParam, L
     case WM_PAINT:
         hdc = BeginPaint(hWnd, &ps);
 		{
-			GR_Graphics * pG = new GR_Win32Graphics(hdc, hwndSplash);
+			// TODO: find XAP_App pointer for this
+			GR_Graphics * pG = new GR_Win32Graphics(hdc, hwndSplash, 0);
 			pG->drawImage(pSplash, 0, 0);
 			DELETEP(pG);
 		}
@@ -641,7 +660,8 @@ static GR_Image * _showSplash(HINSTANCE hInstance, XAP_Args * pArgs, const char
 		if (hwndSplash) 
 		{
 			// create image first
-			GR_Graphics * pG = new GR_Win32Graphics(GetDC(hwndSplash), hwndSplash);
+			// TODO: find XAP_App pointer for this
+			GR_Graphics * pG = new GR_Win32Graphics(GetDC(hwndSplash), hwndSplash, 0);
 			pSplash = pG->createNewImage("splash", pBB, iSplashWidth, iSplashHeight);
 			DELETEP(pG);
 
@@ -672,7 +692,7 @@ int AP_Win32App::WinMain(const char * szAppName, HINSTANCE hInstance,
 
 	_CrtSetReportMode( _CRT_WARN, _CRTDBG_MODE_DEBUG );
 	_CrtSetReportMode( _CRT_ERROR, _CRTDBG_MODE_DEBUG );
-	_CrtSetReportMode( _CRT_ASSERT, _CRTDBG_MODE_DEBUG );
+	_CrtSetReportMode( _CRT_ASSERT, _CRTDBG_MODE_DEBUG | _CRTDBG_MODE_WNDW);
 
 	// Ensure that common control DLL is loaded
 	HINSTANCE hinstCC = LoadLibrary("comctl32.dll");
@@ -755,7 +775,10 @@ int AP_Win32App::WinMain(const char * szAppName, HINSTANCE hInstance,
 			// Note: we do not call TranslateMessage() because
 			// Note: the keybinding mechanism is responsible
 			// Note: for deciding if/when to do this.
-				
+
+			if( pMyWin32App->handleModelessDialogMessage( &msg ) )
+				continue;
+
 			DispatchMessage(&msg);
 		}
 	}
@@ -910,3 +933,28 @@ UT_Error AP_Win32App::fileOpen(XAP_Frame * pFrame, const char * pNewFile)
 {
 	return ::fileOpen(pFrame, pNewFile, IEFT_Unknown);
 }
+
+UT_Bool AP_Win32App::handleModelessDialogMessage( MSG * msg )
+{
+	int iCounter;
+	HWND hWnd = NULL;
+
+	// Try to knock off the easy case quickly
+	if( m_IdTable[ 0 ].id == -1 )
+		return UT_FALSE;
+
+    for( iCounter = 0; iCounter <= NUM_MODELESSID; iCounter++ )
+	{
+		if( m_IdTable[ iCounter ].id != -1 )
+		{
+			hWnd = (HWND)m_IdTable[ iCounter ].pDialog->pGetWindowHandle();
+
+			if( hWnd && IsDialogMessage( hWnd, msg ) )
+				return UT_TRUE;
+		}
+		else
+			break;
+	}
+
+	return UT_FALSE;
+}
