@@ -48,6 +48,9 @@
 #	include "ap_Win32Dialog_WordCount.h"
 #	include "ap_Win32Dialog_Field.h"
 #	include "ap_Win32Dialog_Goto.h"
+#	include "ap_Win32Dialog_Columns.h"
+#	include "ap_Win32Dialog_Lists.h"
+#	include "ap_Win32Dialog_Tab.h"
 
 	// ... add new dialogs here ...
 
@@ -63,6 +66,7 @@
 	DeclareDialog(XAP_DIALOG_ID_ZOOM,			XAP_Win32Dialog_Zoom)
 	DeclareDialog(XAP_DIALOG_ID_ABOUT,			XAP_Win32Dialog_About)
 	DeclareDialog(XAP_DIALOG_ID_INSERT_SYMBOL,	XAP_Win32Dialog_Insert_Symbol)
+	DeclareDialog(XAP_DIALOG_ID_INSERT_PICTURE, XAP_Win32Dialog_FileOpenSaveAs)
 
 	DeclareDialog(AP_DIALOG_ID_REPLACE,			AP_Win32Dialog_Replace)
 	DeclareDialog(AP_DIALOG_ID_FIND,			AP_Win32Dialog_Replace)
@@ -74,6 +78,9 @@
 	DeclareDialog(AP_DIALOG_ID_WORDCOUNT,		AP_Win32Dialog_WordCount)
 	DeclareDialog(AP_DIALOG_ID_FIELD,			AP_Win32Dialog_Field)
 	DeclareDialog(AP_DIALOG_ID_GOTO,			AP_Win32Dialog_Goto)
+	DeclareDialog(AP_DIALOG_ID_COLUMNS,			AP_Win32Dialog_Columns)
+	DeclareDialog(AP_DIALOG_ID_LISTS,			AP_Win32Dialog_Lists)
+	DeclareDialog(AP_DIALOG_ID_TAB,				AP_Win32Dialog_Tab)
 	
 	// ... also add new dialogs here ...
 
