@@ -216,6 +216,7 @@ struct {
 #define SPELL_INDEX		0
 #define PREF_INDEX		1
 #define VIEW_INDEX		2
+#define OTHER_INDEX 3
 
 BOOL AP_Win32Dialog_Options::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lParam)
 {
@@ -265,6 +266,10 @@ BOOL AP_Win32Dialog_Options::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lPar
 		tie.lParam = AP_RID_DIALOG_OPT_VIEW;
 		TabCtrl_InsertItem(m_hwndTab, VIEW_INDEX, &tie); 
 
+		tie.pszText = (LPSTR) _GV(DLG_Options_TabLabel_Other); 
+		tie.lParam = AP_RID_DIALOG_OPT_OTHER;
+		TabCtrl_InsertItem(m_hwndTab, OTHER_INDEX, &tie); 
+
 		// finally, create the (modeless) child dialogs
 		
 		tp.which = AP_RID_DIALOG_OPT_SPL;
@@ -290,6 +295,14 @@ BOOL AP_Win32Dialog_Options::_onInitDialog(HWND hWnd, WPARAM wParam, LPARAM lPar
 		UT_ASSERT((w
 				   && (m_vecSubDlgHWnd.getItemCount()>0)
 				   && (w == m_vecSubDlgHWnd.getLastItem())));
+		
+		tp.which = AP_RID_DIALOG_OPT_OTHER;
+		pTemplate = UT_LockDlgRes(hinst, MAKEINTRESOURCE(tp.which));
+		w = CreateDialogIndirectParam(hinst, pTemplate, m_hwndTab, 
+										(DLGPROC)s_tabProc, (LPARAM)&tp); 
+		UT_ASSERT((w
+				   && (m_vecSubDlgHWnd.getItemCount()>0)
+				   && (w == m_vecSubDlgHWnd.getLastItem())));
 	}
 
 	// let XP code tell us what all of the values should be.
@@ -377,7 +390,10 @@ BOOL AP_Win32Dialog_Options::_onInitTab(HWND hWnd, WPARAM wParam, LPARAM lParam)
 			_DS(OPTIONS_CHK_ViewShowRuler,			DLG_Options_Label_ViewRuler);
             _DS(OPTIONS_CHK_ViewUnprintable,        DLG_Options_Label_ViewUnprintable);
 			_DS(OPTIONS_CHK_ViewCursorBlink,		DLG_Options_Label_ViewCursorBlink);
-			_DS(OPTIONS_CHK_ViewShowToolbars,		DLG_Options_Label_ViewToolbars);
+			_DS(OPTIONS_CHK_ViewShowStandardBar,	DLG_Options_Label_ViewStandardTB);
+			_DS(OPTIONS_CHK_ViewShowFormatBar,		DLG_Options_Label_ViewFormatTB);
+			_DS(OPTIONS_CHK_ViewShowExtraBar,		DLG_Options_Label_ViewExtraTB);
+			_DS(OPTIONS_CHK_ViewShowStatusBar,		DLG_Options_Label_ViewStatusBar);
 			_DS(OPTIONS_CHK_ViewAll,				DLG_Options_Label_ViewAll);
 			_DS(OPTIONS_CHK_ViewHiddenText,			DLG_Options_Label_ViewHiddenText);
 			_DS(OPTIONS_CHK_ViewUnprintable,		DLG_Options_Label_ViewUnprintable);
@@ -405,6 +421,12 @@ BOOL AP_Win32Dialog_Options::_onInitTab(HWND hWnd, WPARAM wParam, LPARAM lParam)
 		}
 		break;
 	
+	case AP_RID_DIALOG_OPT_OTHER:
+		{
+			_DS(OPTIONS_CHK_SmartQuotesEnable,		DLG_Options_Label_SmartQuotesEnable);
+		}
+		break;
+
 	default:
 		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 		break;
@@ -452,12 +474,17 @@ BOOL AP_Win32Dialog_Options::_onCommandTab(HWND hWnd, WPARAM wParam, LPARAM lPar
 
 	case AP_RID_DIALOG_OPTIONS_CHK_PrefsAutoSave:		_enableDisableLogic(id_CHECK_PREFS_AUTO_SAVE);		return 0;
 
+	case AP_RID_DIALOG_OPTIONS_CHK_SmartQuotesEnable:	_enableDisableLogic(id_CHECK_SMART_QUOTES_ENABLE);	return 0;
+
 	case AP_RID_DIALOG_OPTIONS_COMBO_CURRENTSCHEME:
 		return 0;
 
 	case AP_RID_DIALOG_OPTIONS_CHK_ViewShowRuler:		_enableDisableLogic(id_CHECK_VIEW_SHOW_RULER);		return 0;
 	case AP_RID_DIALOG_OPTIONS_CHK_ViewCursorBlink:		_enableDisableLogic(id_CHECK_VIEW_CURSOR_BLINK);	return 0;
-	case AP_RID_DIALOG_OPTIONS_CHK_ViewShowToolbars:	_enableDisableLogic(id_CHECK_VIEW_SHOW_TOOLBARS);	return 0;
+	case AP_RID_DIALOG_OPTIONS_CHK_ViewShowStandardBar:	_enableDisableLogic(id_CHECK_VIEW_SHOW_STANDARD_TOOLBAR);	return 0;
+	case AP_RID_DIALOG_OPTIONS_CHK_ViewShowFormatBar:	_enableDisableLogic(id_CHECK_VIEW_SHOW_FORMAT_TOOLBAR);		return 0;
+	case AP_RID_DIALOG_OPTIONS_CHK_ViewShowExtraBar:	_enableDisableLogic(id_CHECK_VIEW_SHOW_EXTRA_TOOLBAR);		return 0;
+	case AP_RID_DIALOG_OPTIONS_CHK_ViewShowStatusBar:	_enableDisableLogic(id_CHECK_VIEW_SHOW_STATUS_BAR);	return 0;
 	case AP_RID_DIALOG_OPTIONS_CHK_ViewAll:				_enableDisableLogic(id_CHECK_VIEW_ALL);				return 0;
 	case AP_RID_DIALOG_OPTIONS_CHK_ViewHiddenText:		_enableDisableLogic(id_CHECK_VIEW_HIDDEN_TEXT);		return 0;
 	case AP_RID_DIALOG_OPTIONS_CHK_ViewUnprintable:		_enableDisableLogic(id_CHECK_VIEW_UNPRINTABLE);		return 0;
@@ -556,10 +583,22 @@ void AP_Win32Dialog_Options::_controlEnable( tControl id, UT_Bool value )
 		EnableWindow(GetDlgItem((HWND)m_vecSubDlgHWnd.getNthItem(VIEW_INDEX),AP_RID_DIALOG_OPTIONS_CHK_ViewCursorBlink),value);
 		return;
 		
-	case id_CHECK_VIEW_SHOW_TOOLBARS:
-		EnableWindow(GetDlgItem((HWND)m_vecSubDlgHWnd.getNthItem(VIEW_INDEX),AP_RID_DIALOG_OPTIONS_CHK_ViewShowToolbars),value);
+	case id_CHECK_VIEW_SHOW_STANDARD_TOOLBAR:
+		EnableWindow(GetDlgItem((HWND)m_vecSubDlgHWnd.getNthItem(VIEW_INDEX),AP_RID_DIALOG_OPTIONS_CHK_ViewShowStandardBar),value);
 		return;
 		
+	case id_CHECK_VIEW_SHOW_FORMAT_TOOLBAR:
+		EnableWindow(GetDlgItem((HWND)m_vecSubDlgHWnd.getNthItem(VIEW_INDEX),AP_RID_DIALOG_OPTIONS_CHK_ViewShowFormatBar),value);
+		return;
+	
+	case id_CHECK_VIEW_SHOW_EXTRA_TOOLBAR:
+		EnableWindow(GetDlgItem((HWND)m_vecSubDlgHWnd.getNthItem(VIEW_INDEX),AP_RID_DIALOG_OPTIONS_CHK_ViewShowExtraBar),value);
+		return;
+
+	case id_CHECK_VIEW_SHOW_STATUS_BAR:
+		EnableWindow(GetDlgItem((HWND)m_vecSubDlgHWnd.getNthItem(VIEW_INDEX),AP_RID_DIALOG_OPTIONS_CHK_ViewShowStatusBar),value);
+		return;
+
 	case id_CHECK_VIEW_ALL:
 		EnableWindow(GetDlgItem((HWND)m_vecSubDlgHWnd.getNthItem(VIEW_INDEX),AP_RID_DIALOG_OPTIONS_CHK_ViewAll),value);
 		return;
@@ -572,6 +611,10 @@ void AP_Win32Dialog_Options::_controlEnable( tControl id, UT_Bool value )
 		EnableWindow(GetDlgItem((HWND)m_vecSubDlgHWnd.getNthItem(VIEW_INDEX),AP_RID_DIALOG_OPTIONS_CHK_ViewUnprintable),value);
 		return;
 		
+	case id_CHECK_SMART_QUOTES_ENABLE:
+		EnableWindow(GetDlgItem((HWND)m_vecSubDlgHWnd.getNthItem(OTHER_INDEX),id_CHECK_SMART_QUOTES_ENABLE),value);
+		return;
+		
 	default:
 //		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
 		return;
@@ -599,11 +642,16 @@ DEFINE_GET_SET_BOOL(SPELL_INDEX,SpellUppercase);
 DEFINE_GET_SET_BOOL(SPELL_INDEX,SpellNumbers);
 DEFINE_GET_SET_BOOL(SPELL_INDEX,SpellInternet);
 
+DEFINE_GET_SET_BOOL(OTHER_INDEX,SmartQuotesEnable);
+
 DEFINE_GET_SET_BOOL(PREF_INDEX,PrefsAutoSave);
 
 DEFINE_GET_SET_BOOL(VIEW_INDEX,ViewShowRuler);
+DEFINE_GET_SET_BOOL(VIEW_INDEX,ViewShowStandardBar);                                       
+DEFINE_GET_SET_BOOL(VIEW_INDEX,ViewShowFormatBar);                                         
+DEFINE_GET_SET_BOOL(VIEW_INDEX,ViewShowExtraBar);
+DEFINE_GET_SET_BOOL(VIEW_INDEX,ViewShowStatusBar);
 DEFINE_GET_SET_BOOL(VIEW_INDEX,ViewCursorBlink);
-DEFINE_GET_SET_BOOL(VIEW_INDEX,ViewShowToolbars);
 
 DEFINE_GET_SET_BOOL(VIEW_INDEX,ViewAll);
 DEFINE_GET_SET_BOOL(VIEW_INDEX,ViewHiddenText);
