@@ -101,9 +101,14 @@ void AP_Dialog_Options::_storeWindowData(void)
 	Save_Pref_Bool( pPrefsScheme, AP_PREF_KEY_SpellCheckNumbers, _gatherSpellNumbers() );
 	Save_Pref_Bool( pPrefsScheme, AP_PREF_KEY_SpellCheckInternet, _gatherSpellInternet() );
 
+	Save_Pref_Bool( pPrefsScheme, XAP_PREF_KEY_SmartQuotesEnable, _gatherSmartQuotesEnable() );
+
 	Save_Pref_Bool( pPrefsScheme, AP_PREF_KEY_CursorBlink, _gatherViewCursorBlink() );
 	Save_Pref_Bool( pPrefsScheme, AP_PREF_KEY_RulerVisible, _gatherViewShowRuler() );
-    Save_Pref_Bool( pPrefsScheme, AP_PREF_KEY_ParaVisible, _gatherViewUnprintable() );
+	Save_Pref_Bool( pPrefsScheme, AP_PREF_KEY_StandardBarVisible, _gatherViewShowStandardBar() );
+	Save_Pref_Bool( pPrefsScheme, AP_PREF_KEY_FormatBarVisible, _gatherViewShowFormatBar() );
+	Save_Pref_Bool( pPrefsScheme, AP_PREF_KEY_ExtraBarVisible, _gatherViewShowExtraBar() );
+	Save_Pref_Bool( pPrefsScheme, AP_PREF_KEY_ParaVisible, _gatherViewUnprintable() );
 
 	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 	// If we changed whether the ruler is to be visible
@@ -115,6 +120,25 @@ void AP_Dialog_Options::_storeWindowData(void)
 		m_pFrame->toggleRuler( _gatherViewShowRuler() );
 	}
 
+	// TODO: Don't use 0, 1, 2, but AP_TOOLBAR_STANDARD, AP_TOOLBAR_FORMAT, AP_TOOLBAR_EXTRA...
+	if (_gatherViewShowStandardBar() != pFrameData->m_bShowBar[0])
+	{
+		pFrameData->m_bShowBar[0] = _gatherViewShowStandardBar();
+		m_pFrame->toggleBar(0, _gatherViewShowStandardBar());
+	}
+
+	if (_gatherViewShowFormatBar() != pFrameData->m_bShowBar[1])
+	{
+		pFrameData->m_bShowBar[1] = _gatherViewShowFormatBar();
+		m_pFrame->toggleBar(1, _gatherViewShowFormatBar());
+	}
+
+	if (_gatherViewShowExtraBar() != pFrameData->m_bShowBar[2])
+	{
+		pFrameData->m_bShowBar[2] = _gatherViewShowExtraBar();
+		m_pFrame->toggleBar(2, _gatherViewShowExtraBar());
+	}
+
     if ( _gatherViewUnprintable() != pFrameData->m_bShowPara )
     {
         pFrameData->m_bShowPara = _gatherViewUnprintable() ;
@@ -128,7 +152,8 @@ void AP_Dialog_Options::_storeWindowData(void)
 
 	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 	// save ruler units value
-	pPrefsScheme->setValue( AP_PREF_KEY_RulerUnits, UT_dimensionName( _gatherViewRulerUnits()) );
+	pPrefsScheme->setValue((XML_Char*)AP_PREF_KEY_RulerUnits,
+			       (XML_Char*)UT_dimensionName( _gatherViewRulerUnits()) );
 
 	// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 	// allow XAP_Prefs to notify all the listeners of changes
@@ -137,7 +162,8 @@ void AP_Dialog_Options::_storeWindowData(void)
 	// TODO: change to snprintf
 	XML_Char szBuffer[40];
 	sprintf( szBuffer, "%i", _gatherNotebookPageNum() );
-	pPrefsScheme->setValue( AP_PREF_KEY_OptionsTabNumber, szBuffer );
+	pPrefsScheme->setValue((XML_Char*)AP_PREF_KEY_OptionsTabNumber,
+			       (XML_Char*)szBuffer );
 
 	// allow the prefListeners to receive their calls
 	pPrefs->endBlockChange();
@@ -171,36 +197,50 @@ void AP_Dialog_Options::_populateWindowData(void)
 	UT_ASSERT( pPrefs );
 
 	// ------------ Spell
-	if (pPrefs->getPrefsValueBool(AP_PREF_KEY_AutoSpellCheck,&b))
+	if (pPrefs->getPrefsValueBool((XML_Char*)AP_PREF_KEY_AutoSpellCheck,&b))
 		_setSpellCheckAsType (b);
 
-	if (pPrefs->getPrefsValueBool(AP_PREF_KEY_SpellCheckCaps,&b))
+	if (pPrefs->getPrefsValueBool((XML_Char*)AP_PREF_KEY_SpellCheckCaps,&b))
 		_setSpellUppercase (b);
 
-	if (pPrefs->getPrefsValueBool(AP_PREF_KEY_SpellCheckNumbers,&b))
+	if (pPrefs->getPrefsValueBool((XML_Char*)AP_PREF_KEY_SpellCheckNumbers,&b))
 		_setSpellNumbers (b);
 
-	if (pPrefs->getPrefsValueBool(AP_PREF_KEY_SpellCheckInternet,&b))
+	if (pPrefs->getPrefsValueBool((XML_Char*)AP_PREF_KEY_SpellCheckInternet,&b))
 		_setSpellInternet (b);
 	
+	// ------------ Smart Quotes
+	if (pPrefs->getPrefsValueBool((XML_Char*)XAP_PREF_KEY_SmartQuotesEnable,&b))
+		_setSmartQuotesEnable (b);
+
 	// ------------ Prefs	
 	_setPrefsAutoSave( pPrefs->getAutoSavePrefs() );
 
 	// ------------ View
-	if (pPrefs->getPrefsValue(AP_PREF_KEY_RulerUnits,&pszBuffer))
+	if (pPrefs->getPrefsValue((XML_Char*)AP_PREF_KEY_RulerUnits,&pszBuffer))
 		_setViewRulerUnits (UT_determineDimension(pszBuffer));
 
-	if (pPrefs->getPrefsValueBool(AP_PREF_KEY_RulerVisible,&b))
+	if (pPrefs->getPrefsValueBool((XML_Char*)AP_PREF_KEY_RulerVisible,&b))
 		_setViewShowRuler (b);
 
-    if (pPrefs->getPrefsValueBool(AP_PREF_KEY_ParaVisible,&b))
+
+	if (pPrefs->getPrefsValueBool((XML_Char*)AP_PREF_KEY_StandardBarVisible,&b))
+		_setViewShowStandardBar (b);
+
+	if (pPrefs->getPrefsValueBool((XML_Char*)AP_PREF_KEY_FormatBarVisible,&b))
+		_setViewShowFormatBar (b);
+
+	if (pPrefs->getPrefsValueBool((XML_Char*)AP_PREF_KEY_ExtraBarVisible,&b))
+		_setViewShowExtraBar (b);
+
+	if (pPrefs->getPrefsValueBool((XML_Char*)AP_PREF_KEY_ParaVisible,&b))
         _setViewUnprintable (b);
 
-	if (pPrefs->getPrefsValueBool(AP_PREF_KEY_CursorBlink,&b))
+	if (pPrefs->getPrefsValueBool((XML_Char*)AP_PREF_KEY_CursorBlink,&b))
 		_setViewCursorBlink (b);
 
 	// ------------ the page tab number 
-	if (pPrefs->getPrefsValue(AP_PREF_KEY_OptionsTabNumber,&pszBuffer))
+	if (pPrefs->getPrefsValue((XML_Char*)AP_PREF_KEY_OptionsTabNumber,&pszBuffer))
 		_setNotebookPageNum (atoi(pszBuffer));
 
 
@@ -246,7 +286,6 @@ void AP_Dialog_Options::_initEnableControls()
 	_controlEnable( id_COMBO_PREFS_SCHEME,			UT_FALSE );
 
 	// view
-	_controlEnable( id_CHECK_VIEW_SHOW_TOOLBARS,	UT_FALSE );
 	_controlEnable( id_CHECK_VIEW_ALL,				UT_FALSE );
 	_controlEnable( id_CHECK_VIEW_HIDDEN_TEXT,		UT_FALSE );
 
@@ -273,7 +312,7 @@ void AP_Dialog_Options::_event_SetDefaults(void)
 
 	currentPage = _gatherNotebookPageNum();
 
-	pPrefs->setCurrentScheme("_builtin_");		
+	pPrefs->setCurrentScheme((XML_Char*)"_builtin_");		
 	
 	_populateWindowData();
 
