@@ -228,8 +228,8 @@ static struct ap_bs_NVK s_NVKTable[] =
 						  "",					"",					"",				""					}},
 //	{EV_NVK_F11,		{ "",					"",					"",				"",
 //						  "",					"",					"",				""					}},
-	{EV_NVK_F12,		{ "cycleInputMode",		"",					"",				"",
-						  FN_TEST_DUMP,			"",					"",				""					}},
+	{EV_NVK_F12,		{ "cycleInputMode",		"",					FN_TEST_FTR,				"",
+						  FN_TEST_DUMP,			        "",					"",				""					}},
 // 	{EV_NVK_F13,		{
 // 	{EV_NVK_F14,		{
 // 	{EV_NVK_F15,		{
@@ -310,9 +310,9 @@ static struct ap_bs_Char s_CharTable[] =
 	{0x2e, /* .      */ { "insertData",			"",					"",				""					}},
 	{0x2f, /* /      */ { "insertData",			"",					"",				""					}},
 	{0x30, /* 0      */ { "insertData",			"",					"",				""					}},
-	{0x31, /* 1      */ { "insertData",			"singleSpace",		"",				""					}},
-	{0x32, /* 2      */ { "insertData",			"doubleSpace",		"",				""					}},
-	{0x33, /* 3      */ { "insertData",			"",					"",				""					}},
+	{0x31, /* 1      */ { "insertData",			"singleSpace",		"setStyleHeading1",				""					}},
+	{0x32, /* 2      */ { "insertData",			"doubleSpace",		"setStyleHeading2",				""					}},
+	{0x33, /* 3      */ { "insertData",			"",					"setStyleHeading3",				""					}},
 	{0x34, /* 4      */ { "insertData",			"",					"",				""					}},
 	{0x35, /* 5      */ { "insertData",			"middleSpace",		"",				""					}},
 	{0x36, /* 6      */ { "insertData",			"",					"",				""					}},
