@@ -74,6 +74,7 @@ BeginSet(de,DE,UT_TRUE)
 	MenuLabel(AP_MENU_ID_VIEW_TOOLBARS,		"&Symbolleisten",	NULL)
 	MenuLabel(AP_MENU_ID_VIEW_TB_STD,		"&Standard",		"Symbolleiste <Standard> anzeigen oder verbergen")
 	MenuLabel(AP_MENU_ID_VIEW_TB_FORMAT,	"&Format",			"Symbolleiste <Format> anzeigen oder verbergen")
+	MenuLabel(AP_MENU_ID_VIEW_TB_EXTRA,		"&Extra",			"Symbolleiste <Extra> anzeigen oder Verbergen")	
 	MenuLabel(AP_MENU_ID_VIEW_RULER,		"&Lineal",			"Lineale anzeigen oder verbergen")
 	MenuLabel(AP_MENU_ID_VIEW_STATUSBAR,	"Status&leiste",	"Statusleiste anzeigen oder verbergen")
 	MenuLabel(AP_MENU_ID_VIEW_SHOWPARA,		"Para&graph",		"Paragraphen anzeigen oder verbergen")
@@ -129,7 +130,14 @@ BeginSet(de,DE,UT_TRUE)
 	MenuLabel(AP_MENU_ID_WINDOW_MORE,		"&Mehr Fenster",        	"Liste aller Fenster zeigen")
 
 	MenuLabel(AP_MENU_ID_HELP,				"&Hilfe",		NULL)
-	MenuLabel(AP_MENU_ID_HELP_ABOUT,		"&ber %s",	        	"Programminformation, Verion und Copyright anzeigen")
+	MenuLabel(AP_MENU_ID_HELP_CONTENTS,		"&Inhalt",	"Inhalt der Hilfe anzeigen (lokal)")
+	MenuLabel(AP_MENU_ID_HELP_INDEX,		"&Benutzerhandbuch",		"Benutzerhandbuch anzeigen (lokal)")
+	MenuLabel(AP_MENU_ID_HELP_CHECKVER,		"&Version",	"Versionsnummer anzeigen (WWW)")
+	MenuLabel(AP_MENU_ID_HELP_SEARCH,		"In der Hilfe &suchen",	"Aufrufen der Suchseite (WWW)")
+	MenuLabel(AP_MENU_ID_HELP_ABOUT,		"&ber %s",	        	"Programminformation, Version und Copyright anzeigen")
+	MenuLabel(AP_MENU_ID_HELP_ABOUTOS,		"ber &Open Source",	"Informationen ber Open Source anzeigen (lokal)")
+
+
 
 	MenuLabel(AP_MENU_ID_SPELL_SUGGEST_1,	"%s",				"ndere in diesen Vorschlag")
 	MenuLabel(AP_MENU_ID_SPELL_SUGGEST_2,	"%s",				"ndere in diesen Vorschlag")
