@@ -72,11 +72,12 @@ BeginSet(es,ES,UT_TRUE)
 	
 	MenuLabel(AP_MENU_ID_VIEW,				"&Ver",				NULL)
 	MenuLabel(AP_MENU_ID_VIEW_TOOLBARS,		"&Barras de herramientas",	NULL)
-	MenuLabel(AP_MENU_ID_VIEW_TB_STD,		"&Estandard",		"Mostrar u ocultar la barra de herramientas estndar")
+	MenuLabel(AP_MENU_ID_VIEW_TB_STD,		"&Estndar",		"Mostrar u ocultar la barra de herramientas estndar")
 	MenuLabel(AP_MENU_ID_VIEW_TB_FORMAT,	"&Formato",			"Mostrar u ocultar la barra de herramientas de formato")
+	MenuLabel(AP_MENU_ID_VIEW_TB_EXTRA,		"&Extra",			"Mostrar u ocultar la barra de herramientas extra")
 	MenuLabel(AP_MENU_ID_VIEW_RULER,		"&Regla",			"Mostrar u ocultar las reglas")
 	MenuLabel(AP_MENU_ID_VIEW_STATUSBAR,	"&Barra de estado",	"Mostrar u ocultar la barra de estado")
-	MenuLabel(AP_MENU_ID_VIEW_SHOWPARA,		"Mostrar Prra&fos","Mostrar los carcteres no imprimibles")
+	MenuLabel(AP_MENU_ID_VIEW_SHOWPARA,		"Mostrar prra&fos","Mostrar los carcteres no imprimibles")
 	MenuLabel(AP_MENU_ID_VIEW_HEADFOOT,		"&Cabecera y pie de pgina",	"Edita el texto de la cabecera y el pie de cada pgina")
 	MenuLabel(AP_MENU_ID_VIEW_ZOOM,			"&Zoom",			"Reducir o aumentar el documento")
 
@@ -129,7 +130,23 @@ BeginSet(es,ES,UT_TRUE)
 	MenuLabel(AP_MENU_ID_WINDOW_MORE,		"&Ms ventanas",	"Mostrar la lista completa de ventanas")
 
 	MenuLabel(AP_MENU_ID_HELP,				"&Ayuda",			NULL)
+	MenuLabel(AP_MENU_ID_HELP_CONTENTS,		"&Contenidos",		"Mostrar el contenido de la ayuda")
+	MenuLabel(AP_MENU_ID_HELP_INDEX,		"&ndice",			"Mostrar el ndice de la ayuda")
+	MenuLabel(AP_MENU_ID_HELP_CHECKVER,		"&Versin",			"Mostrar el nmero de versin del programa")
+	MenuLabel(AP_MENU_ID_HELP_SEARCH,		"&Buscar ayuda",	"Mostrar informacin acerca del programa, nmero de versin y copyright")
 	MenuLabel(AP_MENU_ID_HELP_ABOUT,		"&Acerca de %s",	"Mostrar informacin del programa, versin y derechos de autor")
+	MenuLabel(AP_MENU_ID_HELP_ABOUTOS,		"Acerca del Software &libre",	"Mostrar informacin acerca del software libre")
+	MenuLabel(AP_MENU_ID_SPELL_SUGGEST_1,	"%s",				"Cambiar por la sugerencia")
+	MenuLabel(AP_MENU_ID_SPELL_SUGGEST_2,	"%s",				"Cambiar por la sugerencia")
+	MenuLabel(AP_MENU_ID_SPELL_SUGGEST_3,	"%s",				"Cambiar por la sugerencia")
+	MenuLabel(AP_MENU_ID_SPELL_SUGGEST_4,	"%s",				"Cambiar por la sugerencia")
+	MenuLabel(AP_MENU_ID_SPELL_SUGGEST_5,	"%s",				"Cambiar por la sugerencia")
+	MenuLabel(AP_MENU_ID_SPELL_SUGGEST_6,	"%s",				"Cambiar por la sugerencia")
+	MenuLabel(AP_MENU_ID_SPELL_SUGGEST_7,	"%s",				"Cambiar por la sugerencia")
+	MenuLabel(AP_MENU_ID_SPELL_SUGGEST_8,	"%s",				"Cambiar por la sugerencia")
+	MenuLabel(AP_MENU_ID_SPELL_SUGGEST_9,	"%s",				"Cambiar por la sugerencia")
+	MenuLabel(AP_MENU_ID_SPELL_IGNOREALL,	"&Ignorar todas", 	"Ignorar todas las ocurrencias de esta palabra en el documento")
+	MenuLabel(AP_MENU_ID_SPELL_ADD,			"&Aadir", 			"Aadir esta palabra al diccionario personal")
 
 	// ... add others here ...
 
