@@ -36,7 +36,7 @@ dcl(MSG_ConfirmSave,	        "Save changes to %s?")
 dcl(MSG_ImportError,	        "Error importing file %s.")
 dcl(MSG_IE_FileNotFound,        "File %s not found")
 dcl(MSG_IE_NoMemory,            "Out of memory attempting to open %s")
-dcl(MSG_IE_UnknownType,         "File %s is of unkown type")
+dcl(MSG_IE_UnknownType,         "File %s is of unknown type")
 dcl(MSG_IE_FakeType,            "File %s is not of the type it claims to be")
 dcl(MSG_IE_UnsupportedType,     "File %s is not of a currently supported file type")
 dcl(MSG_IE_BogusDocument,       "File %s is a bogus document")
@@ -149,6 +149,16 @@ dcl(DLG_Para_PushKeepWithNext,			"Keep with ne&xt")
 
 dcl(DLG_Para_ButtonTabs,		"&Tabs...")
 
+/* Columns dialog */
+dcl(DLG_Column_ColumnTitle,		"Columns")
+dcl(DLG_Column_Number,			"Number of columns")
+dcl(DLG_Column_Preview,			"Preview")
+dcl(DLG_Column_One,				"One")
+dcl(DLG_Column_Two,				"Two")
+dcl(DLG_Column_Three,			"Three")
+dcl(DLG_Column_Line_Between,	"Line between")
+
+
 /* when translating these, the important thing to remember is to get a
    similar amount of text in the translation.  Microsoft Word uses
    strings just like this to simulate text, and if your translation
@@ -161,9 +171,10 @@ dcl(DLG_Para_PreviewPrevParagraph,  "Previous Paragraph Previous Paragraph Previ
 dcl(DLG_Para_PreviewFollowParagraph,"Following Paragraph Following Paragraph Following Paragraph Following Paragraph Following Paragraph Following Paragraph Following Paragraph")
 
 /* Options dialog */
-dcl(DLG_Options_OptionsTitle,			"Options")
+dcl(DLG_Options_OptionsTitle,			"Preferences")
 
 dcl(DLG_Options_TabLabel_Spelling,		"Spelling")
+dcl(DLG_Options_TabLabel_Other,			"Other")
 dcl(DLG_Options_TabLabel_Preferences,	"Preference Schemes")
 dcl(DLG_Options_TabLabel_View,			"View")
 
@@ -184,6 +195,8 @@ dcl(DLG_Options_Btn_CustomDict,			"&Dictionary...")
 dcl(DLG_Options_Btn_IgnoreReset,		"&Reset")
 dcl(DLG_Options_Btn_IgnoreEdit,			"&Edit")
 
+dcl(DLG_Options_Label_SmartQuotesEnable,"Enable smart quotes")
+
 dcl(DLG_Options_Label_PrefsAutoSave,	"&Automatically save this Scheme")
 dcl(DLG_Options_Label_PrefsCurrentScheme,	"Current Preferences Scheme")
 
@@ -191,15 +204,40 @@ dcl(DLG_Options_Label_ViewShowHide,		"Show/Hide...")
 dcl(DLG_Options_Label_ViewRuler,		"&Ruler")
 dcl(DLG_Options_Label_ViewUnits,		"&units:")
 dcl(DLG_Options_Label_ViewCursorBlink,	"Cursor &blink")
-dcl(DLG_Options_Label_ViewToolbars,		"&Toolbars")
+dcl(DLG_Options_Label_ViewStandardTB,		"&Standard Toolbar")
+dcl(DLG_Options_Label_ViewFormatTB,		"&Format Toolbar")
+dcl(DLG_Options_Label_ViewExtraTB,		"&Extra Toolbar")
+dcl(DLG_Options_Label_ViewStatusBar,		"Status bar")
 dcl(DLG_Options_Label_ViewViewFrame,	"View...")
 dcl(DLG_Options_Label_ViewAll,			"&All")
 dcl(DLG_Options_Label_ViewHiddenText,	"&Hidden Text")
-dcl(DLG_Options_Label_ViewUnprintable,	"Un&printable characters")
+dcl(DLG_Options_Label_ViewUnprintable,	"Invisible &Layout Marks")
 
 dcl(DLG_Options_Prompt_IgnoreResetCurrent,	"Do you want to reset ignored words in the current document?" )
 dcl(DLG_Options_Prompt_IgnoreResetAll,		"Do you want to reset ignored words in all the documents?" )
 
+dcl(DLG_Tab_TabTitle,					"Tabs")
+dcl(DLG_Tab_Label_TabPosition,			"Tab stop position:")
+dcl(DLG_Tab_Label_TabToClear,			"Tab stops to be cleared:")
+dcl(DLG_Tab_Label_DefaultTS,			"Default tab stops:")
+
+dcl(DLG_Tab_Label_Alignment,			"Alignment")
+dcl(DLG_Tab_Radio_Left,					"Left")
+dcl(DLG_Tab_Radio_Center,				"Center")
+dcl(DLG_Tab_Radio_Right,				"Right")
+dcl(DLG_Tab_Radio_Decimal,				"Decimal")
+dcl(DLG_Tab_Radio_Bar,					"Bar")
+
+dcl(DLG_Tab_Label_Leader,				"Leader")
+dcl(DLG_Tab_Radio_None,					"&1 None")
+dcl(DLG_Tab_Radio_Dot,					"&2 ..........")
+dcl(DLG_Tab_Radio_Dash,					"&3	----------")
+dcl(DLG_Tab_Radio_Underline,			"&4 __________")
+
+dcl(DLG_Tab_Button_Set,					"Set")
+dcl(DLG_Tab_Button_Clear,				"Clear")
+dcl(DLG_Tab_Button_ClearAll,			"Clear &All")
+
 dcl(DLG_DateTime_DateTimeTitle,			"Date and Time")
 dcl(DLG_DateTime_AvailableFormats,		"&Available formats:")
 
@@ -207,6 +245,16 @@ dcl(DLG_Field_FieldTitle,				"Field")
 dcl(DLG_Field_Types,					"&Types:")
 dcl(DLG_Field_Fields,					"&Fields:")
 
+dcl(FIELD_Type_Datetime,		"Date and Time")
+dcl(FIELD_Type_Numbers,			"Numbers")
+dcl(FIELD_Type_PieceTable,		"Piece Table")
+dcl(FIELD_Datetime_CurrentTime,		"Current time")
+dcl(FIELD_Numbers_PageNumber,		"Page number")
+dcl(FIELD_Numbers_PagesCount,		"Number of pages")
+dcl(FIELD_Numbers_ListLabel,		"List Label")
+dcl(FIELD_PieceTable_Test,		"Kevins Test")
+dcl(FIELD_PieceTable_MartinTest,		"Martins Test")
+
 dcl(DLG_Goto_Title,						"Go to...")
 dcl(DLG_Goto_Label_Help,				"Choose your target in the left side.\n"
 										"If you want to use the \"Go To\" button, "
@@ -221,3 +269,57 @@ dcl(DLG_Goto_Btn_Next,					"Next >>")
 dcl(DLG_Goto_Label_What,				"Go To &What:")
 dcl(DLG_Goto_Label_Number,				"&Number:")
 dcl(DLG_Goto_Btn_Goto,					"Go To")
+dcl(DLG_Goto_Target_Page,				"Page")
+dcl(DLG_Goto_Target_Line,				"Line")
+dcl(DLG_Goto_Target_Picture,				"Picture")
+
+// Lists Dialog
+
+dcl(DLG_Lists_Title,                     "Lists for ")
+dcl(DLG_Lists_Start_New_List,             "Start New List")
+dcl(DLG_Lists_Stop_Current_List,             "Stop Current List")
+dcl(DLG_Lists_Resume_Previous_List,             "Resume Previous List")
+dcl(DLG_Lists_New_List_Type,             "New List \nType")
+dcl(DLG_Lists_Numbered_List,             "Numbered List")
+dcl(DLG_Lists_Lower_Case_List,             "Lower Case List")
+dcl(DLG_Lists_Upper_Case_List,             "Upper Case List")
+dcl(DLG_Lists_Lower_Roman_List,             "Lower Roman List")
+dcl(DLG_Lists_Upper_Roman_List,             "Upper Roman List")
+dcl(DLG_Lists_Bullet_List,                 "Bullet List")
+dcl(DLG_Lists_Dashed_List,                 "Dashed List")
+dcl(DLG_Lists_Square_List,                 "Square List")
+dcl(DLG_Lists_Triangle_List,                 "Triangle List")
+dcl(DLG_Lists_Diamond_List,                 "Diamond List")
+dcl(DLG_Lists_Star_List,                 "Star List")
+dcl(DLG_Lists_Implies_List,                 "Implies List")
+dcl(DLG_Lists_Tick_List,                 "Tick List")
+dcl(DLG_Lists_Box_List,                 "Box List")
+dcl(DLG_Lists_Hand_List,                 "Hand List")
+dcl(DLG_Lists_Heart_List,                 "Heart List")
+dcl(DLG_Lists_Starting_Value,             "New Starting \nValue")
+dcl(DLG_Lists_New_List_Label,             "New List Label")
+dcl(DLG_Lists_Current_List_Type,          "Current List Type")
+dcl(DLG_Lists_Current_List_Label,          "Current List Label")
+dcl(DLG_Lists_Cur_Change_Start,          "Change Current \nList")
+dcl(DLG_Lists_Type,                     "Type:")
+dcl(DLG_Lists_Type_none,                     "None")
+dcl(DLG_Lists_Type_bullet,                     "Bullet")
+dcl(DLG_Lists_Type_numbered,                     "Numbered")
+dcl(DLG_Lists_Style,                          "Style:")
+dcl(DLG_Lists_Customize,                          "Customized List")
+dcl(DLG_Lists_Format,                        "Format:")
+dcl(DLG_Lists_Font,                     "Font:")
+dcl(DLG_Lists_Level,                     "Level Delimiter:")
+dcl(DLG_Lists_Start,                     "Start At:")
+dcl(DLG_Lists_Align,                     "Text Align:")
+dcl(DLG_Lists_Indent,                     "Label Align:")
+dcl(DLG_Lists_Current_Font,                     "Current Font")
+dcl(DLG_Lists_Preview,                     "Preview")
+dcl(DLG_Lists_Start_New,                     "Start New List")
+dcl(DLG_Lists_Apply_Current,                "Apply to Current List")
+dcl(DLG_Lists_Start_Sub,                "Start Sublist")
+dcl(DLG_Lists_Resume,                "Resume Previous List")
+
+
+
+
