@@ -24,6 +24,8 @@
 #include "ut_assert.h"
 #include "ut_string.h"
 #include "ut_units.h"
+#include "ut_debugmsg.h"
+
 #include "ap_Toolbar_Id.h"
 #include "ap_Toolbar_Functions.h"
 #include "ev_Toolbar_Actions.h"
@@ -33,9 +35,11 @@
 #include "xap_Frame.h"
 #include "fv_View.h"
 #include "gr_Graphics.h"
+#include "fl_AutoNum.h"
 
 #define ABIWORD_VIEW  	FV_View * pView = static_cast<FV_View *>(pAV_View)
 
+
 /****************************************************************/
 
 Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Changes)
@@ -153,6 +157,40 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Style)
 	return s;
 }
 
+
+Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Bullets)
+{
+	ABIWORD_VIEW;
+	UT_ASSERT(pView);
+
+	EV_Toolbar_ItemState s = EV_TIS_ZERO;
+
+	fl_BlockLayout * pBlock = pView->getCurrentBlock();
+        UT_ASSERT(pBlock);
+	if(pBlock->isListItem() == UT_FALSE)
+	        return s;
+	if(pBlock->getListType() == BULLETED_LIST)
+	        s = EV_TIS_Toggled;
+        return s;
+}
+
+
+Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Numbers)
+{
+	ABIWORD_VIEW;
+	UT_ASSERT(pView);
+
+	EV_Toolbar_ItemState s = EV_TIS_ZERO;
+
+	fl_BlockLayout * pBlock = pView->getCurrentBlock();
+        UT_ASSERT(pBlock);
+	if(pBlock->isListItem() == UT_FALSE)
+	        return s;
+	if(pBlock->getListType() == NUMBERED_LIST)
+	        s = EV_TIS_Toggled;
+        return s;
+}
+
 Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_CharFmt)
 {
 	ABIWORD_VIEW;
@@ -242,9 +280,8 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_CharFmt)
 		{
 			if (bSize)
 			{	
-				static char buf[5];
-				int pt = (int) UT_convertToPoints(sz);
-				sprintf(buf, "%d", pt);
+				static char buf[7];
+				sprintf(buf, "%s", std_size_string((float)UT_convertToPoints(sz)));
 				*pszState = buf;
 				s = EV_TIS_UseString;
 			}
@@ -441,6 +478,9 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Zoom)
 	ABIWORD_VIEW;
 	UT_ASSERT(pView);
 
+	XAP_Frame * pFrame = static_cast<XAP_Frame *> ( pView->getParentData());
+	const XAP_StringSet * pSS = XAP_App::getApp()->getStringSet();
+	
 	EV_Toolbar_ItemState s = EV_TIS_UseString;
 
 	static char buf[10];
@@ -448,6 +488,19 @@ Defun_EV_GetToolbarItemState_Fn(ap_ToolbarGetState_Zoom)
 	UT_uint32 iZoom = pView->getGraphics()->getZoomPercentage();
 	
 	sprintf(buf, "%d%%", iZoom);
+	switch(pFrame->getZoomType())
+	{
+	// special cases
+	case XAP_Frame::z_PAGEWIDTH:
+		sprintf(buf, "%s", pSS->getValue(XAP_STRING_ID_TB_Zoom_PageWidth));
+		break;
+	case XAP_Frame::z_WHOLEPAGE:
+		sprintf(buf, "%s", pSS->getValue(XAP_STRING_ID_TB_Zoom_WholePage));
+		break;	
+	default:
+		;
+	}
+
 	*pszState = buf;
 
 	return s;
