@@ -24,6 +24,7 @@
 #include "ut_assert.h"
 #include "ut_debugmsg.h"
 #include "ut_string.h"
+#include "ut_timer.h"
 #include "ap_TopRuler.h"
 #include "gr_Graphics.h"
 #include "ap_Ruler.h"
@@ -34,19 +35,11 @@
 #include "ap_StatusBar.h"
 #include "ap_Strings.h"
 
-// HACK: private copy of constants from fl_BlockLayout.h
-// TODO: find a better way of passing iType for tabs?
-// NOTE: this ordering is convenient for cycling m_iDefaultTabType 
-
-#define FL_TAB_LEFT				1
-#define FL_TAB_CENTER			2
-#define FL_TAB_RIGHT			3
-#define FL_TAB_DECIMAL			4
-#define FL_TAB_BAR				5
-
 #define	tr_TABINDEX_NEW			-1
 #define	tr_TABINDEX_NONE		-2
 
+#define tr_AUTOSCROLL_PIXELS	25
+#define tr_AUTOSCROLL_INTERVAL	300 // miliseconds
 
 /*****************************************************************/
 
@@ -64,6 +57,7 @@ AP_TopRuler::AP_TopRuler(XAP_Frame * pFrame)
 	m_bValidMouseClick = UT_FALSE;
 	m_draggingWhat = DW_NOTHING;
 	m_iDefaultTabType = FL_TAB_LEFT;
+	m_pAutoScrollTimer = NULL;
 
 	m_bGuide = UT_FALSE;
 	m_xGuide = 0;
@@ -103,6 +97,7 @@ AP_TopRuler::~AP_TopRuler(void)
 	//UT_DEBUGMSG(("AP_TopRuler::~AP_TopRuler (this=%p scroll=%p)\n", this, m_pScrollObj));
 
 	DELETEP(m_pScrollObj);
+	DELETEP(m_pAutoScrollTimer);
 }
 
 /*****************************************************************/
@@ -587,12 +582,11 @@ void AP_TopRuler::_getTabToggleRect(UT_Rect * prToggle)
 void AP_TopRuler::_getTabStopXAnchor(AP_TopRulerInfo * pInfo,
 										UT_sint32 k,
 										UT_sint32 * pTab,
-										unsigned char & iType)
+										eTabType & iType)
 {
 	UT_sint32 xAbsLeft = _getFirstPixelInColumn(pInfo,pInfo->m_iCurrentColumn);
 
 	UT_sint32 iPosition;
-	UT_uint32 iOffset;
 
 	if (k == tr_TABINDEX_NEW)
 	{
@@ -605,9 +599,12 @@ void AP_TopRuler::_getTabStopXAnchor(AP_TopRulerInfo * pInfo,
 		// look it up in the document
 		UT_ASSERT(k<pInfo->m_iTabStops);
 
+		fl_TabStop TabInfo;
 		UT_Bool bRes = pInfo->m_pfnEnumTabStops(pInfo->m_pVoidEnumTabStopsData, 
-												k, iPosition, iType, iOffset);
+												k, &TabInfo);
 		UT_ASSERT(bRes);
+		iPosition = TabInfo.getPosition();
+		iType = TabInfo.getType();
 	}
 
 	if (pTab)
@@ -636,7 +633,7 @@ void AP_TopRuler::_drawTabProperties(const UT_Rect * pClipRect,
 {
 	UT_sint32 anchor;
 	UT_Rect rect;
-	unsigned char iType;
+	eTabType iType;
 
 	if (m_draggingWhat == DW_TABSTOP)
 	{
@@ -646,7 +643,8 @@ void AP_TopRuler::_drawTabProperties(const UT_Rect * pClipRect,
 		_getTabStopRect(pInfo, anchor, &rect);
 
 		_drawTabStop(rect, m_draggingTabType, UT_FALSE);
-		_drawTabStop(m_draggingRect, m_draggingTabType, UT_TRUE);
+		if (m_draggingRect.left + m_draggingRect.width > (UT_sint32)MyMax(m_iLeftRulerWidth,s_iFixedWidth))
+			_drawTabStop(m_draggingRect, m_draggingTabType, UT_TRUE);
 	}
 	
 	/*
@@ -705,7 +703,7 @@ void AP_TopRuler::_drawTabProperties(const UT_Rect * pClipRect,
 }
 
 UT_sint32 AP_TopRuler::_findTabStop(AP_TopRulerInfo * pInfo, 
-									UT_uint32 x, UT_uint32 y, unsigned char & iType)
+									UT_uint32 x, UT_uint32 y, eTabType & iType)
 {
 	// hit-test all the existing tabs
 	// return the index of the one found
@@ -741,15 +739,13 @@ const char * AP_TopRuler::_getTabStopString(AP_TopRulerInfo * pInfo, UT_sint32 k
 {
 	// return pointer to static buffer -- use it quickly.
 
-	UT_sint32 iPosition;
-	UT_uint32 iOffset;
-	unsigned char iType;
+	fl_TabStop TabInfo;
 
 	UT_Bool bRes = pInfo->m_pfnEnumTabStops(pInfo->m_pVoidEnumTabStopsData, 
-											k, iPosition, iType, iOffset);
+											k, &TabInfo);
 	UT_ASSERT(bRes);
 
-	const char* pStart = &pInfo->m_pszTabStops[iOffset];
+	const char* pStart = &pInfo->m_pszTabStops[TabInfo.getOffset()];
 	const char* pEnd = pStart;
 	while (*pEnd && (*pEnd != ','))
 	{
@@ -838,6 +834,7 @@ void AP_TopRuler::_drawMarginProperties(const UT_Rect * /* pClipRect */,
 	UT_Rect rLeft, rRight;
 
 	_getMarginMarkerRects(pInfo,rLeft,rRight);
+	
 	m_pG->fillRect(GR_Graphics::CLR3D_Background, rLeft);
 
 	m_pG->setColor3D(GR_Graphics::CLR3D_Foreground);
@@ -1085,6 +1082,7 @@ void AP_TopRuler::mousePress(EV_EditModifierState /* ems */, EV_EditMouseButton
 			case FL_TAB_CENTER:		m_iDefaultTabType = FL_TAB_RIGHT;	break;
 			case FL_TAB_RIGHT:		m_iDefaultTabType = FL_TAB_DECIMAL;	break;
 			case FL_TAB_DECIMAL:	m_iDefaultTabType = FL_TAB_LEFT;	break;
+			default:	UT_DEBUGMSG(("Should not happen, tab type %d\n", m_iDefaultTabType));
 		}
 		_drawTabToggle(NULL, UT_TRUE);
 		return;
@@ -1092,8 +1090,8 @@ void AP_TopRuler::mousePress(EV_EditModifierState /* ems */, EV_EditMouseButton
 
 	// next hit-test against the tabs
 
-	unsigned char iType;
-	UT_sint32 iTab = _findTabStop(&m_infoCache, x, y, iType);
+ 	eTabType iType;
+	UT_sint32 iTab = _findTabStop(&m_infoCache, x, s_iFixedHeight/2 + s_iFixedHeight/4 - 3, iType);
 	if (iTab >= 0)
 	{
 		UT_DEBUGMSG(("hit tab %ld\n",iTab));
@@ -1220,9 +1218,10 @@ void AP_TopRuler::mouseRelease(EV_EditModifierState /* ems */, EV_EditMouseButto
 	if (!m_bValidMouseClick || (m_bEventIgnored && m_draggingWhat != DW_TABSTOP))
 	{
 		m_draggingWhat = DW_NOTHING;
+		m_bValidMouseClick = UT_FALSE;
 		return;
 	}
-
+	
 	m_bValidMouseClick = UT_FALSE;
 
 	// if they drag vertically off the ruler, we ignore the whole thing.
@@ -1281,7 +1280,6 @@ void AP_TopRuler::mouseRelease(EV_EditModifierState /* ems */, EV_EditMouseButto
 			m_draggingWhat = DW_NOTHING;
             FV_View *pView = static_cast<FV_View *>(m_pView);
             pView->setSectionFormat(properties);
-            pView->draw(NULL);
 		}
 		return;
 
@@ -1301,7 +1299,6 @@ void AP_TopRuler::mouseRelease(EV_EditModifierState /* ems */, EV_EditMouseButto
 			m_draggingWhat = DW_NOTHING;
             FV_View *pView = static_cast<FV_View *>(m_pView);
             pView->setSectionFormat(properties);
-            pView->draw(NULL);
 		}
 		return;
 
@@ -1319,7 +1316,6 @@ void AP_TopRuler::mouseRelease(EV_EditModifierState /* ems */, EV_EditMouseButto
 			m_draggingWhat = DW_NOTHING;
             FV_View *pView = static_cast<FV_View *>(m_pView);
             pView->setSectionFormat(properties);
-            pView->draw(NULL);
 		}
 		return;
 		
@@ -1410,18 +1406,24 @@ void AP_TopRuler::mouseRelease(EV_EditModifierState /* ems */, EV_EditMouseButto
 
 	case DW_TABSTOP:
 		{
-			unsigned char iType;
-			UT_sint32 iTab = _findTabStop(&m_infoCache, x, y, iType);
+       			eTabType iType;
 
-			if (iTab >= 0)
+			UT_sint32 iTab = _findTabStop(&m_infoCache, xgrid+xAbsLeft, s_iFixedHeight/2 + s_iFixedHeight/4 - 3, iType);
+			
+			UT_DEBUGMSG (("iTab: %i, m_draggingTab: %i\n", iTab, m_draggingTab));
+			
+			if (iTab >= 0 && iTab != m_draggingTab)
 			{
-				// a tabstop is already set here ==> delete the one we're dragging
-				m_draggingWhat = DW_NOTHING;
-				_setTabStops(tick, iTab, UT_TRUE); // UT_TRUE for the last arg will cause this to be deleted
+				UT_DEBUGMSG (("This tab was released over an existing tab. It will be deleted.\n"));
+				_setTabStops(tick, m_draggingTab, UT_TRUE); // UT_TRUE for the last arg will cause this to be deleted
 			}
-			_setTabStops(tick, iTab, UT_FALSE);
+			else
+			{
+				_setTabStops(tick, iTab, UT_FALSE);
+			}
+			m_draggingWhat = DW_NOTHING;
+			return;
 		}
-		return;
 
 	default:
 		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
@@ -1485,6 +1487,8 @@ void AP_TopRuler::_setTabStops(ap_RulerTicks tick, UT_sint32 iTab, UT_Bool bDele
 
 void AP_TopRuler::mouseMotion(EV_EditModifierState ems, UT_sint32 x, UT_sint32 y)
 {
+	// The X and Y that are passed to this function are x and y on the screen, not on the ruler.
+	
 	if (!m_bValidMouseClick)
 		return;
 		
@@ -1504,8 +1508,9 @@ void AP_TopRuler::mouseMotion(EV_EditModifierState ems, UT_sint32 x, UT_sint32 y
 		return;
 	}
 
-	// if they drag horizontally off the ruler, we probably want to ignore
-	// the whole thing.  but this may interact with the current scroll.
+	// the following segment of code used to test whether the mouse was on the ruler horizontally
+	// now it makes sure the values are sane, disregarding mouse area
+	// for example, it will not let a left indent be moved to a place before the end of the left page view margin
 
 	UT_sint32 xFixed = (UT_sint32)MyMax(m_iLeftRulerWidth,s_iFixedWidth);
 	UT_sint32 xStartPixel = xFixed + (UT_sint32) m_infoCache.m_xPageViewMargin;
@@ -1513,17 +1518,68 @@ void AP_TopRuler::mouseMotion(EV_EditModifierState ems, UT_sint32 x, UT_sint32 y
 		m_infoCache.u.c.m_xColumnWidth + m_infoCache.u.c.m_xaRightMargin;
 	ap_RulerTicks tick(m_pG,m_dim);
 
-	if (x < xStartPixel)
+	// now to test if we are on the view, and scroll if the mouse isn't
+	
+	// by the way, my OGR client just hit 50% of the block!! i'm so excited!!! http://www.distributed.net
+
+	if (x < xFixed || x > (UT_sint32) m_iWidth)
 	{
-		x = xStartPixel;
+		// set m_aScrollDirection here instead of in the timer creation block because there is a change,
+		// though small, that the direction will change immediately with no on-ruler in-between state.
+		// the user could have the mouse to the left of the ruler, bring it up onto the toolbar, go to the
+		// right of the window, and bring it down to the right of the ruler. i'm crazy! :-)))
+		if (x < xFixed)
+		{
+			m_aScrollDirection = 'L';
+		}
+		else
+		{
+			m_aScrollDirection = 'R';
+		}
+
+		if (!m_pAutoScrollTimer)
+		{
+			// the timer DOESNT exist, it's NOT already being taken care of, we are the FIRST mouse motion event
+			// off the ruler since the last time it was on the ruler and we have MAJOR SELF-ESTEEM PROBLEMS!!!
+			m_pAutoScrollTimer = UT_Timer::static_constructor(_autoScroll, this, m_pG);
+			if (m_pAutoScrollTimer)
+			{
+				m_pAutoScrollTimer->set(tr_AUTOSCROLL_INTERVAL);
+			}
+		}
+
+		if (m_aScrollDirection == 'L')
+		{
+			x = xFixed + 1;
+		}
+		else
+		{
+			x = m_iWidth - 10;
+		}
+		draw(NULL, &m_infoCache);
 	}
+
+	// by now, if the cursor if off the ruler we will have created a timer and set it and returned.
+	// so, the cursor is on the ruler. make sure that any timer for autoscroll is stopped.
+	else if (m_pAutoScrollTimer)
+	{
+		m_pAutoScrollTimer->stop();
+		DELETEP(m_pAutoScrollTimer);
+		m_pAutoScrollTimer = NULL;
+		draw(NULL, &m_infoCache);
+		_xorGuide(UT_TRUE);
+	}
+
+	// make sure the item is within the page...
+	if (x + m_xScrollOffset < xStartPixel)
+		x = xStartPixel - m_xScrollOffset;
+	// Aiiiiaag... xAbsRight is a screen coordinate. I figured this out the hard way.
 	else if (x > xAbsRight)
-	{
 		x = xAbsRight;
-	}
-
-	// mouse motion was in the ruler portion of the window, we cannot ignore it.
 
+	// if we are this far along, the mouse motion is significant
+	// we cannot ignore it.
+		
 	switch (m_draggingWhat)
 	{
 	case DW_NOTHING:
@@ -1599,7 +1655,6 @@ void AP_TopRuler::mouseMotion(EV_EditModifierState ems, UT_sint32 x, UT_sint32 y
 	case DW_RIGHTMARGIN:
 		{
 		UT_sint32 oldDragCenter = m_draggingCenter;
-        UT_sint32 iAbsLeft = _getFirstPixelInColumn(&m_infoCache,m_infoCache.m_iCurrentColumn);
         UT_sint32 iRightShift = UT_MAX(0,m_infoCache.m_xrRightIndent);
         UT_sint32 iLeftShift = UT_MAX(0,UT_MAX(m_infoCache.m_xrLeftIndent,m_infoCache.m_xrLeftIndent + m_infoCache.m_xrFirstLineIndent));
         UT_sint32 newMargin;
@@ -1767,7 +1822,7 @@ void AP_TopRuler::mouseMotion(EV_EditModifierState ems, UT_sint32 x, UT_sint32 y
             UT_sint32 iRightIndentPos = xAbsLeft + m_infoCache.u.c.m_xColumnWidth - m_infoCache.m_xrRightIndent - iFirstIndentShift;
 
 			// Prevent the first-line indent from being dragged off the page
-			if (m_dragging2Center < xFixed + m_infoCache.m_xPageViewMargin)
+			if (m_dragging2Center < xFixed + (UT_sint32) m_infoCache.m_xPageViewMargin)
 			{
 				m_dragging2Center = oldDragging2Center;
 				m_draggingCenter = oldDraggingCenter;
@@ -1865,21 +1920,28 @@ void AP_TopRuler::mouseMotion(EV_EditModifierState ems, UT_sint32 x, UT_sint32 y
 
 	case DW_TABSTOP:
 		{
-			UT_sint32 xAbsLeft = _getFirstPixelInColumn(&m_infoCache,m_infoCache.m_iCurrentColumn);
-			UT_sint32 xrel = ((UT_sint32)x) - xAbsLeft;
+			if (x < xStartPixel - m_xScrollOffset + m_infoCache.u.c.m_xaLeftMargin)
+			{
+				x = xStartPixel - m_xScrollOffset + m_infoCache.u.c.m_xaLeftMargin;
+			}
+			else if (x > xAbsRight - m_infoCache.u.c.m_xaRightMargin)
+			{
+				x = xAbsRight - m_infoCache.u.c.m_xaRightMargin;
+			}
+			UT_sint32 xrel = ((UT_sint32)x) - xStartPixel - 1; // TODO why is the -1 necessary? w/o it problems arise.
 			UT_sint32 xgrid = _snapPixelToGrid(xrel,tick);
 			double dgrid = _scalePixelDistanceToUnits(xrel,tick);
 			UT_DEBUGMSG(("SettingTabStop: %s\n",m_pG->invertDimension(tick.dimType,dgrid)));
 			UT_sint32 oldDraggingCenter = m_draggingCenter;
 			UT_Rect oldDraggingRect = m_draggingRect;
-			m_draggingCenter = xAbsLeft + xgrid;
+			m_draggingCenter = xStartPixel + xgrid;
 			_getTabStopRect(&m_infoCache,m_draggingCenter,&m_draggingRect);
 			if (!m_bBeforeFirstMotion && (m_draggingCenter != oldDraggingCenter))
 				draw(&oldDraggingRect,&m_infoCache);
 			_drawTabProperties(NULL,&m_infoCache,UT_FALSE);
 			_xorGuide();
 
-			double dxrel = _scalePixelDistanceToUnits(m_draggingCenter - xAbsLeft,tick);
+			double dxrel = _scalePixelDistanceToUnits(m_draggingCenter - xStartPixel,tick);
 			_displayStatusMessage(AP_STRING_ID_TabStopStatus, tick, dxrel);
 		}
 		m_bBeforeFirstMotion = UT_FALSE;
@@ -2191,7 +2253,7 @@ void AP_TopRuler::_drawTabToggle(const UT_Rect * pClipRect, UT_Bool bErase)
 	}
 }
 
-void AP_TopRuler::_drawTabStop(UT_Rect & rect, unsigned char iType, UT_Bool bFilled)
+void AP_TopRuler::_drawTabStop(UT_Rect & rect, eTabType iType, UT_Bool bFilled)
 {
 	GR_Graphics::GR_Color3D clr3d;
 	if (bFilled)
@@ -2293,7 +2355,7 @@ void AP_TopRuler::_drawColumnGapMarker(UT_Rect & rect)
 	//UT_DEBUGMSG(("AP_TopRuler::_prefsListener (this=%p)\n", data));
 
 	const XML_Char *pszBuffer;
-	pPrefs->getPrefsValue( AP_PREF_KEY_RulerUnits, &pszBuffer );
+	pPrefs->getPrefsValue((XML_Char*)AP_PREF_KEY_RulerUnits, &pszBuffer );
 
 	// or should I just default to inches or something?
 	UT_Dimension dim = UT_determineDimension( pszBuffer, DIM_none );
@@ -2334,3 +2396,34 @@ void AP_TopRuler::_displayStatusMessage(XAP_String_Id messageID, const ap_RulerT
 	AP_FrameData * pFrameData = (AP_FrameData *)m_pFrame->getFrameData();
 	pFrameData->m_pStatusBar->setStatusMessage(temp);
 }
+
+/* lambda */ void AP_TopRuler::_autoScroll(UT_Timer * pTimer)
+{
+	// this is a static callback method and does not have a 'this' pointer.
+	AP_TopRuler * pRuler = (AP_TopRuler *) pTimer->getInstanceData();
+	UT_ASSERT(pRuler);
+	
+	pRuler->_xorGuide(UT_TRUE);
+
+	UT_sint32 newXScrollOffset = pRuler->m_xScrollOffset;
+	if (pRuler->m_aScrollDirection == 'L')
+		newXScrollOffset = pRuler->m_xScrollOffset - tr_AUTOSCROLL_PIXELS;
+	else if (pRuler->m_aScrollDirection == 'R')
+		newXScrollOffset = pRuler->m_xScrollOffset + tr_AUTOSCROLL_PIXELS;
+	else
+		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+	
+	if (newXScrollOffset >= 0)
+			pRuler->m_pView->sendHorizontalScrollEvent(newXScrollOffset); // YAY it works!!
+
+	// IT'S A TRICK!!!
+	UT_sint32 fakeY = pRuler->s_iFixedHeight/2 + pRuler->s_iFixedHeight/4 - 3;
+	if (pRuler->m_aScrollDirection == 'L')
+	{
+		pRuler->mouseMotion(NULL, 0, fakeY); // it wants to see something < xFixed and 0 is gonna be
+	}
+	else
+	{
+		pRuler->mouseMotion(NULL, (UT_sint32) pRuler->m_iWidth + 1, fakeY); // m_iWidth+1 will be greater than m_iWidth
+	}
+}
