@@ -17,11 +17,11 @@
  * 02111-1307, USA.
  */
 
-
 #include "ut_string.h"
 #include "ut_types.h"
 #include "ut_bytebuf.h"
 #include "ut_base64.h"
+#include "ut_debugmsg.h"
 #include "pt_Types.h"
 #include "ie_exp_AbiWord_1.h"
 #include "pd_Document.h"
@@ -32,6 +32,9 @@
 #include "px_CR_Strux.h"
 #include "xap_App.h"
 #include "pd_Style.h"
+#include "fd_Field.h"
+#include "xap_EncodingManager.h"
+#include "fl_AutoNum.h"
 
 /*****************************************************************/
 /*****************************************************************/
@@ -112,11 +115,13 @@ protected:
 	void				_closeSection(void);
 	void				_closeBlock(void);
 	void				_closeSpan(void);
+	void				_closeField(void);
 	void				_openSpan(PT_AttrPropIndex apiSpan);
 	void				_openTag(const char * szPrefix, const char * szSuffix,
 								 UT_Bool bNewLineAfter, PT_AttrPropIndex api);
 	void				_outputData(const UT_UCSChar * p, UT_uint32 length);
 	void				_handleStyles(void);
+	void				_handleLists(void);
 	void				_handleDataItems(void);
 	
 	PD_Document *		m_pDocument;
@@ -125,6 +130,7 @@ protected:
 	UT_Bool				m_bInBlock;
 	UT_Bool				m_bInSpan;
 	PT_AttrPropIndex	m_apiLastSpan;
+    fd_Field *             m_pCurrentField;
 };
 
 void s_AbiWord_1_Listener::_closeSection(void)
@@ -156,6 +162,15 @@ void s_AbiWord_1_Listener::_closeSpan(void)
 	m_bInSpan = UT_FALSE;
 	return;
 }
+void s_AbiWord_1_Listener::_closeField(void)
+{
+	if (!m_pCurrentField)
+		return;
+    _closeSpan();
+	m_pie->write("</field>");
+    m_pCurrentField = NULL;
+	return;
+}
 
 void s_AbiWord_1_Listener::_openSpan(PT_AttrPropIndex apiSpan)
 {
@@ -197,19 +212,19 @@ void s_AbiWord_1_Listener::_openTag(const char * szPrefix, const char * szSuffix
 			// TODO in it and escaping it or using single-quotes.
 			
 			m_pie->write(" ");
-			m_pie->write(szName);
+			m_pie->write((char*)szName);
 			m_pie->write("=\"");
-			m_pie->write(szValue);
+			m_pie->write((char*)szValue);
 			m_pie->write("\"");
 		}
 		if (pAP->getNthProperty(0,szName,szValue))
 		{
 			m_pie->write(" ");
-			m_pie->write(PT_PROPS_ATTRIBUTE_NAME);
+			m_pie->write((char*)PT_PROPS_ATTRIBUTE_NAME);
 			m_pie->write("=\"");
-			m_pie->write(szName);
+			m_pie->write((char*)szName);
 			m_pie->write(":");
-			m_pie->write(szValue);
+			m_pie->write((char*)szValue);
 			UT_uint32 j = 1;
 			while (pAP->getNthProperty(j++,szName,szValue))
 			{
@@ -219,9 +234,9 @@ void s_AbiWord_1_Listener::_openTag(const char * szPrefix, const char * szSuffix
 				if (*szValue)
 				{
 					m_pie->write("; ");
-					m_pie->write(szName);
+					m_pie->write((char*)szName);
 					m_pie->write(":");
-					m_pie->write(szValue);
+					m_pie->write((char*)szValue);
 				}
 			}
 			m_pie->write("\"");
@@ -311,6 +326,7 @@ void s_AbiWord_1_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length
 			if (*pData > 0x007f)
 			{
 #if 1
+#	if 0
 				// convert non us-ascii into numeric entities.
 				// this has the advantage that our file format is
 				// 7bit clean and safe for email and other network
@@ -320,6 +336,32 @@ void s_AbiWord_1_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length
 				sprintf(localBuf,"&#x%x;",*pData++);
 				while (*plocal)
 					*pBuf++ = (UT_Byte)*plocal++;
+#	else
+				/*
+				Try to convert to native encoding and if
+				character fits into byte, output raw byte. This 
+				is somewhat essential for single-byte non-latin
+				languages like russian or polish - since
+				tools like grep and sed can be used then for
+				these files without any problem.
+				Networks and mail transfers are 8bit clean
+				these days.  - VH
+				*/
+				UT_UCSChar c = XAP_EncodingManager::instance->try_UToNative(*pData);
+				if (c==0 || c>255)
+				{
+					char localBuf[20];
+					char * plocal = localBuf;
+					sprintf(localBuf,"&#x%x;",*pData++);
+					while (*plocal)
+						*pBuf++ = (UT_Byte)*plocal++;
+				}
+				else
+				{
+					*pBuf++ = (UT_Byte)c;
+					pData++;
+				}
+#	endif
 #else
 				// convert to UTF8
 				// TODO if we choose this, do we have to put the ISO header in
@@ -354,13 +396,19 @@ s_AbiWord_1_Listener::s_AbiWord_1_Listener(PD_Document * pDocument,
 	m_bInBlock = UT_FALSE;
 	m_bInSpan = UT_FALSE;
 	m_apiLastSpan = 0;
-
+    m_pCurrentField = 0;
 	// Be nice to XML apps.  See the notes in _outputData() for more 
 	// details on the charset used in our documents.  By not declaring 
 	// any encoding, XML assumes we're using UTF-8.  Note that US-ASCII 
 	// is a strict subset of UTF-8. 
 
-	m_pie->write("<?xml version=\"1.0\"?>\n");
+	if (!XAP_EncodingManager::instance->cjk_locale()) {
+	    m_pie->write("<?xml version=\"1.0\" encoding=\"");
+	    m_pie->write(XAP_EncodingManager::instance->getNativeEncodingName());
+	    m_pie->write("\"?>\n");
+	} else {
+	    m_pie->write("<?xml version=\"1.0\"?>\n");
+	};
 
 	// We write this first so that the sniffer can detect AbiWord 
 	// documents more easily.   
@@ -456,11 +504,13 @@ s_AbiWord_1_Listener::s_AbiWord_1_Listener(PD_Document * pDocument,
 	
 	
 	_handleStyles();
+	_handleLists();
 }
 
 s_AbiWord_1_Listener::~s_AbiWord_1_Listener()
 {
 	_closeSpan();
+	_closeField();
 	_closeBlock();
 	_closeSection();
 	_handleDataItems();
@@ -476,7 +526,10 @@ UT_Bool s_AbiWord_1_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 	case PX_ChangeRecord::PXT_InsertSpan:
 		{
 			const PX_ChangeRecord_Span * pcrs = static_cast<const PX_ChangeRecord_Span *> (pcr);
-
+            if (pcrs->getField()!=m_pCurrentField)
+            {
+                _closeField();
+            }
 			PT_AttrPropIndex api = pcr->getIndexAP();
 			_openSpan(api);
 			
@@ -494,14 +547,19 @@ UT_Bool s_AbiWord_1_Listener::populate(PL_StruxFmtHandle /*sfh*/,
 			{
 			case PTO_Image:
 				_closeSpan();
+                _closeField();
 				_openTag("image","/",UT_FALSE,api);
 				return UT_TRUE;
 
 			case PTO_Field:
-				_closeSpan();
-				_openTag("field","/",UT_FALSE,api);
-				return UT_TRUE;
-
+                {
+                    _closeSpan();
+                    _closeField();
+                    _openTag("field","",UT_FALSE,api);
+                    m_pCurrentField = pcro->getField(); 
+                    UT_ASSERT(m_pCurrentField);
+                    return UT_TRUE;
+                }
 			default:
 				UT_ASSERT(0);
 				return UT_FALSE;
@@ -530,6 +588,7 @@ UT_Bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 	case PTX_Section:
 		{
 			_closeSpan();
+            _closeField();
 			_closeBlock();
 			_closeSection();
 			_openTag("section","",UT_TRUE,pcr->getIndexAP());
@@ -540,6 +599,7 @@ UT_Bool s_AbiWord_1_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 	case PTX_Block:
 		{
 			_closeSpan();
+            _closeField();
 			_closeBlock();
 			_openTag("p","",UT_FALSE,pcr->getIndexAP());
 			m_bInBlock = UT_TRUE;
@@ -625,16 +685,64 @@ void s_AbiWord_1_Listener::_handleStyles(void)
 	return;
 }
 
+void s_AbiWord_1_Listener::_handleLists(void)
+{
+	UT_Bool bWroteOpenListSection = UT_FALSE;
+	
+ 	//const char * szID;
+	//const char * szPid;
+	//const char * szProps;
+
+	fl_AutoNum * pAutoNum;
+	const char ** attr;
+
+#define LCheck(str) (0 == UT_stricmp(attr[0], str))
+
+	for (UT_uint32 k = 0; (m_pDocument->enumLists(k, &pAutoNum )); k++)
+	{	
+		if (pAutoNum->isEmpty() == UT_TRUE)
+			continue;
+		
+		if (!bWroteOpenListSection)
+		{
+			m_pie->write("<lists>\n");
+			bWroteOpenListSection = UT_TRUE;
+		}
+		m_pie->write("<l");
+		for (attr = pAutoNum->getAttributes(); (*attr); attr++)
+		{
+			if (LCheck("id") || LCheck("parentid") || LCheck("type") || LCheck("start-value") || LCheck("list-delim") || LCheck("list-decimal"))
+			{
+				m_pie->write(" ");
+				m_pie->write(attr[0]);
+				m_pie->write("=\""); 
+				m_pie->write(attr[1]);
+				m_pie->write("\"");
+			}
+			//attr++;
+		}
+		m_pie->write("/>\n");
+	}
+
+#undef LCheck			
+	
+	if (bWroteOpenListSection)
+		m_pie->write("</lists>\n");
+	
+	return;
+}
+
 void s_AbiWord_1_Listener::_handleDataItems(void)
 {
 	UT_Bool bWroteOpenDataSection = UT_FALSE;
 
 	const char * szName;
+   	const char * szMimeType;
 	const UT_ByteBuf * pByteBuf;
 
-	UT_ByteBuf bb64(1024);
+	UT_ByteBuf bbEncoded(1024);
 
-	for (UT_uint32 k=0; (m_pDocument->enumDataItems(k,NULL,&szName,&pByteBuf,NULL)); k++)
+	for (UT_uint32 k=0; (m_pDocument->enumDataItems(k,NULL,&szName,&pByteBuf,(void**)&szMimeType)); k++)
 	{
 		if (!bWroteOpenDataSection)
 		{
@@ -642,23 +750,67 @@ void s_AbiWord_1_Listener::_handleDataItems(void)
 			bWroteOpenDataSection = UT_TRUE;
 		}
 
-		if (UT_Base64Encode(&bb64, pByteBuf))
+	   	UT_Bool status = UT_FALSE;
+	   	UT_Bool encoded = UT_TRUE;
+	   
+		if (szMimeType && (UT_stricmp(szMimeType, "image/svg-xml") == 0 || UT_stricmp(szMimeType, "text/mathml") == 0))
+	     	{
+		   bbEncoded.truncate(0);
+		   bbEncoded.append((UT_Byte*)"<![CDATA[", 9);
+		   UT_uint32 off = 0;
+		   UT_uint32 len = pByteBuf->getLength();
+		   const UT_Byte * buf = pByteBuf->getPointer(0);
+		   while (off < len) {
+		      if (buf[off] == ']' && buf[off+1] == ']' && buf[off+2] == '>') {
+			 bbEncoded.append(buf, off-1);
+			 bbEncoded.append((UT_Byte*)"]]&gt;", 6);
+			 off += 3;
+			 len -= off;
+			 buf = pByteBuf->getPointer(off);
+			 off = 0;
+			 continue;
+		      }
+		      off++;
+		   }
+		   bbEncoded.append(buf, off);
+		   bbEncoded.append((UT_Byte*)"]]>\n", 4);
+		   status = UT_TRUE;
+		   encoded = UT_FALSE;
+		}
+	   	else  
 		{
-			m_pie->write("<d name=\"");
+		   status = UT_Base64Encode(&bbEncoded, pByteBuf);
+		   encoded = UT_TRUE;
+		}
+
+	   	if (status)
+	     	{
+	   		m_pie->write("<d name=\"");
 			m_pie->write(szName);
-			m_pie->write("\">\n");
+		   	if (szMimeType) {
+			   m_pie->write("\" mime-type=\"");
+			   m_pie->write(szMimeType);
+			}
+		   	if (encoded) 
+		     	{
+			   	m_pie->write("\" base64=\"yes\">\n");
+				// break up the Base64 blob as a series lines
+				// like MIME does.
 
-			// break up the Base64 blob as a series lines
-			// like MIME does.
-
-			UT_uint32 jLimit = bb64.getLength();
-			UT_uint32 jSize;
-			UT_uint32 j;
-			for (j=0; j<jLimit; j+=72)
-			{
-				jSize = MyMin(72,(jLimit-j));
-				m_pie->write((const char *)bb64.getPointer(j),jSize);
-				m_pie->write("\n");
+				UT_uint32 jLimit = bbEncoded.getLength();
+				UT_uint32 jSize;
+				UT_uint32 j;
+				for (j=0; j<jLimit; j+=72)
+				{
+					jSize = MyMin(72,(jLimit-j));
+					m_pie->write((const char *)bbEncoded.getPointer(j),jSize);
+					m_pie->write("\n");
+				}
+			}
+		   	else
+		     	{
+		     		m_pie->write("\" base64=\"no\">\n");
+			   	m_pie->write((const char*)bbEncoded.getPointer(0), bbEncoded.getLength());
 			}
 			m_pie->write("</d>\n");
 		}
