@@ -25,6 +25,7 @@
 #include "ut_bytebuf.h"
 #include "ut_base64.h"
 #include "ut_units.h"
+#include "ut_wctomb.h"
 #include "pt_Types.h"
 #include "ie_exp_HTML.h"
 #include "pd_Document.h"
@@ -34,6 +35,7 @@
 #include "px_CR_Span.h"
 #include "px_CR_Strux.h"
 #include "xap_App.h"
+#include "xap_EncodingManager.h"
 
 /*****************************************************************/
 /*****************************************************************/
@@ -134,12 +136,13 @@ protected:
 	UT_Bool				m_bInSection;
 	UT_Bool				m_bInBlock;
 	UT_Bool				m_bInSpan;
+        UT_Bool m_bWasSpace;
 	const PP_AttrProp*	m_pAP_Span;
 
 	// Need to look up proper type, and place to stick #defines...
 
 	UT_uint16		m_iBlockType;	// BT_*
-
+	UT_Wctomb		m_wmctomb;
 };
 
 void s_HTML_Listener::_closeSection(void)
@@ -204,7 +207,7 @@ void s_HTML_Listener::_openParagraph(PT_AttrPropIndex api)
 		const XML_Char * szValue;
 
                 if (
-			(pAP->getAttribute("style", szValue))
+			(pAP->getAttribute((XML_Char*)"style", szValue))
 			)
 		{
 
@@ -274,14 +277,14 @@ void s_HTML_Listener::_openParagraph(PT_AttrPropIndex api)
 		  wasWritten = UT_TRUE;
 		}
 
-		/* Assumption: never get property set with h1-h3, block text, plain text. Probably true. */
+		/* Assumption: never get property set with block text, plain text. Probably true. */
 
 		if (
-			m_iBlockType == BT_NORMAL && (pAP->getProperty("text-align", szValue))
+			m_iBlockType != BT_PLAINTEXT && m_iBlockType != BT_BLOCKTEXT && (pAP->getProperty((XML_Char*)"text-align", szValue))
 			)
 		{
 			m_pie->write(" align=\"");
-			m_pie->write(szValue);
+			m_pie->write((char*)szValue);
 			m_pie->write("\"");
 		}
 	}
@@ -386,7 +389,7 @@ void s_HTML_Listener::_openSpan(PT_AttrPropIndex api)
 		const XML_Char * szValue;
 
 		if (
-			(pAP->getProperty("font-weight", szValue))
+			(pAP->getProperty((XML_Char*)"font-weight", szValue))
 			&& !UT_stricmp(szValue, "bold")
 			)
 		{
@@ -402,13 +405,13 @@ void s_HTML_Listener::_openSpan(PT_AttrPropIndex api)
 		}
 		
 		if (
-			(pAP->getProperty("font-style", szValue))
+			(pAP->getProperty((XML_Char*)"font-style", szValue))
 			&& !UT_stricmp(szValue, "italic")
 			)
 		{
 		  if (!span)
 			  {
-			    m_pie->write("<span style=\"font-style: italic\"");	
+			    m_pie->write("<span style=\"font-style: italic");	
 			    span = UT_TRUE;
 			  }
 			else
@@ -419,7 +422,7 @@ void s_HTML_Listener::_openSpan(PT_AttrPropIndex api)
 
 		
 		if (
-			(pAP->getProperty("text-decoration", szValue))
+			(pAP->getProperty((XML_Char*)"text-decoration", szValue))
 			)
 		{
 			const XML_Char* pszDecor = szValue;
@@ -461,7 +464,7 @@ void s_HTML_Listener::_openSpan(PT_AttrPropIndex api)
 		}
 
 		if (
-			(pAP->getProperty("text-decoration", szValue))
+			(pAP->getProperty((XML_Char*)"text-decoration", szValue))
 			)
 		{
 			const XML_Char* pszDecor = szValue;
@@ -503,7 +506,7 @@ void s_HTML_Listener::_openSpan(PT_AttrPropIndex api)
 		}
 
 		if (
-			(pAP->getProperty("text-decoration", szValue))
+			(pAP->getProperty((XML_Char*)"text-decoration", szValue))
 			)
 		{
 			const XML_Char* pszDecor = szValue;
@@ -544,7 +547,7 @@ void s_HTML_Listener::_openSpan(PT_AttrPropIndex api)
 			free(p);
 		}
 
-		if (pAP->getProperty("text-position", szValue))
+		if (pAP->getProperty((XML_Char*)"text-position", szValue))
 		{
 			if (!UT_stricmp("superscript", szValue))
 			{
@@ -557,18 +560,18 @@ void s_HTML_Listener::_openSpan(PT_AttrPropIndex api)
 		}
 		
 		if (
-			(pAP->getProperty("color", szValue))
-		    || (pAP->getProperty("font-size", szValue))
-		    || (pAP->getProperty("font-family", szValue))
+			(pAP->getProperty((XML_Char*)"color", szValue))
+		    || (pAP->getProperty((XML_Char*)"font-size", szValue))
+		    || (pAP->getProperty((XML_Char*)"font-family", szValue))
 			)
 		{
 			const XML_Char* pszColor = NULL;
 			const XML_Char* pszFontSize = NULL;
 			const XML_Char* pszFontFamily = NULL;
 
-			pAP->getProperty("color", pszColor);
-		    pAP->getProperty("font-size", pszFontSize);
-		    pAP->getProperty("font-family", pszFontFamily);
+			pAP->getProperty((XML_Char*)"color", pszColor);
+		    pAP->getProperty((XML_Char*)"font-size", pszFontSize);
+		    pAP->getProperty((XML_Char*)"font-family", pszFontFamily);
 
 			if (pszColor)
 			{
@@ -576,7 +579,7 @@ void s_HTML_Listener::_openSpan(PT_AttrPropIndex api)
 				    {
 					m_pie->write("<span style=\"color: ");	
 					char szColor[16];
-					_convertColor(szColor, pszColor);
+					_convertColor(szColor,(char*)pszColor);
 					m_pie->write(szColor);
 					m_pie->write(";");
 					span = UT_TRUE;
@@ -585,7 +588,7 @@ void s_HTML_Listener::_openSpan(PT_AttrPropIndex api)
 				    {
 					m_pie->write(" color: ");	
 					char szColor[16];
-					_convertColor(szColor, pszColor);
+					_convertColor(szColor,(char*)pszColor);
 					m_pie->write(szColor);
 					m_pie->write(";");
 				    }
@@ -596,14 +599,14 @@ void s_HTML_Listener::_openSpan(PT_AttrPropIndex api)
 				  if (!span)
 				    {
 					m_pie->write("<span style=\"font-family: ");	
-					m_pie->write(pszFontFamily);
+					m_pie->write((char*)pszFontFamily);
 					m_pie->write(";");
 					span = UT_TRUE;
 				    }
 				  else 
 				    {
 					m_pie->write(" font-family: ");	
-					m_pie->write(pszFontFamily);
+					m_pie->write((char*)pszFontFamily);
 					m_pie->write(";");
 				    }
 
@@ -655,15 +658,15 @@ void s_HTML_Listener::_closeSpan(void)
 		const XML_Char * szValue;
 		
 		if (
-			(pAP->getProperty("color", szValue))
-		    || (pAP->getProperty("font-size", szValue))
-		    || (pAP->getProperty("font-family", szValue))
+			(pAP->getProperty((XML_Char*)"color", szValue))
+		    || (pAP->getProperty((XML_Char*)"font-size", szValue))
+		    || (pAP->getProperty((XML_Char*)"font-family", szValue))
 			)
 		{
 		  closeSpan = UT_TRUE;
 		}
 
-		if (pAP->getProperty("text-position", szValue))
+		if (pAP->getProperty((XML_Char*)"text-position", szValue))
 		{
 			if (!UT_stricmp("superscript", szValue))
 			{
@@ -678,7 +681,7 @@ void s_HTML_Listener::_closeSpan(void)
 
 
 		if (
-			(pAP->getProperty("text-decoration", szValue))
+			(pAP->getProperty((XML_Char*)"text-decoration", szValue))
 			&& UT_stricmp(szValue, "none")
 			)
 		{
@@ -686,7 +689,7 @@ void s_HTML_Listener::_closeSpan(void)
 		}
 
 		if (
-			(pAP->getProperty("font-style", szValue))
+			(pAP->getProperty((XML_Char*)"font-style", szValue))
 			&& !UT_stricmp(szValue, "italic")
 			)
 		{
@@ -694,7 +697,7 @@ void s_HTML_Listener::_closeSpan(void)
 		}
 		
 		if (
-			(pAP->getProperty("font-weight", szValue))
+			(pAP->getProperty((XML_Char*)"font-weight", szValue))
 			&& !UT_stricmp(szValue, "bold")
 			)
 		{
@@ -728,6 +731,8 @@ void s_HTML_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 	char buf[MY_BUFFER_SIZE];
 	char * pBuf;
 	const UT_UCSChar * pData;
+	char mbbuf[30];
+	int mblen;
 
 	for (pBuf=buf, pData=data; (pData<data+length); /**/)
 	{
@@ -736,8 +741,6 @@ void s_HTML_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 			m_pie->write(buf,(pBuf-buf));
 			pBuf = buf;
 		}
-
-		UT_ASSERT(*pData < 256);
 		switch (*pData)
 		{
 		case '<':
@@ -775,22 +778,41 @@ void s_HTML_Listener::_outputData(const UT_UCSChar * data, UT_uint32 length)
 			pData++;
 			break;
 			
+		case ' ':
+		case '\t':
+		  // try to honor multiple spaces
+		  // tabs get treated as a single space
+		  //
+		  if(m_bWasSpace)
+		    {
+		      *pBuf++ = '&';
+		      *pBuf++ = 'n';
+		      *pBuf++ = 'b';
+		      *pBuf++ = 's';
+		      *pBuf++ = 'p';
+		      *pBuf++ = ';';
+		      pData++;
+		    }
+		  else
+		    {
+		      // just tack on a single space to the textrun
+		      m_bWasSpace = UT_TRUE;
+		      *pBuf++ = ' ';
+		      pData++;
+		    }
+		  break;
+			
 		default:
-			if (*pData > 0x007f)
-			{
-				// convert non us-ascii into numeric entities.
-				// we could convert them into UTF-8 multi-byte
-				// sequences, but i prefer these.
+			if (m_wmctomb.wctomb(mbbuf,mblen,(wchar_t)*pData++)) {
+				for(int i=0;i<mblen;++i)
+					*pBuf++ = mbbuf[i];
+			} else {
 				char localBuf[20];
 				char * plocal = localBuf;
-				sprintf(localBuf,"&#%d;",*pData++);
+				sprintf(localBuf,"&#%d;",*pData);
 				while (*plocal)
 					*pBuf++ = (UT_Byte)*plocal++;
-			}
-			else
-			{
-				*pBuf++ = (UT_Byte)*pData++;
-			}
+			};
 			break;
 		}
 	}
@@ -807,6 +829,7 @@ s_HTML_Listener::s_HTML_Listener(PD_Document * pDocument,
 	m_bInSection = UT_FALSE;
 	m_bInBlock = UT_FALSE;
 	m_bInSpan = UT_FALSE;
+	m_bWasSpace = UT_FALSE;
 	
 	m_pie->write("<!-- ================================================================================  -->\n");
 	m_pie->write("<!-- This HTML file was created by AbiWord.                                            -->\n");
@@ -857,6 +880,9 @@ s_HTML_Listener::s_HTML_Listener(PD_Document * pDocument,
 	m_pie->write("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml/DTD/xhtml1-strict.dtd\">\n");
 	m_pie->write("<html>\n");
 	m_pie->write("<head>\n");
+	m_pie->write("<meta http-equiv=\"content-type\" content=\"text/html;charset=");
+	m_pie->write(XAP_EncodingManager::instance->getNativeEncodingName());
+	m_pie->write("\"></meta>\n");//</meta> looks ugly, but it should work.
 	m_pie->write("<title>AbiWord Document</title>\n");
 	m_pie->write("<style type=\"text/css\">\n");
 	m_pie->write("<!-- \n P.norm { margin-top: 0pt; margin-bottom: 0pt } \n -->\n");
@@ -954,7 +980,7 @@ UT_Bool s_HTML_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 		if (m_pDocument->getAttrProp(indexAP, &pAP) && pAP)
 		{
 			const XML_Char* pszSectionType = NULL;
-			pAP->getAttribute("type", pszSectionType);
+			pAP->getAttribute((XML_Char*)"type", pszSectionType);
 			if (
 				!pszSectionType
 				|| (0 == UT_stricmp(pszSectionType, "doc"))
