@@ -40,6 +40,7 @@
 #include "px_CR_Span.h"
 #include "px_CR_Strux.h"
 
+#include "xap_EncodingManager.h"
 void s_RTF_ListenerWriteDoc::_closeSection(void)
 {
 	m_apiThisSection = 0;
@@ -82,7 +83,7 @@ void s_RTF_ListenerWriteDoc::_openSpan(PT_AttrPropIndex apiSpan)
 	m_pDocument->getAttrProp(apiSpan,&pSpanAP);
 
 	const XML_Char * szColor = PP_evalProperty("color",pSpanAP,pBlockAP,pSectionAP,m_pDocument,UT_TRUE);
-	UT_sint32 ndxColor = m_pie->_findColor(szColor);
+	UT_sint32 ndxColor = m_pie->_findColor((char*)szColor);
 	UT_ASSERT(ndxColor != -1);
 	if (ndxColor != 0)
 		m_pie->_rtf_keyword("cf",ndxColor);
@@ -143,6 +144,8 @@ void s_RTF_ListenerWriteDoc::_outputData(const UT_UCSChar * data, UT_uint32 leng
 	char buf[MY_BUFFER_SIZE];
 	char * pBuf;
 	const UT_UCSChar * pData;
+	char mbbuf[30];
+	int mblen;
 
 	for (pBuf=buf, pData=data; (pData<data+length); /**/)
 	{
@@ -189,41 +192,121 @@ void s_RTF_ListenerWriteDoc::_outputData(const UT_UCSChar * data, UT_uint32 leng
 			break;
 
 		default:
-			if (*pData > 0x00ff)		// emit unicode character
+			if (XAP_EncodingManager::instance->cjk_locale())
 			{
-				FlushBuffer();
+				/*FIXME: can it happen that wctomb will fail under CJK locales? */
+				m_wctomb.wctomb_or_fallback(mbbuf,mblen,*pData++);
+				if (mbbuf[0] & 0x80)
+				{
+					FlushBuffer();
+					for(int i=0;i<mblen;++i) {
+						unsigned char c = mbbuf[i];
+						m_pie->_rtf_nonascii_hex2(c);
+					}
+				}
+				else
+				{
+					for(int i=0;i<mblen;++i) {
+						switch (mbbuf[i]) 
+						{
+							case '\\':
+							case '{':
+							case '}':
+								*pBuf++ = '\\';
+						}
+						*pBuf++ = mbbuf[i];
+					}
+				}
+			} else if (!m_pie->m_atticFormat) 
+			{
+				if (*pData > 0x00ff)		// emit unicode character
+				{
+					FlushBuffer();
 
-				// RTF spec says that we should emit an ASCII-equivalent
-				// character for each unicode character, so that dumb/older
-				// readers don't lose a char.  i don't have a good algorithm
-				// for deciding how to do this, so i'm not going to put out
-				// any chars.  so i'm setting \uc0 before emitting \u<u>.
-				// TODO decide if we should be smarter here and do a \uc1\u<u><A> ??
-				// TODO if so, we may need to begin a sub-brace level to avoid
-				// TODO polluting the global context w/r/t \uc.
-				
-				m_pie->_rtf_keyword("uc",0);
-				unsigned short ui = ((unsigned short)(*pData));	// RTF is limited to +/-32K ints
-				signed short si = *((signed short *)(&ui));		// so we need to write negative
-				m_pie->_rtf_keyword("u",si);					// numbers for large unicode values.
-				pData++;
-			}
-			else if (*pData > 0x007f)
-			{
-				FlushBuffer();
+					// RTF spec says that we should emit an ASCII-equivalent
+					// character for each unicode character, so that dumb/older
+					// readers don't lose a char.  i don't have a good algorithm
+					// for deciding how to do this, so i'm not going to put out
+					// any chars.  so i'm setting \uc0 before emitting \u<u>.
+					// TODO decide if we should be smarter here and do a \uc1\u<u><A> ??
+					// TODO if so, we may need to begin a sub-brace level to avoid
+					// TODO polluting the global context w/r/t \uc.
+					
+					UT_UCSChar lc = XAP_EncodingManager::instance->try_UToWindows(*pData);
+					m_pie->_rtf_keyword("uc",lc && lc<256 ? 1 : 0);
+					unsigned short ui = ((unsigned short)(*pData));	// RTF is limited to +/-32K ints
+					signed short si = *((signed short *)(&ui));		// so we need to write negative
+					m_pie->_rtf_keyword("u",si);					// numbers for large unicode values.
+					if (lc && lc <256)
+						m_pie->_rtf_nonascii_hex2(lc);
+					pData++;
+				}
+				else if (*pData > 0x007f)
+				{
+					FlushBuffer();
 
-				// for chars between 7f and ff, we could just send them
-				// out as is, or we could send them out in hex or as a
-				// unicode sequence.  when i originally did this, i chose
-				// hex, so i'm not going to change it now.
+					// for chars between 7f and ff, we could just send them
+					// out as is, or we could send them out in hex or as a
+					// unicode sequence.  when i originally did this, i chose
+					// hex, so i'm not going to change it now.
 				
-				m_pie->_rtf_nonascii_hex2(*pData);
-				pData++;
-			}
-			else
-			{
-				*pBuf++ = (UT_Byte)*pData++;
-			}
+					m_pie->_rtf_nonascii_hex2(*pData);
+					pData++;
+				}
+				else
+				{
+					*pBuf++ = (UT_Byte)*pData++;
+				}
+			} else {
+				/* 
+				    wordpad (and probably word6/7 don't understand
+				    \uc0\u<UUUU> format at all.
+				*/
+				UT_UCSChar c = *pData++;
+				UT_UCSChar lc = XAP_EncodingManager::instance->try_UToWindows(c);
+				if (lc==0 || lc >255) 
+				{
+					/*
+					    can't be represented in windows encoding. 
+					    So emit unicode (though attic apps won't understand it.
+					    This branch is shamelessly copied from
+					    branch if (*pData > 0x00ff) above.
+					*/
+					FlushBuffer();
+
+					// RTF spec says that we should emit an ASCII-equivalent
+					// character for each unicode character, so that dumb/older
+					// readers don't lose a char.  i don't have a good algorithm
+					// for deciding how to do this, so i'm not going to put out
+					// any chars.  so i'm setting \uc0 before emitting \u<u>.
+					// TODO decide if we should be smarter here and do a \uc1\u<u><A> ??
+					// TODO if so, we may need to begin a sub-brace level to avoid
+					// TODO polluting the global context w/r/t \uc.
+					
+					m_pie->_rtf_keyword("uc",0);
+					unsigned short ui = ((unsigned short)(*pData));	// RTF is limited to +/-32K ints
+					signed short si = *((signed short *)(&ui));		// so we need to write negative
+					m_pie->_rtf_keyword("u",si);					// numbers for large unicode values.
+				}
+				else 
+				{
+					if (lc > 0x007f) 
+					{
+						FlushBuffer();
+
+						// for chars between 7f and ff, we could just send them
+						// out as is, or we could send them out in hex or as a
+						// unicode sequence.  when i originally did this, i chose
+						// hex, so i'm not going to change it now.
+				
+						m_pie->_rtf_nonascii_hex2(lc);
+					}
+					else
+					{
+						*pBuf++ = (UT_Byte)lc;
+					}
+				}
+			};
 			break;
 		}
 	}
@@ -256,7 +339,8 @@ s_RTF_ListenerWriteDoc::s_RTF_ListenerWriteDoc(PD_Document * pDocument,
 	// and block.  when going to a file we should not.
 	m_bJustStartingDoc = !m_bToClipboard;
 	m_bJustStartingSection = !m_bToClipboard;
-	
+
+	m_wctomb.setOutCharset(XAP_EncodingManager::instance->WindowsCharsetName());
 	// TODO emit <info> if desired
 
 	_rtf_docfmt();						// deal with <docfmt>
@@ -292,7 +376,7 @@ UT_Bool s_RTF_ListenerWriteDoc::populate(PL_StruxFmtHandle /*sfh*/,
 	case PX_ChangeRecord::PXT_InsertObject:
 		{
 			const PX_ChangeRecord_Object * pcro = static_cast<const PX_ChangeRecord_Object *> (pcr);
-			//PT_AttrPropIndex api = pcr->getIndexAP();
+			PT_AttrPropIndex api = pcr->getIndexAP();
 		       switch (pcro->getObjectType())
 			{
 			case PTO_Image:
@@ -300,7 +384,7 @@ UT_Bool s_RTF_ListenerWriteDoc::populate(PL_StruxFmtHandle /*sfh*/,
 				_writeImageInRTF(pcro);
 				return UT_TRUE;
 
-#if 0
+				//#if 0
 			// TODO deal with these other inline objects....
 			
 			case PTO_Field:
@@ -308,9 +392,8 @@ UT_Bool s_RTF_ListenerWriteDoc::populate(PL_StruxFmtHandle /*sfh*/,
 				_openTag("field","/",UT_FALSE,api);
 				return UT_TRUE;
 
+				//#endif
 			default:
-				UT_ASSERT(0);
-#endif
 				return UT_FALSE;
 			}
 		}
@@ -324,6 +407,13 @@ UT_Bool s_RTF_ListenerWriteDoc::populate(PL_StruxFmtHandle /*sfh*/,
 	}
 }
 
+
+void	 s_RTF_ListenerWriteDoc::_openTag(const char * szPrefix, const char * szSuffix,
+								 UT_Bool bNewLineAfter, PT_AttrPropIndex api)
+{
+  UT_DEBUGMSG(("TODO: Write code to go in here. In _openTag, szPrefix = %s  api = %x \n",szPrefix,api));
+}
+
 UT_Bool s_RTF_ListenerWriteDoc::populateStrux(PL_StruxDocHandle /*sdh*/,
 										   const PX_ChangeRecord * pcr,
 										   PL_StruxFmtHandle * psfh)
@@ -404,7 +494,7 @@ void s_RTF_ListenerWriteDoc::_rtf_docfmt(void)
 	const XML_Char * szDefaultTabs = PP_evalProperty("default-tab-interval",
 													 pSpanAP,pBlockAP,pSectionAP,
 													 m_pDocument,UT_TRUE);
-	m_pie->_rtf_keyword_ifnotdefault_twips("deftab",szDefaultTabs,720);
+	m_pie->_rtf_keyword_ifnotdefault_twips("deftab",(char*)szDefaultTabs,720);
 
 	// <docfmt> -- document views and zoom level
 	
@@ -415,26 +505,26 @@ void s_RTF_ListenerWriteDoc::_rtf_docfmt(void)
 	// <docfmt> -- page information
 
 	const XML_Char * szPaperWidth = "8.5in"; // TODO look this up in the document
-	m_pie->_rtf_keyword_ifnotdefault_twips("paperw",szPaperWidth,0);
+	m_pie->_rtf_keyword_ifnotdefault_twips("paperw",(char*)szPaperWidth,0);
 	const XML_Char * szPaperHeight = "11in"; // TODO look this up in the document
-	m_pie->_rtf_keyword_ifnotdefault_twips("paperh",szPaperHeight,0);
+	m_pie->_rtf_keyword_ifnotdefault_twips("paperh",(char*)szPaperHeight,0);
 	
 	const XML_Char * szLeftMargin = PP_evalProperty("page-margin-left",
 													 pSpanAP,pBlockAP,pSectionAP,
 													 m_pDocument,UT_TRUE);
-	m_pie->_rtf_keyword_ifnotdefault_twips("margl",szLeftMargin,1800);
+	m_pie->_rtf_keyword_ifnotdefault_twips("margl",(char*)szLeftMargin,1800);
 	const XML_Char * szRightMargin = PP_evalProperty("page-margin-right",
 													 pSpanAP,pBlockAP,pSectionAP,
 													 m_pDocument,UT_TRUE);
-	m_pie->_rtf_keyword_ifnotdefault_twips("margr",szRightMargin,1800);
+	m_pie->_rtf_keyword_ifnotdefault_twips("margr",(char*)szRightMargin,1800);
 	const XML_Char * szTopMargin = PP_evalProperty("page-margin-top",
 													 pSpanAP,pBlockAP,pSectionAP,
 													 m_pDocument,UT_TRUE);
-	m_pie->_rtf_keyword_ifnotdefault_twips("margt",szTopMargin,1440);
+	m_pie->_rtf_keyword_ifnotdefault_twips("margt",(char*)szTopMargin,1440);
 	const XML_Char * szBottomMargin = PP_evalProperty("page-margin-Bottom",
 													 pSpanAP,pBlockAP,pSectionAP,
 													 m_pDocument,UT_TRUE);
-	m_pie->_rtf_keyword_ifnotdefault_twips("margb",szBottomMargin,1440);
+	m_pie->_rtf_keyword_ifnotdefault_twips("margb",(char*)szBottomMargin,1440);
 	m_pie->_rtf_keyword("widowctl");	// enable widow and orphan control
 	
 	// TODO <docfmt> -- linked styles
@@ -477,8 +567,8 @@ void s_RTF_ListenerWriteDoc::_rtf_open_section(PT_AttrPropIndex api)
 
 	m_pie->_rtf_keyword("sectd");								// restore all defaults for this section
 	m_pie->_rtf_keyword("sbknone");								// no page break implied
-	m_pie->_rtf_keyword_ifnotdefault("cols",szColumns,1);
-	m_pie->_rtf_keyword_ifnotdefault_twips("colsx",szColumnGap,720);
+	m_pie->_rtf_keyword_ifnotdefault("cols",(char*)szColumns,1);
+	m_pie->_rtf_keyword_ifnotdefault_twips("colsx",(char*)szColumnGap,720);
 
 }
 
@@ -551,11 +641,11 @@ void s_RTF_ListenerWriteDoc::_rtf_open_block(PT_AttrPropIndex api)
 	else if (UT_stricmp(szTextAlign,"justify")==0)
 		m_pie->_rtf_keyword("qj");
 
-	m_pie->_rtf_keyword_ifnotdefault_twips("fi",szFirstLineIndent,0);
-	m_pie->_rtf_keyword_ifnotdefault_twips("li",szLeftIndent,0);
-	m_pie->_rtf_keyword_ifnotdefault_twips("ri",szRightIndent,0);
-	m_pie->_rtf_keyword_ifnotdefault_twips("sb",szTopMargin,0);
-	m_pie->_rtf_keyword_ifnotdefault_twips("sa",szBottomMargin,0);
+	m_pie->_rtf_keyword_ifnotdefault_twips("fi",(char*)szFirstLineIndent,0);
+	m_pie->_rtf_keyword_ifnotdefault_twips("li",(char*)szLeftIndent,0);
+	m_pie->_rtf_keyword_ifnotdefault_twips("ri",(char*)szRightIndent,0);
+	m_pie->_rtf_keyword_ifnotdefault_twips("sb",(char*)szTopMargin,0);
+	m_pie->_rtf_keyword_ifnotdefault_twips("sa",(char*)szBottomMargin,0);
 
 	if (strcmp(szLineHeight,"1.0") != 0)
 	{
@@ -669,7 +759,7 @@ void s_RTF_ListenerWriteDoc::_writeImageInRTF(const PX_ChangeRecord_Object * pcr
 	// fetch the "name" of the image and use it to fetch the actual image data.
 	
 	const XML_Char * szDataID = NULL;
-	UT_Bool bFoundDataID = pImageAP->getAttribute("dataid",szDataID);
+	UT_Bool bFoundDataID = pImageAP->getAttribute((XML_Char*)"dataid",szDataID);
 	if (!bFoundDataID)
 	{
 		UT_DEBUGMSG(("RTF_Export: cannot get dataid for image\n"));
@@ -678,7 +768,7 @@ void s_RTF_ListenerWriteDoc::_writeImageInRTF(const PX_ChangeRecord_Object * pcr
 	const UT_ByteBuf * pbb = NULL;
 	void * pToken = NULL;
 	void * pHandle = NULL;
-	UT_Bool bFoundDataItem = m_pDocument->getDataItemDataByName(szDataID,&pbb,&pToken,&pHandle);
+	UT_Bool bFoundDataItem = m_pDocument->getDataItemDataByName((char*)szDataID,&pbb,&pToken,&pHandle);
 	if (!bFoundDataItem)
 	{
 		UT_DEBUGMSG(("RTF_Export: cannot get dataitem for image\n"));
@@ -690,8 +780,8 @@ void s_RTF_ListenerWriteDoc::_writeImageInRTF(const PX_ChangeRecord_Object * pcr
 	
 	const XML_Char * szWidthProp = NULL;
 	const XML_Char * szHeightProp = NULL;
-	UT_Bool bFoundWidthProperty = pImageAP->getProperty("width",szWidthProp);
-	UT_Bool bFoundHeightProperty = pImageAP->getProperty("height",szHeightProp);
+	UT_Bool bFoundWidthProperty = pImageAP->getProperty((XML_Char*)"width",szWidthProp);
+	UT_Bool bFoundHeightProperty = pImageAP->getProperty((XML_Char*)"height",szHeightProp);
 
 	// get the width/height of the image from the image itself.
 
@@ -729,9 +819,9 @@ void s_RTF_ListenerWriteDoc::_writeImageInRTF(const PX_ChangeRecord_Object * pcr
 			m_pie->_rtf_keyword("picw",iImageWidth);
 			m_pie->_rtf_keyword("pich",iImageHeight);
 			if (bFoundWidthProperty)
-				m_pie->_rtf_keyword_ifnotdefault_twips("picwgoal",szWidthProp,0);
+				m_pie->_rtf_keyword_ifnotdefault_twips("picwgoal",(char*)szWidthProp,0);
 			if (bFoundHeightProperty)
-				m_pie->_rtf_keyword_ifnotdefault_twips("pichgoal",szHeightProp,0);
+				m_pie->_rtf_keyword_ifnotdefault_twips("pichgoal",(char*)szHeightProp,0);
 			// we use the default values for picscale[xy]==100, piccrop[tblr]==0
 			
 			// TODO deal with <metafileinfo>
