@@ -32,7 +32,10 @@
 #include "ie_imp_RTF.h"
 #include "ie_imp_Text.h"
 #include "ie_imp_UTF8.h"
-
+#include "ie_imp_WML.h"
+#include "ie_imp_GraphicAsDocument.h"
+#include "ie_imp_XHTML.h"
+#include "ie_imp_DocBook.h"
 /*****************************************************************/
 /*****************************************************************/
 
@@ -54,11 +57,14 @@ struct _imp
 static struct _imp s_impTable[] =
 {
 	DeclareImporter(IE_Imp_AbiWord_1),
-      DeclareImporter(IE_Imp_GZipAbiWord),
-	DeclareImporter(IE_Imp_RTF),
+	DeclareImporter(IE_Imp_DocBook),
 	DeclareImporter(IE_Imp_MsWord_97),
-	DeclareImporter(IE_Imp_UTF8),
+	DeclareImporter(IE_Imp_XHTML),
+	DeclareImporter(IE_Imp_RTF),
 	DeclareImporter(IE_Imp_Text),
+	DeclareImporter(IE_Imp_UTF8),
+	DeclareImporter(IE_Imp_WML),
+	DeclareImporter(IE_Imp_GZipAbiWord)
 };
 
 		
@@ -137,9 +143,10 @@ IEFileType IE_Imp::fileTypeForSuffix(const char * szSuffix)
 }
 
 UT_Error IE_Imp::constructImporter(PD_Document * pDocument,
-								   const char * szFilename,
-								   IEFileType ieft,
-								   IE_Imp ** ppie)
+				   const char * szFilename,
+				   IEFileType ieft,
+				   IE_Imp ** ppie,
+				   IEFileType * pieft)
 {
 	// construct an importer of the right type.
 	// caller is responsible for deleting the importer object
@@ -169,14 +176,39 @@ UT_Error IE_Imp::constructImporter(PD_Document * pDocument,
 			ieft = IE_Imp::fileTypeForContents(szBuf, iNumbytes);
 		}
 	}
-	// as a last resort, just try importing it as text  :(
 	if (ieft == IEFT_Unknown)
 	{
-		ieft = IEFT_Text ;
+	   	// maybe they're trying to open an image directly?
+	   	IE_ImpGraphic *pIEG;
+ 		UT_Error errorCode = IE_ImpGraphic::constructImporter(szFilename, IEGFT_Unknown, &pIEG);
+		if (!errorCode && pIEG) 
+ 		{
+			// tell the caller the type of importer they got
+		   	if (pieft != NULL) *pieft = IEFT_Unknown; // to force a save-as
+
+		   	// create the importer 
+			*ppie = new IE_Imp_GraphicAsDocument(pDocument);
+		   	if (*ppie) {
+			   	// tell the importer where to get the graphic
+			   	((IE_Imp_GraphicAsDocument*)(*ppie))->setGraphicImporter(pIEG);
+			   	return UT_OK;
+			} else {
+			   	delete pIEG;
+				return UT_IE_NOMEMORY;
+			}
+		}
+		else
+ 		{
+	   		// as a last resort, just try importing it as text  :(
+			ieft = IEFT_Text ;
+		}
 	}
 
 	UT_ASSERT(ieft != IEFT_Unknown);
 
+	// tell the caller the type of importer they got
+	if (pieft != NULL) *pieft = ieft;
+
 	// use the importer for the specified file type
 	for (UT_uint32 k=0; (k < NrElements(s_impTable)); k++)
 	{
