@@ -30,9 +30,14 @@
 #include "ut_units.h"
 #include "pd_Document.h"
 
+#include "xap_EncodingManager.h"
+
 #include "ie_types.h"
 #include "ie_imp_MsWord_97.h"
 
+#include "ie_impGraphic.h"
+
+
 #define X_ReturnIfFail(exp,error)     do { UT_Bool b = (exp); if (!b) return (error); } while (0)
 #define X_ReturnNoMemIfError(exp)   X_ReturnIfFail(exp,UT_IE_NOMEMORY)
 
@@ -43,11 +48,12 @@
                                       {  m_error = UT_ERROR;            \
                                          return 0; } } while (0)
 
-
-int CharProc(wvParseStruct *ps,U16 eachchar,U8 chartype, U16 lid);
-int SpecCharProc(wvParseStruct *ps,U16 eachchar, CHP* achp);
-int ElementProc(wvParseStruct *ps,wvTag tag, void *props, int dirty);
-int DocProc(wvParseStruct *ps,wvTag tag);
+extern "C" {
+  int CharProc(wvParseStruct *ps,U16 eachchar,U8 chartype, U16 lid);
+  int SpecCharProc(wvParseStruct *ps,U16 eachchar, CHP* achp);
+  int ElementProc(wvParseStruct *ps,wvTag tag, void *props, int dirty);
+  int DocProc(wvParseStruct *ps,wvTag tag);
+}
 
 // a little look-up table for mapping Word text colors 
 // (the comments) to Abiword's superior RGB color encoding
@@ -70,13 +76,45 @@ static int word_colors[][3] = {
    {0xc0, 0xc0, 0xc0}, /* light gray */
 };
 
+/*****************************************************************/
+
+/*
+ * This next bit of code is so we can hopefully import 
+ * At least some of MSWord's fields
+ */
+
+static TokenTable s_Tokens[] =
+{
+	{"TIME",      FC_TIME},
+	{"\\@",       FC_DateTimePicture},
+	{"HYPERLINK", FC_HYPERLINK},
+	{"TOC",       FC_TOC},
+	{"\\o",       FC_TOC_FROM_RANGE},
+	{"PAGEREF",   FC_PAGEREF},
+	{"EMBED",     FC_EMBED},
+	{"EDITTIME",  FC_EDITTIME},
+	{ "*",        FC_OTHER}
+};
+
+static unsigned int s_mapNameToToken(const char* name)
+{
+  unsigned int k;
+  for (k=0; k<FieldCodeTableSize; k++)
+    {
+      if (s_Tokens[k].m_name[0] == '*')
+	return k;
+      else if (!(UT_stricmp(s_Tokens[k].m_name,name)))
+	return k;
+    }
+    return 0;
+}
 
 /*****************************************************************/
 
 UT_Error IE_Imp_MsWord_97::importFile(const char * szFilename)
 {
 	FILE *fp = NULL;
-	UT_DEBUGMSG(("got to import file\n"));
+	xxx_UT_DEBUGMSG(("got to import file\n"));
 
 	fp = fopen(szFilename, "rb");
 	if (!fp)
@@ -85,7 +123,7 @@ UT_Error IE_Imp_MsWord_97::importFile(const char * szFilename)
 	   m_error = UT_IE_FILENOTFOUND;
 	   return m_error;
 	}
-	UT_DEBUGMSG(("wv importer\n"));
+	xxx_UT_DEBUGMSG(("wv importer\n"));
 	fclose(fp);
 
 	wvParseStruct ps;
@@ -97,7 +135,7 @@ UT_Error IE_Imp_MsWord_97::importFile(const char * szFilename)
 		return m_error;
 		}
 	
-	UT_DEBUGMSG(("just here\n"));
+	xxx_UT_DEBUGMSG(("just here\n"));
 	ps.userData = this;
 	wvSetElementHandler(ElementProc);
 	wvSetCharHandler(CharProc);
@@ -119,8 +157,9 @@ int CharProc(wvParseStruct *ps,U16 eachchar,U8 chartype,U16 lid)
 	   // convert incoming character to unicode
 	   if (chartype)
 	     eachchar = wvHandleCodePage(eachchar, lid);
-
-	   UT_DEBUGMSG(("word 97 char is %c (%d), type is %d\n",eachchar,(int)eachchar,chartype));
+	     //eachchar = XAP_EncodingManager::instance->UToNative(eachchar);
+	     
+	   xxx_UT_DEBUGMSG(("word 97 char is %c (%d), type is %d\n",eachchar,(int)eachchar,chartype));
 
 	   // take care of any oddities in Microsoft's character "encoding"
 	   // TODO: does the above code page handler take care of these?
@@ -132,7 +171,7 @@ int CharProc(wvParseStruct *ps,U16 eachchar,U8 chartype,U16 lid)
 	      case 13: // paragraph end
 		return 0;
 	      case 11: // hard line break
-		UT_DEBUGMSG(("a line break\n"));
+		xxx_UT_DEBUGMSG(("a line break\n"));
 		eachchar = UCS_LF;
 		break;
 	      case 12: // page breaks, section marks
@@ -144,37 +183,43 @@ int CharProc(wvParseStruct *ps,U16 eachchar,U8 chartype,U16 lid)
 		// the last character in the text buffer. since we just 
 		// flushed it above, there's no chance of auto-flushing
 		// before the end-of-section hits.
-		UT_DEBUGMSG(("a page break/section mark\n"));
+		xxx_UT_DEBUGMSG(("a page break/section mark\n"));
 		eachchar = UCS_FF;
 		break;
 	      case 14: // column break
-		UT_DEBUGMSG(("a column break\n"));
+		xxx_UT_DEBUGMSG(("a column break\n"));
 		eachchar = UCS_VTAB;
 		break;
 	      case 19: // field begin
 		// flush current text buffer
 		pDocReader->_charData(pDocReader->m_pTextRun, pDocReader->m_iTextRunLength);
 		pDocReader->m_iTextRunLength = 0;
-		UT_DEBUGMSG(("a field is beginning\n"));
+		xxx_UT_DEBUGMSG(("a field is beginning\n"));
 		ps->fieldstate++;
 		ps->fieldmiddle = 0;
+		pDocReader->_fieldProc(ps, eachchar, chartype, lid);	/* temp */
 		return 0;
 	      case 20: // field separator
-		UT_DEBUGMSG(("a field separator\n"));
+		xxx_UT_DEBUGMSG(("a field separator\n"));
+		pDocReader->_fieldProc(ps, eachchar, chartype, lid);
 		ps->fieldmiddle = 1;
 		return 0;
 	      case 21: // field end
-		UT_DEBUGMSG(("a field has ended\n"));
+		xxx_UT_DEBUGMSG(("a field has ended\n"));
 		ps->fieldstate--;
 		ps->fieldmiddle = 0;
+		pDocReader->_fieldProc(ps, eachchar, chartype, lid);	/* temp */
 		return 0;
 	     }
 
-	   // HACK: it seems the text which is displayed by a field is contained
-	   // HACK: after the field separator. since I haven't written real field
-	   // HACK: import support, yet, this will fake it somewhat...
-	   if (ps->fieldstate && !ps->fieldmiddle) return 0;
-	   
+	   // TODO: it seems the text which is displayed by a field is contained
+	   // TODO: after the field separator. since I haven't written real field
+	   // TODO: import support, yet, this will fake it somewhat...
+	   if (ps->fieldstate) {
+	     if(pDocReader->_fieldProc(ps, eachchar, chartype, lid))
+	       return 0;
+	   }
+
 	   // add character to our current text run
 	   pDocReader->m_pTextRun[pDocReader->m_iTextRunLength++] = 
 	     (UT_UCSChar) eachchar;
@@ -213,37 +258,44 @@ int SpecCharProc(wvParseStruct *ps, U16 eachchar, CHP* achp)
 	// flush current text buffer
 	pDocReader->_charData(pDocReader->m_pTextRun, pDocReader->m_iTextRunLength);
 	pDocReader->m_iTextRunLength = 0;
-	UT_DEBUGMSG(("a field is beginning\n"));
+	xxx_UT_DEBUGMSG(("a field is beginning\n"));
 	ps->fieldstate++;
 	ps->fieldmiddle = 0;
+	pDocReader->_fieldProc(ps, eachchar, 0, 0x400); /* temp */
 	return 0;
       case 20: // field separator
-	UT_DEBUGMSG(("a field separator\n"));
+	if (achp->fOle2)
+	  {
+	    xxx_UT_DEBUGMSG(("field has associated embedded OLE object\n"));
+	  }
+	xxx_UT_DEBUGMSG(("a field separator\n"));
 	ps->fieldmiddle = 1;
+	pDocReader->_fieldProc(ps, eachchar, 0, 0x400); /* temp */
 	return 0;
       case 21: // field end
-	UT_DEBUGMSG(("a field has ended\n"));
+	xxx_UT_DEBUGMSG(("a field has ended\n"));
 	ps->fieldstate--;
 	ps->fieldmiddle = 0;
+	pDocReader->_fieldProc(ps, eachchar, 0, 0x400); /* temp */
 	return 0;
      }
-   
+
    /* it seems some fields characters slip through here which tricks
     * the import into thinking it has an image with it really does
     * not. this catches special characters in a field
     */
    if (ps->fieldstate) {
-      /* HACK: we should do something with these field characters... */
-      return 0;
+     if(pDocReader->_fieldProc(ps, eachchar, 0, 0x400))
+       return 0;
    }
    
    switch (eachchar) 
      {
-      case 0x01:
-	
+     case 0x01:
+
 	if (achp->fOle2) {
 	   // TODO: support embedded OLE2 components...
-	   UT_DEBUGMSG(("embedded OLE2 component. currently unsupported"));
+	   xxx_UT_DEBUGMSG(("embedded OLE2 component. currently unsupported"));
 	   return 0;
 	}
 	
@@ -259,7 +311,11 @@ int SpecCharProc(wvParseStruct *ps, U16 eachchar, CHP* achp)
 	  {
 	     pDocReader->_handleImage(&blip, picf.dxaGoal, picf.dyaGoal);
 	  }
-	
+	else
+	  {
+	    xxx_UT_DEBUGMSG(("Dom: strange no graphic data 1\n"));
+	  }
+
 	wvStream_goto(ps->data, pos);
 
 	return 0;
@@ -273,17 +329,28 @@ int SpecCharProc(wvParseStruct *ps, U16 eachchar, CHP* achp)
     	     {
  
 	     	fspa = wvGetFSPAFromCP(ps->currentcp, ps->fspa,
-		    			    ps->fspapos, ps->nooffspa);
-	     
+				       ps->fspapos, ps->nooffspa);
+
+		if(!fspa)
+		  {
+		    UT_DEBUGMSG(("No fspa! Panic and Insanity Abounds!\n"));
+		    return 0;
+		  }	     
+
 	     	if (wv0x08(&blip, fspa->spid, ps))
 	        {
 		    pDocReader->_handleImage(&blip, fspa->xaRight-fspa->xaLeft,
-					   fspa->yaBottom-fspa->yaTop);
+					     fspa->yaBottom-fspa->yaTop);
 	      	}
+		else
+		  {
+		    xxx_UT_DEBUGMSG(("Dom: strange no graphic data 2\n"));
+		    return 0;
+		  }
 	    }
 	  else
 	    {
-	      UT_DEBUGMSG(("nooffspa was <=0 -- ignoring"));
+	      xxx_UT_DEBUGMSG(("nooffspa was <=0 -- ignoring"));
 	    } 
 	  }
 	else
@@ -291,8 +358,7 @@ int SpecCharProc(wvParseStruct *ps, U16 eachchar, CHP* achp)
 	     UT_DEBUGMSG(("pre Word8 0x08 graphic -- unsupported at the moment"));
 	     fdoa = wvGetFDOAFromCP(ps->currentcp, NULL, ps->fdoapos, ps->nooffdoa);
 	     
-	     // TODO: do something with the data in this fdoa someday...
-	     
+	     // TODO: do something with the data in this fdoa someday...	     
 	  }
 	
 	return 0;
@@ -312,7 +378,7 @@ int DocProc(wvParseStruct *ps,wvTag tag)
 int ElementProc(wvParseStruct *ps,wvTag tag,void *props, int dirty)
 	{
 	IE_Imp_MsWord_97* pDocReader = (IE_Imp_MsWord_97 *) ps->userData;
-	UT_DEBUGMSG(("element tag = %d\n", tag));
+	xxx_UT_DEBUGMSG(("element tag = %d\n", tag));
 	return(pDocReader->_eleProc(ps, tag, props, dirty));
 	}
 
@@ -347,6 +413,143 @@ int IE_Imp_MsWord_97::_docProc(wvParseStruct * ps, wvTag tag)
 	return(0);
 	}
 
+int IE_Imp_MsWord_97::_handleCommandField(char *command)
+{
+      int ret = 0;
+      unsigned int tokenIndex = 0;
+      char *token = NULL;
+
+      xxx_UT_DEBUGMSG(("DOM: handleCommandField '%s'\n", command));
+
+      const XML_Char* atts[3];
+      atts[2] = NULL;
+
+      if (*command!= 0x13)
+	{
+	  UT_DEBUGMSG(("DOM: field did not begin with 0x13\n"));
+	  return 1;
+	}
+      strtok(command,"\t, ");
+      while((token = strtok(NULL,"\t, ")))
+	{
+	  tokenIndex = s_mapNameToToken(token);
+	  xxx_UT_DEBUGMSG(("DOM: token: %s %d\n", token, tokenIndex));
+	  switch (s_Tokens[tokenIndex].m_type)
+	    {
+	    case FC_TIME:
+	      xxx_UT_DEBUGMSG(("DOM: Field: time token\n"));
+	      ret = 1;
+
+	      atts[0] = "type";
+	      atts[1] = "time";
+
+	      /* 
+	       * Technically we should wait and append a formatted version of the time
+	       * field when we see FC_DateTimePicture, but this should be good enough
+	       */
+	      X_CheckError0(m_pDocument->appendObject(PTO_Field,atts));
+	      break;
+
+	      // TODO: get Abi to support more field types :-)
+
+	    default: 
+	      break;
+	    }
+	}
+
+      return ret;
+}
+
+/*
+ * Caolan had this set to 40000 in wv/field.c
+ * That seemed a bit excessive to me
+ */
+#define FLD_SZ 4096
+static U16 command[FLD_SZ];
+static U16 argumen[FLD_SZ];
+
+int IE_Imp_MsWord_97::_fieldProc(wvParseStruct *ps, U16 eachchar, U8 chartype, U16 lid)
+{
+	xxx_UT_DEBUGMSG(("DOM: fieldProc: %c %x\n", (char)eachchar, (int)eachchar));
+
+	/* 
+	 * The majority of this code has just been ripped out of
+	 * wv/field.c
+	 */
+
+	static U16 *which;
+	static int i,depth;
+	char *a;
+	static char *c = NULL;
+	static int ret;
+
+	if (eachchar == 0x13)
+	  {
+	    a = NULL;
+	    ret = 1;
+	    if (depth == 0)
+	      {
+		which = command;
+		command[0] = 0;
+		argumen[0] = 0;
+		i=0;
+	      }
+	    depth++;
+	  }
+	else if (eachchar == 0x14)
+	  {
+	    if (depth == 1)
+	      {
+		command[i] = 0;
+		c = wvWideStrToMB(command);
+		if (_handleCommandField(c))
+		  ret = 1;
+		else
+		  ret = 0;
+		
+		xxx_UT_DEBUGMSG(("DOM: Field: command %s, ret is %d\n", wvWideStrToMB(command), ret));
+		wvFree(c);
+		which = argumen;
+		i = 0;
+	      }
+	  }
+
+	if (i >= FLD_SZ)
+	  {
+	    UT_DEBUGMSG(("DOM: Something completely absurd in the fields implementation!\n"));
+	    UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+	    return 1;
+	  }
+
+	/* DOM: note to self: make sure wv's iconv problem is fixed */
+	if (chartype)
+	  which[i] = wvHandleCodePage(eachchar, lid);
+	else
+	  which[i] = eachchar;
+
+	i++;
+
+	if (eachchar == 0x15)
+	  {
+	    depth--;
+	    if (depth == 0)
+	      {
+		which[i] = 0;
+#if 0
+		/* only used for hyperlinks, which abi doesn't support */
+		/* but i'll leave the sample code in here so we don't have */
+		/* to go digging for it later */
+		a = wvWideStrToMB(argumen);
+		c = wvWideStrToMB(command);
+		wvHandleTotalField(c);
+		wvFree(a);
+		wvFree(c);
+#endif
+	      }
+	  }
+	return ret;
+}
+
 int IE_Imp_MsWord_97::_eleProc(wvParseStruct *ps, wvTag tag, void *props, int dirty)
 	{
 	XML_Char propBuffer[1024];
@@ -354,7 +557,7 @@ int IE_Imp_MsWord_97::_eleProc(wvParseStruct *ps, wvTag tag, void *props, int di
 	const XML_Char* propsArray[3];
 	   
 	propBuffer[0] = 0;
-	UT_DEBUGMSG(("element started\n"));
+	xxx_UT_DEBUGMSG(("element started\n"));
 	PAP *apap;
 	CHP *achp;
 	SEP *asep;
@@ -369,26 +572,26 @@ int IE_Imp_MsWord_97::_eleProc(wvParseStruct *ps, wvTag tag, void *props, int di
 		  m_iTextRunLength = 0;
 		  UT_ASSERT(iRes == 0);
 
-		   UT_DEBUGMSG(("section properties...\n"));
+		   xxx_UT_DEBUGMSG(("section properties...\n"));
 		   asep = (SEP*)props;
 
 		   // page margins
 		   // -left
 		   sprintf(propBuffer + strlen(propBuffer),
 			   "page-margin-left:%s;", 
-			   UT_convertToDimensionString(DIM_IN, (((float)asep->dxaLeft) / 1440), "1.4"));
+			   UT_convertInchesToDimensionString(DIM_IN, (((float)asep->dxaLeft) / 1440), "1.4"));
 		   // -right
 		   sprintf(propBuffer + strlen(propBuffer),
 			   "page-margin-right:%s;", 
-			   UT_convertToDimensionString(DIM_IN, (((float)asep->dxaRight) / 1440), "1.4"));
+			   UT_convertInchesToDimensionString(DIM_IN, (((float)asep->dxaRight) / 1440), "1.4"));
 		   // -top
 		   sprintf(propBuffer + strlen(propBuffer),
 			   "page-margin-top:%s;", 
-			   UT_convertToDimensionString(DIM_IN, (((float)asep->dyaTop) / 1440), "1.4"));
+			   UT_convertInchesToDimensionString(DIM_IN, (((float)asep->dyaTop) / 1440), "1.4"));
 		   // -left
 		   sprintf(propBuffer + strlen(propBuffer),
 			   "page-margin-bottom:%s;", 
-			   UT_convertToDimensionString(DIM_IN, (((float)asep->dyaBottom) / 1440), "1.4"));
+			   UT_convertInchesToDimensionString(DIM_IN, (((float)asep->dyaBottom) / 1440), "1.4"));
 
 		   // columns
 		   if (asep->ccolM1) {
@@ -398,14 +601,14 @@ int IE_Imp_MsWord_97::_eleProc(wvParseStruct *ps, wvTag tag, void *props, int di
 		      // gap between columns
 		      sprintf(propBuffer + strlen(propBuffer),
 			      "column-gap:%s;", 
-			      UT_convertToDimensionString(DIM_IN, (((float)asep->dxaColumns) / 1440), "1.4"));
+			      UT_convertInchesToDimensionString(DIM_IN, (((float)asep->dxaColumns) / 1440), "1.4"));
 		   }
 		   
 		   // space after section 
 		   // TODO: this is the gutter, right?
 		   sprintf(propBuffer + strlen(propBuffer),
 			   "section-space-after:%s;",
-			   UT_convertToDimensionString(DIM_IN, (((float)asep->dzaGutter) / 1440), "1.4"));
+			   UT_convertInchesToDimensionString(DIM_IN, (((float)asep->dzaGutter) / 1440), "1.4"));
 
 		   // remove trailing semi-colon
 		   propBuffer[strlen(propBuffer)-1] = 0;
@@ -424,7 +627,7 @@ int IE_Imp_MsWord_97::_eleProc(wvParseStruct *ps, wvTag tag, void *props, int di
 		   m_iTextRunLength = 0;
 		   UT_ASSERT(iRes == 0);
 
-		   UT_DEBUGMSG(("paragraph properties...\n"));
+		   xxx_UT_DEBUGMSG(("paragraph properties...\n"));
 		   apap = (PAP*)props;
 
 		   // break before paragraph?
@@ -477,19 +680,19 @@ int IE_Imp_MsWord_97::_eleProc(wvParseStruct *ps, wvTag tag, void *props, int di
 		   if (apap->dxaRight) {
 		      strcat(propBuffer, "margin-right:");
 		      sprintf(propBuffer + strlen(propBuffer),
-			      "%s;", UT_convertToDimensionString(DIM_IN, (((float)apap->dxaRight) / 1440), "1.4"));
+			      "%s;", UT_convertInchesToDimensionString(DIM_IN, (((float)apap->dxaRight) / 1440), "1.4"));
 		   }
 		   // -left
 		   if (apap->dxaLeft) {
 		      strcat(propBuffer, "margin-left:");
 		      sprintf(propBuffer + strlen(propBuffer),
-			      "%s;", UT_convertToDimensionString(DIM_IN, (((float)apap->dxaLeft) / 1440), "1.4"));
+			      "%s;", UT_convertInchesToDimensionString(DIM_IN, (((float)apap->dxaLeft) / 1440), "1.4"));
 		   }
 		   // -left first line (indent)
 		   if (apap->dxaLeft1) {
 		      strcat(propBuffer, "text-indent:");
 		      sprintf(propBuffer + strlen(propBuffer),
-			      "%s;", UT_convertToDimensionString(DIM_IN, (((float)apap->dxaLeft1) / 1440), "1.4"));
+			      "%s;", UT_convertInchesToDimensionString(DIM_IN, (((float)apap->dxaLeft1) / 1440), "1.4"));
 		   }
 		   // -top
 		   if (apap->dyaBefore) {
@@ -528,7 +731,7 @@ int IE_Imp_MsWord_97::_eleProc(wvParseStruct *ps, wvTag tag, void *props, int di
 		      for (int iTab = 0; iTab < apap->itbdMac; iTab++) {
 			 sprintf(propBuffer + strlen(propBuffer),
 				 "%s/",
-				 UT_convertToDimensionString(DIM_IN, (((float)apap->rgdxaTab[iTab]) / 1440), "1.4"));
+				 UT_convertInchesToDimensionString(DIM_IN, (((float)apap->rgdxaTab[iTab]) / 1440), "1.4"));
 			 switch (apap->rgtbd[iTab].jc) {
 			  case 1:
 			    strcat(propBuffer, "C,");
@@ -558,7 +761,7 @@ int IE_Imp_MsWord_97::_eleProc(wvParseStruct *ps, wvTag tag, void *props, int di
 		   propsArray[0] = pProps;
 		   propsArray[1] = propBuffer;
 		   propsArray[2] = NULL;
-		   UT_DEBUGMSG(("the paragraph propBuffer is %s\n",propBuffer));
+		   UT_DEBUGMSG(("the paragraph propBuffer is: '%s'\n",propBuffer));
 		   X_ReturnNoMemIfError(m_pDocument->appendStrux(PTX_Block, propsArray));
 		   break;
 
@@ -569,7 +772,7 @@ int IE_Imp_MsWord_97::_eleProc(wvParseStruct *ps, wvTag tag, void *props, int di
 		   m_iTextRunLength = 0;
 		   UT_ASSERT(iRes == 0);
 
-		   UT_DEBUGMSG(("character properties...\n"));
+		   xxx_UT_DEBUGMSG(("character properties...\n"));
 		   achp = (CHP*)props;
 
 		   // bold text
@@ -612,19 +815,34 @@ int IE_Imp_MsWord_97::_eleProc(wvParseStruct *ps, wvTag tag, void *props, int di
 		   } else {
 		     fname = wvGetFontnameFromCode(&ps->fonts, achp->ftcFE);
 		      UT_DEBUGMSG(("FE font id = %d\n", achp->ftcFE));
+		      {
+			   if(strlen(fname)>6)
+				 fname[6]='\0';
+			   const char *f=XAP_EncodingManager::cjk_word_fontname_mapping.getFirst(fname);
+			   if(f==fname)
+				 {
+				   FREEP(fname);
+				   fname=UT_strdup("song");
+				 }
+			   else
+				 {
+				   FREEP(fname);
+				   fname=UT_strdup(f ? f : "helvetic");
+				 }			   
+		      }
 		   }
 		   // there are times when we should use the third, Other font, 
 		   // and the logic to know when somehow depends on the
 		   // character sets or encoding types? it's in the docs.
 		   
 		   UT_ASSERT(fname != NULL);
-		   UT_DEBUGMSG(("font-family = %s\n", fname));
+		   xxx_UT_DEBUGMSG(("font-family = %s\n", fname));
 
 		    strcat(propBuffer, "font-family:");
 		    strcat(propBuffer, fname);
 		    strcat(propBuffer, ";");
 
-			FREEP(fname);
+		    FREEP(fname);
 		   
 		   // font size (hps is half-points)
 		   sprintf(propBuffer + strlen(propBuffer), 
@@ -645,7 +863,7 @@ int IE_Imp_MsWord_97::_eleProc(wvParseStruct *ps, wvTag tag, void *props, int di
 		   propsArray[0] = pProps;
 		   propsArray[1] = propBuffer;
 		   propsArray[2] = NULL;
-		   UT_DEBUGMSG(("the character propBuffer is %s\n",propBuffer));
+		   xxx_UT_DEBUGMSG(("the character propBuffer is %s\n",propBuffer));
 		   X_ReturnNoMemIfError(m_pDocument->appendFmt(propsArray));
 		   break;
 
@@ -658,9 +876,9 @@ int IE_Imp_MsWord_97::_eleProc(wvParseStruct *ps, wvTag tag, void *props, int di
 		       m_pTextRun[m_iTextRunLength-1] == UCS_FF)
 		     {
 			m_iTextRunLength--;
-			UT_DEBUGMSG(("section mark removed\n"));
+			xxx_UT_DEBUGMSG(("section mark removed\n"));
 		     }
-		   UT_DEBUGMSG(("section end\n"));
+		   xxx_UT_DEBUGMSG(("section end\n"));
 		   break;
 
 	        case CHARPROPEND: /* not needed */
@@ -668,7 +886,7 @@ int IE_Imp_MsWord_97::_eleProc(wvParseStruct *ps, wvTag tag, void *props, int di
 		default:
 		   break;
 	     }
-	UT_DEBUGMSG(("element ended\n"));
+	xxx_UT_DEBUGMSG(("element ended\n"));
 	return(0);
 	}
 
@@ -683,7 +901,7 @@ IE_Imp_MsWord_97::~IE_Imp_MsWord_97()
 IE_Imp_MsWord_97::IE_Imp_MsWord_97(PD_Document * pDocument)
 	: IE_Imp(pDocument)
 {
-	UT_DEBUGMSG(("constructed wv\n"));
+	xxx_UT_DEBUGMSG(("constructed wv\n"));
 	m_error = UT_OK;
    
 	// to increase the speed and efficiency of the important,
@@ -811,56 +1029,91 @@ UT_Bool IE_Imp_MsWord_97::SupportsFileType(IEFileType ft)
 void IE_Imp_MsWord_97::pasteFromBuffer(PD_DocumentRange * pDocRange,
 				       unsigned char * pData, UT_uint32 lenData)
 {
-	UT_DEBUGMSG(("TODO IE_Imp_MsWord_97::pasteFromBuffer\n"));
+	xxx_UT_DEBUGMSG(("TODO IE_Imp_MsWord_97::pasteFromBuffer\n"));
 }
 
 UT_Error IE_Imp_MsWord_97::_handleImage(Blip * b, long width, long height)
 {
-   int data;
-   
-   UT_ByteBuf * buf = new UT_ByteBuf();
+   int data = 0;
+   const char * mimetype = NULL;
    
+   UT_ByteBuf * buf = NULL;
+   IE_ImpGraphic * converter = NULL;
+   UT_Error err = UT_OK;
+
+   // short-circuit this method if we don't support
+   // the incoming format
    switch(b->type)
      {
+      case msoblipDIB:
+	// this is just a BMP file, so we'll use the BMP image importer
+	// to convert it to a PNG for us.
+	mimetype = UT_strdup("image/png");
+	break;
       case msoblipPNG:
 	// conveniently, PNG is the internal format, so we do nothing here
-	// but copy the bitstream
-	while (EOF != (data = getc((FILE*)(b->blip.bitmap.m_pvBits))))
-	  buf->append((UT_Byte*)&data, 1);
+ 	mimetype = UT_strdup("image/png");
 	break;
       case msoblipWMF:
       case msoblipEMF:
       case msoblipPICT:
       case msoblipJPEG:
-      case msoblipDIB:
       default:
 	// TODO: support other image types
-	DELETEP(buf);
-	return UT_OK;
-	break;
+	return UT_ERROR;
      }
 
+   buf = new UT_ByteBuf();
+
+   while (EOF != (data = getc((FILE*)(b->blip.bitmap.m_pvBits))))
+     buf->append((UT_Byte*)&data, 1);
+   
+   if(b->type == msoblipDIB)
+     {
+	// this is just a BMP file, so we'll use the BMP image importer
+	// to convert it to a PNG for us.
+	err = IE_ImpGraphic::constructImporter("", IEGFT_DIB, &converter);
+     }
+
+   if (err != UT_OK)
+     goto HandleImgEnd;
+
    XML_Char propBuffer[128];
    propBuffer[0] = 0;
    sprintf(propBuffer, "width:%fin; height:%fin", 
 	   (double)width / (double)1440, 
 	   (double)height / (double)1440);
+
    XML_Char propsName[32];
    propsName[0] = 0;
    sprintf(propsName, "image%d", m_iImageCount++);
-   const XML_Char* propsArray[3];
-	   
-   propsArray[0] = "PROPS";
+
+   const XML_Char* propsArray[5];
+   propsArray[0] = "props";
    propsArray[1] = propBuffer;
-   propsArray[2] = "DATAID";
+   propsArray[2] = "dataid";
    propsArray[3] = propsName;
    propsArray[4] = NULL;
 
+   UT_ByteBuf * pBBPNG;
+   if (converter == NULL) 
+     pBBPNG = buf;
+   else 
+     {
+       err = converter->convertGraphic(buf, &pBBPNG);
+       DELETEP(converter);
+       if (err != UT_OK) 
+	 goto HandleImgEnd;
+     }
+
    X_ReturnNoMemIfError(m_pDocument->appendObject(PTO_Image, propsArray));
-   X_CheckError0(m_pDocument->createDataItem(propsName, UT_FALSE,
-					     buf, NULL, NULL));
+   X_CheckError0(m_pDocument->createDataItem((char*)propsName, UT_FALSE,
+					     pBBPNG, (void*)mimetype, NULL));
+
+ HandleImgEnd:
+
+   // TODO: free mimetype??
 
    DELETEP(buf);
-
-   return UT_OK;
+   return err;
 }
