@@ -35,7 +35,8 @@
 #include "ie_types.h"
 #include "ie_imp_RTF.h"
 #include "pd_Document.h"
-
+#include "wv.h"
+#include "xap_EncodingManager.h"
 
 // Font table items
 RTFFontTableItem::RTFFontTableItem(FontFamilyEnum fontFamily, int charSet, int codepage, FontPitch pitch,
@@ -174,7 +175,8 @@ IE_Imp_RTF::IE_Imp_RTF(PD_Document * pDocument)
 	m_newParaFlagged(UT_FALSE),
 	m_newSectionFlagged(UT_FALSE),
 	m_cbBin(0),
-	m_pImportFile(NULL)
+	m_pImportFile(NULL),
+	deflangid(0)
 {
 	m_pPasteBuffer = NULL;
 	m_lenPasteBuffer = 0;
@@ -291,7 +293,7 @@ UT_Error IE_Imp_RTF::_parseFile(FILE* fp)
 					cNibble--;
 					if (!cNibble  &&  ok)
 					{
-						ok = ParseChar(b);
+						ok = ParseChar(b,0);
 						cNibble = 2;
 						b = 0;
 						m_currentRTFState.m_internalState = RTFStateStore::risNorm;
@@ -495,7 +497,7 @@ UT_Bool IE_Imp_RTF::PopRTFState(void)
 
 // Process a single character from the RTF stream
 //
-UT_Bool IE_Imp_RTF::ParseChar(UT_UCSChar ch)
+UT_Bool IE_Imp_RTF::ParseChar(UT_UCSChar ch,bool no_convert)
 {
     // Have we reached the end of the binary skip?
 	if (m_currentRTFState.m_internalState == RTFStateStore::risBin  && --m_cbBin <= 0)
@@ -517,7 +519,13 @@ UT_Bool IE_Imp_RTF::ParseChar(UT_UCSChar ch)
 			// Insert a character into the story
             if ((ch >= 32  ||  ch == 9 || ch == UCS_FF || ch == UCS_LF)  &&  !m_currentRTFState.m_charProps.m_deleted)
 			{
-				return AddChar(ch);
+				if (no_convert==0 && ch<=0xff) 
+				{	
+					wchar_t wc;
+					if (m_mbtowc.mbtowc(wc,(UT_Byte)ch))
+						return AddChar(wc);
+				} else
+					return AddChar(ch);
 			}
 		default:
 			// handle other destinations....
@@ -673,6 +681,12 @@ UT_Bool IE_Imp_RTF::TranslateKeyword(unsigned char* pKeyword, long param, UT_Boo
 	// NB. all RTF keywords are lowercase.
 	switch (*pKeyword)
 	{
+	case 'a':
+		if (strcmp((char*)pKeyword, "ansicpg") == 0)
+		{
+			m_mbtowc.setInCharset(XAP_EncodingManager::instance->charsetFromCodepage((UT_uint32)param));
+		}
+		break;		
 	case 'b':
 		if (strcmp((char*)pKeyword, "b") == 0)
 		{
@@ -706,7 +720,7 @@ UT_Bool IE_Imp_RTF::TranslateKeyword(unsigned char* pKeyword, long param, UT_Boo
 			// bold - either on or off depending on the parameter
 			return HandleDeleted(fParam ? UT_FALSE : UT_TRUE);
 		}
-
+		break;
 	case 'e':
 		if (strcmp((char*)pKeyword, "emdash") == 0)
 		{
@@ -1004,7 +1018,7 @@ UT_Bool IE_Imp_RTF::ApplyCharacterAttributes()
 {
 	XML_Char* pProps = "PROPS";
 	XML_Char propBuffer[1024];	//TODO is this big enough?  better to make it a member and stop running all over the stack
-	XML_Char tempBuffer[128];
+	XML_Char tempBuffer[130];
 
 	propBuffer[0] = 0;
 
@@ -1064,7 +1078,7 @@ UT_Bool IE_Imp_RTF::ApplyCharacterAttributes()
 	}
 
 	// font size
-	sprintf(tempBuffer, "; font-size:%dpt", (int)(m_currentRTFState.m_charProps.m_fontSize+0.5));
+	sprintf(tempBuffer, "; font-size:%spt", std_size_string((float)m_currentRTFState.m_charProps.m_fontSize));	
 	strcat(propBuffer, tempBuffer);
 	// typeface
 	RTFFontTableItem* pFont = GetNthTableFont(m_currentRTFState.m_charProps.m_fontNumber);
@@ -1138,7 +1152,7 @@ UT_Bool IE_Imp_RTF::ApplyParagraphAttributes()
 			UT_sint32 tabTwips = (UT_sint32)m_currentRTFState.m_paraProps.m_tabStops.getNthItem(i);
 			double tabIn = tabTwips/(20.0*72.0);
 
-			sprintf(tempBuffer, "%s/L", UT_convertToDimensionString(DIM_IN,tabIn,"04"));	// TODO - left tabs only
+			sprintf(tempBuffer, "%s/L", UT_convertInchesToDimensionString(DIM_IN,tabIn,"04"));	// TODO - left tabs only
 			strcat(propBuffer, tempBuffer);
 		}
 
@@ -1167,15 +1181,15 @@ UT_Bool IE_Imp_RTF::ApplyParagraphAttributes()
 	strcat(propBuffer, "; ");
 
 	// indents - first, left and right, top and bottom
-	sprintf(tempBuffer, "margin-top:%s; ",		UT_convertToDimensionString(DIM_IN, (double)m_currentRTFState.m_paraProps.m_spaceBefore/1440));
+	sprintf(tempBuffer, "margin-top:%s; ",		UT_convertInchesToDimensionString(DIM_IN, (double)m_currentRTFState.m_paraProps.m_spaceBefore/1440));
 	strcat(propBuffer, tempBuffer);
-	sprintf(tempBuffer, "margin-bottom:%s; ",	UT_convertToDimensionString(DIM_IN, (double)m_currentRTFState.m_paraProps.m_spaceAfter/1440));
+	sprintf(tempBuffer, "margin-bottom:%s; ",	UT_convertInchesToDimensionString(DIM_IN, (double)m_currentRTFState.m_paraProps.m_spaceAfter/1440));
 	strcat(propBuffer, tempBuffer);
-	sprintf(tempBuffer, "margin-left:%s; ",		UT_convertToDimensionString(DIM_IN, (double)m_currentRTFState.m_paraProps.m_indentLeft/1440));
+	sprintf(tempBuffer, "margin-left:%s; ",		UT_convertInchesToDimensionString(DIM_IN, (double)m_currentRTFState.m_paraProps.m_indentLeft/1440));
 	strcat(propBuffer, tempBuffer);
-	sprintf(tempBuffer, "margin-right:%s; ",	UT_convertToDimensionString(DIM_IN, (double)m_currentRTFState.m_paraProps.m_indentRight/1440));
+	sprintf(tempBuffer, "margin-right:%s; ",	UT_convertInchesToDimensionString(DIM_IN, (double)m_currentRTFState.m_paraProps.m_indentRight/1440));
 	strcat(propBuffer, tempBuffer);
-	sprintf(tempBuffer, "text-indent:%s; ",		UT_convertToDimensionString(DIM_IN, (double)m_currentRTFState.m_paraProps.m_indentFirst/1440));
+	sprintf(tempBuffer, "text-indent:%s; ",		UT_convertInchesToDimensionString(DIM_IN, (double)m_currentRTFState.m_paraProps.m_indentFirst/1440));
 	strcat(propBuffer, tempBuffer);
 	
 	// line spacing
@@ -1383,10 +1397,12 @@ UT_Bool IE_Imp_RTF::ReadOneFontFromTable()
 	// Now (possibly) comes some optional keyword before the fontname
 	if (!ReadCharFromFile(&ch))
 		return UT_FALSE;
+	int nesting = 0;
 	while (ch == '\\'  ||  ch == '{')
 	{
 		if (ch == '{')
 		{
+			++nesting;
 			if (!ReadCharFromFile(&ch))
 				return UT_FALSE;
 		}
@@ -1419,6 +1435,7 @@ UT_Bool IE_Imp_RTF::ReadOneFontFromTable()
 				unsigned char val = (unsigned char)(atoi((char*)buf));
 				panose[i] = val;
 			}
+			--nesting;/*since this loop will break on '}'*/
 		}
 
 		//TODO - handle the other keywords
@@ -1426,27 +1443,45 @@ UT_Bool IE_Imp_RTF::ReadOneFontFromTable()
 		if (!ReadCharFromFile(&ch))
 			return UT_FALSE;
 	}
-
+	//we fall back here when space between parameter of keyword and font name
+	//is seen
 	// Now comes the font name, terminated by either a close brace or a slash or a semi-colon
 	int count = 0;
-	while ( ch != '}'  &&  ch != '\\'  &&  ch != ';' )
+	/*
+	    FIXME: CJK font names come in form \'aa\'cd\'ef - so we have to 
+	    parse \'HH correctly (currently we ignore them!) - VH
+	*/
+	while ( ch != '}'  &&  ch != '\\'  &&  ch != ';' && ch!= '{')
 	{
 		keyword[count++] = ch;
 		if (!ReadCharFromFile(&ch))
 			return UT_FALSE;
 	}
+	if (ch=='{')
+		++nesting;
+		
 	keyword[count] = 0;
+	/*work around "helvetica" font name -replace it with "Helvetic"*/
+	if (!UT_stricmp((const char*)keyword,"helvetica"))
+		strcpy((char*)keyword,"Helvetic");
 
 	if (!UT_cloneString(pFontName, (char*)keyword))
 	{
 		// TODO outofmem
 	}			
-
-	// Munch the remaining control words down to the close brace
-	while (ch != '}')
+	for(int i=0;i<=nesting;++i)
 	{
-		if (!ReadCharFromFile(&ch))
-			return UT_FALSE;
+		// Munch the remaining control words down to the close brace
+		while (ch != '}')
+		{
+			if (!ReadCharFromFile(&ch))
+				return UT_FALSE;
+			if (ch=='{')
+				++nesting;
+		}
+		if (nesting>0 && i!=nesting) //we need to skip '}' we've just seen.
+			if (!ReadCharFromFile(&ch))
+				return UT_FALSE;		
 	}
 
 	// Create the font entry and put it into the font table
